package br.gov.arquitetura.bean;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.faces.bean.ManagedBean;
import javax.faces.bean.SessionScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpSession;

import org.primefaces.component.menuitem.MenuItem;
import org.primefaces.component.submenu.Submenu;
import org.primefaces.model.DefaultMenuModel;
import org.primefaces.model.MenuModel;

import br.gov.arquitetura.comparator.ComparatorMenu;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.mc.seg.client.consulta.entidade.Grupo;
import br.gov.mc.seg.client.consulta.entidade.Menu;
import br.gov.mc.seg.client.consulta.entidade.Pagina;
import br.gov.mc.seg.client.consulta.entidade.Usuario;
import br.gov.mc.seg.client.web.Constants;

@ManagedBean(name = "SEGBean")
@SessionScoped
public class SEGBean {

    private MenuModel model;
    private String nome;

    /**
     * Apartir do usuário logado na sessão busca seus grupos e cria o menu.
     * 
     * @return
     */
    public MenuModel getMenuModel() {
	if (model == null) {
	    ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
	    HttpSession session = (HttpSession) context.getSession(false);
	    Usuario usuario = (Usuario) session.getAttribute(Constants.USER_PARAMETER);
	    if (usuario != null) {
	    	Menu[] menus = this.agregaMenus(usuario.getGrupos());
	    	model = this.createMenu(menus);
	    }
	}
	return model;
    }

    public void carregarNome() {
	if (nome == null || nome.trim().equals("")) {
	    ExternalContext context = FacesContext.getCurrentInstance().getExternalContext();
	    HttpSession session = (HttpSession) context.getSession(false);
	    Usuario usuario = (Usuario) session.getAttribute(Constants.USER_PARAMETER);
	    if (usuario != null) {
		nome = usuario.getNome();
		if(nome == null){
		    nome = usuario.getLogin();    
		}
	    }
	}
    }

    /**
     * Cria o menu apartir da lista de menus já agregada. Usando o MenuModel
     * doPrimeFaces
     * 
     * @param items
     * @return Menu renderizado pelo primefaces
     */
    private MenuModel createMenu(Menu[] items) {
	MenuModel modelSeg = new DefaultMenuModel();

	try {
	    if ((items == null) || (items.length == 0)) {
		return null;
	    }

	    List<Menu> list = Arrays.asList(items);

	    int count = 0;
	    for (Menu m : sort(list)) {
		// Validar o menu
		if (!validaMenu(m)) {
		    continue;
		}

		// Adiciona menu que não possuem ação e possuem submenus
		if ((m.getSubMenus() != null) && (Arrays.asList(m.getSubMenus()).size() > 0)) {
		    Submenu subMenu = new Submenu();
		    subMenu.setId("subMenu" + count++);
		    subMenu.getChildren().addAll(this.getSubMenus(subMenu, m.getSubMenus()));
		    subMenu.setLabel(m.getNome());
		    modelSeg.addSubmenu(subMenu);
		} else {
		    // Adiciona menus de ação, ou seja, que não possuem
		    // submenus
		    MenuItem menuItem = new MenuItem();
		    menuItem.setId("menuItem" + count++);
		    if (m.getPagina() != null) {
			menuItem.setValue(m.getNome());
			menuItem.setUrl(m.getPagina().getUrl());

		    } else {
			menuItem.setValue(m.getNome());
			menuItem.setUrl(m.getUrl());
			// Devido a necessidade do sistema ASPAR, deverá abrir
			// o documento em uma
			// nova pagina. - Como não existe campo para setar esta
			// funcionalidade no segWeb
			// foi definido pelo nome do menu que sera aberto em uma
			// nova pagina.
			if (m.getNome().equalsIgnoreCase("Ajuda")) {
			    menuItem.setTarget("_blank");
			}
		    }
		    modelSeg.addMenuItem(menuItem);
		}
	    }
	    // Adiciona o menu sair ao final de tudo
	    modelSeg.addMenuItem(adicionarMenuSair(count));
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	}
	return modelSeg;
    }

    /**
     * Adiciona item de menu Sair sem a necessidade do menu Sistema permitindo
     * assim a saída direta do sistema.
     * 
     * @param Contador
     *            para definir o ID do item de menu
     * @return Item de menu de primeiro nível
     */
    private MenuItem adicionarMenuSair(int count) {
	MenuItem menuItem = new MenuItem();
	menuItem.setId("menuItem" + ++count);
	menuItem.setValue("Sair");
	menuItem.setUrl("/sair.jsf");
	return menuItem;
    }

    /**
     * Adiciona menu Sistema com o item Sair. Método utilizado quando o menu
     * sistema se faz necessário para sair do mesmo.
     * 
     * @param menus
     * @return Menu Sistema com item de menu Sair
     */
    @SuppressWarnings("unused")
    private Menu[] adicionarMenuSair(Menu[] menus) {
	if (menus.length > 0) {

	    // Aumenta o tamanho do array
	    Menu[] arrayMenu = new Menu[menus.length + 1];

	    // Adiciona os item no array novo
	    int cont = 0;
	    for (Menu item : menus) {
		arrayMenu[cont++] = item;
	    }

	    // Pega o ultimo index
	    // Sort o ultimo registro vai ter o maior index
	    Arrays.sort(menus, new ComparatorMenu());

	    Menu menu = new Menu();
	    // Pega o ultimo registro do array com o maior index e adicionar +1
	    if (arrayMenu.length > 0) {
		menu.setIndex(arrayMenu[menus.length - 1].getIndex() + 1);
		menu.setNome("Sistema");
	    }

	    Pagina pagina = new Pagina();
	    pagina.setDescricao("Sair");
	    pagina.setUrl("/sair.jsf");

	    Menu subMenu = new Menu();
	    subMenu.setIndex(0);
	    subMenu.setNome("Sair");
	    subMenu.setPagina(pagina);

	    Menu[] subMenus = new Menu[1];
	    subMenus[0] = subMenu;

	    menu.setSubMenus(subMenus);

	    arrayMenu[menus.length] = menu;
	    return arrayMenu;
	} else {
	    return null;
	}

    }

    /**
     * Adiciona os submenus ao menu principal.
     * 
     * @param subMenu
     * @param subMenus
     * @return Lista de itens de menu
     */
    public List<MenuItem> getSubMenus(Submenu subMenu, Menu[] subMenus) {

	List<MenuItem> menus = new ArrayList<MenuItem>();

	List<Menu> list = Arrays.asList(subMenus);

	int cont = 0;
	for (Menu menu : sort(list)) {
	    MenuItem menuItem = new MenuItem();

	    menuItem.setId("menuItem" + cont++ + subMenu.getId());
	    menuItem.setValue(menu.getNome());
	    // Pagina
	    if (menu.getPagina() != null) {
		menuItem.setUrl(menu.getPagina().getUrl());
	    } else {
		menuItem.setUrl(menu.getUrl());
	    }

	    subMenu.getChildren().add(menuItem);

	}

	return menus;

    }

    /**
     * Recupera o usuário da sessão
     * 
     * @return Usuario do Seg
     */
    public Usuario getUsuario() {
	FacesContext context = FacesContext.getCurrentInstance();
	HttpSession session = (HttpSession) context.getExternalContext().getSession(false);
	Usuario usuario = new Usuario();
	usuario.setAtivo(true);
	usuario = (Usuario) session.getAttribute(Constants.USER_PARAMETER);
	return usuario;
    }

    /**
     * Agrega os menus de grupos diferentes, mesclado-os
     * 
     * @param grupos
     * @return Array de menus
     */
    private Menu[] agregaMenus(Grupo[] grupos) {
	List<Menu[]> menus = new ArrayList<Menu[]>();
	if (true) {
	    return null;
	}
	// Adicionando os menus ‡ uma lista comum
	for (Grupo grupo : grupos) {
	    menus.add(grupo.getMenus());
	}
	// Lista que conter· os menus j· agregados
	List<Menu> menu = new ArrayList<Menu>();
	// Lista que possuir· os ids dos menus j· adicionados com a finalidade
	// de controle
	List<Integer> ids = new ArrayList<Integer>();
	for (Menu[] m : menus) {
	    for (Menu mn : m) {
		addMenu(menu, ids, mn);
	    }
	}
	return toArray(menu);
    }

    /**
     * Adiciona na lista de menus o menu corrente e seus submenus
     * 
     * @param menus
     * @param ids
     * @param mn
     */
    private void addMenu(List<Menu> menus, List<Integer> ids, Menu mn) {
	// Se o menu já existir é necessário remove-lo para que não haja
	// deuplicação
	if (!ids.contains(mn.getId())) {
	    // Controle para verificar se o menu j· existe tendo em vista que
	    // são objetos diferentes
	    ids.add(mn.getId());
	    // Adicionando submenus se houver
	    this.addSubMenu(mn, null);
	    // Adicionando o menu corrente a lista de menus
	    menus.add(mn);
	} else {
	    Menu menu = null;
	    for (Menu m : menus) {
		if (m.getId().intValue() == mn.getId().intValue()) {
		    menu = m;
		}
	    }
	    this.addSubMenu(mn, menu);
	}
    }

    /**
     * Adiciona SubMenu
     * 
     * @param mn
     * @param destino
     */
    private void addSubMenu(Menu mn, Menu destino) {
	// Lista que conter· os submenus
	List<Menu> subs = new ArrayList<Menu>();
	// SubMenus atuais
	List<Menu> subMenus = this.asList(mn.getSubMenus());
	// Ids dos submenus com a finalidade de controle
	List<Integer> idsSubMenus = new ArrayList<Integer>();
	if (destino != null) {
	    for (Menu menu : this.asList(destino.getSubMenus())) {
		idsSubMenus.add(menu.getId());
	    }
	}
	for (Menu subMenu : subMenus) {
	    addMenu(subs, idsSubMenus, subMenu);
	}

	if (destino == null) {
	    // setando no menu corrente seus respectivos submenus
	    mn.setSubMenus(toArray(subs));
	} else {
	    // caso o menu pai já exista
	    List<Menu> list = this.asList(destino.getSubMenus());
	    list.addAll(subs);
	    destino.setSubMenus(toArray(list));
	}
    }

    /**
     * Método que transforma um list num array. Foi necessário pois os
     * métodos já existentes em java.util.Arrays não atendia as necessidades
     * 
     * @param menus
     * @return
     */
    private Menu[] toArray(List<Menu> lista) {
	Menu[] menus = new Menu[lista.size()];
	for (int i = 0; i < lista.size(); i++) {
	    menus[i] = lista.get(i);
	}
	return menus;
    }

    /**
     * Método que transforma um list num array. Foi necessário pois os
     * métodos já existentes em java.util.Arrays não atendia as necessidades
     * 
     * @param menus
     * @return
     */
    private List<Menu> asList(Menu[] menus) {
	List<Menu> retorno = new ArrayList<Menu>();
	for (Menu menu : menus) {
	    retorno.add(menu);
	}
	return retorno;
    }

    /**
     * Ordena o menu pela ordem cadastrada no SEG
     * 
     * @param items
     * @return
     */
    protected Collection<Menu> sort(Collection<Menu> items) {
	List<Menu> ordered = new ArrayList<Menu>();
	ordered.addAll(items);

	Collections.sort(ordered, new Comparator<Menu>() {

	    public int compare(Menu m1, Menu m2) {
		if (m1.getIndex() != null) {
		    if (m2.getIndex() != null) {
			return m1.getIndex().compareTo(m2.getIndex());
		    }
		    return -1;
		} else if (m2.getIndex() != null) {
		    return 1;
		}
		return 0;
	    }

	});

	return ordered;
    }

    /**
     * Verifica se o menu possui alguma ação
     * 
     * @param m
     * @return
     */
    protected boolean validaMenu(Menu m) {
	if (m == null) {
	    return false;
	}
	if (((m.getUrl() == null) || (m.getUrl().trim().length() == 0)) && (m.getPagina() == null)) {
	    if ((m.getSubMenus() == null) || (m.getSubMenus().length == 0)) {
		return false;
	    }
	}
	return true;
    }

    public String getNome() {
	return nome;
    }

    public void setNome(String nome) {
	this.nome = nome;
    }

    public MenuModel getModel() {
	return model;
    }

    public void setModel(MenuModel model) {
	this.model = model;
    }

}