package br.gov.arquitetura.excecao;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.util.Date;

import javax.faces.context.FacesContext;
import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import br.gov.arquitetura.mail.SimpleMail;
import br.gov.arquitetura.mail.SimpleMailSender;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.DateUtil;
import br.gov.arquitetura.util.SpringUtil;

/**
 * Classe usada para tratamento de exceo <br>
 * <li>Escreve o erro no log</li> <li>Direciona para tela de erro
 * <b>error.jsf</b></li>
 * 
 * @author Igor Galvo
 */
public class ExcecaoUtil {

	private static Log log = LogFactory.getLog(ExcecaoUtil.class);

	private SimpleMailSender simpleMailSender;

	private SimpleMail simpleMail;

	/**
	 * Escreve informaes da exceo no log e direciona para pgina de erro.
	 * 
	 * @param Exception
	 *            e
	 */

	public ExcecaoUtil() {
		simpleMailSender = (SimpleMailSender) SpringUtil.getBean("simpleMailSender", "applicationContext-service.xml");
		simpleMail = (SimpleMail) SpringUtil.getBean("simpleMail", "applicationContext-service.xml");
	}

	public static void tratarExcecao(Exception e) {
		if (e == null) {
			log.error("Data Hora:" + DateUtil.getDataHora(new java.util.Date(), "dd/MM/yyyy HH:mm:ss"));
			log.error("-->Exceo ocorrida  nula");
			return;
		}

		log.error("Data Hora:" + DateUtil.getDataHora(new java.util.Date(), "dd/MM/yyyy HH:mm:ss"));
		log.error("Exceo do tipo: " + e.getClass().getSimpleName());
		log.error("printStackTrace", e);
		try {
			if (FacesContext.getCurrentInstance() != null
					&& FacesContext.getCurrentInstance().getExternalContext() != null) {

				FacesContext context = FacesContext.getCurrentInstance();

				ServletContext servletContext = (ServletContext) context.getExternalContext().getContext();

				String path = servletContext.getContextPath();

				context.getExternalContext().redirect(path + "/pages/error/error.jsf");

				if (ConstantesARQ.ENVIAR_EMAIL_ERRO) {
					ExcecaoUtil excecao = new ExcecaoUtil();
					excecao.enviarEmail(printStackTraceToString(e));
				}

			} else {
				log.error("Data Hora:" + DateUtil.getDataHora(new java.util.Date(), "dd/MM/yyyy HH:mm:ss"));
				log.error("-->Exceo ocorrida fora do Context JSF");
			}
		} catch (IOException ioe) {
			log.error("Data Hora:" + DateUtil.getDataHora(new java.util.Date(), "dd/MM/yyyy HH:mm:ss"));
			log.error("-->No foi possvel redirecionar para tela: error.jsf");
			log.error("", ioe);
		} finally {
			e = null;
		}
	}

	private void enviarEmail(String error) {
		simpleMail.setSubject(ConstantesARQ.ASSUNTO_EMAIL_ERRO);
		simpleMail.setText(error);

		simpleMail.addTo(ConstantesARQ.EMAIL_ERRO);
		simpleMailSender.send(simpleMail);

		simpleMail.setSubject("");
		simpleMail.setText("");
		simpleMail.addTo("");

	}

	private static String printStackTraceToString(Exception e) {
		StringWriter sw = new StringWriter();
		PrintWriter pw = new PrintWriter(sw);
		e.printStackTrace(pw);

		return "<font color='red'>Data e Hora: " + DateUtil.getDataHora(new Date(), "dd/MM/yyyy HH:mm:ss")
				+ "</font><br/><br/>" + "------<br/>" + sw.toString() + "------<br/>";
	}
}
