package br.gov.arquitetura.service.impl;

import java.io.Serializable;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.arquitetura.DAO.UniversalDAO;
import br.gov.arquitetura.service.UniversalManager;

/*MANDATORY
 Support a current transaction, throw an exception if none exists.
 NESTED
 Execute within a nested transaction if a current transaction exists, behave like PROPAGATION_REQUIRED else.
 NEVER
 Execute non-transactionally, throw an exception if a transaction exists.
 NOT_SUPPORTED
 Execute non-transactionally, suspend the current transaction if one exists.
 REQUIRED
 Support a current transaction, create a new one if none exists.
 REQUIRES_NEW
 Create a new transaction, suspend the current transaction if one exists.
 SUPPORTS
 Support a current transaction, execute non-transactionally if none exists.*/

/**
 * Readonly : true - FlushMode -MANUAL 
 * A sesso s  liberado quando j Session.flush ()  explicitamente chamado pelo aplicativo.<br>
 *  Este modo  muito eficiente para transaes de leitura apenas.
 */
/**
 * Readonly : False - FlushMode -AUTO
 * A sesso  s vezes lavada antes da execuo da consulta, a fim de garantir <br>
 *  que nunca consultas retornam estado obsoleto. Este  o modo padro flush.
 */

/**
 * rollbackFor : defini que tipo de execo ira dar um rollback
 */
/**
 * noRollbackFor : defini que tipo de execo NO ira dar um rollback
 */

@Service(value = "universalManager")
@Transactional(readOnly = true, rollbackFor = Exception.class, propagation = Propagation.SUPPORTS)
public class UniversalManagerImpl implements UniversalManager {

	@Autowired(required = true)
	@Qualifier(value = "universalDAO")
	protected UniversalDAO dao;

	
	@Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
	public void remove(Class clazz, Serializable id) throws Exception {
		dao.remove(clazz, id);
	}

	
	@Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
	public Object save(Object o) throws Exception {
		return dao.save(o);
	}

	
	public Object get(Class clazz, Serializable id) {
		return dao.get(clazz, id);
	}

	
	public List getAll(Class clazz) {
		return dao.getAll(clazz);
	}

	
	public List listBy(Object o) throws Exception {
		return dao.listBy(o);
	}

	/**
	 * Paginacao
	 * 
	 * @param first
	 * @param pageSize
	 * @param sortField
	 * @param sortOrder
	 * @param filters
	 * @param o
	 * @return
	 */
	
	public List paginate(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filters,
			Class clazz) throws Exception {
		return dao.paginate(first, pageSize, sortField, sortOrder, filters, clazz);
	}

	
	public Long getCount(String sortField, boolean sortOrder, Map<String, String> filters, Class clazz)
			throws Exception {
		return dao.getCount(sortField, sortOrder, filters, clazz);
	}

	
	public Long getCount(Class clazz, Map filters) throws Exception {

		return dao.getCount(clazz, filters);
	}
}
