package br.gov.mc.arquitetura.converter;

import java.lang.reflect.Array;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import br.gov.mc.arquitetura.ws.client.entidade.Cargo;
import br.gov.mc.arquitetura.ws.client.entidade.CorRaca;
import br.gov.mc.arquitetura.ws.client.entidade.Email;
import br.gov.mc.arquitetura.ws.client.entidade.EmailPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Entidade;
import br.gov.mc.arquitetura.ws.client.entidade.EntidadePessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Escolaridade;
import br.gov.mc.arquitetura.ws.client.entidade.Pessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Profissao;
import br.gov.mc.arquitetura.ws.client.entidade.Servico;
import br.gov.mc.arquitetura.ws.client.entidade.TipoEmail;
import br.gov.mc.arquitetura.ws.client.entidade.TipoEndereco;
import br.gov.mc.arquitetura.ws.client.entidade.TipoTelefone;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpCorRaca;
import br.gov.mc.corporativo.entidade.CrpEscolaridade;
import br.gov.mc.corporativo.entidade.CrpPessoa;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpProfissao;
import br.gov.mc.corporativo.entidade.CrpQualificacao;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;
import br.gov.mc.corporativo.entidade.CrpServico;
import br.gov.mc.corporativo.entidade.CrpTipoEndereco;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;

public class ConverterSupport {

	@SuppressWarnings("unchecked")
	public static <T> T[] toArray(Collection<T> collection) {
		T[] array = null;
		if (collection != null && !collection.isEmpty()) {
			Iterator<T> iterator = collection.iterator();
			T object = iterator.next();
			while (iterator.hasNext() && object == null) {
				object = iterator.next();
			}
			if (object != null) {
				array = (T[]) Array.newInstance(object.getClass(), 0);
				array = collection.toArray(array);
			}
		}
		return array;
	}

	public static TipoEndereco convert(CrpTipoEndereco tipoEndereco) {
		TipoEndereco t = null;
		if (tipoEndereco != null) {
			t = new TipoEndereco();
			t.setId(tipoEndereco.getId());
			t.setDescricao(tipoEndereco.getDescricao());
		}
		return t;
	}

	public static CrpTipoEndereco convert(TipoEndereco tipoEndereco) {
		CrpTipoEndereco t = null;
		if (tipoEndereco != null) {
			t = new CrpTipoEndereco();
			t.setId(tipoEndereco.getId());
			t.setDescricao(tipoEndereco.getDescricao());
		}
		return t;
	}

	public static TipoTelefone convert(CrpTipoTelefone tipoTelefone) {
		TipoTelefone t = null;
		if (tipoTelefone != null) {
			t = new TipoTelefone();
			t.setId(tipoTelefone.getId());
			t.setDescricao(tipoTelefone.getDescricao());
		}
		return t;
	}

	public static CrpTipoTelefone convert(TipoTelefone tipoTelefone) {
		CrpTipoTelefone t = null;
		if (tipoTelefone != null) {
			t = new CrpTipoTelefone();
			t.setId(tipoTelefone.getId());
			t.setDescricao(tipoTelefone.getDescricao());
		}
		return t;
	}

	public static CrpCorRaca convert(CorRaca corRaca) {
		CrpCorRaca c = null;
		if (corRaca != null) {
			c = new CrpCorRaca();
			c.setId(corRaca.getId());
			c.setNomCorRaca(corRaca.getNomCorRaca());
		}
		return c;
	}

	public static CrpEscolaridade convert(Escolaridade escolaridade) {
		CrpEscolaridade c = null;
		if (escolaridade != null) {
			c = new CrpEscolaridade();
			c.setId(escolaridade.getId());
			c.setDscEscolaridade(escolaridade.getDscEscolaridade());
		}
		return c;
	}

	public static CrpProfissao convert(Profissao profissao) {
		CrpProfissao c = null;
		if (profissao != null) {
			c = new CrpProfissao();
			c.setId(profissao.getId());
			c.setDscProfissao(profissao.getDscProfissao());
		}
		return c;
	}

	public static EntidadePessoa convert(CrpRepresentantePesJur representante) {
		EntidadePessoa e = null;
		if (representante != null) {
			e = new EntidadePessoa();
			e.setId(representante.getId());

			e.setPessoa(new Pessoa(representante.getPessoaFisica() != null ? representante.getPessoaFisica().getId()
							: null));

			e.setEntidade(new Entidade(representante.getPessoaJuridica() != null ? representante.getPessoaJuridica()
							.getId() : null));

			e.setCargo(convert(representante.getQualificacao()));
		}
		return e;
	}

	public static List<EntidadePessoa> convert(CrpRepresentantePesJur[] representantes) {
		List<EntidadePessoa> set = new ArrayList<EntidadePessoa>();
		if (representantes != null) {
			for (CrpRepresentantePesJur rep : representantes) {
				set.add(convert(rep));
			}
		}
		return set;
	}

	public static CrpRepresentantePesJur convert(EntidadePessoa entidadePessoa) {
		CrpRepresentantePesJur rpj = null;
		if (entidadePessoa != null) {
			rpj = new CrpRepresentantePesJur();
			rpj.setId(entidadePessoa.getId());
			rpj.setPessoaFisica(new CrpPessoaFisica(entidadePessoa.getPessoa() != null ? entidadePessoa.getPessoa()
							.getId() : null));
			rpj.setPessoaJuridica(new CrpPessoaJuridica(entidadePessoa.getEntidade() != null ? entidadePessoa
							.getEntidade().getId() : null));
			rpj.setQualificacao(convert(entidadePessoa.getCargo()));
			rpj.setDataInicio(new Date());
			rpj.setRepresentante(true);
			rpj.setSituacaoEspecial(null);
		}
		return rpj;
	}

	public static Set<CrpRepresentantePesJur> convert(EntidadePessoa[] entidadesPessoa) {
		Set<CrpRepresentantePesJur> set = new HashSet<CrpRepresentantePesJur>();
		if (entidadesPessoa != null) {
			for (EntidadePessoa ep : entidadesPessoa) {
				set.add(convert(ep));
			}
		}
		return set;
	}

	public static Servico convert(CrpServico servico) {
		Servico s = null;
		if (servico != null) {
			s = new Servico();
			s.setId(servico.getId());
			s.setDescricao(servico.getDescricao());
			s.setNumero(servico.getNumero() != null ? servico.getNumero().doubleValue() : null);
			s.setSigla(servico.getSigla());
		}
		return s;
	}

	public static Cargo convert(CrpQualificacao qualificacao) {
		Cargo c = null;
		if (qualificacao != null) {
			c = new Cargo();
			c.setId(c.getId() != null ? Integer.valueOf(qualificacao.getId()) : null);
			c.setDescricao(qualificacao.getDescricao());
		}
		return c;
	}

	public static CrpQualificacao convert(Cargo cargo) {
		CrpQualificacao q = null;
		if (cargo != null) {
			q = new CrpQualificacao();
			q.setId(q.getId() != null ? cargo.getId().toString() : null);
			q.setDescricao(cargo.getDescricao());
		}
		return q;
	}

	public static CrpContatoEmail convert(Email email) {
		CrpContatoEmail ce = null;

		if (email != null) {
			ce = new CrpContatoEmail();
			ce.setDescricao(email.getDescricao());
			ce.setEmailPessoal(false);
			ce.setId(email.getId());
		}

		return ce;
	}

	public static Email convert(CrpContatoEmail ce) {
		Email email = null;

		if (ce != null) {
			email = new Email();
			email.setDescricao(ce.getDescricao());
			email.setId(ce.getId());
			email.setTipo(new TipoEmail(1));
		}

		return email;
	}

	public static CrpContatoEmail convert(EmailPessoa ep) {
		CrpContatoEmail ce = null;

		if (ep != null) {
			ce = new CrpContatoEmail();
			ce.setDescricao(ep.getDescricao());
			ce.setEmailPessoal(false);
			ce.setId(ep.getId());
			ce.setPessoa(new CrpPessoa(ep.getPessoa().getId()));

		}

		return ce;
	}

	public static EmailPessoa convertEmailPessoa(CrpContatoEmail ce) {
		EmailPessoa ep = null;

		if (ce != null) {
			ep = new EmailPessoa();
			ep.setDescricao(ce.getDescricao());
			ep.setId(ce.getId());
			ep.setPessoa(new Pessoa(ce.getPessoa().getId()));
			ep.setTipo(new TipoEmail(1));
		}

		return ep;
	}
}