package br.gov.mc.arquitetura.converter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import br.gov.mc.arquitetura.ws.client.entidade.EmailEntidade;
import br.gov.mc.arquitetura.ws.client.entidade.EnderecoEntidade;
import br.gov.mc.arquitetura.ws.client.entidade.Entidade;
import br.gov.mc.arquitetura.ws.client.entidade.Fistel;
import br.gov.mc.arquitetura.ws.client.entidade.TelefoneEntidade;
import br.gov.mc.arquitetura.ws.client.entidade.TipoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridicaAnatel;

public class EntidadeWSConverter extends ConverterSupport {

	public static Entidade convert(CrpPessoaJuridica pessoa) {
		Entidade e = null;
		if (pessoa != null) {
			e = new Entidade();
			e.setId(pessoa.getId());
			e.setCnpj(pessoa.getCnpj());
			e.setRazaoSocial(pessoa.getRazaoSocial());
			e.setNomeFantasia(pessoa.getNomeFantasia());
			e.setUrl(pessoa.getUrl());
			e.setAtivo(true);
			e.setEnderecos(convert(toArray(pessoa.getEnderecos())));
			e.setTelefones(convert(toArray(pessoa.getTelefones())));
			e.setEmails(convert(toArray(pessoa.getEmails())));
			e.setEntidadesPessoa(convert(toArray(pessoa.getRepresentantesLegais())));
			e.setNumerosFistel(convert(toArray(pessoa.getPessoaJuridicasAnatel())));
			if(pessoa.getCnae() != null && pessoa.getCnae().getDescricao() != null && !pessoa.getCnae().getDescricao().equals("")){
				e.setDescricaoCnae(pessoa.getCnae().getDescricao());
			}
			if(pessoa.getRepresentantesLegais() != null && pessoa.getRepresentantesLegais().size() > 0){
				e.setDescricaoRepresentanteLegal(pessoa.getRepresentantesLegais().iterator().next().getPessoaFisica().getNome());
			}
		}
		return e;
	}

	public static Entidade[] convert(List<CrpPessoaJuridica> pessoas) {
		Entidade[] array = null;
		if (pessoas != null && !pessoas.isEmpty()) {
			array = new Entidade[pessoas.size()];
			int contador = 0;
			for (CrpPessoaJuridica crpPessoa : pessoas) {
				array[contador] = convert(crpPessoa);
				contador++;
			}

		}
		return array;
	}

	public static List<CrpPessoaJuridica> convert(Collection<CrpPessoaJuridica> pjs) {
		List<CrpPessoaJuridica> ps = null;
		if (pjs != null && pjs.size() > 0) {
			ps = new ArrayList<CrpPessoaJuridica>();
			for (CrpPessoaJuridica p : pjs) {
				ps.add(p);
			}

			return ps;
		}
		return null;
	}

	public static EnderecoEntidade convert(CrpEndereco endereco) {
		EnderecoEntidade e = null;
		if (endereco != null) {
			e = new EnderecoEntidade();
			e.setEntidade(new Entidade(endereco.getPessoa().getId()));
			// e.setEntidade(convert((CrpPessoaJuridica) endereco.getPessoa()));
			e.setId(endereco.getId());
			e.setLogradouro(endereco.getLogradouro());
			e.setNumero(endereco.getNumero());
			e.setComplemento(endereco.getComplemento());
			e.setBairro(endereco.getBairro());
			e.setCep(endereco.getCep());
			e.setMunicipio(LocalizacaoWSConverter.convert(endereco.getMunicipio()));
			e.setTipo(convert(endereco.getTipoEndereco()));
		}
		return e;
	}

	public static List<EnderecoEntidade> convert(CrpEndereco[] enderecos) {
		List<EnderecoEntidade> set = new ArrayList<EnderecoEntidade>();
		if (enderecos != null) {
			for (CrpEndereco e : enderecos) {
				set.add(convert(e));
			}
		}
		return set;
	}

	public static TelefoneEntidade convert(CrpContatoTelefone telefone) {
		TelefoneEntidade t = null;
		if (telefone != null) {
			t = new TelefoneEntidade();
			t.setEntidade(new Entidade(telefone.getPessoa().getId()));
			t.setId(telefone.getId());
			t.setDdd(telefone.getNumDdd());
			t.setNumero(telefone.getNumTelefone());
			t.setTipo(convert(telefone.getTipoTelefone()));
		}
		return t;
	}

	public static List<TelefoneEntidade> convert(CrpContatoTelefone[] contatosTelefones) {
		List<TelefoneEntidade> set = new ArrayList<TelefoneEntidade>();
		if (contatosTelefones != null) {
			for (CrpContatoTelefone ct : contatosTelefones) {
				set.add(convert(ct));
			}
		}
		return set;
	}

	public static EmailEntidade convert(CrpContatoEmail email) {
		EmailEntidade e = null;
		if (email != null) {
			e = new EmailEntidade();
			e.setId(email.getId());
			e.setEntidade(new Entidade(email.getPessoa().getId()));
			e.setDescricao(email.getDescricao());
			e.setTipo(email.getEmailPessoal() ? new TipoEmail(1) : new TipoEmail(2));
		}
		return e;
	}

	public static List<EmailEntidade> convert(CrpContatoEmail[] contatosEmails) {
		List<EmailEntidade> set = new ArrayList<EmailEntidade>();
		if (contatosEmails != null) {
			for (CrpContatoEmail ce : contatosEmails) {
				set.add(convert(ce));
			}
		}
		return set;
	}

	public static Fistel convert(CrpPessoaJuridicaAnatel fistel) {
		Fistel e = null;
		if (fistel != null) {
			e = new Fistel();
			e.setId(fistel.getId());
			e.setEntidade(new Entidade(fistel.getPessoaJuridica().getId()));
			e.setNumeroFistel(fistel.getCodigo());
			e.setServico(convert(fistel.getServico()));
			e.setCanal(fistel.getCanal());
		}
		return e;
	}

	public static List<Fistel> convert(CrpPessoaJuridicaAnatel[] fisteis) {
		List<Fistel> set = new ArrayList<Fistel>();
		if (fisteis != null) {
			for (CrpPessoaJuridicaAnatel pja : fisteis) {
				set.add(convert(pja));
			}
		}
		return set;
	}
}