package br.gov.mc.arquitetura.converter;

import java.util.HashSet;
import java.util.Set;

import br.gov.mc.arquitetura.ws.client.entidade.EmailPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.EnderecoPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Pessoa;
import br.gov.mc.arquitetura.ws.client.entidade.TelefonePessoa;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpMunicipio;
import br.gov.mc.corporativo.entidade.CrpPais;
import br.gov.mc.corporativo.entidade.CrpPessoa;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoDocPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpUf;

public class PessoaFisicaWSConverter extends ConverterSupport {

	public static CrpPessoaFisica convert(Pessoa pessoa) {
		CrpPessoaFisica p = null;
		if (pessoa != null) {
			p = new CrpPessoaFisica();
			convertDocumentosPessoa(pessoa, p);
			p.setId(pessoa.getId());
			p.setNome(pessoa.getNome());
			p.setDataNascimento(pessoa.getDataNascimento() != null ? pessoa.getDataNascimento() : null);
			p.setDataEmissao(pessoa.getDataEmissaoRG() != null ? pessoa.getDataEmissaoRG() : null);
			p.setSexo(pessoa.getSexo() != null ? pessoa.getSexo() : null);
			//convertMunicipio(pessoa, p);
			p.setEnderecos(convert(toArray(pessoa.getEnderecos()), p));
			p.setTelefones(convert(toArray(pessoa.getTelefones()), p));
			p.setEmails(convert(toArray(pessoa.getEmails()), p));
			p.setRepresentacoes(convert(toArray(pessoa.getEntidadesPessoa())));
			p.setMunicipio(LocalizacaoWSConverter.convert(pessoa.getMunicipio()));
			p.setCorRaca(pessoa.getCorRaca() != null ? convert(pessoa.getCorRaca()) : null);
			p.setEscolaridade((pessoa.getEscolaridade() != null ? convert(pessoa.getEscolaridade()) : null));
			p.setProfissao(pessoa.getProfissao() != null ? convert(pessoa.getProfissao()) : null);
			if (pessoa.getUfNaturalidade() != null && pessoa.getUfNaturalidade().getPais() != null
							&& pessoa.getUfNaturalidade().getPais().getId() != null) {
				p.setPais(new CrpPais(pessoa.getUfNaturalidade().getPais().getId().toString()));
			} else {
				p.setPais(null);
			}
		}
		return p;
	}

	public static void convertDocumentosPessoa(Pessoa pessoa, CrpPessoaFisica p) {
		p.setDocumentos(new HashSet<CrpDocumentoPessoaFisica>());
		if (pessoa.getCpf() != null && !"".equals(pessoa.getCpf())) {
			CrpDocumentoPessoaFisica cpf = new CrpDocumentoPessoaFisica();
			cpf.setTipoDocumento(new CrpTipoDocPessoaFisica(CrpTipoDocPessoaFisica.CPF));
			cpf.setNumero(pessoa.getCpf());
			cpf.setPessoaFisica(p);
			cpf.setOrgaoEmissor(pessoa.getOrgaoEmissorRG());
			p.getDocumentos().add(cpf);
		}
		if (pessoa.getRg() != null && !"".equals(pessoa.getRg())) {
			CrpDocumentoPessoaFisica rg = new CrpDocumentoPessoaFisica();
			rg.setTipoDocumento(new CrpTipoDocPessoaFisica(CrpTipoDocPessoaFisica.RG));
			rg.setNumero(pessoa.getRg());
			rg.setOrgaoEmissor(pessoa.getOrgaoEmissorRG());
			rg.setDataEmissao(pessoa.getDataEmissaoRG());
			rg.setPessoaFisica(p);
			p.getDocumentos().add(rg);
		}
	}

	private static void convertMunicipio(Pessoa pessoa, CrpPessoaFisica p) {
		CrpMunicipio m = null;
		if (pessoa.getNaturalidade() != null && !"".equals(pessoa.getNaturalidade())) {
			m = new CrpMunicipio();
			m.setNome(pessoa.getNaturalidade());
		}
		if (pessoa.getUfNaturalidade() != null && !"".equals(pessoa.getUfNaturalidade())) {
			if (m == null) {
				m = new CrpMunicipio();
			}
			CrpUf uf = LocalizacaoWSConverter.convert(pessoa.getUfNaturalidade());
			m.setUf(uf);
		}
		p.setMunicipio(m);
	}

	public static CrpEndereco convert(EnderecoPessoa endereco, CrpPessoaFisica pessoaFisica) {
		CrpEndereco e = null;
		if (endereco != null) {
			e = new CrpEndereco();
			e.setPessoa(new CrpPessoa(pessoaFisica != null ? pessoaFisica.getId() : null));
			e.setId(endereco.getId());
			e.setLogradouro(endereco.getLogradouro());
			e.setNumero(endereco.getNumero());
			e.setComplemento(endereco.getComplemento());
			e.setBairro(endereco.getBairro());
			e.setCep(endereco.getCep());
			e.setMunicipio(LocalizacaoWSConverter.convert(endereco.getMunicipio()));
			e.setTipoEndereco(convert(endereco.getTipo()));
		}
		return e;
	}

	public static Set<CrpEndereco> convert(EnderecoPessoa[] enderecos, CrpPessoaFisica pessoaFisica) {
		Set<CrpEndereco> set = new HashSet<CrpEndereco>();
		if (enderecos != null) {
			for (EnderecoPessoa e : enderecos) {
				set.add(convert(e, pessoaFisica));
			}
		}
		return set;
	}

	public static CrpContatoTelefone convert(TelefonePessoa telefone, CrpPessoaFisica pessoaFisica) {
		CrpContatoTelefone t = null;
		if (telefone != null) {
			t = new CrpContatoTelefone();
			t.setPessoa(new CrpPessoa(pessoaFisica != null ? pessoaFisica.getId() : null));
			t.setId(telefone.getId());
			t.setNumDdd(telefone.getDdd());
			t.setNumTelefone(telefone.getNumero());
			t.setTipoTelefone(convert(telefone.getTipo()));
		}
		return t;
	}

	public static Set<CrpContatoTelefone> convert(TelefonePessoa[] telefones, CrpPessoaFisica pessoaFisica) {
		Set<CrpContatoTelefone> set = new HashSet<CrpContatoTelefone>();
		if (telefones != null) {
			for (TelefonePessoa t : telefones) {
				set.add(convert(t, pessoaFisica));
			}
		}
		return set;
	}

	public static CrpContatoEmail convert(EmailPessoa email, CrpPessoaFisica pessoaFisica) {
		CrpContatoEmail e = null;
		if (email != null) {
			e = new CrpContatoEmail();
			e.setId(email.getId());
			e.setPessoa(new CrpPessoa(pessoaFisica != null ? pessoaFisica.getId() : null));
			e.setDescricao(email.getDescricao());
			if (email.getTipo() != null && email.getTipo().getId() != null) {
				Integer tipoEmail = email.getTipo().getId();
				e.setEmailPessoal(tipoEmail == 1);
			}
		}
		return e;
	}

	public static Set<CrpContatoEmail> convert(EmailPessoa[] emails, CrpPessoaFisica pessoaFisica) {
		Set<CrpContatoEmail> set = new HashSet<CrpContatoEmail>();
		if (emails != null) {
			for (EmailPessoa e : emails) {
				set.add(convert(e, pessoaFisica));
			}
		}
		return set;
	}
}