package br.gov.mc.arquitetura.converter;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import br.gov.mc.arquitetura.ws.client.entidade.EmailEntidade;
import br.gov.mc.arquitetura.ws.client.entidade.EnderecoEntidade;
import br.gov.mc.arquitetura.ws.client.entidade.Entidade;
import br.gov.mc.arquitetura.ws.client.entidade.Fistel;
import br.gov.mc.arquitetura.ws.client.entidade.TelefoneEntidade;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpPessoa;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridicaAnatel;

public class PessoaJuridicaWSConverter extends ConverterSupport {

	public static CrpPessoaJuridica convert(Entidade entidade) {
		CrpPessoaJuridica p = null;
		if (entidade != null) {
			p = new CrpPessoaJuridica();
			p.setId(entidade.getId());
			p.setCnpj(entidade.getCnpj());
			p.setRazaoSocial(entidade.getRazaoSocial());
			p.setNomeFantasia(entidade.getNomeFantasia());
			p.setUrl(entidade.getUrl());
			p.setEnderecos(convert(toArray(entidade.getEnderecos()), p));
			p.setTelefones(convert(toArray(entidade.getTelefones()), p));
			p.setEmails(convert(toArray(entidade.getEmails()), p));
			p.setRepresentantesLegais(convert(toArray(entidade.getEntidadesPessoa())));
			p.setPessoaJuridicasAnatel(convert(toArray(entidade.getNumerosFistel())));
			// p.setDataSituacaoCadastral(new Date());
			// p.setDataAlteracao(new Date());
			// p.setSituacaoPessoaOrgao(new CrpSituacaoPessoaOrgao(1));
			// p.setPorte(new CrpPortePessoaJuridica(3));
			// p.setTipoPessoaJuridica(new CrpTipoPessoaJuridica(9));
			// p.setTipoOperacao("I");
			// p.setTipoPessoa(new CrpTipoPessoa(2));
		}
		return p;
	}
	
	/*public static CrpPessoaJuridica convert(Entidade entidade, CrpPessoaJuridica p) {		
		if (entidade != null) {
			p = new CrpPessoaJuridica();
			p.setId(entidade.getId());
			p.setCnpj(entidade.getCnpj());
			p.setRazaoSocial(entidade.getRazaoSocial());
			p.setNomeFantasia(entidade.getNomeFantasia());
			p.setUrl(entidade.getUrl());
			p.setEnderecos(convert(toArray(entidade.getEnderecos()), p));
			p.setTelefones(convert(toArray(entidade.getTelefones()), p));
			p.setEmails(convert(toArray(entidade.getEmails()), p));
			p.setRepresentantesLegais(convert(toArray(entidade.getEntidadesPessoa())));
			p.setPessoaJuridicasAnatel(convert(toArray(entidade.getNumerosFistel())));
			// p.setDataSituacaoCadastral(new Date());
			// p.setDataAlteracao(new Date());
			// p.setSituacaoPessoaOrgao(new CrpSituacaoPessoaOrgao(1));
			// p.setPorte(new CrpPortePessoaJuridica(3));
			// p.setTipoPessoaJuridica(new CrpTipoPessoaJuridica(9));
			// p.setTipoOperacao("I");
			// p.setTipoPessoa(new CrpTipoPessoa(2));
		}
		return p;
	}*/

	public static List<CrpPessoaJuridica> convert(Entidade[] entidades) {
		List<CrpPessoaJuridica> pjs = null;
		if (entidades != null && entidades.length > 0) {
			pjs = new ArrayList<CrpPessoaJuridica>();

			for (Entidade e : entidades) {
				pjs.add(convert(e));
			}
		}

		return pjs;
	}

	public static CrpEndereco convert(EnderecoEntidade endereco, CrpPessoaJuridica pessoaJuridica) {
		CrpEndereco e = null;
		if (endereco != null) {
			e = new CrpEndereco();
			e.setPessoa(new CrpPessoa(pessoaJuridica != null ? pessoaJuridica.getId() : null));
			e.setId(endereco.getId());
			e.setLogradouro(endereco.getLogradouro());
			e.setNumero(endereco.getNumero());
			e.setComplemento(endereco.getComplemento());
			e.setBairro(endereco.getBairro());
			e.setCep(endereco.getCep());
			e.setMunicipio(LocalizacaoWSConverter.convert(endereco.getMunicipio()));
			e.setTipoEndereco(convert(endereco.getTipo()));
		}
		return e;
	}

	public static Set<CrpEndereco> convert(EnderecoEntidade[] enderecos, CrpPessoaJuridica pessoaJuridica) {
		Set<CrpEndereco> list = new HashSet<CrpEndereco>();
		if (enderecos != null) {
			for (EnderecoEntidade e : enderecos) {
				list.add(convert(e, pessoaJuridica));
			}
		}
		return list;
	}

	public static CrpContatoTelefone convert(TelefoneEntidade telefone, CrpPessoaJuridica pessoaJuridica) {
		CrpContatoTelefone t = null;
		if (telefone != null) {
			t = new CrpContatoTelefone();
			t.setPessoa(new CrpPessoa(pessoaJuridica != null ? pessoaJuridica.getId() : null));
			t.setId(telefone.getId());
			t.setNumDdd(telefone.getDdd());
			t.setNumTelefone(telefone.getNumero());
			t.setTipoTelefone(convert(telefone.getTipo()));
		}
		return t;
	}

	public static Set<CrpContatoTelefone> convert(TelefoneEntidade[] telefones, CrpPessoaJuridica pessoaFisica) {
		Set<CrpContatoTelefone> set = new HashSet<CrpContatoTelefone>();
		if (telefones != null) {
			for (TelefoneEntidade t : telefones) {
				set.add(convert(t, pessoaFisica));
			}
		}
		return set;
	}

	public static CrpContatoEmail convert(EmailEntidade email, CrpPessoaJuridica pessoaFisica) {
		CrpContatoEmail e = null;
		if (email != null) {
			e = new CrpContatoEmail();
			e.setId(email.getId());
			e.setPessoa(new CrpPessoa(pessoaFisica != null ? pessoaFisica.getId() : null));
			e.setDescricao(email.getDescricao());
			if (email.getTipo() != null && email.getTipo().getId() != null) {
				Integer tipoEmail = email.getTipo().getId();
				e.setEmailPessoal(tipoEmail == 1);
			}
		}
		return e;
	}

	public static Set<CrpContatoEmail> convert(EmailEntidade[] emails, CrpPessoaJuridica pessoaFisica) {
		Set<CrpContatoEmail> set = new HashSet<CrpContatoEmail>();
		if (emails != null) {
			for (EmailEntidade e : emails) {
				set.add(convert(e, pessoaFisica));
			}
		}
		return set;
	}

	public static CrpPessoaJuridicaAnatel convert(Fistel fistel) {
		CrpPessoaJuridicaAnatel pja = null;
		if (fistel != null) {
			pja = new CrpPessoaJuridicaAnatel();
			pja.setId(fistel.getId());
			pja.setCodigo(fistel.getNumeroFistel());
			pja.setPessoaJuridica(new CrpPessoaJuridica(fistel.getEntidade() != null ? fistel.getEntidade().getId()
							: null));
			pja.setServico(ServicoWSConverter.convert(fistel.getServico()));
			pja.setCanal(fistel.getCanal());
		}
		return pja;
	}

	public static Set<CrpPessoaJuridicaAnatel> convert(Fistel[] numerosFistel) {
		Set<CrpPessoaJuridicaAnatel> set = new HashSet<CrpPessoaJuridicaAnatel>();
		if (numerosFistel != null) {
			for (Fistel ep : numerosFistel) {
				set.add(convert(ep));
			}
		}
		return set;
	}
}