package br.gov.mc.arquitetura.converter;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import br.gov.mc.arquitetura.ws.client.entidade.CorRaca;
import br.gov.mc.arquitetura.ws.client.entidade.EmailPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.EnderecoPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Escolaridade;
import br.gov.mc.arquitetura.ws.client.entidade.Pais;
import br.gov.mc.arquitetura.ws.client.entidade.Pessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Profissao;
import br.gov.mc.arquitetura.ws.client.entidade.TelefonePessoa;
import br.gov.mc.arquitetura.ws.client.entidade.TipoEmail;
import br.gov.mc.arquitetura.ws.client.entidade.UF;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpCorRaca;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpEscolaridade;
import br.gov.mc.corporativo.entidade.CrpPessoa;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpProfissao;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;

public class PessoaWSConverter extends ConverterSupport {

	public static Pessoa convert(CrpPessoaFisica pessoa) {
		Pessoa p = null;
		if (pessoa != null) {
			p = new Pessoa();
			convertDocumentosPessoa(pessoa, p);
			p.setId(pessoa.getId());
			p.setNome(pessoa.getNome());
			p.setDataNascimento(pessoa.getDataNascimento());
			p.setSexo(pessoa.getSexo());
			p.setNacionalidade(pessoa.getPais() == null ? 'B' : 'E');
			p.setNaturalidade(pessoa.getMunicipio() != null ? pessoa.getMunicipio().getNome() : null);
			if (pessoa.getPais() != null) {
				Pais pais = LocalizacaoWSConverter.convert(pessoa.getPais());
				p.setUfNaturalidade(new UF());
				p.getUfNaturalidade().setPais(pais);

			} else {
				p.setUfNaturalidade(LocalizacaoWSConverter.convert(pessoa.getMunicipio() != null ? pessoa
								.getMunicipio().getUf() : null));
			}
			p.setEnderecos(convert(toArray(pessoa.getEnderecos())));
			p.setTelefones(convert(toArray(pessoa.getTelefones())));
			p.setEmails(convert(toArray(pessoa.getEmails())));
			p.setEntidadesPessoa(convert(toArray(pessoa.getRepresentacoes())));
			p.setCorRaca(convert(pessoa.getCorRaca()));
			p.setEscolaridade(convert(pessoa.getEscolaridade()));
			p.setProfissao(convert(pessoa.getProfissao()));
			
			p.setMunicipio((pessoa.getMunicipio() != null)?LocalizacaoWSConverter.convert(pessoa.getMunicipio()):null);
			p.setPais((pessoa.getPais() != null)?LocalizacaoWSConverter.convert(pessoa.getPais()):null);
		}
		return p;
	}

	private static CorRaca convert(CrpCorRaca corRaca) {
		if (corRaca != null) {
			CorRaca cr = new CorRaca();
			cr.setId(corRaca.getId());
			cr.setNomCorRaca(corRaca.getNomCorRaca());
			return cr;
		}
		return null;
	}

	private static Escolaridade convert(CrpEscolaridade escolaridade) {
		if (escolaridade != null) {
			Escolaridade cr = new Escolaridade();
			cr.setId(escolaridade.getId());
			cr.setDscEscolaridade(escolaridade.getDscEscolaridade());
			return cr;
		}
		return null;
	}

	private static Profissao convert(CrpProfissao profissao) {
		if (profissao != null) {
			Profissao cr = new Profissao();
			cr.setId(profissao.getId());
			cr.setDscProfissao(profissao.getDscProfissao());
			return cr;
		}
		return null;
	}
	
	public static Pessoa[] convert(List<CrpPessoaFisica> pessoas) {
		Pessoa[] array = null;
		if (pessoas != null && !pessoas.isEmpty()) {
			array = new Pessoa[pessoas.size()];
			int i = 0;
			for (CrpPessoaFisica pf : pessoas) {
				array[i] = convert(pf);
				i++;
			}
		}
		return array;
	}

	public static List<CrpPessoaFisica> convert(Collection<CrpPessoaFisica> pessoas) {
		List<CrpPessoaFisica> ps = null;
		if (pessoas != null) {
			ps = new ArrayList<CrpPessoaFisica>();
			for (CrpPessoaFisica p : pessoas) {
				ps.add(p);
			}
		}

		return ps;
	}

	private static void convertDocumentosPessoa(CrpPessoaFisica pessoa, Pessoa p) {
		if (pessoa.getDocumentos() != null) {
			for (CrpDocumentoPessoaFisica doc : pessoa.getDocumentos()) {
				if ("CPF".equalsIgnoreCase(doc.getTipoDocumento().getDescricao())) {
					p.setCpf(doc.getNumero());
					continue;
				} else if ("RG".equalsIgnoreCase(doc.getTipoDocumento().getDescricao())) {
					p.setRg(doc.getNumero());
					p.setOrgaoEmissorRG(doc.getOrgaoEmissor());
					p.setDataEmissaoRG(pessoa.getDataEmissao());
					continue;
				}
			}
		}
	}

	public static EnderecoPessoa convert(CrpEndereco endereco) {
		EnderecoPessoa e = null;
		if (endereco != null) {
			e = new EnderecoPessoa();
			e.setPessoa(new Pessoa(endereco.getPessoa().getId()));
			e.setId(endereco.getId());
			e.setLogradouro(endereco.getLogradouro());
			e.setNumero(endereco.getNumero());
			e.setComplemento(endereco.getComplemento());
			e.setBairro(endereco.getBairro());
			e.setCep(endereco.getCep());
			e.setMunicipio(LocalizacaoWSConverter.convert(endereco.getMunicipio()));
			e.setTipo(convert(endereco.getTipoEndereco()));
		}
		return e;
	}

	public static CrpEndereco convert(EnderecoPessoa ep) {
		CrpEndereco e = null;

		if (ep != null) {
			e = new CrpEndereco();
			e.setBairro(ep.getBairro());
			e.setCep(ep.getCep());
			e.setCidade(ep.getMunicipio() != null ? ep.getMunicipio().getNome() : "");
			e.setComplemento(ep.getComplemento());
			e.setCorrespondencia(false);
			e.setId(ep.getId());
			e.setLogradouro(ep.getLogradouro());
			e.setMunicipio(LocalizacaoWSConverter.convert(ep.getMunicipio()));
			e.setNumero(ep.getNumero());
			e.setPais(null);
			e.setPessoa(new CrpPessoa(ep.getPessoa().getId()));
		}

		return e;
	}

	public static List<EnderecoPessoa> convert(CrpEndereco[] enderecos) {
		List<EnderecoPessoa> list = new ArrayList<EnderecoPessoa>();
		if (enderecos != null) {
			for (CrpEndereco e : enderecos) {
				list.add(convert(e));
			}
		}
		return list;
	}

	public static TelefonePessoa convert(CrpContatoTelefone telefone) {
		TelefonePessoa t = null;
		if (telefone != null) {
			t = new TelefonePessoa();
			t.setPessoa(new Pessoa(telefone.getPessoa().getId()));
			t.setId(telefone.getId());
			t.setDdd(telefone.getNumDdd());
			t.setNumero(telefone.getNumTelefone());
			t.setTipo(convert(telefone.getTipoTelefone()));
		}
		return t;
	}

	public static List<TelefonePessoa> convert(CrpContatoTelefone[] contatosTelefones) {
		List<TelefonePessoa> set = new ArrayList<TelefonePessoa>();
		if (contatosTelefones != null) {
			for (CrpContatoTelefone ct : contatosTelefones) {
				set.add(convert(ct));
			}
		}
		return set;
	}

	public static EmailPessoa convert(CrpContatoEmail email) {
		EmailPessoa e = null;
		if (email != null) {
			e = new EmailPessoa();
			e.setId(email.getId());
			e.setPessoa(new Pessoa(email.getPessoa().getId()));
			e.setDescricao(email.getDescricao());
			e.setTipo(email.getEmailPessoal() ? new TipoEmail(1) : new TipoEmail(2));
		}
		return e;
	}

	public static List<EmailPessoa> convert(CrpContatoEmail[] contatosEmails) {
		List<EmailPessoa> set = new ArrayList<EmailPessoa>();
		if (contatosEmails != null) {
			for (CrpContatoEmail ce : contatosEmails) {
				set.add(convert(ce));
			}
		}
		return set;
	}

	public static CrpContatoTelefone convert(TelefonePessoa telefone) {
		CrpContatoTelefone ct = null;

		if (telefone != null) {
			ct = new CrpContatoTelefone();
			ct.setId(telefone.getId());
			ct.setNumDdd(telefone.getDdd());
			ct.setNumTelefone(telefone.getNumero());
			ct.setPessoa(new CrpPessoa(telefone.getPessoa().getId()));
			ct.setTipoTelefone(new CrpTipoTelefone(telefone.getTipo().getId()));
		}
		return ct;
	}
}