package br.gov.mc.arquitetura.persistencia.impl;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.util.Collection;

import javax.persistence.Transient;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Example;
import org.hibernate.criterion.MatchMode;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.arquitetura.persistencia.UniversalDao;

@Repository("universalDao")
public class UniversalDaoImpl extends HibernateDaoSupport implements UniversalDao {

	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}
	public Integer count(Object o) {
		Criteria c = getSession().createCriteria(o.getClass());
		c.setProjection(Projections.rowCount());
		return ((Integer) c.list().get(0)).intValue();
	}

	public Object get(Class clazz, Serializable id) {
		return getHibernateTemplate().get(clazz, id);
	}

	public Object get(String hql) {
		return getSession().createQuery(hql).list();
	}

	public Collection getAll(Class clazz) {
		return getHibernateTemplate().loadAll(clazz);
	}

	
	public Collection listByExample(Object o) throws Exception {
		Criteria c = criarCriteria(o);

		return c.list();
	}

	
	public Collection listByExample(String hql) {
		return getSession().createQuery(hql).list();
	}

	
	public Collection listByExample(Object o, int startPage, int pageSize) throws Exception {
		return this.getHibernateTemplate().findByExample(o, (startPage - 1) * pageSize, pageSize);
	}

	
	public Collection listByExampleWithOrderBy(Object o, String... camposOrdenacao) {
		Example example = Example.create(o).enableLike(MatchMode.ANYWHERE);

		Criteria c = getSession().createCriteria(o.getClass()).add(example);

		for (String s : camposOrdenacao) {
			c.addOrder(Order.asc(s));
		}

		return c.list();
	}

	
	public void remove(Class clazz, Serializable id) {
		getHibernateTemplate().delete(get(clazz, id));
	}

	
	public Object save(Object o) {
		return getHibernateTemplate().save(o);
	}

	
	public Collection listByExample(Criteria c) {
		return c.list();
	}

	
	public void update(Object o) {
		getHibernateTemplate().update(o);
	}

	
	public Integer getMax(Class clazz, String propriedade) {
		Criteria c = getSession().createCriteria(clazz);
		c.setProjection(Projections.max(propriedade));

		return (Integer) c.uniqueResult();
	}

	
	public Integer gerMin(Class clazz, String propriedade) {
		Criteria c = getSession().createCriteria(clazz);
		c.setProjection(Projections.min(propriedade));

		return (Integer) c.uniqueResult();
	}

	
	public Session getCurrentSession() {
		return getSession();
	}

	public HibernateTemplate getCurrentHibernateTemplate() {
		return getHibernateTemplate();
	}

	public Criteria criarCriteria(Object o) throws Exception {
		if (o != null) {

			Class clazz = o.getClass();

			Criteria criteria = getSession().createCriteria(clazz);

			for (final Field f : clazz.getDeclaredFields()) {
				f.setAccessible(true);	
				
				if (f.get(o) != null) {
					if (!f.isAnnotationPresent(Transient.class) && !Modifier.isTransient(f.getModifiers()) && !Modifier.isFinal(f.getModifiers())) {
						if(f.getName().contains("serialVersion")) continue;
						if (f.getType().getName().contains("java")) {
							if (f.getType().getName().contains("java.lang")) {
								if (f.getType().getName().contains("String")) {
									criteria.add(Restrictions.ilike(f.getName(), "%" + f.get(o) + "%"));
								} else if (f.getType().getName().contains("Integer")) {
									criteria.add(Restrictions.eq(f.getName(), f.get(o)));
								} else if (f.getType().getName().contains("Double")) {
									criteria.add(Restrictions.eq(f.getName(), f.get(o)));
								} else if (f.getType().getName().contains("Float")) {
									criteria.add(Restrictions.eq(f.getName(), f.get(o)));
								} else if (f.getType().getName().contains("Boolean")) {
									criteria.add(Restrictions.eq(f.getName(), f.get(o)));
								}
							}
						} else {
							criteria = criarSubCriteria(f.get(o), f.getName(), criteria);
						}
					}
				}

			}
			return criteria;
		}

		return null;
	}

	private Criteria criarSubCriteria(Object o, String alias, Criteria c) throws Exception {
		Criteria subCriteria = c.createCriteria(alias);

		Class clazz = o.getClass();

		for (final Field f : clazz.getDeclaredFields()) {
			f.setAccessible(true);			

			if (f.get(o) != null) {
				if (!f.isAnnotationPresent(Transient.class) && !Modifier.isTransient(f.getModifiers()) && !Modifier.isFinal(f.getModifiers())) {
					if(f.getName().contains("serialVersion")) continue;
					if (f.getType().getName().contains("java")) {
						if (f.getType().getName().contains("java.lang")) {
							if (f.getType().getName().contains("String")) {
								subCriteria.add(Restrictions.ilike(f.getName(), "%" + f.get(o) + "%"));
							} else if (f.getType().getName().contains("Integer")) {
								subCriteria.add(Restrictions.eq(f.getName(), f.get(o)));
							} else if (f.getType().getName().contains("Double")) {
								subCriteria.add(Restrictions.eq(f.getName(), f.get(o)));
							} else if (f.getType().getName().contains("Float")) {
								subCriteria.add(Restrictions.eq(f.getName(), f.get(o)));
							} else if (f.getType().getName().contains("Boolean")) {
								subCriteria.add(Restrictions.eq(f.getName(), f.get(o)));
							}
						}
					} else {
						subCriteria = criarSubCriteria(f.get(o), f.getName(), subCriteria);
					}
				}
			}

		}

		return c;
	}
	
}
