package br.gov.mc.arquitetura.util;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.logging.Logger;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.springframework.stereotype.Repository;

import br.gov.mc.arquitetura.exceptions.PessoaJuridicaException;
import br.gov.mc.arquitetura.persistencia.impl.UniversalDaoImpl;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridicaAnatel;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridicaSucessora;
import br.gov.mc.corporativo.entidade.CrpQuadroSocioAdm;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;

@Repository("pessoaJuridicaUtil")
public class PessoaJuridicaUtil extends UniversalDaoImpl {

    // @Autowired
    // @Qualifier("receita_empresaDAO")
    // protected EmpresaDAO empresaDao;

    private Logger log;

    /**
     * MÃ©todo para pesquisar Pessoa Juridica e atualizar na receita federal A
     * pesquisa Ã© realizada pelo ID da pessoa jurÃ­dica
     * 
     * @param ID
     *            da pessoa jurÃ­dica
     * @return Pessoa jurÃ­dida atualizada
     * @throws PessoaJuridicaException
     *             - ExceÃ§Ã£o lanÃ§ada quando nenhum registro for
     *             encontrado na base da Receita Federal Exception -
     *             ExceÃ§Ã£o lanÃ§ada para qualquer outro erro que nÃ£o
     *             seja de negÃ³cio.
     */
    public CrpPessoaJuridica recuperar(Integer id) throws Exception {
	CrpPessoaJuridica pj = new CrpPessoaJuridica();
	pj.setId(id);
	List<CrpPessoaJuridica> lista = (List<CrpPessoaJuridica>) pesquisar(pj);
	if (lista != null && !lista.isEmpty()) {
	    return lista.get(0);
	}
	return null;
    }

    /**
     * Recupera informaÃ§Ãµes da pessoa jurÃ­dica na base de dados da
     * Receita Federal. A pesquisa na base da receita somente Ã© feita caso a
     * pessoa jurÃ­dica nÃ£o exista na base corporativa ou caso esteja
     * cadastrada a mais de 15 dias sem atualizaÃ§Ãµes.
     * 
     * @param Objeto
     *            CrpPessoaJuridica com campos informados para filtro da
     *            pesquisa na base corporativa.
     * @return Collection<CrpPessoaJuridica> com os registros encontrados
     * @throws PessoaJuridicaException
     *             - ExceÃ§Ã£o lanÃ§ada quando nenhum registro for
     *             encontrado na base da Receita Federal Exception -
     *             ExceÃ§Ã£o lanÃ§ada para qualquer outro erro que nÃ£o
     *             seja de negÃ³cio.
     * 
     */
    @SuppressWarnings({ "unchecked", "unused" })
    public Collection<CrpPessoaJuridica> pesquisar(CrpPessoaJuridica pj) throws Exception {
	Collection<CrpPessoaJuridica> pjs = Collections.EMPTY_LIST;

	Criteria c = getCurrentSession().createCriteria(pj.getClass());

	Criteria count = getCurrentSession().createCriteria(pj.getClass());

	try {
	    if (pj != null) {

		if (pj.getId() != null) {
		    c.add(Restrictions.idEq(pj.getId()));
		}

		if (pj.getCapitalSocial() != null) {
		    c.add(Restrictions.eq("capitalSocial", pj.getCapitalSocial()));
		    count.add(Restrictions.eq("capitalSocial", pj.getCapitalSocial()));
		}
		if (pj.getCnae() != null && pj.getCnae().getId() != null) {

		    c.createCriteria("cnae").add(Restrictions.idEq(pj.getCnae().getId()));

		    count.createCriteria("cnae").add(Restrictions.idEq(pj.getCnae().getId()));
		}

		if (pj.getCnpj() != null && !"".equals(pj.getCnpj())) {
		    /**
		     * O campo razao social vai ser lipo, pelo fato do usuario
		     * pode digitar um nome errado com cnpj valido
		     */
		    pj.setRazaoSocial("");
		    c.add(Restrictions.eq("cnpj", pj.getCnpj()));
		    count.add(Restrictions.eq("cnpj", pj.getCnpj()));
		}
		if (pj.getDataAlteracao() != null) {
		    c.add(Restrictions.eq("dataAlteracao", pj.getDataAlteracao()));
		    count.add(Restrictions.eq("dataAlteracao", pj.getDataAlteracao()));
		}
		if (pj.getDataSituacaoCadastral() != null) {
		    c.add(Restrictions.eq("dataSituacaoCadastral", pj.getDataSituacaoCadastral()));
		    count.add(Restrictions.eq("dataSituacaoCadastral", pj.getDataSituacaoCadastral()));
		}
		if (pj.getDataSituacaoEspecial() != null) {
		    c.add(Restrictions.eq("dataSituacaoEspecial", pj.getDataSituacaoEspecial()));
		    count.add(Restrictions.eq("dataSituacaoEspecial", pj.getDataSituacaoEspecial()));
		}

		if (pj.getId() != null) {
		    c.add(Restrictions.idEq(pj.getId()));
		    count.add(Restrictions.idEq(pj.getId()));
		}
		if (pj.getMatriz() != null && pj.getMatriz().getId() != null) {

		    c.createCriteria("matriz").add(Restrictions.idEq(pj.getMatriz().getId()));

		    count.createCriteria("matriz").add(Restrictions.idEq(pj.getMatriz().getId()));
		}
		if (pj.getMotivo() != null && pj.getMotivo().getId() != null) {

		    c.createCriteria("motivo").add(Restrictions.idEq(pj.getMotivo().getId()));

		    count.createCriteria("motivo").add(Restrictions.idEq(pj.getMotivo().getId()));
		}
		if (pj.getMunicipio() != null && !"".equals(pj.getMunicipio().getId())) {
		    c.createCriteria("municipio").add(Restrictions.ilike("id", pj.getMunicipio().getId()));
		    count.createCriteria("municipio").add(Restrictions.ilike("id", pj.getMunicipio().getId()));
		}
		if (pj.getNaturezaJuridica() != null && pj.getNaturezaJuridica().getId() != null) {

		    c.createCriteria("naturejaJuridica").add(Restrictions.idEq(pj.getNaturezaJuridica().getId()));

		    count.createCriteria("naturejaJuridica").add(Restrictions.idEq(pj.getNaturezaJuridica().getId()));
		}
		if (pj.getNomeFantasia() != null && !"".equals(pj.getNomeFantasia())) {
		    c.add(Restrictions.ilike("nomeFantasia", "%" + pj.getNomeFantasia() + "%"));
		    count.add(Restrictions.ilike("nomeFantasia", "%" + pj.getNomeFantasia() + "%"));
		}
		if (pj.getObjetoSocial() != null && "".equals(pj.getObjetoSocial())) {
		    c.add(Restrictions.ilike("objetoSocial", "%" + pj.getObjetoSocial() + "%"));
		    count.add(Restrictions.ilike("objetoSocial", "%" + pj.getObjetoSocial() + "%"));
		}
		if (pj.getPorte() != null && pj.getPorte().getId() != null) {

		    c.createCriteria("porte").add(Restrictions.idEq(pj.getPorte().getId()));

		    count.createCriteria("porte").add(Restrictions.idEq(pj.getPorte().getId()));
		}
		if (pj.getRazaoSocial() != null && !"".equals(pj.getRazaoSocial())) {
		    c.add(Restrictions.ilike("razaoSocial", "%" + pj.getRazaoSocial() + "%"));
		    count.add(Restrictions.ilike("razaoSocial", "%" + pj.getRazaoSocial() + "%"));
		}
		if (pj.getSituacaoEspecial() != null && pj.getSituacaoEspecial().getId() != null) {

		    c.createCriteria("situacaoEspecial").add(Restrictions.idEq(pj.getSituacaoEspecial().getId()));

		    count.createCriteria("situacaoEspecial").add(Restrictions.idEq(pj.getSituacaoEspecial().getId()));
		}
		if (pj.getSituacaoPessoaOrgao() != null && pj.getSituacaoPessoaOrgao().getId() != null) {

		    c.createCriteria("situacaoPessoaOrgao").add(Restrictions.idEq(pj.getSituacaoPessoaOrgao().getId()));

		    count.createCriteria("situacaoPessoaOrgao").add(
			    Restrictions.idEq(pj.getSituacaoPessoaOrgao().getId()));
		}
		if (pj.getSituacaoPessoaReceitaFederal() != null && pj.getSituacaoPessoaOrgao() != null) {
		    c.createCriteria("situacaoPessoaReceitaFederal").add(

		    Restrictions.idEq(pj.getSituacaoPessoaReceitaFederal().getId()));
		    count.createCriteria("situacaoPessoaReceitaFederal").add(

		    Restrictions.idEq(pj.getSituacaoPessoaReceitaFederal().getId()));
		}
		if (pj.getTipoOperacao() != null && !"".equals(pj.getTipoOperacao())) {
		    c.add(Restrictions.ilike("tipoOperacao", "%" + pj.getTipoOperacao() + "%"));
		    count.add(Restrictions.ilike("tipoOperacao", "%" + pj.getTipoOperacao() + "%"));
		}
		if (pj.getTipoPessoa() != null && pj.getTipoPessoa().getId() != null) {

		    c.createCriteria("tipoPessoa").add(Restrictions.idEq(pj.getTipoPessoa().getId()));

		    count.createCriteria("tipoPessoa").add(Restrictions.idEq(pj.getTipoPessoa().getId()));
		}
		if (pj.getTipoPessoaJuridica() != null && pj.getTipoPessoaJuridica().getId() != null) {

		    c.createCriteria("tipoPessoaJuridica").add(Restrictions.idEq(pj.getTipoPessoaJuridica().getId()));

		    count.createCriteria("tipoPessoaJuridica").add(
			    Restrictions.idEq(pj.getTipoPessoaJuridica().getId()));
		}
		if (pj.getUrl() != null && !"".equals(pj.getUrl())) {
		    c.add(Restrictions.ilike("url", "%" + pj.getUrl() + "%"));
		    count.add(Restrictions.ilike("url", "%" + pj.getUrl() + "%"));
		}

		/* Colunas adicionadas por necessidade descrita na OS 104137 */
		if (pj.getDataAbertura() != null) {
		    c.add(Restrictions.eq("dataAbertura", pj.getDataAbertura()));
		    count.add(Restrictions.eq("dataAbertura", pj.getDataAbertura()));
		}
		if (pj.getFlgSimples() != null) {
		    c.add(Restrictions.eq("flgSimples", pj.getFlgSimples()));
		    count.add(Restrictions.eq("flgSimples", pj.getFlgSimples()));
		}
		if (pj.getNumCnpjSucedida() != null && !pj.getNumCnpjSucedida().isEmpty()) {
		    c.add(Restrictions.eq("numCnpjSucedida", pj.getNumCnpjSucedida()));
		    count.add(Restrictions.eq("numCnpjSucedida", pj.getNumCnpjSucedida()));
		}
		/* OS 104137 */

		count.setProjection(Projections.rowCount());

		Object o = count.uniqueResult();
		Integer valor = null;
		if (o instanceof Integer) {
		    valor = (Integer) o;
		} else if (o != null) {
		    valor = ((Long) o).intValue();
		}

		if (valor != null && valor > 50) {
		    throw new PessoaJuridicaException(
			    "Para essa consulta foram encontrados mais de 50 registros. Favor aumentar os crit&eacute;rios da pesquisa!");
		}

		pjs = null;
		pjs = listByExample(c);

		/**
		 * Lista usada para adicionar SOMENTE CNPJ VÃ�LIDOS. Conforme
		 * vai entrando no catch vai sendo retirado os cnpj invÃ¡lidos
		 */
		Collection<CrpPessoaJuridica> listaPjs = new ArrayList<CrpPessoaJuridica>();
		listaPjs.addAll(pjs);
		if (pjs != null && pjs.size() > 0) {

		    Calendar anoPassado = Calendar.getInstance();
		    // anoPassado.add(Calendar.YEAR, -1);
		    anoPassado.add(Calendar.DAY_OF_MONTH, -15);
		    for (CrpPessoaJuridica cpj : pjs) {

			// Caso a data de atualizaÃ§Ã£o tenha mais de 15
			// dias, realizar nova
			// pesquisa na receita e atualizar informaÃ§Ãµes da
			// pessoa jurÃ­dica.
			if (cpj.getDtAtualizacao() == null || cpj.getDtAtualizacao().before(anoPassado.getTime())) {

			    String[] cnpjs = { cpj.getCnpj().trim() };
			    // Empresa e = null;
			    // Empresa[] es = null;
			    // Try catch usado para nÃ£o para o fluxo ao achar
			    // um cnpj invÃ¡lido
			    try {

				// Caso a pessoa jurÃ­dica nÃ£o exista na
				// base da receita lanÃ§a uma exceÃ§Ã£o
				// es = empresaDao.consultar(cnpjs,
				// TipoConsulta.COMPLETA);
				// if(es == null || es.length == 0){
				// throw new PessoaJuridicaException(
				// "CNPJ n&atilde;o encontrado na base de dados da Receita Federal.");
				// }
				//
				// e = es[0];

				/**
				 * IMPORTANTE: NÃ£o remover as
				 * inicializaÃ§Ãµes das listas
				 */
				cpj.setEmails(new ArrayList<CrpContatoEmail>());
				cpj.setEnderecos(new ArrayList<CrpEndereco>());
				cpj.setTelefones(new ArrayList<CrpContatoTelefone>());
				cpj.setRepresentantesLegais(new ArrayList<CrpRepresentantePesJur>());
				cpj.setQuadroSocios(new ArrayList<CrpQuadroSocioAdm>());
				cpj.setPessoaJuridicasAnatel(new ArrayList<CrpPessoaJuridicaAnatel>());
				cpj.setSucessoras(new ArrayList<CrpPessoaJuridicaSucessora>());
				// ReceitaWSConverter.convert(e, true, cpj,
				// getCurrentHibernateTemplate());

				/**
				 * Mantis foi criado para criar uma
				 * exceÃ§Ã£o personalizada para quando for
				 * cnpj invÃ¡lido,assim nÃ£o tratamos o erro
				 * de forma Ã© generico.
				 */

			    } catch (Exception ia) {
				listaPjs.remove(cpj);
				getLog().info(ia.getMessage());
				throw ia;
			    }
			}
		    }
		    // Senao existe no corporativo
		} else {
		    if (pj.getCnpj() != null) {
			String[] cnpjs = { pj.getCnpj() };
			// Empresa e = null;
			// Empresa[] es = null;
			// Try catch usado para nÃ£o para o fluxo ao achar um
			// cnpj invÃ¡lido
			try {
			    // Caso a pessoa jurÃ­dica nÃ£o exista na base
			    // da receita lanÃ§a uma exceÃ§Ã£o
			    // es = empresaDao.consultar(cnpjs,
			    // TipoConsulta.COMPLETA);
			    // if(es == null || es.length == 0){
			    // throw new PessoaJuridicaException(
			    // "CNPJ n&atilde;o encontrado na base de dados da Receita Federal.");
			    // }
			    // e = es[0];
			    // ReceitaWSConverter.convert(e, false, pj,
			    // getCurrentHibernateTemplate());
			    listaPjs = new ArrayList<CrpPessoaJuridica>();
			    listaPjs.add(pj);
			} catch (Exception ia) {
			    getLog().info(ia.getMessage());
			    throw ia;
			}
		    }
		}

		/**
		 * Retornando uma lista de CrpPessoaJuridica. Pesquisado
		 * novamente para recuperar as collections, que foram retiradas
		 * ao atualizar o registro
		 */
		List<CrpPessoaJuridica> listaRetorno = new ArrayList<CrpPessoaJuridica>();
		for (CrpPessoaJuridica crpPJ : listaPjs) {
		    if (crpPJ.getId() != null) {
			crpPJ = (CrpPessoaJuridica) getCurrentSession().get(CrpPessoaJuridica.class, crpPJ.getId());
			// ..
			Hibernate.initialize(crpPJ.getEmails());
			// crpPJ.setEmails(getCurrentSession().createQuery(
			// "from CrpContatoEmail cce where cce.pessoa.id = " +
			// crpPJ.getId()).list());
			Hibernate.initialize(crpPJ.getEnderecos());
			// crpPJ.setEnderecos(getCurrentSession().createQuery(
			// "from CrpEndereco ce where ce.pessoa.id = " +
			// crpPJ.getId()).list());
			Hibernate.initialize(crpPJ.getTelefones());
			// crpPJ.setTelefones(getCurrentSession().createQuery(
			// "from CrpContatoTelefone ct where ct.pessoa.id = " +
			// crpPJ.getId()).list());
			Hibernate.initialize(crpPJ.getRepresentantesLegais());
			// crpPJ.setRepresentantesLegais(getCurrentSession().createQuery(
			// "from CrpRepresentantePesJur crpj where crpj.pessoaJuridica.id = "
			// + crpPJ.getId()).list());
			Hibernate.initialize(crpPJ.getQuadroSocios());
			// crpPJ.setQuadroSocios(getCurrentSession().createQuery(
			// "from CrpQuadroSocioAdm cqsa where cqsa.pessoaJuridica.id = "
			// + crpPJ.getId()).list());
			Hibernate.initialize(crpPJ.getSucessoras());
			listaRetorno.add(crpPJ);
		    }
		}
		return listaRetorno;

	    }
	} catch (IllegalArgumentException e) {
	    getLog().info(e.getMessage());
	    throw e;
	} catch (Exception e) {
	    getLog().info(e.getMessage());
	    throw e;
	}

	return getAll(pj.getClass());
    }

    /**
     * Recupera informaÃ§Ãµes da pessoa jurÃ­dica na base de dados da
     * Receita Federal. A pesquisa na base da receita somente Ã© feita caso a
     * pessoa jurÃ­dica nÃ£o exista na base corporativa ou caso esteja
     * cadastrada a mais de 15 dias sem atualizaÃ§Ãµes.
     * 
     * @param Objeto
     *            CrpPessoaJuridica com campos informados para filtro da
     *            pesquisa na base corporativa.
     * @return Collection<CrpPessoaJuridica> com os registros encontrados
     * @throws PessoaJuridicaException
     *             - ExceÃ§Ã£o lanÃ§ada quando nenhum registro for
     *             encontrado na base da Receita Federal Exception -
     *             ExceÃ§Ã£o lanÃ§ada para qualquer outro erro que nÃ£o
     *             seja de negÃ³cio.
     * 
     */
    @SuppressWarnings({ "unchecked", "unused" })
    public Collection<CrpPessoaJuridica> pesquisarInteligente(CrpPessoaJuridica pj) throws Exception {
	Collection<CrpPessoaJuridica> pjs = Collections.EMPTY_LIST;

	Criteria c = getCurrentSession().createCriteria(pj.getClass());

	Criteria count = getCurrentSession().createCriteria(pj.getClass());

	try {
	    if (pj != null) {

		if (pj.getId() != null) {
		    c.add(Restrictions.idEq(pj.getId()));
		}

		if (pj.getCapitalSocial() != null) {
		    c.add(Restrictions.eq("capitalSocial", pj.getCapitalSocial()));
		    count.add(Restrictions.eq("capitalSocial", pj.getCapitalSocial()));
		}
		if (pj.getCnae() != null && pj.getCnae().getId() != null) {

		    c.createCriteria("cnae").add(Restrictions.idEq(pj.getCnae().getId()));

		    count.createCriteria("cnae").add(Restrictions.idEq(pj.getCnae().getId()));
		}

		if (pj.getCnpj() != null && !"".equals(pj.getCnpj())) {
		    /**
		     * O campo razao social vai ser lipo, pelo fato do usuario
		     * pode digitar um nome errado com cnpj valido
		     */
		    pj.setRazaoSocial("");
		    c.add(Restrictions.eq("cnpj", pj.getCnpj()));
		    count.add(Restrictions.eq("cnpj", pj.getCnpj()));
		}
		if (pj.getDataAlteracao() != null) {
		    c.add(Restrictions.eq("dataAlteracao", pj.getDataAlteracao()));
		    count.add(Restrictions.eq("dataAlteracao", pj.getDataAlteracao()));
		}
		if (pj.getDataSituacaoCadastral() != null) {
		    c.add(Restrictions.eq("dataSituacaoCadastral", pj.getDataSituacaoCadastral()));
		    count.add(Restrictions.eq("dataSituacaoCadastral", pj.getDataSituacaoCadastral()));
		}
		if (pj.getDataSituacaoEspecial() != null) {
		    c.add(Restrictions.eq("dataSituacaoEspecial", pj.getDataSituacaoEspecial()));
		    count.add(Restrictions.eq("dataSituacaoEspecial", pj.getDataSituacaoEspecial()));
		}
		if (pj.getId() != null) {
		    c.add(Restrictions.idEq(pj.getId()));
		    count.add(Restrictions.idEq(pj.getId()));
		}
		if (pj.getMatriz() != null && pj.getMatriz().getId() != null) {

		    c.createCriteria("matriz").add(Restrictions.idEq(pj.getMatriz().getId()));

		    count.createCriteria("matriz").add(Restrictions.idEq(pj.getMatriz().getId()));
		}
		if (pj.getMotivo() != null && pj.getMotivo().getId() != null) {

		    c.createCriteria("motivo").add(Restrictions.idEq(pj.getMotivo().getId()));

		    count.createCriteria("motivo").add(Restrictions.idEq(pj.getMotivo().getId()));
		}
		if (pj.getMunicipio() != null && !"".equals(pj.getMunicipio().getId())) {
		    c.createCriteria("municipio").add(Restrictions.ilike("id", pj.getMunicipio().getId()));
		    count.createCriteria("municipio").add(Restrictions.ilike("id", pj.getMunicipio().getId()));
		}
		if (pj.getNaturezaJuridica() != null && pj.getNaturezaJuridica().getId() != null) {

		    c.createCriteria("naturejaJuridica").add(Restrictions.idEq(pj.getNaturezaJuridica().getId()));

		    count.createCriteria("naturejaJuridica").add(Restrictions.idEq(pj.getNaturezaJuridica().getId()));
		}
		if (pj.getNomeFantasia() != null && !"".equals(pj.getNomeFantasia())) {
		    c.add(Restrictions.ilike("nomeFantasia", "%" + pj.getNomeFantasia() + "%"));
		    count.add(Restrictions.ilike("nomeFantasia", "%" + pj.getNomeFantasia() + "%"));
		}
		if (pj.getObjetoSocial() != null && "".equals(pj.getObjetoSocial())) {
		    c.add(Restrictions.ilike("objetoSocial", "%" + pj.getObjetoSocial() + "%"));
		    count.add(Restrictions.ilike("objetoSocial", "%" + pj.getObjetoSocial() + "%"));
		}
		if (pj.getPorte() != null && pj.getPorte().getId() != null) {

		    c.createCriteria("porte").add(Restrictions.idEq(pj.getPorte().getId()));

		    count.createCriteria("porte").add(Restrictions.idEq(pj.getPorte().getId()));
		}
		if (pj.getRazaoSocial() != null && !"".equals(pj.getRazaoSocial())) {
		    c.add(Restrictions.ilike("razaoSocial", "%" + pj.getRazaoSocial() + "%"));
		    count.add(Restrictions.ilike("razaoSocial", "%" + pj.getRazaoSocial() + "%"));
		}
		if (pj.getSituacaoEspecial() != null && pj.getSituacaoEspecial().getId() != null) {

		    c.createCriteria("situacaoEspecial").add(Restrictions.idEq(pj.getSituacaoEspecial().getId()));

		    count.createCriteria("situacaoEspecial").add(Restrictions.idEq(pj.getSituacaoEspecial().getId()));
		}
		if (pj.getSituacaoPessoaOrgao() != null && pj.getSituacaoPessoaOrgao().getId() != null) {

		    c.createCriteria("situacaoPessoaOrgao").add(Restrictions.idEq(pj.getSituacaoPessoaOrgao().getId()));

		    count.createCriteria("situacaoPessoaOrgao").add(
			    Restrictions.idEq(pj.getSituacaoPessoaOrgao().getId()));
		}
		if (pj.getSituacaoPessoaReceitaFederal() != null && pj.getSituacaoPessoaOrgao() != null) {
		    c.createCriteria("situacaoPessoaReceitaFederal").add(

		    Restrictions.idEq(pj.getSituacaoPessoaReceitaFederal().getId()));
		    count.createCriteria("situacaoPessoaReceitaFederal").add(

		    Restrictions.idEq(pj.getSituacaoPessoaReceitaFederal().getId()));
		}
		if (pj.getTipoOperacao() != null && !"".equals(pj.getTipoOperacao())) {
		    c.add(Restrictions.ilike("tipoOperacao", "%" + pj.getTipoOperacao() + "%"));
		    count.add(Restrictions.ilike("tipoOperacao", "%" + pj.getTipoOperacao() + "%"));
		}
		if (pj.getTipoPessoa() != null && pj.getTipoPessoa().getId() != null) {

		    c.createCriteria("tipoPessoa").add(Restrictions.idEq(pj.getTipoPessoa().getId()));

		    count.createCriteria("tipoPessoa").add(Restrictions.idEq(pj.getTipoPessoa().getId()));
		}
		if (pj.getTipoPessoaJuridica() != null && pj.getTipoPessoaJuridica().getId() != null) {

		    c.createCriteria("tipoPessoaJuridica").add(Restrictions.idEq(pj.getTipoPessoaJuridica().getId()));

		    count.createCriteria("tipoPessoaJuridica").add(
			    Restrictions.idEq(pj.getTipoPessoaJuridica().getId()));
		}
		if (pj.getUrl() != null && !"".equals(pj.getUrl())) {
		    c.add(Restrictions.ilike("url", "%" + pj.getUrl() + "%"));
		    count.add(Restrictions.ilike("url", "%" + pj.getUrl() + "%"));
		}

		/* Colunas adicionadas por necessidade descrita na OS 104137 */
		if (pj.getDataAbertura() != null) {
		    c.add(Restrictions.eq("dataAbertura", pj.getDataAbertura()));
		    count.add(Restrictions.eq("dataAbertura", pj.getDataAbertura()));
		}
		if (pj.getFlgSimples() != null) {
		    c.add(Restrictions.eq("flgSimples", pj.getFlgSimples()));
		    count.add(Restrictions.eq("flgSimples", pj.getFlgSimples()));
		}
		if (pj.getNumCnpjSucedida() != null && !pj.getNumCnpjSucedida().isEmpty()) {
		    c.add(Restrictions.eq("numCnpjSucedida", pj.getNumCnpjSucedida()));
		    count.add(Restrictions.eq("numCnpjSucedida", pj.getNumCnpjSucedida()));
		}
		/* OS 104137 */

		count.setProjection(Projections.rowCount());

		Integer valor = (Integer) count.uniqueResult();

		if (valor != null && valor > 50) {
		    throw new PessoaJuridicaException(
			    "Para essa consulta foram encontrados mais de 50 registros. Favor aumentar os crit&eacute;rios dessa pesquisa!");
		}

		pjs = null;
		pjs = listByExample(c);

		// Calendar anoPassado = Calendar.getInstance();
		// anoPassado.add(Calendar.YEAR, -1);
		// c.add(
		// Restrictions.or(
		// Restrictions.lt("dtAtualizacao", anoPassado.getTime()),
		// Restrictions.isNull("dtAtualizacao")));
		// Collection<CrpPessoaJuridica> pjNaoAtualizados = c.list();
		// if(!pjNaoAtualizados.isEmpty()){
		// System.out.println();
		// }

		/**
		 * Lista usada para adicionar SOMENTE CNPJ VÃ�LIDOS. Conforme
		 * vai entrando no catch vai sendo retirado os cnpj invÃ¡lidos
		 */
		Collection<CrpPessoaJuridica> listaPjs = new ArrayList<CrpPessoaJuridica>();
		listaPjs.addAll(pjs);
		// pjs.retainAll(pjNaoAtualizados);

		// se existe no corporativo
		if (pjs != null && pjs.size() > 0) {

		    Calendar anoPassado = Calendar.getInstance();
		    // anoPassado.add(Calendar.YEAR, -1);
		    anoPassado.add(Calendar.DAY_OF_MONTH, -15);

		    for (CrpPessoaJuridica cpj : pjs) {

			// Caso a data de atualizaÃ§Ã£o tenha mais de 15
			// dias, realizar nova
			// pesquisa na receita e atualizar informaÃ§Ãµes da
			// pessoa jurÃ­dica.
			if (cpj.getDtAtualizacao() == null || cpj.getDtAtualizacao().before(anoPassado.getTime())) {
			    String[] cnpjs = { cpj.getCnpj().trim() };
			    // Empresa e = null;
			    // Empresa[] es = null;
			    // Try catch usado para nÃ£o para o fluxo ao achar
			    // um cnpj invÃ¡lido
			    try {

				// Caso a pessoa jurÃ­dica nÃ£o exista na
				// base da receita lanÃ§a uma exceÃ§Ã£o
				// es = empresaDao.consultar(cnpjs,
				// TipoConsulta.COMPLETA);
				// if(es == null || es.length == 0){
				// throw new PessoaJuridicaException(
				// "CNPJ n&atilde;o encontrado na base de dados da Receita Federal.");
				// }
				//
				// e = es[0];
				//
				/**
				 * IMPORTANTE: NÃ£o remover a
				 * inicializaÃ§Ã£o das listas
				 */
				cpj.setEmails(new ArrayList<CrpContatoEmail>());
				cpj.setEnderecos(new ArrayList<CrpEndereco>());
				cpj.setTelefones(new ArrayList<CrpContatoTelefone>());
				cpj.setRepresentantesLegais(new ArrayList<CrpRepresentantePesJur>());
				cpj.setQuadroSocios(new ArrayList<CrpQuadroSocioAdm>());
				cpj.setPessoaJuridicasAnatel(new ArrayList<CrpPessoaJuridicaAnatel>());
				cpj.setSucessoras(new ArrayList<CrpPessoaJuridicaSucessora>());
				// ReceitaWSConverter.convert(e, true, cpj,
				// getCurrentHibernateTemplate());

				/**
				 * Mantis foi criado para criar uma
				 * exceÃ§Ã£o personalizada para quando for
				 * cnpj invÃ¡lido,assim nÃ£o tratamos o erro
				 * de forma tÃ£o generico
				 */

			    } catch (Exception ia) {
				listaPjs.remove(cpj);
				getLog().info(ia.getMessage());
				throw ia;
			    }
			}
		    }
		    // Senao existe no corporativo
		} else {
		    if (pj.getCnpj() != null) {
			String[] cnpjs = { pj.getCnpj() };
			// Empresa e = null;
			// Empresa[] es = null;
			// Try catch usado para nÃ£o para o fluxo ao achar um
			// cnpj invÃ¡lido
			try {
			    // Caso a pessoa jurÃ­dica nÃ£o exista na base
			    // da receita lanÃ§a uma exceÃ§Ã£o
			    // es = empresaDao.consultar(cnpjs,
			    // TipoConsulta.COMPLETA);
			    // if(es == null || es.length == 0){
			    // throw new PessoaJuridicaException(
			    // "CNPJ n&atilde;o encontrado na base de dados da Receita Federal.");
			    // }
			    // e = es[0];
			    // ReceitaWSConverter.convert(e, false, pj,
			    // getCurrentHibernateTemplate());
			    listaPjs = new ArrayList<CrpPessoaJuridica>();
			    listaPjs.add(pj);
			} catch (Exception ia) {
			    getLog().info(ia.getMessage());
			    throw ia;
			}
		    }
		}

		/**
		 * Retornando uma lista de CrpPessoaJuridica. Pesquisado
		 * novamente para recuperar as collections, que foram retiradas
		 * ao atualizar o registro
		 */
		List<CrpPessoaJuridica> listaRetorno = new ArrayList<CrpPessoaJuridica>();
		for (CrpPessoaJuridica crpPJ : listaPjs) {
		    crpPJ = (CrpPessoaJuridica) getCurrentSession().get(CrpPessoaJuridica.class, crpPJ.getId());
		    // ..
		    Hibernate.initialize(crpPJ.getEmails());
		    // crpPJ.setEmails(getCurrentSession().createQuery(
		    // "from CrpContatoEmail cce where cce.pessoa.id = " +
		    // crpPJ.getId()).list());
		    Hibernate.initialize(crpPJ.getEnderecos());
		    // crpPJ.setEnderecos(getCurrentSession().createQuery(
		    // "from CrpEndereco ce where ce.pessoa.id = " +
		    // crpPJ.getId()).list());
		    Hibernate.initialize(crpPJ.getTelefones());
		    // crpPJ.setTelefones(getCurrentSession().createQuery(
		    // "from CrpContatoTelefone ct where ct.pessoa.id = " +
		    // crpPJ.getId()).list());
		    Hibernate.initialize(crpPJ.getRepresentantesLegais());
		    // crpPJ.setRepresentantesLegais(getCurrentSession().createQuery(
		    // "from CrpRepresentantePesJur crpj where crpj.pessoaJuridica.id = "
		    // + crpPJ.getId()).list());
		    Hibernate.initialize(crpPJ.getQuadroSocios());
		    // crpPJ.setQuadroSocios(getCurrentSession().createQuery(
		    // "from CrpQuadroSocioAdm cqsa where cqsa.pessoaJuridica.id = "
		    // + crpPJ.getId()).list());
		    Hibernate.initialize(crpPJ.getSucessoras());
		    listaRetorno.add(crpPJ);
		}
		return listaRetorno;

	    }
	} catch (IllegalArgumentException e) {
	    getLog().info(e.getMessage());
	    throw e;
	} catch (Exception e) {
	    getLog().info(e.getMessage());
	    throw e;
	}

	return getAll(pj.getClass());
    }

    protected Logger getLog() {
	if (this.log == null) {
	    this.log = Logger.getLogger("ACAO : ");
	}
	return this.log;
    }

    // private Integer recuperarIdSistema() {
    // ServletContext sc = this.getServletContext();
    // if(sc != null){
    // // A propriedade display-name do web.xml serÃ¡ utilizada para definir o
    // nome do sistema cadastrado no SEG.
    // // Essa propriedade deve ter o mesmo valor do parÃ¢metro "sistema" do
    // filtro acesso.
    // Query query =
    // getSession().createSQLQuery("SELECT s.ID_SISTEMA FROM SEG_NOVO.dbo.SEG_SISTEMA s WHERE UPPER(s.SGL_SISTEMA) = ?");
    // query.setString(0, sc.getServletContextName().toString().toUpperCase());
    // return (Integer) query.uniqueResult();
    // }
    // return null;
    // }

    // private ServletContext getServletContext(){
    // if(FacesContext.getCurrentInstance() != null){
    // return (ServletContext)
    // FacesContext.getCurrentInstance().getExternalContext().getContext();
    // }else if(ServletActionContext.getServletContext() != null){
    // return (ServletContext) ServletActionContext.getServletContext();
    // }
    // return null;
    // }
    //
}