package br.gov.mc.arquitetura.ws.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.criterion.Order;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import br.gov.mc.arquitetura.comparator.ComparatorDistrito;
import br.gov.mc.arquitetura.comparator.ComparatorMunicipio;
import br.gov.mc.arquitetura.comparator.ComparatorRegiao;
import br.gov.mc.arquitetura.comparator.ComparatorSubDistrito;
import br.gov.mc.arquitetura.comparator.ComparatorUF;
import br.gov.mc.arquitetura.converter.LocalizacaoWSConverter;
import br.gov.mc.arquitetura.util.LocalizacaoUtil;
import br.gov.mc.arquitetura.ws.client.entidade.Distrito;
import br.gov.mc.arquitetura.ws.client.entidade.Municipio;
import br.gov.mc.arquitetura.ws.client.entidade.MunicipioSub;
import br.gov.mc.arquitetura.ws.client.entidade.Pais;
import br.gov.mc.arquitetura.ws.client.entidade.Regiao;
import br.gov.mc.arquitetura.ws.client.entidade.SubDistrito;
import br.gov.mc.arquitetura.ws.client.entidade.UF;
import br.gov.mc.corporativo.entidade.CrpDistrito;
import br.gov.mc.corporativo.entidade.CrpMunicipio;
import br.gov.mc.corporativo.entidade.CrpPais;
import br.gov.mc.corporativo.entidade.CrpRegiao;
import br.gov.mc.corporativo.entidade.CrpSubdistrito;
import br.gov.mc.corporativo.entidade.CrpUf;

@Repository("localizacaoDAO")
public class LocalizacaoDAO {

	@Autowired
	protected LocalizacaoUtil dao;

	public LocalizacaoDAO() throws Exception {
		super();

	}

	public Collection<Regiao> pesquisarRegiao(Regiao regiao) throws Exception {
		CrpRegiao r = LocalizacaoWSConverter.convert(regiao);
		Collection<CrpRegiao> rs = dao.pesquisar(r);

		List<Regiao> listaRegiao = new ArrayList<Regiao>();
		listaRegiao.addAll(LocalizacaoWSConverter.convert(rs));

		Collections.sort(listaRegiao, new ComparatorRegiao());

		return listaRegiao;
	}

	public Collection<UF> pesquisarUF(UF uf) {
		try {
			CrpUf u = LocalizacaoWSConverter.convert(uf);
			Collection<CrpUf> us = dao.pesquisar(u);

			List<UF> listaUf = new ArrayList<UF>();
			listaUf.addAll(LocalizacaoWSConverter.convertUfs(us));
			Collections.sort(listaUf, new ComparatorUF());

			return listaUf;
		} catch (Exception e) {
			e.printStackTrace();
			return new ArrayList<UF>();
		}
	}

	public Collection<Municipio> pesquisarMunicipio(Municipio municipio) throws Exception {
		CrpMunicipio m = LocalizacaoWSConverter.convert(municipio);
		Collection<CrpMunicipio> ms = dao.pesquisar(m);

		List<Municipio> listaMunicipio = new ArrayList<Municipio>();
		listaMunicipio.addAll(LocalizacaoWSConverter.convertMunicipios(ms));

		Collections.sort(listaMunicipio, new ComparatorMunicipio());

		return listaMunicipio;
	}

	public Collection<MunicipioSub> pesquisarMunicipioSub(MunicipioSub municipio) throws Exception {
		return null;
	}

	public Collection<Distrito> pesquisarDistrito(Distrito distrito) throws Exception {
		CrpDistrito d = LocalizacaoWSConverter.convert(distrito);
		Collection<CrpDistrito> ds = dao.pesquisar(d);

		List<Distrito> lista = new ArrayList<Distrito>();
		lista.addAll(LocalizacaoWSConverter.convertDistritos(ds));
		Collections.sort(lista, new ComparatorDistrito());

		return lista;
	}

	public Collection<SubDistrito> pesquisarSubDistrito(SubDistrito subDistrito) throws Exception {
		CrpSubdistrito sd = LocalizacaoWSConverter.convert(subDistrito);
		Collection<CrpSubdistrito> sds = dao.pesquisar(sd);
		List<SubDistrito> lista = new ArrayList<SubDistrito>();
		lista.addAll(LocalizacaoWSConverter.convertSubDistritos(sds));
		Collections.sort(lista, new ComparatorSubDistrito());

		return lista;
	}

	public Regiao recuperarRegiao(Regiao regiao) throws Exception {
		CrpRegiao r = LocalizacaoWSConverter.convert(regiao);
		r = (CrpRegiao) dao.get(r.getClass(), r.getId());
		return LocalizacaoWSConverter.convert(r);
	}

	public Pais recuperarPais(Pais pais) throws Exception {
		CrpPais p = LocalizacaoWSConverter.convert(pais);
		p = (CrpPais) dao.get(p.getClass(), p.getId());
		return LocalizacaoWSConverter.convert(p);
	}

	public UF recuperarUF(UF uf) throws Exception {
		CrpUf u = LocalizacaoWSConverter.convert(uf);
		u = (CrpUf) dao.get(u.getClass(), u.getId());
		return LocalizacaoWSConverter.convert(u);
	}

	public Municipio recuperarMunicipio(Municipio municipio) throws Exception {
		CrpMunicipio m = LocalizacaoWSConverter.convert(municipio);
		m = (CrpMunicipio) dao.get(m.getClass(), m.getId());
		return LocalizacaoWSConverter.convert(m);
	}

	public MunicipioSub recuperarMunicipioSub(MunicipioSub municipio) throws Exception {
		return null;
	}

	public Distrito recuperarDistrito(Distrito distrito) throws Exception {
		CrpDistrito dis = LocalizacaoWSConverter.convert(distrito);
		dis = (CrpDistrito) dao.get(dis.getClass(), dis.getId());
		return LocalizacaoWSConverter.convert(dis);
	}

	public SubDistrito recuperarSubDistrito(SubDistrito subDistrito) throws Exception {
		CrpSubdistrito sub = LocalizacaoWSConverter.convert(subDistrito);
		sub = (CrpSubdistrito) dao.get(sub.getClass(), sub.getId());
		return LocalizacaoWSConverter.convert(sub);
	}

	public Collection<CrpPais> pesquisarPais(CrpPais pais) {
		Criteria c = dao.getCurrentSession().createCriteria(pais.getClass());

		if (pais != null) {
			if (pais.getAtivo() != null) {
				c.add(Restrictions.eq("ativo", pais.getAtivo()));
			}
			if (pais.getId() != null) {
				c.add(Restrictions.idEq(pais.getId()));
			}
			if (pais.getNome() != null && !"".equals(pais.getNome())) {
				c.add(Restrictions.ilike("nome", "%" + pais.getNome() + "%"));
			}
			if (pais.getSigla() != null && !"".equals(pais.getSigla())) {
				c.add(Restrictions.ilike("sigla", "%" + pais.getSigla() + "%"));
			}
			c.addOrder(Order.asc("nome"));
		}

		return dao.listByExample(c);
	}

	public Collection<CrpUf> pesquisarUf(CrpUf uf) {
		Criteria c = dao.getCurrentSession().createCriteria(uf.getClass());

		if (uf != null) {
			if (uf.getId() != null) {
				c.add(Restrictions.idEq(uf.getId()));
			}
			if (uf.getNome() != null && !"".equals(uf.getNome())) {
				c.add(Restrictions.ilike("nome", "%" + uf.getNome() + "%"));
			}
			if (uf.getSigla() != null && !"".equals(uf.getSigla())) {
				c.add(Restrictions.ilike("sigla", "%" + uf.getSigla() + "%"));
			}
			if (uf.getRegiao() != null && !"".equals(uf.getRegiao())) {
				c.createCriteria("regiao").add(Restrictions.idEq(uf.getRegiao().getId()));
			}
			if (uf.getPais() != null && !"".equals(uf.getPais())) {
				c.createCriteria("pais").add(Restrictions.idEq(uf.getPais().getId()));
			}
			c.addOrder(Order.asc("sigla"));
		}

		return dao.listByExample(c);
	}

	public Collection<Pais> pesquisarPais(Pais pais) {
		CrpPais p = new CrpPais();
		p = LocalizacaoWSConverter.convert(pais);
		Criteria c = dao.getCurrentSession().createCriteria(p.getClass());

		if (p != null) {
			if (p.getId() != null) {
				c.add(Restrictions.idEq(p.getId()));
			}
			if (p.getNome() != null && !"".equals(p.getNome())) {
				c.add(Restrictions.ilike("nome", "%" + p.getNome() + "%"));
			}
			if (p.getSigla() != null && !"".equals(p.getSigla())) {
				c.add(Restrictions.ilike("sigla", "%" + p.getSigla() + "%"));
			}
			c.addOrder(Order.asc("nome"));
		}

		return LocalizacaoWSConverter.convertPais(dao.listByExample(c));
	}

}
