package br.gov.mc.arquitetura.ws.dao;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;

import br.gov.mc.arquitetura.converter.ConverterSupport;
import br.gov.mc.arquitetura.converter.PessoaFisicaWSConverter;
import br.gov.mc.arquitetura.converter.PessoaJuridicaWSConverter;
import br.gov.mc.arquitetura.converter.PessoaWSConverter;
import br.gov.mc.arquitetura.util.PessoaFisicaUtil;
import br.gov.mc.arquitetura.ws.client.entidade.EmailPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.EnderecoPessoa;
import br.gov.mc.arquitetura.ws.client.entidade.Pessoa;
import br.gov.mc.arquitetura.ws.client.entidade.TelefonePessoa;
import br.gov.mc.arquitetura.ws.client.entidade.TipoTelefone;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpSituacaoPessoaOrgao;
import br.gov.mc.corporativo.entidade.CrpTipoPessoa;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;

@Repository("pessoaDAO")
public class PessoaDAO {

	@Autowired
	protected PessoaFisicaUtil dao;

	public PessoaDAO() throws Exception {

	}

	public Pessoa[] pesquisar(Pessoa pessoa) throws Exception {
		Collection<CrpPessoaFisica> pfs = dao.pesquisar(PessoaFisicaWSConverter.convert(pessoa));
		return PessoaWSConverter.convert(PessoaWSConverter.convert(pfs));
	}

	public Pessoa recuperar(Pessoa pessoa) throws Exception {
		CrpPessoaFisica pf = PessoaFisicaWSConverter.convert(pessoa);
		pf = (CrpPessoaFisica) dao.get(pf.getClass(), pf.getId());

		return PessoaWSConverter.convert(pf);
	}

	public Integer salvar(Pessoa pessoa) throws Exception {
		CrpPessoaFisica pf = PessoaFisicaWSConverter.convert(pessoa);
		pf.setDataSituacaoCadastral(new Date());
		pf.setDataAlteracao(new Date());
		pf.setTipoOperacao("I");
		pf.setSituacaoPessoaOrgao(new CrpSituacaoPessoaOrgao(1));
		pf.setTipoPessoa(new CrpTipoPessoa(1));
		if (pessoa.getDataNascimento() == null) {
			pf.setDataNascimento(new Date());
		}
		if (pessoa.getDataEmissaoRG() == null) {
			pf.setDataEmissao(new Date());
		}		
		if (pessoa.getSexo() == null) {
			pf.setSexo(new Character('N'));
		}
		dao.getCurrentHibernateTemplate().setCheckWriteOperations(false);
		Integer id = (Integer) dao.save(pf);
		pf.setId(id);
		if (pf.getDocumentos() != null) {
			for (CrpDocumentoPessoaFisica crpDocumentoPessoaFisica : pf.getDocumentos()) {
				crpDocumentoPessoaFisica.setPessoaFisica(pf);
				if (pessoa.getOrgaoEmissorRG() == null || "".equals(pessoa.getOrgaoEmissorRG().trim())) {
					crpDocumentoPessoaFisica.setOrgaoEmissor("RECEITA FEDERAL");
				}
				dao.save(crpDocumentoPessoaFisica);
			}
		}
		if (pf.getEmails() != null) {
			for (CrpContatoEmail contatoEmail : pf.getEmails()) {
				contatoEmail.setPessoa(pf);
				dao.save(contatoEmail);
			}
		}
		if (pf.getEnderecos() != null) {
			for (CrpEndereco crpEndereco : pf.getEnderecos()) {
				crpEndereco.setPessoa(pf);
				dao.save(crpEndereco);
			}
		}
		if (pf.getTelefones() != null) {
			for (CrpContatoTelefone contatoTelefone : pf.getTelefones()) {
				contatoTelefone.setPessoa(pf);
				dao.save(contatoTelefone);
			}
		}
		return id;
	}

	public void alterar(Pessoa pessoa) throws Exception {
		CrpPessoaFisica pf = PessoaFisicaWSConverter.convert(pessoa);
		pf.setDataSituacaoCadastral(new Date());
		pf.setDataAlteracao(new Date());
		pf.setTipoOperacao("I");
		pf.setSituacaoPessoaOrgao(new CrpSituacaoPessoaOrgao(1));
		pf.setTipoPessoa(new CrpTipoPessoa(1));
		if (pessoa.getDataNascimento() == null) {
			pf.setDataNascimento(new Date());
		}
		if (pessoa.getDataEmissaoRG() == null) {
			pf.setDataEmissao(new Date());
		}		
		if (pessoa.getSexo() == null) {
			pf.setSexo(new Character('N'));
		}
		if (pf.getDocumentos() != null) {
			for (CrpDocumentoPessoaFisica crpDocumentoPessoaFisica : pf.getDocumentos()) {
				/* Pesquisa tipo documento RG */
				if (crpDocumentoPessoaFisica.getTipoDocumento().getId() == 3) {
					List<CrpDocumentoPessoaFisica> cdpf = (List<CrpDocumentoPessoaFisica>)dao.pesquisar(pessoa.getId(), crpDocumentoPessoaFisica.getTipoDocumento().getId());
					if (cdpf != null && !cdpf.isEmpty()) {
						for (CrpDocumentoPessoaFisica dcto : cdpf) {
							CrpDocumentoPessoaFisica docPF = new CrpDocumentoPessoaFisica();
							docPF.setId(dcto.getId());
							docPF.setDataEmissao(dcto.getDataEmissao());
							docPF.setNumero(pessoa.getRg());
							docPF.setOrgaoEmissor(pessoa.getOrgaoEmissorRG());
							docPF.setPessoaFisica(dcto.getPessoaFisica());
							docPF.setTipoDocumento(dcto.getTipoDocumento());
							dao.update(docPF);
						}
					}
				}
			}
		}

		dao.getCurrentHibernateTemplate().setCheckWriteOperations(false);
		dao.update(pf);
	}

	public void remover(Pessoa pessoa) throws Exception {
		CrpPessoaFisica pf = PessoaFisicaWSConverter.convert(pessoa);
		dao.remove(pf.getClass(), pf.getId());
	}

	public EmailPessoa recuperar(EmailPessoa email) throws Exception {
		CrpContatoEmail ce = PessoaJuridicaWSConverter.convert(email);
		ce = (CrpContatoEmail) dao.get(ce.getClass(), ce.getId());
		return ConverterSupport.convertEmailPessoa(ce);
	}

	public Integer salvar(EmailPessoa email) throws Exception {
		CrpContatoEmail ce = PessoaJuridicaWSConverter.convert(email);
		dao.save(ce);
		return ce.getId();
	}

	public void alterar(EmailPessoa email) throws Exception {
		CrpContatoEmail ce = PessoaJuridicaWSConverter.convert(email);
		dao.update(ce);
	}

	public void remover(EmailPessoa email) throws Exception {
		dao.remove(CrpContatoEmail.class, email.getId());
	}

	public EnderecoPessoa recuperar(EnderecoPessoa endereco) throws Exception {
		CrpEndereco e = PessoaWSConverter.convert(endereco);
		e = (CrpEndereco) dao.get(e.getClass(), e.getId());
		return PessoaWSConverter.convert(e);
	}

	public Integer salvar(EnderecoPessoa endereco) throws Exception {
		CrpEndereco e = PessoaWSConverter.convert(endereco);
		e = (CrpEndereco) dao.save(e);
		return e.getId();
	}

	public void alterar(EnderecoPessoa endereco) throws Exception {
		CrpEndereco e = PessoaWSConverter.convert(endereco);
		dao.update(e);
	}

	public void remover(EnderecoPessoa endereco) throws Exception {
		CrpEndereco e = PessoaWSConverter.convert(endereco);
		dao.remove(e.getClass(), e.getId());
	}

	public TelefonePessoa recuperar(TelefonePessoa telefone) throws Exception {
		CrpContatoTelefone ct = PessoaWSConverter.convert(telefone);
		ct = (CrpContatoTelefone) dao.get(ct.getClass(), ct.getId());

		return PessoaWSConverter.convert(ct);
	}

	public Integer salvar(TelefonePessoa telefone) throws Exception {
		CrpContatoTelefone ct = PessoaWSConverter.convert(telefone);
		dao.save(ct);
		return ct.getId();
	}

	public void alterar(TelefonePessoa telefone) throws Exception {
		CrpContatoTelefone ct = PessoaWSConverter.convert(telefone);
		dao.update(ct);
	}

	public void remover(TelefonePessoa telefone) throws Exception {
		dao.remove(CrpContatoTelefone.class, telefone.getId());
	}

	public List<TipoTelefone> listarTipoTelefone() throws Exception {
		List<CrpTipoTelefone> lista = (List<CrpTipoTelefone>) dao.listByExample(new CrpTipoTelefone());
		List<TipoTelefone> listaRetorno = new ArrayList<TipoTelefone>();
		for (CrpTipoTelefone tel : lista) {
			listaRetorno.add(PessoaWSConverter.convert(tel));
		}
		return listaRetorno;
	}

	public boolean temLog(Integer idUsuario){
		List lista = dao.getCurrentSession().createQuery("from SegLog s where s.idUsuario = ?").setParameter(0, idUsuario).list();
		if (lista.size() > 0){
			return true;
		}else{
			return false;
		}
	}
	
	public boolean removerUsuario(Integer id) {
		try {
			dao.getCurrentSession().createSQLQuery("delete from SEG_USUARIO_EXTERNO where id_usuario = ?").setParameter(0, id).executeUpdate();
			dao.getCurrentSession().createSQLQuery("delete from SEG_USUARIO where id_usuario = ?").setParameter(0, id).executeUpdate();
		} catch (org.hibernate.exception.ConstraintViolationException e) {
			e.printStackTrace();
			return false;
		}
		return true;
	}

}
