package br.gov.mc.cadsei.bean;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.sql.rowset.serial.SerialBlob;

import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.criterion.Restrictions;
import org.primefaces.event.FileUploadEvent;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.arquitetura.util.StringUtil;
import br.gov.mc.arquitetura.util.PessoaJuridicaUtil;
import br.gov.mc.cadsei.entidade.CadDocumento;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadHistoricoEmpresa;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.CadTipoDocumentoD;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.util.CompareCrpMunicipio;
import br.gov.mc.cadsei.util.CompareCrpUf;
import br.gov.mc.cadsei.util.Constantes;
import br.gov.mc.cadsei.util.CpfCnpjUtil;
import br.gov.mc.cadsei.util.SessionUtil;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpMunicipio;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpPortePessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;
import br.gov.mc.corporativo.entidade.CrpTipoDocPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoEndereco;
import br.gov.mc.corporativo.entidade.CrpTipoPessoa;
import br.gov.mc.corporativo.entidade.CrpTipoPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;
import br.gov.mc.corporativo.entidade.CrpUf;

@ManagedBean(name = "cadastroEmpresaBean")
@ViewScoped
public class CadastroEmpresaBean extends BaseBean<CadEmpresa> {

    private static final long serialVersionUID = 1L;
    private static final String EMPRESA = "idEmpresa";
    private static final String QUERY_ONLY = "somenteConsulta";

    private boolean proponenteRepresentante = true;
    private boolean somenteConsulta;
    private String emailConfirmacao;
    private String cnpjSemFormatacao;
    private String qualifiedName;
    private boolean validated;
    private boolean rgValidado;
    private boolean editarPj;
    private boolean renderizaUploadDocumentos;

    private String pageTitle;

    private CrpContatoTelefone telefone;
    private CrpContatoEmail email;
    private CrpEndereco endereco;
    private CrpUf uf;
    private CrpMunicipio municipio;
    private CrpDocumentoPessoaFisica cpfRepresentante;
    private CrpDocumentoPessoaFisica rgRepresentante;
    private List<CrpUf> listaUf;
    private List<CrpMunicipio> listaMunicipio;
    private CadDocumento licenca;
    private CadDocumento contrato;

    private String confirmacaoEmailAdicional1;
    private String confirmacaoEmailAdicional2;
    private String confirmacaoEmailAdicional3;
    private String confirmacaoEmailAdicional4;

    // OST 92858: Adicionados novos documentos para a pessoa jurídica
    private CadDocumento atosConstitutivos;
    private CadDocumento comprovanteCnpj;
    private CadDocumento comprovanteRepresentanteLegal;

    private List<CadServico> servicos;

    private Integer[] listaIdServicos;

    // Gerado para contemplar a geração de histórico.
    private Map<Integer, Object> valoresOriginais;
    private Map<Integer, Object> valoresSubmetidos;

    @ManagedProperty(value = "#{pessoaJuridicaUtil}")
    private transient PessoaJuridicaUtil pessoaJuridicaUtil;

    @ManagedProperty(value = "#{empresaService}")
    private transient EmpresaService empresaService;

    @ManagedProperty(value = "#{genericService}")
    private transient GenericService genericService;

    private void createTitularModel() {
	this.telefone = new CrpContatoTelefone();
	this.telefone.setTipoTelefone(new CrpTipoTelefone(CrpTipoTelefone.COMERCIAL));
	this.email = new CrpContatoEmail();
	this.endereco = new CrpEndereco();
	this.endereco.setMunicipio(new CrpMunicipio());
	this.endereco.setTipoEndereco(new CrpTipoEndereco(CrpTipoEndereco.COMERCIAL));
	this.uf = new CrpUf();
	this.municipio = new CrpMunicipio();
	this.cpfRepresentante = new CrpDocumentoPessoaFisica();
	this.cpfRepresentante.setTipoDocumento(new CrpTipoDocPessoaFisica(CrpTipoDocPessoaFisica.CPF));
	this.rgRepresentante = new CrpDocumentoPessoaFisica();
	this.rgRepresentante.setTipoDocumento(new CrpTipoDocPessoaFisica(CrpTipoDocPessoaFisica.RG));
    }

    @Override
    public CadEmpresa createModel() {

	// Inicializa os objetos manipulados pela view
	this.cnpjSemFormatacao = null;
	this.editarPj = false;
	this.somenteConsulta = false;
	this.validated = false;
	this.rgValidado = false;
	this.qualifiedName = "Pessoa Jur&iacute;dica";
	// Define o título do frame principal da página
	this.setPageTitle("Nova " + this.qualifiedName);
	this.licenca = new CadDocumento();
	this.contrato = new CadDocumento();
	this.createTitularModel();

	// Cria o model com suas dependências
	CadEmpresa empresa = new CadEmpresa();
	empresa.setPessoaTitular(new CrpPessoaJuridica());
	empresa.setProponente(SessionUtil.getProponenteLogado());
	return empresa;

    }

    @SuppressWarnings("unchecked")
    public boolean validateCPF() {

	// Se o campo estiver preenchido, valida-o
	if (this.cpfRepresentante.getNumero() != null && !this.cpfRepresentante.getNumero().isEmpty()) {

	    this.cpfRepresentante.setNumero(this.cpfRepresentante.getNumero().replace(".", "").replace("-", ""));
	    if (!(CpfCnpjUtil.ValidateCPF(this.cpfRepresentante.getNumero()))) {
		FacesContext.getCurrentInstance().addMessage("cpfRepresentante",
			new FacesMessage(FacesMessage.SEVERITY_ERROR, "", "CPF Inv&aacute;lido"));
		return false;
	    } else {
		List<CrpDocumentoPessoaFisica> doctos = (List<CrpDocumentoPessoaFisica>) this.getGenericService()
			.createCriteria(CrpDocumentoPessoaFisica.class)
			.add(Restrictions.eq("numero", this.cpfRepresentante.getNumero()))
			.createCriteria("pessoaFisica", Criteria.LEFT_JOIN).list();
		if (doctos.size() > 0) {
		    this.setCpfRepresentante(doctos.get(0));
		    this.getModel().getRepresentante().setPessoaFisica(doctos.get(0).getPessoaFisica());
		    for (CrpDocumentoPessoaFisica documento : this.getModel().getRepresentante().getPessoaFisica()
			    .getDocumentos()) {
			if (documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG))
			    this.setRgRepresentante(documento);
		    }
		}
	    }
	}
	return true;
    }

    /*
     * @SuppressWarnings("unchecked") public boolean validateRG() { // Se o
     * campo estiver preenchido, valida-o if (this.rgRepresentante.getNumero()
     * != null && !this.rgRepresentante.getNumero().isEmpty()) {
     * List<CrpDocumentoPessoaFisica> doctos = (List<CrpDocumentoPessoaFisica>)
     * this.getGenericService() .createCriteria(CrpDocumentoPessoaFisica.class)
     * .add(Restrictions.eq("numero", this.rgRepresentante.getNumero())).list();
     * if (doctos.size() > 0) { if (this.getModel().getQtdEmpregados() != null
     * && this.getModel().getQtdEmpregados().doubleValue() == 0D) {
     * this.getModel().setQtdEmpregados(null); } this.rgValidado = false;
     * FacesContext.getCurrentInstance().addMessage("rgRepresentante", new
     * FacesMessage(FacesMessage.SEVERITY_ERROR, "",
     * "RG j&aacute; cadastrado!")); return false; } } this.rgValidado = true;
     * return true; }
     */
    private boolean validaComunicacaoOficial() {

	if (this.getEmail().getDescricao().isEmpty() || this.getEmailConfirmacao().isEmpty()
		|| this.getTelefone().getNumDdd().isEmpty() || this.getTelefone().getNumTelefone().isEmpty()) {

	    FacesContext.getCurrentInstance().addMessage(
		    null,
		    new FacesMessage(FacesMessage.SEVERITY_ERROR,
			    "Informa&ccedil;&otilde;es de comunica&ccedil;&atilde;o oficial  obrigat&oacute;rias!",
			    null));
	    return false;
	}

	if (!this.getEmailConfirmacao().isEmpty()) {
	    if (!this.getEmailConfirmacao().equals(this.getEmail().getDescricao())) {
		FacesContext.getCurrentInstance().addMessage("emailConfirmacao",
			new FacesMessage(FacesMessage.SEVERITY_ERROR, "E-mail informado n&atilde;o confere!", null));
		return false;
	    }
	}

	if (!this.getConfirmacaoEmailAdicional1().isEmpty()) {
	    if (!this.getConfirmacaoEmailAdicional1().equals(this.getModel().getEmailAdicional1())) {
		FacesContext.getCurrentInstance().addMessage(
			"emailConfirmacaoAdiciona1",
			new FacesMessage(FacesMessage.SEVERITY_ERROR,
				"E-mail adicional 1 informado n&atilde;o confere!", null));
		return false;
	    }
	}

	if (!this.getConfirmacaoEmailAdicional2().isEmpty()) {
	    if (!this.getConfirmacaoEmailAdicional2().equals(this.getModel().getEmailAdicional2())) {
		FacesContext.getCurrentInstance().addMessage(
			"emailConfirmacaoAdiciona2",
			new FacesMessage(FacesMessage.SEVERITY_ERROR,
				"E-mail adicional 2 informado n&atilde;o confere!", null));
		return false;
	    }
	}

	if (!this.getConfirmacaoEmailAdicional3().isEmpty()) {
	    if (!this.getConfirmacaoEmailAdicional3().equals(this.getModel().getEmailAdicional3())) {
		FacesContext.getCurrentInstance().addMessage(
			"emailConfirmacaoAdiciona3",
			new FacesMessage(FacesMessage.SEVERITY_ERROR,
				"E-mail adicional 3 informado n&atilde;o confere!", null));
		return false;
	    }
	}

	if (!this.getConfirmacaoEmailAdicional4().isEmpty()) {
	    if (!this.getConfirmacaoEmailAdicional4().equals(this.getModel().getEmailAdicional4())) {
		FacesContext.getCurrentInstance().addMessage(
			"emailConfirmacaoAdiciona4",
			new FacesMessage(FacesMessage.SEVERITY_ERROR,
				"E-mail adicional 4 informado n&atilde;o confere!", null));
		return false;
	    }
	}

	return true;
    }

    public boolean validateCNPJ() {

	// Se o campo estiver preenchido
	if (this.getModel().getPessoaTitular().getCnpj() != null) {

	    List<CrpPessoaJuridica> listPj = null;
	    CrpPessoaJuridica pj = null;
	    this.cnpjSemFormatacao = this.getModel().getPessoaTitular().getCnpj().replace("-", "").replace("/", "")
		    .replace(".", "");
	    this.getModel().getPessoaTitular().setCnpj(this.cnpjSemFormatacao);
	    if (!CpfCnpjUtil.validarCNPJ(this.cnpjSemFormatacao)) {
		this.clearCampos();
		FacesContext.getCurrentInstance().addMessage("cnpj",
			new FacesMessage(FacesMessage.SEVERITY_ERROR, "", "CNPJ inv&aacute;lido."));
		return false;
	    } else {
		// Mantis 363/364: Invocar o mc-arquitetura o qual validará se
		// a PJ existe na base corporativa
		// pj = validatePessoaJuridica();

		// Verifica se a pessoa existe na Base do Corporativo
		listPj = (List<CrpPessoaJuridica>) this.getGenericService().createCriteria(CrpPessoaJuridica.class)
			.add(Restrictions.eq("cnpj", this.cnpjSemFormatacao)).list();
		if (listPj.size() > 0) {
		    pj = listPj.get(0);
		}
	    }

	    this.validated = true;
	    if (pj == null || pj.getId() == null) {
		return true;
	    }
	    this.getModel().getPessoaTitular().setId(pj.getId());
	    this.getModel().getPessoaTitular().setCnpj(pj.getCnpj());
	    this.getModel()
		    .getPessoaTitular()
		    .setNomeFantasia(
			    (pj.getNomeFantasia() != null && "".equals(pj.getNomeFantasia().trim())) ? null : pj
				    .getNomeFantasia());
	    this.getModel()
		    .getPessoaTitular()
		    .setRazaoSocial(
			    (pj.getRazaoSocial() != null && "".equals(pj.getRazaoSocial().trim())) ? null : pj
				    .getRazaoSocial());
	    this.getModel().getPessoaTitular()
		    .setUrl((pj.getUrl() != null && "".equals(pj.getUrl().trim())) ? null : pj.getUrl());
	    this.getModel().getPessoaTitular().setFlgAtualizadoReceita(true);
	    this.getModel().getPessoaTitular().setTipoPessoa(pj.getTipoPessoa());
	    this.getModel().getPessoaTitular().setTipoPessoaJuridica(pj.getTipoPessoaJuridica());

	    for (CrpEndereco endereco : pj.getEnderecos()) {
		this.setEndereco(endereco);
		if (this.getEndereco().getLogradouro() != null && "".equals(this.getEndereco().getLogradouro().trim())) {
		    this.getEndereco().setLogradouro(null);
		}
		if (this.getEndereco().getNumero() != null && "".equals(this.getEndereco().getNumero().trim())) {
		    this.getEndereco().setNumero(null);
		}
		if (this.getEndereco().getBairro() != null && "".equals(this.getEndereco().getBairro().trim())) {
		    this.getEndereco().setBairro(null);
		}
		if (!(this.getEndereco().getMunicipio() == null)) {
		    this.setMunicipio(this.getEndereco().getMunicipio());
		    this.setUf(this.getMunicipio().getUf());

		    // Recarrega a lista de municípios e define o
		    // município da pessoa jurídica
		    this.changeUF();
		}

		break;
	    }

	    for (CrpContatoTelefone telefone : pj.getTelefones()) {
		if (telefone.getTipoTelefone().getId().equals(CrpTipoTelefone.COMERCIAL)) {
		    this.setTelefone(telefone);
		    if (this.getTelefone().getNumDdd() != null && "".equals(this.getTelefone().getNumDdd().trim())) {
			this.getTelefone().setNumDdd(null);
		    }
		    if (this.getTelefone().getNumTelefone() != null
			    && "".equals(this.getTelefone().getNumTelefone().trim())) {
			this.getTelefone().setNumTelefone(null);
		    }
		}
	    }

	    for (CrpContatoEmail email : pj.getEmails()) {
		this.setEmail(email);
		if (this.getEmail().getDescricao() != null && "".equals(this.getEmail().getDescricao().trim())) {
		    this.getEmail().setDescricao(null);
		}
		this.setEmailConfirmacao(this.getEmail().getDescricao());
	    }

	    boolean achouCpfRepresentante = false;
	    boolean achouRgRepresentante = false;
	    for (CrpRepresentantePesJur representante : pj.getRepresentantesLegais()) {
		// Pega o representante com a Data fim nula (indicando que
		// esta ativo)
		if (representante.getDataFim() == null) {

		    for (CrpDocumentoPessoaFisica documento : representante.getPessoaFisica().getDocumentos()) {
			if (documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.CPF)) {
			    if (SessionUtil.getProponenteLogado().getLoginProponente().equals(documento.getNumero())) {
				this.getModel().setRepresentante(representante);
				this.setCpfRepresentante(documento);
				if (this.getCpfRepresentante().getNumero() != null
					&& "".equals(this.getCpfRepresentante().getNumero().trim())) {
				    this.getCpfRepresentante().setNumero(null);
				}
				achouCpfRepresentante = true;
			    }
			} else if (documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG)) {
			    if (achouCpfRepresentante) {
				this.setRgRepresentante(documento);
				if (this.getRgRepresentante().getNumero() != null
					&& "".equals(this.getRgRepresentante().getNumero().trim())) {
				    this.getRgRepresentante().setNumero(null);
				}
				if (this.getRgRepresentante().getOrgaoEmissor() != null
					&& "".equals(this.getRgRepresentante().getOrgaoEmissor().trim())) {
				    this.getRgRepresentante().setOrgaoEmissor(null);
				}
				// Sempre que o RG já existir considera-o
				// válido
				this.rgValidado = true;
				achouRgRepresentante = true;
			    }

			}
			if (achouCpfRepresentante && achouRgRepresentante)
			    break;

		    }
		    // break;
		}
	    }

	}

	return true;

    }

    private CrpPessoaJuridica validatePessoaJuridica() {
	// Metodo para recuperar CNPJ da base da Receita Federal.
	List<CrpPessoaJuridica> listaPessoaJuridica = null;
	CrpPessoaJuridica pj = new CrpPessoaJuridica();
	pj.setCnpj(this.cnpjSemFormatacao);
	try {
	    // Metodo da arquitetura para recuperar CNPJ da Receita.
	    listaPessoaJuridica = (List<CrpPessoaJuridica>) this.pessoaJuridicaUtil.pesquisar(pj);
	    if (listaPessoaJuridica.size() > 0) {
		return listaPessoaJuridica.get(0);
	    }
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	}

	return null;

    }

    public void handleUploadAtosConstitutivos(FileUploadEvent event) {
	try {
	    this.getAtosConstitutivos().setDocumento(new SerialBlob(event.getFile().getContents()));
	    this.getAtosConstitutivos().setNomDocumento(event.getFile().getFileName());
	    if (this.getAtosConstitutivos().getId() == null) {
		this.getModel().addDocumento(this.getAtosConstitutivos());
	    }
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	}
    }

    public void handleUploadComprovanteCnpj(FileUploadEvent event) {
	try {
	    this.getComprovanteCnpj().setDocumento(new SerialBlob(event.getFile().getContents()));
	    this.getComprovanteCnpj().setNomDocumento(event.getFile().getFileName());
	    if (this.getComprovanteCnpj().getId() == null) {
		this.getModel().addDocumento(this.getComprovanteCnpj());
	    }
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	}
    }

    public void handleUploadComprovanteRepresentanteLegal(FileUploadEvent event) {
	try {
	    this.getComprovanteRepresentanteLegal().setDocumento(new SerialBlob(event.getFile().getContents()));
	    this.getComprovanteRepresentanteLegal().setNomDocumento(event.getFile().getFileName());
	    if (this.getComprovanteRepresentanteLegal().getId() == null) {
		this.getModel().addDocumento(this.getComprovanteRepresentanteLegal());
	    }
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	}
    }

    private void clearCampos() {
	if (this.getModel().getQtdEmpregados() != null && this.getModel().getQtdEmpregados().doubleValue() == 0D) {
	    this.getModel().setQtdEmpregados(null);
	}
	if (this.getModel().getAnoConstituicao() != null && this.getModel().getAnoConstituicao() == 0) {
	    this.getModel().setAnoConstituicao(null);
	}
	if (this.getModel().getPessoaTitular().getCapitalSocial() != null
		&& this.getModel().getPessoaTitular().getCapitalSocial().doubleValue() == 0D) {
	    this.getModel().getPessoaTitular().setCapitalSocial(null);
	}

    }

    @Override
    public String save() {
	if (!validaServicos(getModel())) {
	    FacesContext.getCurrentInstance().addMessage("servico",
		    new FacesMessage(FacesMessage.SEVERITY_ERROR, "Selecione pelo menos um servio!.", null));
	    return ERROR;
	}
	// Valida o endereço de e-mail informado
	if (!validaComunicacaoOficial()) {
	    return ERROR;
	}

	// Valida CPF do representante legal com o do proponente logado
	/*
	 * if(!Mascara.retirarMascara(SessionUtil.getProponenteLogado().
	 * getLoginProponente())
	 * .equals(Mascara.retirarMascara(this.getCpfRepresentante
	 * ().getNumero()))){ this.setProponenteRepresentante(false);
	 * FacesContext.getCurrentInstance().addMessage(null, new
	 * FacesMessage(FacesMessage.SEVERITY_ERROR, null,
	 * "O CPF do usurio logado n&atilde;o &eacute; o mesmo cadastrado como respons&aacute;vel por "
	 * +
	 * "esta Pessoa Jur&iacute;dica na Receita Federal do Brasil. Apenas o respons&aacute;vel "
	 * +
	 * "pela Pessoa Jur&iacute;dica na Receita Federal pode cadastr&agrave;-la no sistema. "
	 * +
	 * "Atualize os dados da Pessoa Jur&iacute;dica na Receita Federal e tente novamente cadastr&agrave;-la novamente."
	 * 
	 * )); return ERROR; }
	 */
	/*
	 * if(!this.rgValidado && !validateRG()){ return ERROR; }
	 */

	if (this.getModel().getDocumentos().size() > 0) {
	    boolean atosConstitutivos = false;
	    boolean representanteLegal = false;
	    boolean cnpj = false;
	    for (CadDocumento documentos : this.getModel().getDocumentos()) {
		if (documentos.getTipoDocumento().getId().equals(CadTipoDocumentoD.ATOS_CONSTITUTIVOS)) {
		    atosConstitutivos = true;
		}
		if (documentos.getTipoDocumento().getId().equals(CadTipoDocumentoD.REPRESENTANTE_LEGAL)) {
		    representanteLegal = true;
		}
		if (documentos.getTipoDocumento().getId().equals(CadTipoDocumentoD.CNPJ)) {
		    cnpj = true;
		}
	    }
	    if (!atosConstitutivos || !representanteLegal || !cnpj) {
		FacesMessagesUtil.addErrorMessage(Constantes.INFORMAR_DOCUMENTOS);
		return ERROR;
	    }
	} else {
	    FacesMessagesUtil.addErrorMessage(Constantes.INFORMAR_DOCUMENTOS);
	    return ERROR;
	}

	// Somenta valida o CNPJ e CPF no cadastro da pessoa jurídica
	if (!this.editarPj) {
	    if (!this.validated && !validateCNPJ())
		return ERROR;

	    if (!validateCPF())
		return ERROR;

	    // Define a data de cadastro da empresa
	    this.getModel().setDataCadastro(new Date());

	}

	if (this.getModel().getAnoConstituicao() != null) {
	    Calendar anoAtual = Calendar.getInstance();
	    if (this.getModel().getAnoConstituicao() > anoAtual.get(Calendar.YEAR)) {
		this.clearCampos();
		FacesContext.getCurrentInstance().addMessage(
			"anoConstituicao",
			new FacesMessage(FacesMessage.SEVERITY_ERROR, null,
				"N&atilde;o pode ser maior que o ano atual."));
		return ERROR;
	    }
	}

	try {

	    // Grava RG do representante
	    this.getModel().getRepresentante().setRepresentante(true);

	    if (this.getRgRepresentante().getNumero() != null && !this.getRgRepresentante().getNumero().isEmpty()) {
		CrpDocumentoPessoaFisica rg = new CrpDocumentoPessoaFisica();
		rg.setNumero(this.getRgRepresentante().getNumero());
		rg.setOrgaoEmissor(this.getRgRepresentante().getOrgaoEmissor());
		rg.setPessoaFisica(this.getModel().getRepresentante().getPessoaFisica());
		rg.setTipoDocumento(this.getRgRepresentante().getTipoDocumento());
		if (this.getRgRepresentante().getId() != null) {
		    rg.setId(this.getRgRepresentante().getId());
		}
		this.getGenericService().save(rg);
	    }
	    if(this.getModel().getPessoaTitular()!=null && this.getModel().getPessoaTitular().getCnpj()!=null
		    && this.getModel().getPessoaTitular().getCnpj().length()>14){
	    this.getModel().getPessoaTitular()
		    .setCnpj(StringUtil.unformat("##.###.###/####-##", this.getModel().getPessoaTitular().getCnpj()));
	    }
	    this.getGenericService().save(this.getModel().getPessoaTitular());
	    // Grava os e-mails da PJ
	    this.getEmail().setEmailPessoal(Boolean.FALSE);
	    this.getEmail().setPessoa(this.getModel().getPessoaTitular());
	    this.getGenericService().save(this.getEmail());

	    // Grava o telefone da PJ
	    this.getTelefone().setPessoa(this.getModel().getPessoaTitular());
	    this.getGenericService().save(this.getTelefone());

	    if (this.getEndereco() != null && this.getMunicipio()!=null && this.getMunicipio().getId()!=null){
		this.getEndereco().setMunicipio(this.getMunicipio());
		this.getEndereco().setPessoa(this.getModel().getPessoaTitular());
		this.getGenericService().save(this.getEndereco());
	    }
	    // Salva os dados do representante legalsl
	    this.getModel().getRepresentante().setPessoaJuridica(this.getModel().getPessoaTitular());
	    this.getGenericService().save(this.getModel().getRepresentante());

	    // Salva os dados da pessoa jurídica na base corporativa
	    this.getModel().getPessoaTitular().setPorte(new CrpPortePessoaJuridica(3)); // Demais
	    this.getModel().getPessoaTitular().setDataAlteracao(new Date());
	    this.getModel().getPessoaTitular().setDataSituacaoCadastral(new Date());
	    this.getModel().getPessoaTitular().setTipoOperacao("A");
	    this.getModel().getPessoaTitular().setTipoPessoa(new CrpTipoPessoa(CrpTipoPessoa.JURIDICA_CNPJ));
	    this.getModel().getPessoaTitular()
		    .setTipoPessoaJuridica(new CrpTipoPessoaJuridica(CrpTipoPessoaJuridica.NAO_INFORMADO));
	    this.getModel().getPessoaTitular().setFlgAtualizadoReceita(false);
	    if (this.getModel().getPessoaTitular().getCnpj().length() > 14) {
		this.getModel()
			.getPessoaTitular()
			.setCnpj(
				StringUtil.unformat("##.###.###/####-##", this.getModel().getPessoaTitular().getCnpj()));
	    }

	    this.getModel().getRepresentante().setPessoaFisica(SessionUtil.getProponenteLogado().getPessoaProponente());
	    this.getModel().getRepresentante().setPessoaJuridica(this.getModel().getPessoaTitular());
	    this.getGenericService().save(this.getModel().getRepresentante());

	    valoresSubmetidos = new HashMap<Integer, Object>();
	    valoresSubmetidos.put(1, getTelefone().getNumDdd());
	    valoresSubmetidos.put(2, getTelefone().getNumTelefone());
	    valoresSubmetidos.put(3, getEmail().getDescricao());

	    // caso o status seja igual a aprovado, e for uma edição, logo o
	    // status passará a ser NECESSITA_RATFIICAÇÃO".
	    if (getModel().getId() == null) {
		this.getModel().setSituacaoCadastro(new CadSituacaoCadastroD(CadSituacaoCadastroD.AGUARDANDO_ANALISE));
	    }
	    if (this.getModel().getId() != null) {
		if (this.getModel().getSituacaoCadastro() != null) {
		    if (this.getModel().getSituacaoCadastro().getId().equals(CadSituacaoCadastroD.APROVADO)) {
			if (!valoresOriginais.equals(valoresSubmetidos))
			    this.getModel().setSituacaoCadastro(
				    new CadSituacaoCadastroD(CadSituacaoCadastroD.NECESSITA_RATIFICACAO));
		    }
		    if (this.getModel().getId() != null
			    && this.getModel().getSituacaoCadastro().getId().equals(CadSituacaoCadastroD.REPROVADO)) {
			this.getModel().setSituacaoCadastro(
				new CadSituacaoCadastroD(CadSituacaoCadastroD.AGUARDANDO_ANALISE));
		    }
		}
	    }
	    if (this.getModel().getId() != null) {
		if (valoresOriginais != null) {
		    salvarHistorico(valoresOriginais, valoresSubmetidos);
		}
	    }

	    // Salva os dados da empresa e proponente
	    List<CadServico> listaServ = this.servicos;

	    this.getModel().setServicos(new ArrayList<CadServico>());
	    this.getGenericService().saveEmpresa(this.getModel());
	    // para gravar o histórico, obtêm-se os valores submetidos e
	    // adiciona-os em um MAP.

	} catch (Exception e) {
	    FacesMessagesUtil.addErrorMessage(this.getQualifiedName(), ConstantesARQ.ERRO_SALVAR);
	    ExcecaoUtil.tratarExcecao(e);
	    return ERROR;
	}

	FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), this.getSaveMessage() + " "
		+ ConstantesARQ.COM_SUCESSO);
	if (!this.editarPj) {
	    this.setModel(this.createModel());
	}

	FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("showSaveMessage", true);
	return this.redirect("pessoa-juridica.jsf");

    }

    private boolean validaServicos(CadEmpresa model) {
	return true;/* !model.getServicos().isEmpty(); */
    }

    private void salvarHistorico(Map<Integer, Object> valoresOriginais2, Map<Integer, Object> valoresSubmetidos2) {
	if (!valoresOriginais.equals(valoresSubmetidos)) {
	    String camposAlterados = "";
	    CadHistoricoEmpresa h = new CadHistoricoEmpresa();

	    if (!isDDDIgual(valoresOriginais, valoresSubmetidos))
		camposAlterados = camposAlterados.equals("") ? camposAlterados = " DDDTelefone " : camposAlterados
			.concat(", DDDTelefone ");

	    if (!isTelefonelIgual(valoresOriginais, valoresSubmetidos))
		camposAlterados = camposAlterados.equals("") ? camposAlterados = " Telefone " : camposAlterados
			.concat(", Telefone ");

	    if (!isEmailIgual(valoresOriginais, valoresSubmetidos))
		camposAlterados = camposAlterados.equals("") ? camposAlterados = " Email " : camposAlterados
			.concat(", Email ");

	    h.setEmpresa(getModel());
	    h.setCampoAlterado(camposAlterados);
	    h.setDtAlteracao(new Date());
	    h.setSituacaoCadastro(getModel().getSituacaoCadastro());
	    h.setAnalista(null);
	    try {
		this.getGenericService().save(h);

		// a partir do momento em que a tela for salva, os valores
		// originais passarão a ser os admitidos pelo bano, ou seja, os
		// novos valores.
	    } catch (Exception e) {
		FacesMessagesUtil.addErrorMessage(this.getQualifiedName(), ConstantesARQ.ERRO_SALVAR);
		ExcecaoUtil.tratarExcecao(e);
	    }
	}
    }

    private boolean isDDDIgual(Map<Integer, Object> valoresOriginais, Map<Integer, Object> valoresSubmetidos) {
	return valoresOriginais.get(1).equals(valoresSubmetidos.get(1));
    }

    // caso o Telefone submetido igual ao obtido ao carregar a tela, será
    // passível de armazenamento na tabela histórico.
    private Boolean isTelefonelIgual(Map<Integer, Object> valoresOriginais, Map<Integer, Object> valoresSubmetidos) {
	return valoresOriginais.get(2).equals(valoresSubmetidos.get(2));
    }

    // caso o Email submetido igual ao obtido ao carregar a tela, será
    // passível
    // de armazenamento na tabela histórico.
    private Boolean isEmailIgual(Map<Integer, Object> valoresOriginais, Map<Integer, Object> valoresSubmetidos) {
	return valoresOriginais.get(3).equals(valoresSubmetidos.get(3));

    }

    @SuppressWarnings("unchecked")
    public void changeUF() {
	if (this.getUf() != null) {
	    this.setListaMunicipio(this.getGenericService().createCriteria(CrpMunicipio.class)
		    .add(Restrictions.eq("uf", this.getUf())).list());
	    Collections.sort(this.listaMunicipio, new CompareCrpMunicipio());
	}
    }

    @SuppressWarnings("unchecked")
    public List<CrpUf> getListaUf() {
	if (this.listaUf == null) {
	    this.listaUf = this.getUniversalManager().getAll(CrpUf.class);
	    Collections.sort(this.listaUf, new CompareCrpUf());
	}
	return listaUf;
    }

    @PostConstruct
    public void init() {
	endereco = new CrpEndereco();
	endereco.setMunicipio(new CrpMunicipio());
	setValoresOriginais(new HashMap<Integer, Object>());
	setValoresSubmetidos(new HashMap<Integer, Object>());
	setServicos(new ArrayList<CadServico>());
	setServicos(empresaService.buscaServicos());

	getModel().getRepresentante().setPessoaFisica(SessionUtil.getProponenteLogado().getPessoaProponente());
	this.cpfRepresentante.setNumero(SessionUtil.getProponenteLogado().getLoginProponente());
	verificaUploadDocumentos();
	Boolean somenteConsulta = (Boolean) FacesContext.getCurrentInstance().getExternalContext().getSessionMap()
		.remove(QUERY_ONLY);
	if (somenteConsulta != null) {
	    this.setSomenteConsulta(somenteConsulta.booleanValue());
	}
	this.carregarTela();
    }

    private void verificaUploadDocumentos() {
	setRenderizaUploadDocumentos(true);
	if (getModel().getId() != null && getModel().getSituacaoCadastro() != null) {
	    if (getModel().getId() != null && getModel().getSituacaoCadastro().getId() == 3) {
		setRenderizaUploadDocumentos(false);
	    }
	}

	if (getModel().getId() == null) {
	    setRenderizaUploadDocumentos(true);
	}
    }

    public void preload() {
	if (FacesContext.getCurrentInstance().getRenderResponse()) {
	    if (FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(EMPRESA) != null) {
		this.clearCampos();
		if (this.getModel().getPessoaTitular() == null) {
		    this.getModel().setPessoaTitular(new CrpPessoaJuridica());
		}
		this.editarPj = true;
	    }
	}
    }

    public void carregarTela() {
	if (FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(EMPRESA) != null) {
	    // Define o título do frame principal da página
	    this.setPageTitle("Editando " + this.getQualifiedName());

	    Integer idEmpresa = (Integer) FacesContext.getCurrentInstance().getExternalContext().getSessionMap()
		    .get(EMPRESA);
	    CadEmpresa e = (CadEmpresa) empresaService.buscaEmpresaPorId(idEmpresa);
	    e.setServicos(empresaService.buscaServicosPorEmpresa(idEmpresa));
	    e.setDocumentos(empresaService.pesquisaDocumentosPorEmpresa(e.getId()));
	    Hibernate.initialize(e.getRepresentante());
	    if (e.getServicos().isEmpty())
		e.setServicos(new ArrayList<CadServico>());
	    this.setModel(e);

	    if (this.getModel().getQtdEmpregados() != null && this.getModel().getQtdEmpregados().doubleValue() == 0D) {
		this.getModel().setQtdEmpregados(null);
	    }

	    List<CrpEndereco> endereco = (List<CrpEndereco>) e.getPessoaTitular().getEnderecos();
	    if (endereco != null && endereco.size() > 0) {
		this.endereco = endereco.get(0);
		this.municipio = this.endereco.getMunicipio();
		this.uf = this.endereco.getMunicipio().getUf();
		// Recarrega a lista de municípios e define o município da
		// pessoa jurídica
		if (this.endereco != null)
		    this.changeUF();
	    }

	    if (e.getDocumentos() != null && !e.getDocumentos().isEmpty()) {
		for (CadDocumento documento : e.getDocumentos()) {
		    if (CadTipoDocumentoD.ATOS_CONSTITUTIVOS.equals(documento.getTipoDocumento().getId())) {
			this.setAtosConstitutivos(documento);
		    } else if (CadTipoDocumentoD.CNPJ.equals(documento.getTipoDocumento().getId())) {
			this.setComprovanteCnpj(documento);
		    } else if (CadTipoDocumentoD.REPRESENTANTE_LEGAL.equals(documento.getTipoDocumento().getId())) {
			this.setComprovanteRepresentanteLegal(documento);
		    }
		}
	    }

	    List<CrpContatoTelefone> telefones = (List<CrpContatoTelefone>) e.getPessoaTitular().getTelefones();
	    if (telefones != null && !telefones.isEmpty()) {
		for (CrpContatoTelefone telefone : telefones) {
		    if (CrpTipoTelefone.COMERCIAL.equals(telefone.getTipoTelefone().getId())) {
			this.setTelefone(telefone);
		    }
		}
	    }

	    List<CrpContatoEmail> emails = (List<CrpContatoEmail>) e.getPessoaTitular().getEmails();
	    if (emails != null && !emails.isEmpty()) {
		for (CrpContatoEmail email : emails) {
		    this.setEmail(email);
		    this.setEmailConfirmacao(email.getDescricao());
		}
	    }

	    if (this.getModel().getEmailAdicional1() != null) {
		this.setConfirmacaoEmailAdicional1(this.getModel().getEmailAdicional1());
	    }
	    if (this.getModel().getEmailAdicional2() != null) {
		this.setConfirmacaoEmailAdicional2(this.getModel().getEmailAdicional2());
	    }
	    if (this.getModel().getEmailAdicional3() != null) {
		this.setConfirmacaoEmailAdicional3(this.getModel().getEmailAdicional3());
	    }
	    if (this.getModel().getEmailAdicional4() != null) {
		this.setConfirmacaoEmailAdicional4(this.getModel().getEmailAdicional4());
	    }

	    List<CrpDocumentoPessoaFisica> documentos = (List<CrpDocumentoPessoaFisica>) e.getRepresentante()
		    .getPessoaFisica().getDocumentos();
	    if (documentos != null && !documentos.isEmpty()) {
		for (CrpDocumentoPessoaFisica documento : documentos) {
		    if (CrpTipoDocPessoaFisica.CPF.equals(documento.getTipoDocumento().getId())) {
			this.setCpfRepresentante(documento);
		    } else if (CrpTipoDocPessoaFisica.RG.equals(documento.getTipoDocumento().getId())) {
			this.setRgRepresentante(documento);
			this.rgValidado = true;
		    }
		}
	    }

	    // Indica que esta sendo realizada a edição da PJ
	    this.editarPj = true;
	    valoresOriginais = new HashMap<Integer, Object>();
	    valoresSubmetidos = new HashMap<Integer, Object>();
	    if (getModel().getId() != null) {
		// para gravar o histórico, obtêm-se os valores oridinais e
		// submete-se em um MAP.
		CrpContatoTelefone t = getTelefone();
		CrpContatoEmail email = getEmail();
		valoresOriginais = new HashMap<Integer, Object>();
		valoresOriginais.put(1, t.getNumDdd());
		valoresOriginais.put(2, t.getNumTelefone());
		valoresOriginais.put(3, email.getDescricao());
	    }

	}

    }

    @Override
    public String getQualifiedName() {
	return this.qualifiedName;
    }

    @Override
    public boolean isFeminino() {
	return true;
    }

    @Override
    public String getRequiredMessage() {
	return super.getRequiredMessage();
    }

    public void setListaUf(List<CrpUf> listaUf) {
	this.listaUf = listaUf;
    }

    public List<CrpMunicipio> getListaMunicipio() {
	return listaMunicipio;
    }

    public void setListaMunicipio(List<CrpMunicipio> listaMunicipio) {
	this.listaMunicipio = listaMunicipio;
    }

    public PessoaJuridicaUtil getPessoaJuridicaUtil() {
	return pessoaJuridicaUtil;
    }

    public void setPessoaJuridicaUtil(PessoaJuridicaUtil pessoaJuridicaUtil) {
	this.pessoaJuridicaUtil = pessoaJuridicaUtil;
    }

    public CrpContatoTelefone getTelefone() {
	return telefone;
    }

    public void setTelefone(CrpContatoTelefone telefone) {
	this.telefone = telefone;
    }

    public CrpContatoEmail getEmail() {
	return email;
    }

    public void setEmail(CrpContatoEmail email) {
	this.email = email;
    }

    public CrpEndereco getEndereco() {
	return endereco;
    }

    public void setEndereco(CrpEndereco endereco) {
	this.endereco = endereco;
    }

    public CrpDocumentoPessoaFisica getCpfRepresentante() {
	return cpfRepresentante;
    }

    public void setCpfRepresentante(CrpDocumentoPessoaFisica cpfRepresentante) {
	this.cpfRepresentante = cpfRepresentante;
    }

    public CrpDocumentoPessoaFisica getRgRepresentante() {
	return rgRepresentante;
    }

    public void setRgRepresentante(CrpDocumentoPessoaFisica rgRepresentante) {
	this.rgRepresentante = rgRepresentante;
    }

    public CrpMunicipio getMunicipio() {
	return municipio;
    }

    public void setMunicipio(CrpMunicipio municipio) {
	this.municipio = municipio;
    }

    public CrpUf getUf() {
	return uf;
    }

    public void setUf(CrpUf uf) {
	this.uf = uf;
    }

    public boolean isEditarPj() {
	return editarPj;
    }

    public void setEditarPj(boolean editarPj) {
	this.editarPj = editarPj;
    }

    public CadDocumento getLicenca() {
	return licenca;
    }

    public void setLicenca(CadDocumento licenca) {
	this.licenca = licenca;
    }

    public CadDocumento getContrato() {
	return contrato;
    }

    public void setContrato(CadDocumento contrato) {
	this.contrato = contrato;
    }

    public String getPageTitle() {
	return pageTitle;
    }

    public void setPageTitle(String pageTitle) {
	this.pageTitle = pageTitle;
    }

    public CadDocumento getAtosConstitutivos() {
	if (this.atosConstitutivos == null) {
	    this.atosConstitutivos = new CadDocumento();
	    this.atosConstitutivos.setTipoDocumento(new CadTipoDocumentoD(CadTipoDocumentoD.ATOS_CONSTITUTIVOS));
	}
	return this.atosConstitutivos;
    }

    public void setAtosConstitutivos(CadDocumento atosConstitutivos) {
	this.atosConstitutivos = atosConstitutivos;
    }

    public CadDocumento getComprovanteCnpj() {
	if (this.comprovanteCnpj == null) {
	    this.comprovanteCnpj = new CadDocumento();
	    this.comprovanteCnpj.setTipoDocumento(new CadTipoDocumentoD(CadTipoDocumentoD.CNPJ));
	}
	return this.comprovanteCnpj;
    }

    public void setComprovanteCnpj(CadDocumento comprovanteCnpj) {
	this.comprovanteCnpj = comprovanteCnpj;
    }

    public CadDocumento getComprovanteRepresentanteLegal() {
	if (this.comprovanteRepresentanteLegal == null) {
	    this.comprovanteRepresentanteLegal = new CadDocumento();
	    this.comprovanteRepresentanteLegal.setTipoDocumento(new CadTipoDocumentoD(
		    CadTipoDocumentoD.REPRESENTANTE_LEGAL));
	}
	return comprovanteRepresentanteLegal;
    }

    public void setComprovanteRepresentanteLegal(CadDocumento comprovanteRepresentanteLegal) {
	this.comprovanteRepresentanteLegal = comprovanteRepresentanteLegal;
    }

    public String getEmailConfirmacao() {
	return emailConfirmacao;
    }

    public void setEmailConfirmacao(String emailConfirmacao) {
	this.emailConfirmacao = emailConfirmacao;
    }

    public GenericService getGenericService() {
	return genericService;
    }

    public void setGenericService(GenericService genericService) {
	this.genericService = genericService;
    }

    public String getConfirmacaoEmailAdicional1() {
	return confirmacaoEmailAdicional1;
    }

    public void setConfirmacaoEmailAdicional1(String confirmacaoEmailAdicional1) {
	this.confirmacaoEmailAdicional1 = confirmacaoEmailAdicional1;
    }

    public String getConfirmacaoEmailAdicional2() {
	return confirmacaoEmailAdicional2;
    }

    public void setConfirmacaoEmailAdicional2(String confirmacaoEmailAdicional2) {
	this.confirmacaoEmailAdicional2 = confirmacaoEmailAdicional2;
    }

    public String getConfirmacaoEmailAdicional3() {
	return confirmacaoEmailAdicional3;
    }

    public void setConfirmacaoEmailAdicional3(String confirmacaoEmailAdicional3) {
	this.confirmacaoEmailAdicional3 = confirmacaoEmailAdicional3;
    }

    public String getConfirmacaoEmailAdicional4() {
	return confirmacaoEmailAdicional4;
    }

    public void setConfirmacaoEmailAdicional4(String confirmacaoEmailAdicional4) {
	this.confirmacaoEmailAdicional4 = confirmacaoEmailAdicional4;
    }

    public boolean isSomenteConsulta() {
	return somenteConsulta;
    }

    public void setSomenteConsulta(boolean somenteConsulta) {
	this.somenteConsulta = somenteConsulta;
    }

    public boolean isProponenteRepresentante() {
	return proponenteRepresentante;
    }

    public void setProponenteRepresentante(boolean proponenteRepresentante) {
	this.proponenteRepresentante = proponenteRepresentante;
    }

    public Map<Integer, Object> getValoresOriginais() {
	return valoresOriginais;
    }

    public void setValoresOriginais(Map<Integer, Object> valoresOriginais) {
	this.valoresOriginais = valoresOriginais;
    }

    public Map<Integer, Object> getValoresSubmetidos() {
	return valoresSubmetidos;
    }

    public void setValoresSubmetidos(Map<Integer, Object> valoresSubmetidos) {
	this.valoresSubmetidos = valoresSubmetidos;
    }

    public List<CadServico> getServicos() {
	return servicos;
    }

    public void setServicos(List<CadServico> servicos) {
	this.servicos = servicos;
    }

    public EmpresaService getEmpresaService() {
	return empresaService;
    }

    public void setEmpresaService(EmpresaService empresaService) {
	this.empresaService = empresaService;
    }

    public boolean getRenderizaUploadDocumentos() {
	return renderizaUploadDocumentos;
    }

    public Integer[] getListaIdServicos() {
	return listaIdServicos;
    }

    public void setListaIdServicos(Integer[] listaIdServicos) {
	this.listaIdServicos = listaIdServicos;
    }

    public void setRenderizaUploadDocumentos(boolean renderizaUploadDocumentos) {
	this.renderizaUploadDocumentos = renderizaUploadDocumentos;
    }

}