package br.gov.mc.cadsei.bean;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.sql.rowset.serial.SerialBlob;

import org.apache.commons.codec.binary.Base64;
import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.DefaultStreamedContent;
import org.primefaces.model.StreamedContent;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.mc.cadsei.entidade.CadArea;
import br.gov.mc.cadsei.entidade.CadAreaServico;
import br.gov.mc.cadsei.entidade.CadDocumentoPeticao;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadPeticao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadSeiIntegracao;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadServicoSolicitacao;
import br.gov.mc.cadsei.entidade.CadSituacaoPeticaoD;
import br.gov.mc.cadsei.entidade.dto.CprdProtocoloDto;
import br.gov.mc.cadsei.entidade.dto.ResultadoOutorgadoDto;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.service.PeticaoService;
import br.gov.mc.cadsei.service.ProcuracaoService;
import br.gov.mc.cadsei.util.Constantes;
import br.gov.mc.cadsei.util.EnvioMail;
import br.gov.mc.cadsei.util.SessionUtil;
import br.gov.mc.cadsei.util.StringUtil;
import br.gov.mc.sei.client.SeiRepository;
import br.gov.mc.sei.client.util.Constants;
import br.gov.mc.sei.client.util.SeiIntegrationException;
import br.gov.mc.sei.entidade.Assunto;
import br.gov.mc.sei.entidade.Destinatario;
import br.gov.mc.sei.entidade.Documento;
import br.gov.mc.sei.entidade.Interessado;
import br.gov.mc.sei.entidade.Procedimento;
import br.gov.mc.sei.entidade.Remetente;
import br.gov.mc.sei.entidade.RetornoConsultaProcedimento;
import br.gov.mc.sei.entidade.RetornoGeracaoProcedimento;
import br.gov.mc.sei.entidade.RetornoInclusaoDocumento;

@ManagedBean(name = "cadastroPeticaoBean")
@ViewScoped
public class CadastroPeticaoBean extends BaseBean<CadPeticao> {

    private static final long serialVersionUID = 1L;

    private final String PESSOA_JURIDICA = "J";
    private final String PROCURACAO = "P";

    private final String SISTEMA = Constants.PROTOCOLO_ELETRONICO;
    private final String SERVICO = Constants.INTEGRACAO;

    // VISUALIZACAO
    private final String PETICAO = "PETICAO";
    private boolean visualizar;

    private CprdProtocoloDto protocolo;
    private List<SelectItem> empresas;
    private List<SelectItem> listProcuracoes;
    private CadDocumentoPeticao documento;

    private String tipoPessoa;
    private boolean protocoloValido;
    private boolean exibirIconeProtocolo;
    private boolean disableTipoPessoa = false;

    private CadProponente proponente;

    private List<CadArea> listArea;
    private List<CadAreaServico> listAreaServico;
    private List<CadServicoSolicitacao> listServicoSolicitacao;

    private String numeroProtocoloExistente;

    @ManagedProperty(value = "#{peticaoPrtoService}")
    private transient PeticaoService peticaoService;

    @ManagedProperty(value = "#{procuracaoService}")
    private transient ProcuracaoService procuracaoService;

    @ManagedProperty(value = "#{empresaService}")
    private transient EmpresaService empresaService;

    @ManagedProperty(value = "#{seiRepository}")
    private transient SeiRepository sei;

    @ManagedProperty(value = "#{genericService}")
    private transient GenericService genericService;

    private RetornoGeracaoProcedimento processo = null;

    public CadPeticao createModel() {
	return new CadPeticao();
    }

    public String getQualifiedName() {
	return "Peti&ccedil;&atilde;o";
    }

    public boolean isFeminino() {
	return true;
    }

    @PostConstruct
    public void init() {
	Integer idPeticao = (Integer) FacesContext.getCurrentInstance().getExternalContext().getSessionMap()
		.remove(PETICAO);
	this.setProponente((CadProponente) this.getUniversalManager().get(CadProponente.class,
		SessionUtil.getProponenteLogado().getId()));
	if (idPeticao != null) {
	    this.setModel(this.getPeticaoService().getByPeticao(new CadPeticao(idPeticao)));
	    this.setVisualizar(true);
	    if (this.getModel().getEmpresa() != null && this.getModel().getEmpresa().getId() == null
		    && this.getModel().getProcuracao() != null && this.getModel().getProcuracao().getId() == null) {
		this.setTipoPessoa("F");
	    } else if (this.getModel().getEmpresa() != null && this.getModel().getEmpresa().getId() != null) {
		this.setTipoPessoa("J");
	    } else if (this.getModel().getProcuracao() != null && this.getModel().getProcuracao().getId() != null) {
		this.setTipoPessoa("P");
	    }
	    // Preencher os dados de servico e solcitacao
	    populaServico();
	    populaSolicitacao();
	} else {
	    // this.setProponente((CadProponente)
	    // this.getUniversalManager().get(CadProponente.class,
	    // SessionUtil.getProponenteLogado().getId()));
	    if (this.getProponente().getEmpresas().isEmpty() && this.getListProcuracoes().isEmpty()) {
		this.setTipoPessoa("F");
		this.setDisableTipoPessoa(true);
	    }
	    this.getModel().setProponente(SessionUtil.getProponenteLogado());
	}

    }

    public String enviar() {

	// Valida o preenchimento dos campos obrigatórios
	if (StringUtil.isEmpty(this.getTipoPessoa())
		|| this.getModel().getExisteProtocolo() == null
		|| (this.getModel().getExisteProtocolo().booleanValue() && StringUtil.isEmpty(this.getModel()
			.getNumeroProtocolo())) || StringUtil.isEmpty(this.getModel().getAssunto())
//		|| this.getModel().getArea().getId() == null || this.getModel().getServico().getId() == null
		|| this.getModel().getSolicitacao().getId() == null || PESSOA_JURIDICA.equals(this.getTipoPessoa())
		&& this.getModel().getEmpresa().getId() == null
		&& (this.getModel().getProcuracao().getId() == null || PROCURACAO.equals(this.getTipoPessoa()))) {
	    FacesMessagesUtil.addErrorMessage("Campos obrigat&oacute;rios devem ser informados!");
	    return ERROR;
	}

	// Valida se algum documento esta sendo enviado
	if (this.getModel().getDocumentos().isEmpty()) {
	    FacesMessagesUtil
		    .addErrorMessage("N&atilde;o &eacute; poss&iacute;vel enviar a peti&ccedil;&atilde;o sem antes enviar algum documento.");
	    return ERROR;
	}

	// Verifica se o protocolo é válido
	if (this.getModel().getExisteProtocolo()) {
	    String numeroProtocolo = this.getModel().getNumeroProtocolo().toString()
		    + this.getModel().getDigitoNumeroProtocolo();
	    // if (this.getPeticaoService().getProtocolo(numeroProtocolo) ==
	    // null) {
	    // FacesMessagesUtil.addErrorMessage("Protocolo n&atilde;o encontrado.");
	    // return ERROR;
	    // }
	}

	// Define algumas informações para permitir a persistência da
	// petição
	if (!StringUtil.isEmpty(this.getModel().getNumeroProtocolo())) {
	    String protocolo = this.getModel().getNumeroProtocolo() + this.getModel().getDigitoNumeroProtocolo();
	    this.getModel().setProtocolo(new BigDecimal(StringUtil.removeMascara(protocolo)));
	} else {
	    this.getModel().setProtocolo(null);
	}
	if (this.getModel().getEmpresa().getId() != null) {
	    this.getModel().setEmpresa(new CadEmpresa(this.getModel().getEmpresa().getId()));
	} else {
	    this.getModel().setEmpresa(null);
	}
	this.getModel().setDataPeticao(new Date());
	this.getModel().setSituacao(new CadSituacaoPeticaoD(CadSituacaoPeticaoD.AGUARDANDO_ANALISE));

	// Criar Prceodimento no SEI
	if (criarProcedimentoSEI()) {
	    return ERROR;
	}

	// Salva a petição
	try {
	    this.getModel().setProponente(proponente);
	    if (getProcesso() != null && getProcesso().getProcedimentoFormatado() != null) {
		this.getModel().setNumeroProtocolo(getProcesso().getProcedimentoFormatado());
		this.getModel().setProtocolo(
			new BigDecimal(StringUtil.removeMascara(getProcesso().getProcedimentoFormatado())));
	    }
	    this.getPeticaoService().save(this.getModel());
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	    return ERROR;
	}

	// Envia e-mail notificando os analistas do CADSEI
	try {
	    this.getModel().getProponente()
		    .setPessoaProponente(SessionUtil.getProponenteLogado().getPessoaProponente());
	    try {
		    EnvioMail.sendEmailPeticionamento(this.getModel(), SessionUtil.getProponenteLogado().getEmailProponente(),
				    this.getNumeroProtocoloExistente());
		} catch (Exception e) {
			e.printStackTrace();
			if(getProcesso().getProcedimentoFormatado()!=null){
			FacesMessagesUtil.addInfoMessage(null, "Sua peti&ccedil;&atilde;o foi enviada com sucesso! Protocolo nº:"
					+ getProcesso().getProcedimentoFormatado()+", mas não foi possivel enviar um email de confirmação!");
			return "cadastro-peticao.jsf";
			}else{
				e.printStackTrace();
			    ExcecaoUtil.tratarExcecao(e);
			    return ERROR;
			}
		}
	} catch (Exception e) {
		e.printStackTrace();
	    ExcecaoUtil.tratarExcecao(e);
	    return ERROR;
	}

	FacesMessagesUtil.addInfoMessage(null, "Sua peti&ccedil;&atilde;o foi enviada com sucesso! Protocolo nº:"
		+ getProcesso().getProcedimentoFormatado());
	return "cadastro-peticao.jsf";

    }

    private boolean criarProcedimentoSEI() {
	CadPeticao cadPeticao = this.getModel();
	cadPeticao.setServico(new CadServico(1));
	cadPeticao.setArea(new CadArea(1));
	CadSeiIntegracao seiIntegracao = this.getPeticaoService().findSeiIntegracaoByIdPedticao(this.getModel());
	if (seiIntegracao == null) {
	    FacesMessagesUtil.addErrorMessage(Constantes.PARAMETRO_INTEGRACAO);
	    return true;
	}
	try {
	    Procedimento procedimento = new Procedimento();
	    procedimento.setIdTipoProcedimento(seiIntegracao.getIdTipoProcedimento());
	    if (seiIntegracao.getNumAssunto().isEmpty() || seiIntegracao.getNumAssunto() == null) {
		procedimento.setAssuntos(new Assunto[] {});
	    } else {
		procedimento.setAssuntos(new Assunto[] { new Assunto(seiIntegracao.getNumAssunto(), null) });
	    }
	    if (this.getTipoPessoa().equals("J")) {
		CadEmpresa empresa = this.getEmpresaService().buscaEmpresaPorId(this.getModel().getEmpresa().getId());
		procedimento.setInteressados(new Interessado[] { new Interessado(empresa.getPessoaTitular()
			.getRazaoSocial().substring(0, 2), empresa.getPessoaTitular().getRazaoSocial()) });
	    } else {
		procedimento.setInteressados(new Interessado[] { new Interessado(SessionUtil.getProponenteLogado()
			.getPessoaProponente().getNome().substring(0, 2), SessionUtil.getProponenteLogado()
			.getPessoaProponente().getNome()) });
	    }
	    if (seiIntegracao.getNumNivelAcesso() == null) {
		procedimento.setNivelAcesso("null");
	    } else {
		procedimento.setNivelAcesso(seiIntegracao.getNumNivelAcesso());
	    }
	    procedimento.setEspecificacao(this.getModel().getAssunto());
	    SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy HH:mm");
	    String observacao = "Peticao Eletronica realizada pelo Protocolo Eletr�nico. \n" + "Data: "
		    + sdf.format(new Date()) + "\n" + "Usuario : "
		    + SessionUtil.getProponenteLogado().getPessoaProponente().getNome() + "\n";

	    if (this.getTipoPessoa().equalsIgnoreCase("P")) {
		this.getModel().setProcuracao(
			this.getProcuracaoService().getByProcuracao(this.getModel().getProcuracao()));
		if (this.getModel().getProcuracao().getProponenteOutorgante() != null
			&& this.getModel().getProcuracao().getProponenteOutorgante().getId() != null) {
		    observacao = observacao + "Em nome de : "
			    + this.getModel().getProcuracao().getProponenteOutorgante().getPessoaProponente().getNome()
			    + "\n";
		} else {
		    observacao = observacao
			    + "Em nome de : "
			    + this.getModel().getProcuracao().getEmpresaOutorgante().getPessoaTitular()
				    .getRazaoSocial() + "\n";
		}
		observacao = observacao + "Procuracao : " + this.getModel().getProcuracao().getAno() + "."
			+ this.getModel().getProcuracao().getNumeroSequencialZeros();
	    }
	    procedimento.setObservacao(observacao);

	    if (this.getModel().getExisteProtocolo()) {
		RetornoConsultaProcedimento consultaProcedimentoExistente = new RetornoConsultaProcedimento();
		consultaProcedimentoExistente = this.getSei().consultarProcedimento(SISTEMA, SERVICO,
			seiIntegracao.getIdUnidade(), this.getModel().getNumeroProcessoFormatado());
		List<String> idProcedimentos = new ArrayList<String>();
		idProcedimentos.add(consultaProcedimentoExistente.getIdProcedimento());
		processo = this.getSei().gerarProcedimento(SISTEMA, SERVICO, seiIntegracao.getIdUnidade(),
			procedimento, null, idProcedimentos, null);
		// Setar protocolo informado existente
		this.setNumeroProtocoloExistente(consultaProcedimentoExistente.getProcedimentoFormatado());
	    } else {
		processo = this.getSei().gerarProcedimento(SISTEMA, SERVICO, seiIntegracao.getIdUnidade(),
			procedimento, null, null, null);
	    }

	    // Setar numero do processo na peticao.
	    this.getModel().setProtocolo(new BigDecimal(StringUtil.removeMascara(processo.getProcedimentoFormatado())));
	    try {
	    	return saveDocumentosSei(processo.getIdProcedimento(), seiIntegracao);
	    } catch (Exception e) {
	    	e.printStackTrace();
	    }

	} catch (SeiIntegrationException e) {
	    FacesMessagesUtil.addErrorMessage(e.getMessage());
	    e.printStackTrace();
	    return true;
	}
	return false;

    }

    private boolean saveDocumentosSei(String idProcesso, CadSeiIntegracao seiIntegracao) {
	for (CadDocumentoPeticao docs : this.getModel().getDocumentos()) {
	    try {
		Documento documento = new Documento();
		documento.setTipo("R");
		documento.setIdProcedimento(idProcesso);
		if(this.getModel().getSolicitacao()!=null && this.getModel().getSolicitacao().getId()>0){
		    documento.setIdSerie(this.getModel().getSolicitacao().getId().toString());
		}else{
			documento.setIdSerie("596");
		}
		SimpleDateFormat sdf = new SimpleDateFormat("dd/MM/yyyy");
		documento.setData(sdf.format(new Date()));
		documento.setInteressados(new Interessado[] {});
		documento.setDestinatarios(new Destinatario[] {});
		documento.setNivelAcesso("null");
		documento.setNumero("null");
		documento.setDescricao("null");
		String nome = SessionUtil.getProponenteLogado().getPessoaProponente().getNome();
		documento.setRemetente(new Remetente(nome.substring(0, 3), nome));
		documento.setObservacao("null");
		int blobLength;
		byte[] blobAsBytes = null;
		try {
		    blobLength = (int) docs.getArquivo().length();
		    blobAsBytes = docs.getArquivo().getBytes(1, blobLength);
		} catch (SQLException e1) {
		    e1.printStackTrace();
		}
		String encoded = new String(Base64.encodeBase64(blobAsBytes));
		documento.setConteudo(encoded);
		documento.setNomeArquivo(docs.getNome());
		RetornoInclusaoDocumento retorno = this.getSei().incluirDocumento(SISTEMA, SERVICO,
			seiIntegracao.getIdUnidade(), documento);
	    } catch (SeiIntegrationException e) {
		FacesMessagesUtil.addErrorMessage(e.getMessage());
		return true;
	    }
	}
	return false;
    }

    public void changeExisteProcesso() {
	this.getModel().setProtocolo(null);
	this.getModel().setNumeroProtocolo(null);
	this.getModel().setDigitoNumeroProtocolo(null);
	this.setExibirIconeProtocolo(false);
	this.setProtocoloValido(false);
	this.setProtocolo(null);
    }

    public boolean protocoloValidado(String clientId, String message) {
	if (this.getModel().getExisteProtocolo() != null && this.getModel().getExisteProtocolo().booleanValue()
		&& !this.isProtocoloValido()) {
	    FacesContext.getCurrentInstance().addMessage(clientId,
		    new FacesMessage(FacesMessage.SEVERITY_ERROR, null, message));
	    return false;
	}
	return true;
    }

    public void pesquisarProtocolo() {
	if (!StringUtil.isEmpty(this.getModel().getNumeroProtocolo())
		&& !StringUtil.isEmpty(this.getModel().getDigitoNumeroProtocolo())) {
	    String numero = this.getModel().getNumeroProtocolo().concat(this.getModel().getDigitoNumeroProtocolo());
	    // this.setProtocolo(this.getPeticaoService().getProtocolo(numero));
	    // this.setProtocoloValido(this.getProtocolo().getId() != null);
	    this.setProtocoloValido(true);
	    this.setExibirIconeProtocolo(true);
	    this.protocoloValidado("protocolo", "Protocolo n&atilde;o encontrado!");
	}
    }

    public void uploadDocumento(FileUploadEvent event) {
	try {
	    if (event.getFile().getFileName().contains(".pdf") 
		    || event.getFile().getFileName().contains(".jpg") || event.getFile().getFileName().contains(".png")
		    || event.getFile().getFileName().contains(".jpeg")) {
		this.getDocumento().setArquivo(new SerialBlob(event.getFile().getContents()));
		this.getDocumento().setNome(event.getFile().getFileName());
		adicionarDocumento();
	    } else {
		FacesContext.getCurrentInstance().addMessage("documento",
			new FacesMessage(FacesMessage.SEVERITY_ERROR, null, "Tipo de arquivo n&atilde;o autorizado"));
		return;
	    }
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	}
    }

    public StreamedContent getFile() {
	try {
	    if (this.getDocumento() != null) {
		return new DefaultStreamedContent(this.getDocumento().getArquivo().getBinaryStream(), null,
			StringUtil.replaceWhiteSpace(this.getDocumento().getNome()));
	    }
	} catch (SQLException e) {
	    ExcecaoUtil.tratarExcecao(e);
	}
	return null;
    }

    public void adicionarDocumento() {
	if (StringUtil.isEmpty(this.getDocumento().getNome())) {
	    FacesContext.getCurrentInstance().addMessage("documento",
		    new FacesMessage(FacesMessage.SEVERITY_ERROR, null, this.getRequiredMessage()));
	    return;
	}
	this.getModel().addDocumento(this.getDocumento());
	this.setDocumento(null);
    }

    public void excluirDocumento() {
	this.getModel().getDocumentos().remove(this.getDocumento());
	this.setDocumento(null);
    }

    @SuppressWarnings("unchecked")
    public void populaServico() {
	try {
	    if (this.getModel().getArea() != null) {
		CadAreaServico areaServico = new CadAreaServico();
		areaServico.setArea(new CadArea(this.getModel().getArea().getId()));

		if (this.getTipoPessoa() != null && this.getTipoPessoa().equalsIgnoreCase("P")) {
		    if (this.getModel().getProcuracao() != null && this.getModel().getProcuracao().getId() != null) {
			listAreaServico = (List<CadAreaServico>) this.getPeticaoService().findServicoByProcuracao(
				this.getModel().getArea().getId(), this.getModel().getProcuracao().getId());
		    }
		} else {
		    listAreaServico = this.getUniversalManager().listBy(areaServico);
		}

		Collections.sort(this.listAreaServico, new Comparator<CadAreaServico>() {

		    public int compare(CadAreaServico as1, CadAreaServico as2) {
			return as1.getServico().getDscServico().compareTo(as2.getServico().getDscServico());
		    }
		});
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    @SuppressWarnings("unchecked")
	public void populaSolicitacao() {
		try {
			if (this.getModel().getServico() != null) {
				CadServicoSolicitacao servicoSolicitacao = new CadServicoSolicitacao();
				servicoSolicitacao.setServico(new CadServico(this.getModel().getServico().getId()));
				if (this.getTipoPessoa() != null && this.getTipoPessoa().equalsIgnoreCase("P")) {
					if (this.getModel().getProcuracao() != null && this.getModel().getProcuracao().getId() != null) {
						listServicoSolicitacao = (List<CadServicoSolicitacao>) this.getPeticaoService().findSolicitacaoByProcuracao(
								this.getModel().getServico().getId(), this.getModel().getProcuracao().getId());
					}
				} else {
					listServicoSolicitacao = this.getUniversalManager().listBy(servicoSolicitacao);
				}

				Collections.sort(this.listServicoSolicitacao, new Comparator<CadServicoSolicitacao>() {
					
					public int compare(CadServicoSolicitacao as1, CadServicoSolicitacao as2) {
						return as1.getSolicitacao().getDscSolicitacao().compareTo(as2.getSolicitacao().getDscSolicitacao());
					}
				});
			}
			if(listServicoSolicitacao.isEmpty()){
			    listServicoSolicitacao = genericService.createCriteria(CadServicoSolicitacao.class).list();
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    public void exibirListaArea() {
	// this.getModel().setEmpresa(null);
	// this.setListArea(null);
	// this.getModel().setArea(null);
	// this.setListAreaServico(null);
	// this.getModel().setServico(null);
	// this.setListServicoSolicitacao(null);
	// this.getModel().setSolicitacao(null);
    }

    public void exibirListaPessoaJuridica() {
	// this.getModel().setEmpresa(null);
	// this.setListArea(null);
	// this.getModel().setArea(null);
	// this.setListAreaServico(null);
	// this.getModel().setServico(null);
	// this.setListServicoSolicitacao(null);
	// this.getModel().setSolicitacao(null);
	// this.setListProcuracoes(null);
	// this.getModel().setProcuracao(null);
    }

    public boolean isProtocoloValido() {
	return protocoloValido;
    }

    public void setProtocoloValido(boolean protocoloValido) {
	this.protocoloValido = protocoloValido;
    }

    public boolean isExibirIconeProtocolo() {
	return exibirIconeProtocolo;
    }

    public void setExibirIconeProtocolo(boolean exibirIconeProtocolo) {
	this.exibirIconeProtocolo = exibirIconeProtocolo;
    }

    public CadDocumentoPeticao getDocumento() {
	if (this.documento == null) {
	    this.documento = new CadDocumentoPeticao();
	}
	return this.documento;
    }

    public void setDocumento(CadDocumentoPeticao documento) {
	this.documento = documento;
    }

    public String getTipoPessoa() {
	return tipoPessoa;
    }

    public void setTipoPessoa(String tipoPessoa) {
	this.tipoPessoa = tipoPessoa;
    }

    public List<SelectItem> getEmpresas() {
	if (this.empresas == null) {
	    this.empresas = new ArrayList<SelectItem>();
	    if (!this.getProponente().getEmpresas().isEmpty()) {
		if (this.getProponente().getEmpresas().size() > 1) {
		    this.empresas.add(new SelectItem(null, "[Selecione]"));
		}
		for (CadEmpresa empresa : this.getProponente().getEmpresas()) {
		    this.empresas.add(new SelectItem(empresa.getId(), empresa.getPessoaTitular().getRazaoSocial()));
		}
	    }

	}
	return this.empresas;
    }

    public void setEmpresas(List<SelectItem> empresas) {
	this.empresas = empresas;
    }

    public PeticaoService getPeticaoService() {
	return peticaoService;
    }

    public void setPeticaoService(PeticaoService peticaoService) {
	this.peticaoService = peticaoService;
    }

    public CprdProtocoloDto getProtocolo() {
	if (this.protocolo == null) {
	    this.protocolo = new CprdProtocoloDto();
	}
	return this.protocolo;
    }

    public void setProtocolo(CprdProtocoloDto protocolo) {
	this.protocolo = protocolo;
    }

    public CadProponente getProponente() {
	if (proponente == null) {
	    proponente = new CadProponente();
	}
	return proponente;
    }

    public void setProponente(CadProponente proponente) {
	this.proponente = proponente;
    }

    @SuppressWarnings("unchecked")
    public List<CadArea> getListArea() {
	if (listArea == null) {
	    listArea = new ArrayList<CadArea>();
	    // Selecionado a opção de Procuração
	    if (this.getTipoPessoa() != null && this.getTipoPessoa().equalsIgnoreCase("P")) {
		if (this.getModel().getProcuracao() != null && this.getModel().getProcuracao().getId() != null) {
		    listArea = (List<CadArea>) this.getPeticaoService().findAreaByProcuracao(
			    this.getModel().getProcuracao().getId());
		}
	    } else {
		listArea = this.getUniversalManager().getAll(CadArea.class);
	    }
	}
	return listArea;
    }

    public void setListArea(List<CadArea> listArea) {
	this.listArea = listArea;
    }

    public List<CadAreaServico> getListAreaServico() {
	if (listAreaServico == null) {
	    listAreaServico = new ArrayList<CadAreaServico>();
	}
	return listAreaServico;
    }

    public void setListAreaServico(List<CadAreaServico> listAreaServico) {
	this.listAreaServico = listAreaServico;
    }

    @SuppressWarnings("unchecked")
    public List<CadServicoSolicitacao> getListServicoSolicitacao() {
	if (listServicoSolicitacao == null) {
	    listServicoSolicitacao = new ArrayList<CadServicoSolicitacao>();
	}
	if(listServicoSolicitacao.isEmpty()){
	    listServicoSolicitacao = genericService.createCriteria(CadServicoSolicitacao.class).list();
	}
	return listServicoSolicitacao;
    }

    public GenericService getGenericService() {
        return genericService;
    }

    public void setGenericService(GenericService genericService) {
        this.genericService = genericService;
    }

    public void setListServicoSolicitacao(List<CadServicoSolicitacao> listServicoSolicitacao) {
	this.listServicoSolicitacao = listServicoSolicitacao;
    }

    public boolean isDisableTipoPessoa() {
	return disableTipoPessoa;
    }

    public void setDisableTipoPessoa(boolean disableTipoPessoa) {
	this.disableTipoPessoa = disableTipoPessoa;
    }

    public List<SelectItem> getListProcuracoes() {
	if (this.listProcuracoes == null || this.listProcuracoes.isEmpty()) {
	    this.listProcuracoes = new ArrayList<SelectItem>();
	    List<ResultadoOutorgadoDto> list = this.getProcuracaoService().findProcuracaoRecebidas(
		    SessionUtil.getProponenteLogado().getLoginProponente(), true);
	    if (!list.isEmpty()) {
			for (ResultadoOutorgadoDto resultadoOutorgadoDto : list) {
			    this.listProcuracoes.add(new SelectItem(resultadoOutorgadoDto.getProcuracao().getId(),
				    resultadoOutorgadoDto.getNomeOutorgante()));
			}
	    }
	}
	return listProcuracoes;
    }

    public void setListProcuracoes(List<SelectItem> listProcuracoes) {
	this.listProcuracoes = listProcuracoes;
    }

    public ProcuracaoService getProcuracaoService() {
	return procuracaoService;
    }

    public void setProcuracaoService(ProcuracaoService procuracaoService) {
	this.procuracaoService = procuracaoService;
    }

    public SeiRepository getSei() {
	if (sei == null) {
	    try {
		sei = new SeiRepository();
	    } catch (Exception e) {
		e.printStackTrace();
	    }

	}
	return sei;
    }

    public void setSei(SeiRepository sei) {
	this.sei = sei;
    }

    public boolean isVisualizar() {
	return visualizar;
    }

    public void setVisualizar(boolean visualizar) {
	this.visualizar = visualizar;
    }

    public String getNumeroProtocoloExistente() {
	return numeroProtocoloExistente;
    }

    public void setNumeroProtocoloExistente(String numeroProtocoloExistente) {
	this.numeroProtocoloExistente = numeroProtocoloExistente;
    }

    public EmpresaService getEmpresaService() {
	return empresaService;
    }

    public void setEmpresaService(EmpresaService empresaService) {
	this.empresaService = empresaService;
    }

    public RetornoGeracaoProcedimento getProcesso() {
	return processo;
    }

    public void setProcesso(RetornoGeracaoProcedimento processo) {
	this.processo = processo;
    }

}