package br.gov.mc.cadsei.bean;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.hibernate.criterion.Restrictions;
import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.mc.cadsei.entidade.CadArea;
import br.gov.mc.cadsei.entidade.CadAreaServico;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProcuracao;
import br.gov.mc.cadsei.entidade.CadProcuracaoSolicitacao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadServicoSolicitacao;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.CadSolicitacao;
import br.gov.mc.cadsei.entidade.CadStatusProcuracaoD;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.service.ProcuracaoService;
import br.gov.mc.cadsei.util.Constantes;
import br.gov.mc.cadsei.util.CpfCnpjUtil;
import br.gov.mc.cadsei.util.EnvioMail;
import br.gov.mc.cadsei.util.SessionUtil;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;

@ManagedBean(name = "cadastroProcuracaoBean")
@ViewScoped
public class CadastroProcuracaoBean extends BaseBean<CadProcuracao> {

	private static final long serialVersionUID = 1L;

	private static final String ACEITO = "aceito";

	private List<CadEmpresa> listEmpresa;

	private List<String> empresasOutorganteSelecionadas;
	private List<String> proponenteOutorganteSelecionado;

	private CadProponente proponenteLogado;

	private String tipoPessoa;

	private List<String> selecionadoGeral;

	private CadProponente proponenteValidado;
	private CadEmpresa empresaValidado;

	private boolean renderEspecificios;

	private String cnpjSemFormatacao;
	private String numCnpj;
	private String cpfSemFormatacao;
	private String numCpf;

	private boolean cpfValidado;
	private boolean cnpjValidado;

	private TreeNode list;

	private TreeNode[] selectedSolicitacao;

	private boolean visualizacao;

	private Boolean flgPermiteSubstabelecimento;

	private boolean renderTodos;

	@ManagedProperty(value = "#{genericService}")
	private transient GenericService genericService;

	@ManagedProperty(value = "#{procuracaoService}")
	private transient ProcuracaoService procuracaoService;
	
	@Override
	public CadProcuracao createModel() {
		CadProcuracao procuracao = new CadProcuracao();

		// Setar data inicial
		Calendar dt = Calendar.getInstance();
		dt.setTime(new Date());
		dt.set(Calendar.MONTH, dt.get(Calendar.MONTH));
		dt.set(Calendar.DAY_OF_MONTH, dt.get(Calendar.DAY_OF_MONTH));
		dt.set(Calendar.YEAR, dt.get(Calendar.YEAR));
		dt.set(Calendar.HOUR_OF_DAY, 0);
		dt.set(Calendar.MINUTE, 0);
		dt.set(Calendar.SECOND, 0);
		dt.set(Calendar.MILLISECOND, 0);
		procuracao.setDataValidadeInicial(dt.getTime());

		// Data Final adicionar 5 anos.
		procuracao.setDataValidadeFinal(adicionarAnos());

		return procuracao;
	}

	@PostConstruct
	public void init() {
		Integer idProcuracao = (Integer) this.getSessionMap().remove("idProcuracao");
		if (idProcuracao != null) {
			this.setModel(this.getProcuracaoService().getByProcuracao(new CadProcuracao(idProcuracao)));
			this.setVisualizacao(true);

			if (this.getModel().getProponenteOutorgado().getId() != null) {
				this.setProponenteValidado(this.getModel().getProponenteOutorgado());
				this.setNumCpf(this.getModel().getProponenteOutorgado().getLoginProponente());
				this.setTipoPessoa("F");
				this.setCpfValidado(true);
			} else {
				this.setEmpresaValidado(this.getModel().getEmpresaOutorgado());
				this.setNumCnpj(this.getModel().getEmpresaOutorgado().getPessoaTitular().getCnpj());
				this.setTipoPessoa("J");
				this.setCnpjValidado(true);
			}

			this.setFlgPermiteSubstabelecimento(this.getModel().getFlgPermiteSubstabelecimento());

			this.setRenderEspecificios(true);
		} else {
			Boolean aceito = (Boolean) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().remove(ACEITO);
			if (aceito == null || !aceito) {
				try {
					FacesContext.getCurrentInstance().getExternalContext().redirect("declaracao-cadastro-procuracao.jsf");
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			this.setTipoPessoa("F");
			this.setRenderEspecificios(true);
			this.setVisualizacao(false);

			// Proponente Logado
			this.setProponenteLogado(SessionUtil.getProponenteLogado());

			this.setCnpjValidado(false);
			this.setCpfValidado(false);

			this.setListEmpresa(this.getProcuracaoService().getEmpresaByProponente(SessionUtil.getProponenteLogado().getId()));
			if (this.getListEmpresa().size() > 0) {
				this.setRenderTodos(true);
			}

		}
		try {
			popularSolicitacoes();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	public Date adicionarAnos() {
		Calendar dataFinal = Calendar.getInstance();
		dataFinal.setTime(new Date());
		dataFinal.set(Calendar.MONTH, dataFinal.get(Calendar.MONTH));
		dataFinal.set(Calendar.DAY_OF_MONTH, dataFinal.get(Calendar.DAY_OF_MONTH));
		dataFinal.set(Calendar.YEAR, dataFinal.get(Calendar.YEAR) + 5);
		dataFinal.set(Calendar.HOUR_OF_DAY, 0);
		dataFinal.set(Calendar.MINUTE, 0);
		dataFinal.set(Calendar.SECOND, 0);
		dataFinal.set(Calendar.MILLISECOND, 0);
		return dataFinal.getTime();
	}

	@SuppressWarnings("unchecked")
	private void popularSolicitacoes() throws Exception {
		this.setList(new DefaultTreeNode("Root", null));

		// Criar as listas
		List<CadArea> listAreas = null;
		List<CadAreaServico> listAreaServico = null;
		List<CadServicoSolicitacao> listServicoSolicitacao = null;

		int countSolicitacao = 0;
		boolean solicitacaoDiferente = false;

		// Recuperar lista de areas.
		listAreas = this.getGenericService().createCriteria(CadArea.class).list();
		for (CadArea cadArea : listAreas) {
			TreeNode nodeArea = null;
			// Quando eh visualização deverá popular a lista somente com os
			// registro gravados.
			if (this.isVisualizacao()) {
				for (CadProcuracaoSolicitacao procuracaoSolicitacao : this.getModel().getProcuracaoSolicitacao()) {
					if (procuracaoSolicitacao.getArea().getId().equals(cadArea.getId())) {
						// Popular TreeNode da Area
						nodeArea = new DefaultTreeNode("area", cadArea, this.getList());
						break;
					}
				}
				if (nodeArea == null) {
					solicitacaoDiferente = true;
					continue;
				}
			} else {
				nodeArea = new DefaultTreeNode("area", cadArea, this.getList());
			}

			// Recuperar lista de Servicos da area
			CadAreaServico areaServico = new CadAreaServico();
			areaServico.setArea(cadArea);
			listAreaServico = this.getUniversalManager().listBy(areaServico);

			for (CadAreaServico cadAreaServico : listAreaServico) {
				TreeNode nodeServico = null;
				// Quando eh visualização deverá popular a lista somente com os
				// registro gravados.
				if (this.isVisualizacao()) {
					for (CadProcuracaoSolicitacao procuracaoSolicitacao : this.getModel().getProcuracaoSolicitacao()) {
						if (procuracaoSolicitacao.getServico().getId().equals(cadAreaServico.getServico().getId())) {
							// Popular TreeNode de Servico
							nodeServico = new DefaultTreeNode("servico", cadAreaServico.getServico(), nodeArea);
							break;
						}
					}
					if (nodeServico == null) {
						solicitacaoDiferente = true;
						continue;
					}
				} else {
					// Popular TreeNode de Servico
					nodeServico = new DefaultTreeNode("servico", cadAreaServico.getServico(), nodeArea);
				}
				// Recuperar lista de Solicitação.
				CadServicoSolicitacao servicoSolicitacao = new CadServicoSolicitacao();
				servicoSolicitacao.setServico(new CadServico(cadAreaServico.getServico().getId()));
				listServicoSolicitacao = this.getUniversalManager().listBy(servicoSolicitacao);

				for (CadServicoSolicitacao cadServicoSolicitacao : listServicoSolicitacao) {
					// Popular TreeNode de Solicitação.
					TreeNode nodeSolicitacao = null;
					// Quando eh visualização deverá popular a lista somente com
					// os registro gravados.
					if (this.isVisualizacao()) {
						for (CadProcuracaoSolicitacao procuracaoSolicitacao : this.getModel().getProcuracaoSolicitacao()) {
							if (procuracaoSolicitacao.getSolicitacao().getId().equals(cadServicoSolicitacao.getSolicitacao().getId())) {
								nodeSolicitacao = new DefaultTreeNode("solicitacao", cadServicoSolicitacao.getSolicitacao(), nodeServico);
								countSolicitacao++;
								break;
							}
						}
					} else {
						nodeSolicitacao = new DefaultTreeNode("solicitacao", cadServicoSolicitacao.getSolicitacao(), nodeServico);
						countSolicitacao++;
					}
				}
			}
		}
		// Caso o numero de solicitações existente na tabela
		// CAD_PROCURACAO_SOLICITACAO for o mesmo numero de Solicitacoes
		// Com isso foi selecionado todos.
		if (!solicitacaoDiferente && this.getModel().getProcuracaoSolicitacao().size() == countSolicitacao) {
			this.getSelecionadoGeral().add("T");
		}
	}

	public CadProcuracao getCreateModel() {
		return createModel();
	}

	@Override
	public String getQualifiedName() {
		return "Procura&ccedil&atildeo";
	}

	@Override
	public boolean isFeminino() {
		return false;
	}

	@Override
	public String getRequiredMessage() {
		return super.getRequiredMessage();
	}

	public void exibirPessoa() {
		this.setNumCnpj("");
		this.setNumCpf("");
		this.setProponenteValidado(null);
		this.setEmpresaValidado(null);
		this.setCpfValidado(false);
		this.setCnpjValidado(false);
	}

	public String validateCPF() {
		if (this.getNumCpf() != null && !this.getNumCpf().isEmpty()) {
			this.setCpfSemFormatacao(this.getNumCpf().replace("-", "").replace("/", "").replace(".", ""));
			if (!CpfCnpjUtil.ValidateCPF(this.getNumCpf())) {
				FacesContext.getCurrentInstance().addMessage("cpf", new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.CPF_INVALIDO));
				return ERROR;
			}

			// Validar se possui acesso ao sistema.
			this.setProponenteValidado((CadProponente) this.getGenericService().createCriteria(CadProponente.class)
					.add(Restrictions.eq("loginProponente", this.getCpfSemFormatacao())).uniqueResult());
			if (this.getProponenteValidado() == null || this.getProponenteValidado().getId() == null || this.getProponenteValidado().getSituacaoCadastro().equals(CadSituacaoCadastroD.REPROVADO)) {
				FacesContext.getCurrentInstance().addMessage("cpf",
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.USUARIO_NAO_CADASTRADO_APROVADO));
				return ERROR;
			}
		}
		this.setCpfValidado(true);
		return SUCCESS;
	}

	public String validateCNPJ() {
		// Validar CNPJ
		if (this.getNumCnpj() != null && !this.getNumCnpj().isEmpty()) {
			this.setCnpjSemFormatacao(this.getNumCnpj().replace("-", "").replace("/", "").replace(".", ""));
			if (!CpfCnpjUtil.validarCNPJ(this.getCnpjSemFormatacao())) {
				FacesContext.getCurrentInstance().addMessage("cnpj", new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.CNPJ_INVALIDO));
				return ERROR;
			}
		}

		// Verificar se algum representante legal possui cadastro no
		// cad_proponete.
		this.setEmpresaValidado((CadEmpresa) this.getProcuracaoService().recuperarEmpresa(this.getCnpjSemFormatacao()));
		boolean proponenteCadastrado = false;
		for (CrpRepresentantePesJur representanteLegal : this.getEmpresaValidado().getPessoaTitular().getRepresentantesLegais()) {
			CadProponente proponente = (CadProponente) this.getGenericService().createCriteria(CadProponente.class)
					.add(Restrictions.eq("pessoaProponente.id", representanteLegal.getPessoaFisica().getId()))
					.add(Restrictions.eq("situacaoCadastro.id", CadSituacaoCadastroD.APROVADO)).uniqueResult();
			if (proponente != null) {
				proponenteCadastrado = true;
				break;
			}
		}

		if (!proponenteCadastrado) {
			FacesContext.getCurrentInstance().addMessage("cnpj",
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.REPRESENTANTE_LEGAL_NAO_CADASTRADO));
			return ERROR;
		}

		this.setCnpjValidado(true);
		return SUCCESS;
	}

	public void selecionarTodasEmpresas() {
		if (this.getEmpresasOutorganteSelecionadas().contains("T")) {
			for (CadEmpresa empresa : this.getListEmpresa()) {
				this.getEmpresasOutorganteSelecionadas().add(empresa.getId().toString());
			}
			this.getProponenteOutorganteSelecionado().add(this.getProponenteLogado().getId().toString());
		}
	}

	public void renderizarEspecificos() {
		boolean desabilita = false;
		for (int i = 0; i < this.getSelecionadoGeral().size(); i++) {
			if (this.getSelecionadoGeral().get(i).equalsIgnoreCase("T")) {
				this.setRenderEspecificios(false);
				desabilita = true;
				break;
			}
		}
		if (!desabilita) {
			this.setRenderEspecificios(true);
		}
	}

	@SuppressWarnings("unchecked")
	public String salvar() {
		try {

			if (!validar()) {

				// Para cada empresa selecionada deverá ser criada uma
				// procuração
				// SALVAR EMPRESAS SELECIONADAS
				for (String empresaSelecionada : this.getEmpresasOutorganteSelecionadas()) {

					// Popula empresa outorgante
					this.getModel().setEmpresaOutorgante(new CadEmpresa(Integer.parseInt(empresaSelecionada)));

					// Proponente ou Empresa Outorgado informado.
					if (this.getTipoPessoa().equalsIgnoreCase("F")) {
						this.getModel().setProponenteOutorgado(new CadProponente(this.getProponenteValidado().getId()));
					} else {
						this.getModel().setEmpresaOutorgado(new CadEmpresa(this.getEmpresaValidado().getId()));
					}
					this.getModel().setNrSequencial(this.getProcuracaoService().getNumeroSequencial(null, this.getModel().getEmpresaOutorgante().getId()));

					// Status Procuração
					this.getModel().setStatusProcuracao(new CadStatusProcuracaoD(CadStatusProcuracaoD.ATIVA));

					// Popular Procuracao Solicitação
					this.getModel().setProcuracaoSolicitacao(salvarSolicitacoes());

					this.getModel().setFlgPermiteSubstabelecimento(this.getFlgPermiteSubstabelecimento());

					// salvar
					this.getUniversalManager().save(this.getModel());
					
					// Atualiza informações após salvar.
					this.setModel(this.getProcuracaoService().getByProcuracao(this.getModel()));

					// Envia e-mail notificando a procuração
					try {
						List<CrpContatoEmail> listEmails = null;
						if (this.getModel().getEmpresaOutorgado() != null && this.getModel().getEmpresaOutorgado().getId() != null) {
							listEmails = this.getGenericService().createCriteria(CrpContatoEmail.class)
									.add(Restrictions.eq("pessoa.id", this.getModel().getEmpresaOutorgado().getPessoaTitular().getId())).list();
						}
						this.setModel((CadProcuracao) this.getUniversalManager().get(CadProcuracao.class, this.getModel().getId()));
						EnvioMail.sendEmailProcuracao(this.getModel(), listEmails);
					} catch (Exception e) {
						ExcecaoUtil.tratarExcecao(e);
						return ERROR;
					}

					this.setModel(this.createModel());
				}

				// SALVAR CASO PROPONENTE FOR SELECIONADO.
				if (this.getProponenteOutorganteSelecionado() != null && this.getProponenteOutorganteSelecionado().size() > 0
						&& this.getProponenteOutorganteSelecionado().get(0) != null) {
					this.getModel().setProponenteOutorgante(new CadProponente(this.getProponenteLogado().getId()));

					// Proponente ou Empresa Outorgado informado.
					if (this.getTipoPessoa().equalsIgnoreCase("F")) {
						this.getModel().setProponenteOutorgado(new CadProponente(this.getProponenteValidado().getId()));
					} else {
						this.getModel().setEmpresaOutorgado(new CadEmpresa(this.getEmpresaValidado().getId()));
					}

					// Poppular numero sequencial
					this.getModel().setNrSequencial(this.getProcuracaoService().getNumeroSequencial(this.getProponenteLogado().getId(), null));

					// Popular Procuracao Solicitação
					this.getModel().setProcuracaoSolicitacao(salvarSolicitacoes());

					this.getModel().setFlgPermiteSubstabelecimento(this.getFlgPermiteSubstabelecimento());

					// Status Procuração
					this.getModel().setStatusProcuracao(new CadStatusProcuracaoD(CadStatusProcuracaoD.ATIVA));

					// salvar
					this.getUniversalManager().save(this.getModel());

					// Envia e-mail notificando a procuração
					try {
						List<CrpContatoEmail> listEmails = null;
						if (this.getModel().getEmpresaOutorgado() != null && this.getModel().getEmpresaOutorgado().getId() != null) {
							
							listEmails = procuracaoService.obtemEmailsPessoaTitularPorEmpresaOrtogado(getModel().getEmpresaOutorgado().getId());
						}
						this.setModel((CadProcuracao) this.getUniversalManager().get(CadProcuracao.class, this.getModel().getId()));
						EnvioMail.sendEmailProcuracao(this.getModel(), listEmails);
					} catch (Exception e) {
						//ExcecaoUtil.tratarExcecao(e);
						FacesMessagesUtil.addErrorMessage(e.getMessage(), Constantes.ERRO_ENVIAR_EMAIL);
						return ERROR;
					}

				}
				FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), ConstantesARQ.SALVA + " " + ConstantesARQ.COM_SUCESSO);

				this.setVisualizacao(true);
			}

		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		return SUCCESS;
	}

	private boolean validar() {
		if (this.getEmpresasOutorganteSelecionadas().size() == 0 && this.getProponenteOutorganteSelecionado().size() == 0) {
			FacesMessagesUtil.addErrorMessage("Outorgante(s) " + ConstantesARQ.CAMPO_OBRIGATORIO);
			return true;
		}

		if (this.getNumCpf().isEmpty() && this.getNumCnpj().isEmpty()) {
			FacesMessagesUtil.addErrorMessage("Outorgado(s) " + ConstantesARQ.CAMPO_OBRIGATORIO);
			return true;
		}

		// REMOVER A OPÇÂO TODOS AO VALIDAR PARA NAO TER PROBLEMA NA CONVERSÂO.
		if (this.getEmpresasOutorganteSelecionadas().contains("T")) {
			this.getEmpresasOutorganteSelecionadas().remove("T");
		}

		boolean erro = false;
		if (this.getTipoPessoa().equalsIgnoreCase("F")) {

			for (String outorgantesSelecionados : this.getEmpresasOutorganteSelecionadas()) {
				// Verficar se já possui alguma procuracao para este Outorgado
				// em relação ao Outorgante
				CadProcuracao procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCPF(this.getCpfSemFormatacao(),
						Integer.parseInt(outorgantesSelecionados), null);
				if (procuracao != null) {
					erro = true;
				}
			}
			if (this.getProponenteOutorganteSelecionado() != null && this.getProponenteOutorganteSelecionado().size() > 0
					&& this.getProponenteOutorganteSelecionado().get(0) != null) {
				CadProcuracao procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCPF(this.getCpfSemFormatacao(), null,
						this.getProponenteLogado().getId());
				if (procuracao != null) {
					erro = true;
				}
			}
			if (erro) {
				FacesMessagesUtil.addErrorMessage(Constantes.OUTORGANTE_JA_POSSUI_PARA_OUTORGADO);
				return true;
			}
		} else {
			for (String outorgantesSelecionados : this.getEmpresasOutorganteSelecionadas()) {
				// Verficar se já possui alguma procuracao para este Outorgado
				// em relação ao Outorgante
				CadProcuracao procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCNPJ(this.getCnpjSemFormatacao(),
						Integer.parseInt(outorgantesSelecionados), null);
				if (procuracao != null) {
					erro = true;
				}
			}
			if (this.getProponenteOutorganteSelecionado() != null && this.getProponenteOutorganteSelecionado().size() > 0
					&& this.getProponenteOutorganteSelecionado().get(0) != null) {
				CadProcuracao procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCNPJ(this.getCnpjSemFormatacao(), null,
						this.getProponenteLogado().getId());
				if (procuracao != null) {
					erro = true;
				}
			}
			if (erro) {
				FacesMessagesUtil.addErrorMessage(Constantes.OUTORGANTE_JA_POSSUI_PARA_OUTORGADO);
				return true;
			}
		}

		// validar Data
		if (this.getModel().getDataValidadeInicial().after(this.getModel().getDataValidadeFinal())) {
			FacesMessagesUtil.addErrorMessage(Constantes.DATA_INICIAL_MAIOR_DATA_FINAL);
			return true;
		}

		Date dataMaxima = adicionarAnos();
		if (this.getModel().getDataValidadeFinal().getTime() > dataMaxima.getTime()) {
			FacesMessagesUtil.addErrorMessage(Constantes.DATA_PRAZO_MAXIMO);
			return true;
		}

		if (this.getFlgPermiteSubstabelecimento() == null) {
			FacesMessagesUtil.addErrorMessage("Substabelecimento " + ConstantesARQ.CAMPO_OBRIGATORIO);
			return true;
		}
		return false;
	}

	private List<CadProcuracaoSolicitacao> salvarSolicitacoes() {
		List<CadProcuracaoSolicitacao> list = new ArrayList<CadProcuracaoSolicitacao>();
		// Caso opção de serviços estiver marcados Todos
		if (!this.isRenderEspecificios()) {
			List<CadArea> listAreas = null;
			List<CadAreaServico> listAreaServico = null;
			List<CadServicoSolicitacao> listServicoSolicitacao = null;
			// Recuperar lista de areas.
			try {
				listAreas = this.getGenericService().createCriteria(CadArea.class).list();
				for (CadArea cadArea : listAreas) {
					// Recuperar lista de Servicos da area
					CadAreaServico areaServico = new CadAreaServico();
					areaServico.setArea(cadArea);
					listAreaServico = this.getUniversalManager().listBy(areaServico);
					// TODO Auto-generated catch block

					for (CadAreaServico cadAreaServico : listAreaServico) {
						// Recuperar lista de Solicitação.
						CadServicoSolicitacao servicoSolicitacao = new CadServicoSolicitacao();
						servicoSolicitacao.setServico(new CadServico(cadAreaServico.getServico().getId()));
						listServicoSolicitacao = this.getUniversalManager().listBy(servicoSolicitacao);

						for (CadServicoSolicitacao cadServicoSolicitacao : listServicoSolicitacao) {

							CadProcuracaoSolicitacao procuracaoSolicitacao = new CadProcuracaoSolicitacao();
							procuracaoSolicitacao.setProcuracao(this.getModel());
							procuracaoSolicitacao.setArea(cadArea);
							procuracaoSolicitacao.setServico(cadAreaServico.getServico());
							procuracaoSolicitacao.setSolicitacao(cadServicoSolicitacao.getSolicitacao());
							list.add(procuracaoSolicitacao);
						}
					}
				}
			} catch (Exception e) {
				e.printStackTrace();
			}
		} else {
			for (TreeNode selecionados : this.getSelectedSolicitacao()) {
				CadProcuracaoSolicitacao procuracaoSolicitacao = new CadProcuracaoSolicitacao();
				procuracaoSolicitacao.setProcuracao(this.getModel());
				// Somente pegar os selecionados SOLICITAÇÂO.
				if (selecionados.getType().equalsIgnoreCase("solicitacao")) {
					procuracaoSolicitacao.setSolicitacao((CadSolicitacao) selecionados.getData());
					procuracaoSolicitacao.setServico((CadServico) selecionados.getParent().getData());
					procuracaoSolicitacao.setArea((CadArea) selecionados.getParent().getParent().getData());
					list.add(procuracaoSolicitacao);
				}
			}
		}
		return list;
	}

	public String getTipoPessoa() {
		return tipoPessoa;
	}

	public void setTipoPessoa(String tipoPessoa) {
		this.tipoPessoa = tipoPessoa;
	}

	public String getNumCnpj() {
		return numCnpj;
	}

	public void setNumCnpj(String numCnpj) {
		this.numCnpj = numCnpj;
	}

	public String getNumCpf() {
		return numCpf;
	}

	public void setNumCpf(String numCpf) {
		this.numCpf = numCpf;
	}

	public String getCnpjSemFormatacao() {
		return cnpjSemFormatacao;
	}

	public void setCnpjSemFormatacao(String cnpjSemFormatacao) {
		this.cnpjSemFormatacao = cnpjSemFormatacao;
	}

	public String getCpfSemFormatacao() {
		return cpfSemFormatacao;
	}

	public void setCpfSemFormatacao(String cpfSemFormatacao) {
		this.cpfSemFormatacao = cpfSemFormatacao;
	}

	public GenericService getGenericService() {
		return genericService;
	}

	public void setGenericService(GenericService genericService) {
		this.genericService = genericService;
	}

	public List<CadEmpresa> getListEmpresa() {
		if (listEmpresa == null) {
			listEmpresa = new ArrayList<CadEmpresa>();
		}
		return listEmpresa;
	}

	public void setListEmpresa(List<CadEmpresa> listEmpresa) {
		this.listEmpresa = listEmpresa;
	}

	public ProcuracaoService getProcuracaoService() {
		return procuracaoService;
	}

	public void setProcuracaoService(ProcuracaoService procuracaoService) {
		this.procuracaoService = procuracaoService;
	}

	public List<String> getEmpresasOutorganteSelecionadas() {
		if (empresasOutorganteSelecionadas == null) {
			empresasOutorganteSelecionadas = new ArrayList<String>();
		}
		return empresasOutorganteSelecionadas;
	}

	public void setEmpresasOutorganteSelecionadas(List<String> empresasOutorganteSelecionadas) {
		this.empresasOutorganteSelecionadas = empresasOutorganteSelecionadas;
	}

	public TreeNode[] getSelectedSolicitacao() {
		return selectedSolicitacao;
	}

	public void setSelectedSolicitacao(TreeNode[] selectedSolicitacao) {
		this.selectedSolicitacao = selectedSolicitacao;
	}

	public TreeNode getList() {
		return list;
	}

	public void setList(TreeNode list) {
		this.list = list;
	}

	public List<String> getSelecionadoGeral() {
		if (selecionadoGeral == null) {
			selecionadoGeral = new ArrayList<String>();
		}
		return selecionadoGeral;
	}

	public void setSelecionadoGeral(List<String> selecionadoGeral) {
		this.selecionadoGeral = selecionadoGeral;
	}

	public boolean isRenderEspecificios() {
		return renderEspecificios;
	}

	public void setRenderEspecificios(boolean renderEspecificios) {
		this.renderEspecificios = renderEspecificios;
	}

	public CadProponente getProponenteValidado() {
		if (proponenteValidado == null) {
			proponenteValidado = new CadProponente();
		}
		return proponenteValidado;
	}

	public void setProponenteValidado(CadProponente proponenteValidado) {
		this.proponenteValidado = proponenteValidado;
	}

	public CadEmpresa getEmpresaValidado() {
		if (empresaValidado == null) {
			empresaValidado = new CadEmpresa();
		}
		return empresaValidado;
	}

	public void setEmpresaValidado(CadEmpresa empresaValidado) {
		this.empresaValidado = empresaValidado;
	}

	public List<String> getProponenteOutorganteSelecionado() {
		if (proponenteOutorganteSelecionado == null) {
			proponenteOutorganteSelecionado = new ArrayList<String>();
		}
		return proponenteOutorganteSelecionado;
	}

	public void setProponenteOutorganteSelecionado(List<String> proponenteOutorganteSelecionado) {
		this.proponenteOutorganteSelecionado = proponenteOutorganteSelecionado;
	}

	public CadProponente getProponenteLogado() {
		if (proponenteLogado == null) {
			proponenteLogado = new CadProponente();
		}
		return proponenteLogado;
	}

	public void setProponenteLogado(CadProponente proponenteLogado) {
		this.proponenteLogado = proponenteLogado;
	}

	public boolean isCpfValidado() {
		return cpfValidado;
	}

	public void setCpfValidado(boolean cpfValidado) {
		this.cpfValidado = cpfValidado;
	}

	public boolean isCnpjValidado() {
		return cnpjValidado;
	}

	public void setCnpjValidado(boolean cnpjValidado) {
		this.cnpjValidado = cnpjValidado;
	}

	public boolean isVisualizacao() {
		return visualizacao;
	}

	public void setVisualizacao(boolean visualizacao) {
		this.visualizacao = visualizacao;
	}

	public Boolean getFlgPermiteSubstabelecimento() {
		return flgPermiteSubstabelecimento;
	}

	public void setFlgPermiteSubstabelecimento(Boolean flgPermiteSubstabelecimento) {
		this.flgPermiteSubstabelecimento = flgPermiteSubstabelecimento;
	}

	public boolean isRenderTodos() {
		return renderTodos;
	}

	public void setRenderTodos(boolean renderTodos) {
		this.renderTodos = renderTodos;
	}

}