package br.gov.mc.cadsei.bean;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.hibernate.criterion.Restrictions;
import org.primefaces.model.DefaultTreeNode;
import org.primefaces.model.TreeNode;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.mc.cadsei.entidade.CadArea;
import br.gov.mc.cadsei.entidade.CadAreaServico;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProcuracao;
import br.gov.mc.cadsei.entidade.CadProcuracaoSolicitacao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadServicoSolicitacao;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.CadSolicitacao;
import br.gov.mc.cadsei.entidade.CadStatusProcuracaoD;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.service.ProcuracaoService;
import br.gov.mc.cadsei.util.Constantes;
import br.gov.mc.cadsei.util.CpfCnpjUtil;
import br.gov.mc.cadsei.util.SessionUtil;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;

@ManagedBean(name = "cadastroSubstabelecedorBean")
@ViewScoped
public class CadastroSubstabelecedorBean extends BaseBean<CadProcuracao> {

	private static final long serialVersionUID = 1L;

	private String numeroProcuracao;
	private String nomeOutorgante;
	private String cpfCnpjOutorgante;
	private String nomeOutorgado;
	private String cpfCnpjOutorgado;

	private String tipoPessoa;

	private List<String> selecionadoGeral;

	private CadProponente proponenteValidado;
	private CadEmpresa empresaValidado;

	private boolean renderEspecificios;

	private String cnpjSemFormatacao;
	private String numCnpj;
	private String cpfSemFormatacao;
	private String numCpf;

	private boolean cpfValidado;
	private boolean cnpjValidado;

	private TreeNode list;

	private TreeNode[] selectedSolicitacao;

	private CadProcuracao procuracaoPai;

	@ManagedProperty(value = "#{genericService}")
	private transient GenericService genericService;

	@ManagedProperty(value = "#{procuracaoService}")
	private transient ProcuracaoService procuracaoService;

	@Override
	public CadProcuracao createModel() {
		return new CadProcuracao();
	}

	@PostConstruct
	public void init() {

		Integer idProcuracao = (Integer) this.getSessionMap().remove("idProcuracao");
		if (idProcuracao != null) {
			this.setProcuracaoPai(this.getProcuracaoService().getByProcuracao(new CadProcuracao(idProcuracao)));
			this.setRenderEspecificios(true);

			// Setar data inicial
			this.getModel().setDataValidadeInicial(this.getProcuracaoPai().getDataValidadeInicial());

			// Data Final adicionar 5 anos.
			this.getModel().setDataValidadeFinal(this.getProcuracaoPai().getDataValidadeFinal());
			try {
				popularSolicitacoes();
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	@SuppressWarnings({ "unchecked", "unused" })
	private void popularSolicitacoes() throws Exception {
		this.setList(new DefaultTreeNode("Root", null));

		// Criar as listas
		List<CadArea> listAreas = null;
		List<CadAreaServico> listAreaServico = null;
		List<CadServicoSolicitacao> listServicoSolicitacao = null;

		// Recuperar lista de areas.
		listAreas = this.getGenericService().createCriteria(CadArea.class).list();
		for (CadArea cadArea : listAreas) {
			TreeNode nodeArea = null;
			for (CadProcuracaoSolicitacao procuracaoSolicitacao : this.getProcuracaoPai().getProcuracaoSolicitacao()) {
				if (procuracaoSolicitacao.getArea().getId().equals(cadArea.getId())) {
					// Popular TreeNode da Area
					nodeArea = new DefaultTreeNode("area", cadArea, this.getList());
					break;
				}
				if (nodeArea == null) {
					continue;
				}
			}
			// Recuperar lista de Servicos da area
			CadAreaServico areaServico = new CadAreaServico();
			areaServico.setArea(cadArea);
			listAreaServico = this.getUniversalManager().listBy(areaServico);

			for (CadAreaServico cadAreaServico : listAreaServico) {
				TreeNode nodeServico = null;
				for (CadProcuracaoSolicitacao procuracaoSolicitacao : this.getProcuracaoPai().getProcuracaoSolicitacao()) {
					if (procuracaoSolicitacao.getServico().getId().equals(cadAreaServico.getServico().getId())) {
						// Popular TreeNode de Servico
						nodeServico = new DefaultTreeNode("servico", cadAreaServico.getServico(), nodeArea);
						break;
					}
				}
				if (nodeServico == null) {
					continue;
				}
				// Recuperar lista de Solicitação.
				CadServicoSolicitacao servicoSolicitacao = new CadServicoSolicitacao();
				servicoSolicitacao.setServico(new CadServico(cadAreaServico.getServico().getId()));
				listServicoSolicitacao = this.getUniversalManager().listBy(servicoSolicitacao);

				for (CadServicoSolicitacao cadServicoSolicitacao : listServicoSolicitacao) {
					// Popular TreeNode de Solicitação.
					TreeNode nodeSolicitacao = null;
					for (CadProcuracaoSolicitacao procuracaoSolicitacao : this.getProcuracaoPai().getProcuracaoSolicitacao()) {
						if (procuracaoSolicitacao.getSolicitacao().getId().equals(cadServicoSolicitacao.getSolicitacao().getId())) {
							nodeSolicitacao = new DefaultTreeNode("solicitacao", cadServicoSolicitacao.getSolicitacao(), nodeServico);
							break;
						}
					}
				}
			}
		}
	}

	public CadProcuracao getCreateModel() {
		return createModel();
	}

	@Override
	public String getQualifiedName() {
		return "Substabelecimento";
	}

	@Override
	public boolean isFeminino() {
		return false;
	}

	@Override
	public String getRequiredMessage() {
		return super.getRequiredMessage();
	}

	public void exibirPessoa() {
		this.setNumCnpj("");
		this.setNumCpf("");
		this.setProponenteValidado(null);
		this.setEmpresaValidado(null);
		this.setCpfValidado(false);
		this.setCnpjValidado(false);
	}

	public String validateCPF() {
		if (this.getNumCpf() != null && !this.getNumCpf().isEmpty()) {
			this.setCpfSemFormatacao(this.getNumCpf().replace("-", "").replace("/", "").replace(".", ""));
			if (!CpfCnpjUtil.ValidateCPF(this.getNumCpf())) {
				FacesContext.getCurrentInstance().addMessage("cpf", new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.CPF_INVALIDO));
				return ERROR;
			}

			// Validar se possui acesso ao sistema.
			this.setProponenteValidado((CadProponente) this.getGenericService().createCriteria(CadProponente.class)
					.add(Restrictions.eq("loginProponente", this.getCpfSemFormatacao())).uniqueResult());
			if (this.getProponenteValidado() == null || this.getProponenteValidado().getId() == null || this.getProponenteValidado().getSituacaoCadastro().equals(CadSituacaoCadastroD.REPROVADO)) {
				FacesContext.getCurrentInstance().addMessage("cpf",
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.USUARIO_NAO_CADASTRADO_APROVADO));
				return ERROR;
			}
		}
		this.setCpfValidado(true);
		return SUCCESS;
	}

	public String validateCNPJ() {
		// Validar CNPJ
		if (this.getNumCnpj() != null && !this.getNumCnpj().isEmpty()) {
			this.setCnpjSemFormatacao(this.getNumCnpj().replace("-", "").replace("/", "").replace(".", ""));
			if (!CpfCnpjUtil.validarCNPJ(this.getCnpjSemFormatacao())) {
				FacesContext.getCurrentInstance().addMessage("cnpj", new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.CNPJ_INVALIDO));
				return ERROR;
			}
		}

		// Verificar se algum representante legal possui cadastro no
		// cad_proponete.
		this.setEmpresaValidado((CadEmpresa) this.getProcuracaoService().recuperarEmpresa(this.getCnpjSemFormatacao()));
		boolean proponenteCadastrado = false;
		for (CrpRepresentantePesJur representanteLegal : this.getEmpresaValidado().getPessoaTitular().getRepresentantesLegais()) {
			CadProponente proponente = (CadProponente) this.getGenericService().createCriteria(CadProponente.class)
					.add(Restrictions.eq("pessoaProponente.id", representanteLegal.getPessoaFisica().getId())).uniqueResult();
			if (proponente != null) {
				proponenteCadastrado = true;
				break;
			}
		}

		if (!proponenteCadastrado) {
			FacesContext.getCurrentInstance().addMessage("cnpj",
					new FacesMessage(FacesMessage.SEVERITY_ERROR, "", Constantes.REPRESENTANTE_LEGAL_NAO_CADASTRADO));
			return ERROR;
		}

		this.setCnpjValidado(true);
		return SUCCESS;
	}

	public void renderizarEspecificos() {
		boolean desabilita = false;
		for (int i = 0; i < this.getSelecionadoGeral().size(); i++) {
			if (this.getSelecionadoGeral().get(i).equalsIgnoreCase("T")) {
				this.setRenderEspecificios(false);
				desabilita = true;
				break;
			}
		}
		if (!desabilita) {
			this.setRenderEspecificios(true);
		}
	}

	public String salvar() {
		try {

			if (!validar()) {

				// SETAR FLG_SUBSTABELECIMENTO PARA VEDADO, SOMENTE PODERIA
				// SUBSTABELECER UM VEZ.
				this.getModel().setFlgPermiteSubstabelecimento(false);

				// VERIFICAR PELA PROCURACAO PAI, SE EH EMPRESA OU PROPONENTE
				// QUEM ESTA SUBSTABELECENDO A PROCURACAO.
				if (this.getProcuracaoPai().getEmpresaOutorgado().getId() != null) {
					// Empresa Outorgado da procuracao Pai (quem recebeu), esta
					// passando uma procuracao agora no caso Empresa Outorgante.
					this.getModel().setEmpresaOutorgante(this.getProcuracaoPai().getEmpresaOutorgado());
				} else {
					// Proponente Outorgado da procuracao Pai (quem recebeu),
					// esta passando uma procuracao agora no caso Proponente
					// Outorgante.
					this.getModel().setProponenteOutorgante(this.getProcuracaoPai().getProponenteOutorgado());
				}

				// PROPOENETE OU EMPRESA OUTORGADO INFORMADO.
				if (this.isCpfValidado()) {
					this.getModel().setProponenteOutorgado(this.getProponenteValidado());
				} else {
					this.getModel().setEmpresaOutorgado(this.getEmpresaValidado());
				}

				// POPULAR NUMERO SEQUENCIAL
				this.getModel().setNrSequencial(this.getProcuracaoService().getNumeroSequencial(SessionUtil.getProponenteLogado().getId(), null));

				// POPULAR PROCURAÇÃO SOLICITACAO - SOLICITAÇÕES INFORMADA
				this.getModel().setProcuracaoSolicitacao(salvarSolicitacoes());

				// STATUS DA PROCURAÇÃO - SENDO CRIADA SEMPRE SERA ATIVA.
				this.getModel().setStatusProcuracao(new CadStatusProcuracaoD(CadStatusProcuracaoD.ATIVA));

				// Criando relacionamento da procuração PAI.
				this.getModel().setProcuracaoPai(this.getProcuracaoPai());

				this.getModel().setFlgPermiteSubstabelecimento(true);

				// SALVAR
				this.getUniversalManager().save(this.getModel());

				FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), ConstantesARQ.SALVA + " " + ConstantesARQ.COM_SUCESSO);
			}

		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		return SUCCESS;
	}

	private boolean validar() {

		if (this.getNumCpf().isEmpty() && this.getNumCnpj().isEmpty()) {
			FacesMessagesUtil.addErrorMessage("Outorgado(s) " + ConstantesARQ.CAMPO_OBRIGATORIO);
			return true;
		}

		if (!this.getSelecionadoGeral().contains("T")) {
			if (!this.isRenderEspecificios() && this.getSelectedSolicitacao() != null || this.getSelectedSolicitacao().length == 0) {
				FacesMessagesUtil.addErrorMessage("Espec&iacute;ficos " + ConstantesARQ.CAMPO_OBRIGATORIO);
				return true;
			}
		}

		boolean erro = false;
		if (this.getTipoPessoa().equalsIgnoreCase("F")) {
			CadProcuracao procuracao = null;
			if (this.getProcuracaoPai().getEmpresaOutorgado().getId() != null) {
				procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCPF(this.getCpfSemFormatacao(),
						this.getProcuracaoPai().getEmpresaOutorgado().getId(), null);
				if (procuracao != null) {
					erro = true;
				}
			} else {
				procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCPF(this.getCpfSemFormatacao(), null,
						this.getProcuracaoPai().getProponenteOutorgado().getId());
				if (procuracao != null) {
					erro = true;
				}
			}
		} else {
			CadProcuracao procuracao = null;
			if (this.getProcuracaoPai().getEmpresaOutorgado().getId() != null) {
				procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCNPJ(this.getCnpjSemFormatacao(),
						this.getProcuracaoPai().getEmpresaOutorgado().getId(), null);
				if (procuracao != null) {
					erro = true;
				}
			} else {
				procuracao = this.getProcuracaoService().findProcuracaoByOutorgadoCNPJ(this.getCnpjSemFormatacao(), null,
						this.getProcuracaoPai().getProponenteOutorgado().getId());
				if (procuracao != null) {
					erro = true;
				}
			}
		}
		if (erro) {
			FacesMessagesUtil.addErrorMessage(Constantes.OUTORGANTE_JA_POSSUI_PARA_OUTORGADO);
			return true;
		}

		// validar Data
		if (this.getModel().getDataValidadeInicial().getTime() > this.getModel().getDataValidadeFinal().getTime()) {
			FacesMessagesUtil.addErrorMessage(Constantes.DATA_INICIAL_MAIOR_DATA_FINAL);
			return true;
		}

		if (this.getModel().getDataValidadeFinal().getTime() > this.getProcuracaoPai().getDataValidadeFinal().getTime()) {
			FacesMessagesUtil.addErrorMessage(Constantes.DATA_FINAL_DATA_PROCURACAO_ORIGINAL);
			return true;
		}

		Date dataMaxima = adicionarAnos();
		if (this.getModel().getDataValidadeFinal().getTime() > dataMaxima.getTime()) {
			FacesMessagesUtil.addErrorMessage(Constantes.DATA_PRAZO_MAXIMO);
			return true;
		}
		return false;
	}

	public Date adicionarAnos() {
		Calendar dataFinal = Calendar.getInstance();
		dataFinal.setTime(new Date());
		dataFinal.set(Calendar.MONTH, dataFinal.get(Calendar.MONTH));
		dataFinal.set(Calendar.DAY_OF_MONTH, dataFinal.get(Calendar.DAY_OF_MONTH));
		dataFinal.set(Calendar.YEAR, dataFinal.get(Calendar.YEAR) + 5);
		return dataFinal.getTime();
	}

	private List<CadProcuracaoSolicitacao> salvarSolicitacoes() {
		List<CadProcuracaoSolicitacao> list = new ArrayList<CadProcuracaoSolicitacao>();
		// Caso opção de serviços estiver marcados Todos
		if (!this.isRenderEspecificios()) {
			for (CadProcuracaoSolicitacao cadProcuracaoSolicitacao : this.getProcuracaoPai().getProcuracaoSolicitacao()) {
				CadProcuracaoSolicitacao solProcuracao = new CadProcuracaoSolicitacao();
				solProcuracao.setProcuracao(this.getModel());
				solProcuracao.setArea(cadProcuracaoSolicitacao.getArea());
				solProcuracao.setServico(cadProcuracaoSolicitacao.getServico());
				solProcuracao.setSolicitacao(cadProcuracaoSolicitacao.getSolicitacao());
				list.add(solProcuracao);
			}
			// list.addAll(this.getProcuracaoPai().getProcuracaoSolicitacao());
		} else {
			for (TreeNode selecionados : this.getSelectedSolicitacao()) {
				CadProcuracaoSolicitacao procuracaoSolicitacao = new CadProcuracaoSolicitacao();
				procuracaoSolicitacao.setProcuracao(this.getModel());
				// Somente pegar os selecionados SOLICITAÇÂO.
				if (selecionados.getType().equalsIgnoreCase("solicitacao")) {
					procuracaoSolicitacao.setSolicitacao((CadSolicitacao) selecionados.getData());
					procuracaoSolicitacao.setServico((CadServico) selecionados.getParent().getData());
					procuracaoSolicitacao.setArea((CadArea) selecionados.getParent().getParent().getData());
					list.add(procuracaoSolicitacao);
				}
			}
		}
		return list;
	}

	public String getTipoPessoa() {
		return tipoPessoa;
	}

	public void setTipoPessoa(String tipoPessoa) {
		this.tipoPessoa = tipoPessoa;
	}

	public String getNumCnpj() {
		return numCnpj;
	}

	public void setNumCnpj(String numCnpj) {
		this.numCnpj = numCnpj;
	}

	public String getNumCpf() {
		return numCpf;
	}

	public void setNumCpf(String numCpf) {
		this.numCpf = numCpf;
	}

	public String getCnpjSemFormatacao() {
		return cnpjSemFormatacao;
	}

	public void setCnpjSemFormatacao(String cnpjSemFormatacao) {
		this.cnpjSemFormatacao = cnpjSemFormatacao;
	}

	public String getCpfSemFormatacao() {
		return cpfSemFormatacao;
	}

	public void setCpfSemFormatacao(String cpfSemFormatacao) {
		this.cpfSemFormatacao = cpfSemFormatacao;
	}

	public GenericService getGenericService() {
		return genericService;
	}

	public void setGenericService(GenericService genericService) {
		this.genericService = genericService;
	}

	public ProcuracaoService getProcuracaoService() {
		return procuracaoService;
	}

	public void setProcuracaoService(ProcuracaoService procuracaoService) {
		this.procuracaoService = procuracaoService;
	}

	public TreeNode[] getSelectedSolicitacao() {
		return selectedSolicitacao;
	}

	public void setSelectedSolicitacao(TreeNode[] selectedSolicitacao) {
		this.selectedSolicitacao = selectedSolicitacao;
	}

	public TreeNode getList() {
		return list;
	}

	public void setList(TreeNode list) {
		this.list = list;
	}

	public List<String> getSelecionadoGeral() {
		if (selecionadoGeral == null) {
			selecionadoGeral = new ArrayList<String>();
		}
		return selecionadoGeral;
	}

	public void setSelecionadoGeral(List<String> selecionadoGeral) {
		this.selecionadoGeral = selecionadoGeral;
	}

	public boolean isRenderEspecificios() {
		return renderEspecificios;
	}

	public void setRenderEspecificios(boolean renderEspecificios) {
		this.renderEspecificios = renderEspecificios;
	}

	public CadProponente getProponenteValidado() {
		if (proponenteValidado == null) {
			proponenteValidado = new CadProponente();
		}
		return proponenteValidado;
	}

	public void setProponenteValidado(CadProponente proponenteValidado) {
		this.proponenteValidado = proponenteValidado;
	}

	public CadEmpresa getEmpresaValidado() {
		if (empresaValidado == null) {
			empresaValidado = new CadEmpresa();
		}
		return empresaValidado;
	}

	public void setEmpresaValidado(CadEmpresa empresaValidado) {
		this.empresaValidado = empresaValidado;
	}

	public boolean isCpfValidado() {
		return cpfValidado;
	}

	public void setCpfValidado(boolean cpfValidado) {
		this.cpfValidado = cpfValidado;
	}

	public boolean isCnpjValidado() {
		return cnpjValidado;
	}

	public void setCnpjValidado(boolean cnpjValidado) {
		this.cnpjValidado = cnpjValidado;
	}

	public CadProcuracao getProcuracaoPai() {
		if (procuracaoPai == null) {
			procuracaoPai = new CadProcuracao();
		}
		return procuracaoPai;
	}

	public void setProcuracaoPai(CadProcuracao procuracaoPai) {
		this.procuracaoPai = procuracaoPai;
	}

	public String getNumeroProcuracao() {
		if (this.getProcuracaoPai() != null && this.getProcuracaoPai().getId() != null) {
			return this.getProcuracaoPai().getAno() + this.getProcuracaoPai().getNumeroSequencialZeros();
//			if (this.getProcuracaoPai().getProponenteOutorgado().getId() == null) {
//				return this.getProcuracaoPai().getEmpresaOutorgado().getPessoaTitular().getCnpjFormat() + "/" + this.getProcuracaoPai().getAno() + "."
//						+ this.getProcuracaoPai().getNumeroSequencialZeros();
//			} else {
//				return this.getProcuracaoPai().getProponenteOutorgado().getLoginFormated() + "/" + this.getProcuracaoPai().getAno() + "."
//						+ this.getProcuracaoPai().getNumeroSequencialZeros();
//			}
		}
		return numeroProcuracao;
	}

	public void setNumeroProcuracao(String numeroProcuracao) {
		this.numeroProcuracao = numeroProcuracao;
	}

	public String getNomeOutorgante() {
		if (this.getProcuracaoPai() != null && this.getProcuracaoPai().getId() != null) {
			if (this.getProcuracaoPai().getProponenteOutorgante().getId() == null) {
				return this.getProcuracaoPai().getEmpresaOutorgante().getPessoaTitular().getRazaoSocial();
			} else {
				return this.getProcuracaoPai().getProponenteOutorgante().getPessoaProponente().getNome();
			}
		}
		return nomeOutorgante;
	}

	public void setNomeOutorgante(String nomeOutorgante) {
		this.nomeOutorgante = nomeOutorgante;
	}

	public String getCpfCnpjOutorgante() {
		if (this.getProcuracaoPai() != null && this.getProcuracaoPai().getId() != null) {
			if (this.getProcuracaoPai().getProponenteOutorgante().getId() == null) {
				return this.getProcuracaoPai().getEmpresaOutorgante().getPessoaTitular().getCnpjFormat();
			} else {
				return this.getProcuracaoPai().getProponenteOutorgante().getLoginFormated();
			}
		}
		return cpfCnpjOutorgante;
	}

	public void setCpfCnpjOutorgante(String cpfCnpjOutorgante) {
		this.cpfCnpjOutorgante = cpfCnpjOutorgante;
	}

	public String getNomeOutorgado() {
		if (this.getProcuracaoPai() != null && this.getProcuracaoPai().getId() != null) {
			if (this.getProcuracaoPai().getProponenteOutorgado().getId() == null) {
				return this.getProcuracaoPai().getEmpresaOutorgado().getPessoaTitular().getRazaoSocial();
			} else {
				return this.getProcuracaoPai().getProponenteOutorgado().getPessoaProponente().getNome();
			}
		}
		return nomeOutorgado;
	}

	public void setNomeOutorgado(String nomeOutorgado) {
		this.nomeOutorgado = nomeOutorgado;
	}

	public String getCpfCnpjOutorgado() {
		if (this.getProcuracaoPai() != null && this.getProcuracaoPai().getId() != null) {
			if (this.getProcuracaoPai().getProponenteOutorgado().getId() == null) {
				return this.getProcuracaoPai().getEmpresaOutorgado().getPessoaTitular().getCnpjFormat();
			} else {
				return this.getProcuracaoPai().getProponenteOutorgado().getLoginFormated();
			}
		}
		return cpfCnpjOutorgado;
	}

	public void setCpfCnpjOutorgado(String cpfCnpjOutorgado) {
		this.cpfCnpjOutorgado = cpfCnpjOutorgado;
	}

}