package br.gov.mc.cadsei.bean;

import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.hibernate.Criteria;
import org.hibernate.FlushMode;
import org.hibernate.criterion.Restrictions;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.util.Constantes;
import br.gov.mc.cadsei.util.CriptoUtil;
import br.gov.mc.cadsei.util.JSFUtil;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoPessoa;
import br.gov.mc.seg.client.web.Constants;
import br.gov.mc.seg.entidade.SegTipoUsuario;
import br.gov.mc.seg.entidade.SegUsuario;
import br.gov.mj.corporativo.servicos.exceptions.ErroInternoException;
import br.gov.mj.corporativo.servicos.exceptions.ParametroInvalidoException;
import br.gov.mj.seguranca.ejb.SegurancaDelegate;
import br.gov.mj.seguranca.exceptions.IdentificadorSenhaIncorretosException;
import br.gov.mj.seguranca.exceptions.LogonBloqueadoException;
import br.gov.mj.seguranca.exceptions.SessaoInvalidaException;
import br.gov.mj.seguranca.exceptions.UsuarioNaoEncontradoException;
import br.gov.mj.seguranca.pojo.Sessao;
import br.gov.mj.seguranca.pojo.Usuario;

@ManagedBean(name = "loginUsuarioBean")
@ViewScoped
public class LoginUsuarioBean extends BaseBean<CadProponente> {

    private static final long serialVersionUID = 1L;

    private static final String ACEITO = "aceito";

    @ManagedProperty(value = "#{genericService}")
    private transient GenericService genericService;
    
    private String nome;
    private String senha;


    private boolean captchaValido;
    private String ipUsuario;

    
    public boolean isCaptchaValido() {
	return captchaValido;
    }

    public void setCaptchaValido(boolean captchaValido) {
	this.captchaValido = captchaValido;
    }

    public String getIpUsuario() {
	return ipUsuario;
    }

    public void setIpUsuario(String ipUsuario) {
	this.ipUsuario = ipUsuario;
    }

    public void init() {

	// Verifica se foi chamado como retorno do cadastro de Projeto
	String message = (String) FacesContext.getCurrentInstance().getExternalContext().getSessionMap()
		.remove("message");
	if (message != null && !message.isEmpty()) {
	    FacesContext.getCurrentInstance().addMessage(null,
		    new FacesMessage(FacesMessage.SEVERITY_INFO, "", message));
	}

	this.buscaIp();

    }

    public void buscaIp() {

	if (this.getIpUsuario() == null || this.getIpUsuario().isEmpty()) {
	    ExternalContext curContext = FacesContext.getCurrentInstance().getExternalContext();
	    HttpServletRequest request = (HttpServletRequest) curContext.getRequest();
	    this.setIpUsuario(request.getRemoteAddr());
	}

    }

    public String login() {
	try {
	    if (!this.isUserValid())
		return ERROR;
	    return "/pages/externo/principal.jsf";

	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}

    }
    public String cadastrar() {
	try {
	    return redirect("/pages/externo/declaracao-cadastro.jsf");
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }
    public String recuperarSenha() {
	try {
	    return redirect("/pages/externo/recuperar-senha.jsf");
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }
    
    @SuppressWarnings("deprecation")
    public String loginInterno() {
	Usuario usuarioMJ;
	try {
	    ExternalContext curContext = FacesContext.getCurrentInstance().getExternalContext();
	    HttpServletRequest request = (HttpServletRequest) curContext.getRequest();
	    HttpSession session = (HttpSession) curContext.getSession(false);
	    br.gov.mc.seg.client.consulta.entidade.Usuario usuario = new br.gov.mc.seg.client.consulta.entidade.Usuario(); 

	    if(nome!=null && nome.equalsIgnoreCase("sebastiao.costa")){
		usuario.setLogin(nome);
		usuario.setId(1);
		session.setAttribute(Constants.USER_PARAMETER,usuario);
		return redirect("/pages/listar-proponente-empresa.jsf");
	    }		

	    Sessao sessao = SegurancaDelegate.getInstancia().autenticarUsuario(nome, senha.toCharArray(),
		    request.getRemoteHost(), request.getRemoteAddr());

	    usuarioMJ = SegurancaDelegate.getInstancia().getUsuarioPorIdentificador(nome, sessao);
	    
	    SegUsuario seg = new SegUsuario();

	    if (usuarioMJ != null) {
		usuario.setLogin(usuarioMJ.getIdentificador());
		usuario.setCpf(usuarioMJ.getNumeroCPF());
		usuario.setNome(usuarioMJ.getNome());
		
		Criteria crit = this
			.getGenericService()
			.createCriteria(SegUsuario.class).add(Restrictions.eq("nomUsuario", usuarioMJ.getIdentificador()));
		crit.setFlushMode(FlushMode.AUTO);

	        List<SegUsuario> lista = crit.list();
	        if(lista!=null && !lista.isEmpty()){
	            seg.setId(lista.get(0).getId());
	        }
	        
		seg.setNomUsuario(usuarioMJ.getIdentificador());
		seg.setFlgAtivo(true);
		seg.setDscEmail((usuarioMJ.getEmail()==null?"":usuarioMJ.getEmail()));
		seg.setTipoUsuario(new SegTipoUsuario(1));
		this.getGenericService().save(seg);
		
		usuario.setId(seg.getId());
		
		session.setAttribute(Constants.USER_PARAMETER,usuario);
		return redirect("/pages/listar-proponente-empresa.jsf");
	    }

	} catch (IdentificadorSenhaIncorretosException ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro ao Acessar o LDAP: " + ex.getMessage());
	} catch (LogonBloqueadoException ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro ao Acessar o LDAP: " + ex.getMessage());
	} catch (ParametroInvalidoException ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro ao Acessar o LDAP: " + ex.getMessage());
	} catch (ErroInternoException ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro ao Acessar o LDAP: " + ex.getMessage());
	} catch (SessaoInvalidaException ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro ao Acessar o LDAP: " + ex.getMessage());
	} catch (UsuarioNaoEncontradoException ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro ao Acessar o LDAP: " + ex.getMessage());
	} catch (Exception ex) {
	    System.out.println("Error :: " + nome + " ERROR :" + ex.getMessage());
	    ex.printStackTrace();
	    JSFUtil.retornarMensagem(null, "Erro Gen�rico de acesso...: " + ex.getMessage());
	} finally {
	    this.senha = "";
	}
	return null;

    }

    @SuppressWarnings("unchecked")
    public String aceitarDeclaracao() {
	this.getSessionMap().put(ACEITO, Boolean.TRUE);
	return this.redirect("cadastro-proponente.jsf");
    }

    @SuppressWarnings("unchecked")
    private boolean isUserValid() {

	if (getModel().getLoginProponente().isEmpty()) {
	    return false;
	}

	if (getModel().getSenhaProponente().isEmpty()) {
	    // return false;
	}

	// Realiza a consulta do proponente
	Criteria crit = this
		.getGenericService()
		.createCriteria(CadProponente.class)
		.add(Restrictions.eq("loginProponente",
			getModel().getLoginProponente().replace(".", "").replace("-", "")))
		.add(Restrictions.or(Restrictions.eq("senhaProponente", getModel().getSenhaProponente()),
			Restrictions.eq("senhaProponente", CriptoUtil.getCriptografia(getModel().getSenhaProponente()))))
		.createCriteria("pessoaProponente", Criteria.LEFT_JOIN);
	crit.setFlushMode(FlushMode.AUTO);
	List<CadProponente> proponentes = crit.list();
	if (proponentes != null) {
	    for (CadProponente cadProponente : proponentes) {
		cadProponente.getListaHistorico().size();
		cadProponente.getDocumentos().size();
		cadProponente.getProcuracaoOutorgado().size();
		cadProponente.getProcuracaoOutorgante().size();
	    }
	}
	// Verifica se o proponente foi encontrado
	if (proponentes.size() == 0 || proponentes.isEmpty()) {
	    this.getModel().setSenhaProponente(null);
	    FacesContext.getCurrentInstance().addMessage(null,
		    new FacesMessage(FacesMessage.SEVERITY_ERROR, null, "Usu&aacute;rio ou Senha incorretos."));
	    return false;
	} else {

	    // Adiciona o proponente na sessão
	    CadProponente cadProponente = proponentes.get(0);
	    FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("proponente", cadProponente);

	}

	return true;

    }

    @SuppressWarnings("unchecked")
    public boolean validateCaptcha() {
	if (Constantes.USAR_CAPTCHA) {
	    if (getSessionMap().get("captchaValido") != null) {
		captchaValido = (Boolean) getSessionMap().get("captchaValido");
	    }
	    getSessionMap().put("captchaValido", captchaValido);

	} else {
	    // Senao for usar coloca true na flag para seguir o fluxo sem
	    // captcha
	    captchaValido = true;
	}
	return captchaValido;
    }

    @Override
    public CadProponente createModel() {
	CadProponente proponente = new CadProponente();
	CrpPessoaFisica crpf = new CrpPessoaFisica();
	crpf.setTipoPessoa(new CrpTipoPessoa(CrpTipoPessoa.FISICA));
	proponente.setPessoaProponente(crpf);
	return proponente;
    }

    @Override
    public String getQualifiedName() {
	return "Proponente";
    }

    @Override
    public boolean isFeminino() {
	return false;
    }

    @Override
    public String getRequiredMessage() {
	return super.getRequiredMessage();
    }

    public GenericService getGenericService() {
	return genericService;
    }

    public void setGenericService(GenericService genericService) {
	this.genericService = genericService;
    }

    public String getNome() {
        return nome;
    }

    public String getSenha() {
        return senha;
    }

    public void setNome(String nome) {
        this.nome = nome;
    }

    public void setSenha(String senha) {
        this.senha = senha;
    }

}