package br.gov.mc.cadsei.bean;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.primefaces.event.SelectEvent;
import org.primefaces.event.UnselectEvent;
import org.springframework.dao.DataIntegrityViolationException;

import br.gov.arquitetura.bean.PaginableBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.util.SessionUtil;

@ManagedBean(name = "pessoaJuridicaBean")
@ViewScoped
public class PessoaJuridicaBean extends PaginableBean<CadEmpresa> {

	private static final long serialVersionUID = 1L;

	private static final String EMPRESA = "idEmpresa";
	private static final String QUERY_ONLY = "somenteConsulta";

	@ManagedProperty(value = "#{genericService}")
	private transient GenericService genericService;

	@ManagedProperty(value = "#{empresaService}")
	private transient EmpresaService empresaService;

	private List<CadEmpresa> empresas;

	private boolean selectedRecord;
	private CadEmpresa selected;

	@Override
	public CadEmpresa createModel() {
		this.setSelectedRecord(true);
		return new CadEmpresa();
	}

	@Override
	public String getQualifiedName() {
		return "Pessoa Jur&iacute;dica";
	}

	@Override
	public boolean isFeminino() {
		return true;
	}

	public void init() {

		// Verifica se foi chamado como retorno do cadastro de Projeto
		Boolean verifica = (Boolean) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get("showSaveMessage");
		if (verifica != null && verifica) {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, this.getQualifiedName(), this.getSaveMessage() + " " + ConstantesARQ.COM_SUCESSO));
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put("showSaveMessage", false);
		}

	}

	public String novo() {
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(EMPRESA, null);
		return redirect("cadastro-empresa.jsf");
	}

	public String editar() {
		this.setModel(this.getSelected());
		if (this.getModel() != null) {
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(EMPRESA, this.getModel().getId());
		} else {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, "", "Selecione uma pessoa jur&iacute;dica para editar."));
			return ERROR;
		}
		return redirect("cadastro-empresa.jsf");
	}

	public String visualizar() {
		this.setModel(this.getSelected());
		if (this.getModel() != null) {
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(EMPRESA, this.getModel().getId());
			FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(QUERY_ONLY, Boolean.TRUE);
		} else {
			FacesContext.getCurrentInstance().addMessage(null,
					new FacesMessage(FacesMessage.SEVERITY_INFO, "", "Selecione uma pessoa jur&iacute;dica para visualizar."));
			return ERROR;
		}
		return redirect("cadastro-empresa.jsf");
	}

	@Override
	public String delete() {
		this.setModel(this.getSelected());
		try {
			this.getUniversalManager().remove(this.getModel().getClass(), this.getModel().getId());
			FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), this.getRemoveMessage() + " " + ConstantesARQ.COM_SUCESSO);
		} catch (DataIntegrityViolationException dive) {
			FacesMessagesUtil.addErrorMessage(this.getQualifiedName(), ConstantesARQ.ERRO_EXCLUIR_DEPENDENCIA);
			this.setModel(this.createModel());
			return ERROR;
		} catch (Exception e) {
			FacesMessagesUtil.addErrorMessage(this.getQualifiedName(), ConstantesARQ.ERRO_EXCLUIR);
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		this.setModel(this.createModel());
		this.setEmpresas(null);
		return SUCCESS;
	}

	public void enableToolbar(SelectEvent event) {
		this.setSelectedRecord(false);
	}

	public void disableToolbar(UnselectEvent event) {
		this.setSelectedRecord(true);
	}

	public boolean isSelectedRecord() {
		return selectedRecord;
	}

	public void setSelectedRecord(boolean selectedRecord) {
		this.selectedRecord = selectedRecord;
	}

	public CadEmpresa getSelected() {
		return selected;
	}

	public void setSelected(CadEmpresa selected) {
		this.selected = selected;
	}

	@Override
	public Map<String, String> getFilters() {
		CadProponente proponente = SessionUtil.getProponenteLogado();
		if (filters == null) {
			filters = new HashMap<String, String>();
		}
		filters.put("proponente.id", proponente.getId().toString());
		return filters;
	}

	public List<CadEmpresa> getEmpresas() {
		// Deixado de utilizar getLazyDataModel devido ao problema de pesquisa
		// com like
		// mesmo quando o campos é um inteiro.
		if (this.empresas == null) {
			setEmpresas(empresaService.listaEmpresaPorRepresentante(SessionUtil.getProponenteLogado().getLoginProponente()));
		}
		return this.empresas;
	}

	public void setEmpresas(List<CadEmpresa> empresas) {
		this.empresas = empresas;
	}

	public GenericService getGenericService() {
		return genericService;
	}

	public void setGenericService(GenericService genericService) {
		this.genericService = genericService;
	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}

}