package br.gov.mc.cadsei.bean.interno;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;
import javax.faces.event.AjaxBehaviorEvent;
import javax.sql.rowset.serial.SerialBlob;

import org.primefaces.event.FileUploadEvent;
import org.primefaces.model.DefaultStreamedContent;
import org.primefaces.model.StreamedContent;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.bean.SEGBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.mc.cadsei.entidade.CadDocumento;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.CadTipoDocumentoD;
import br.gov.mc.cadsei.entidade.dto.AnaliseCadastroPessoaJuridicaDto;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.service.ProponenteService;
import br.gov.mc.cadsei.util.EnvioMail;
import br.gov.mc.cadsei.util.StringUtil;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpTipoDocPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoEndereco;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;
import br.gov.mc.seg.entidade.SegUsuario;

@ManagedBean(name = "analisarCadastroPessoaJuridicaBean")
@ViewScoped
public class AnalisarCadastroPessoaJuridicaBean extends BaseBean<CadProponente>{

	private static final long serialVersionUID = 1L;
	
	private AnaliseCadastroPessoaJuridicaDto dto;
	
	@ManagedProperty(value = "#{proponenteService}")
	private transient ProponenteService proponenteService;
	
	@ManagedProperty(value = "#{empresaService}")
	private transient EmpresaService empresaService;
	
	private CadProponente proponente;
	
	private String comentarioAnaliseTecnica;
	private boolean renderTabEmpresa;

	@ManagedProperty(value = "#{SEGBean}")
	private transient SEGBean segBean;
	
	private final String PROPONENTE = "idProponente";
	private final String EMPRESA = "idEmpresa";
	
	@ManagedProperty(value = "#{genericService}")
	private transient GenericService genericService;
	
	@Override
	public CadProponente createModel() {
		return new CadProponente();
	}

	@Override
	public String getQualifiedName() {
		return "Cadastro";
	}

	@Override
	public boolean isFeminino() {
		return false;
	}
	
	public String analisar() {
		return this.redirect("analisar-cadastro.jsf");
	}
	public String voltar() {
		return this.redirect("listar-proponente-empresa.jsf");
	}
	public String voltarVisualizar() {
		return this.redirect("listar-proponente-empresa-consulta.jsf");
	}
	
	
	@PostConstruct
	public void init(){
		carregarTela(); 
		setProponente(new CadProponente());
		Integer idProponente = (Integer) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(PROPONENTE);
		Integer idEmpresa = (Integer) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(EMPRESA);
		
		setModel(proponenteService.buscaProponentePorId(idProponente));
		dto.setRepresentante(empresaService.buscaRepresentantePorEmpresa(idEmpresa));
		getModel().setEmpresas(empresaService.buscaEmpresaPorRepresentante(dto.getRepresentante().getId()));
		setRenderTabEmpresa(!getModel().getEmpresas().isEmpty());
		dto.setDescricaoAnaliseTecnicaProponente(getModel().getDescricaoAnaliseTecnica());
		
		for (CadDocumento documento : getModel().getDocumentos()) {
			if(CadTipoDocumentoD.RG.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoRg(documento);
			}else if(CadTipoDocumentoD.CPF.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoCpf(documento);
			}else if(CadTipoDocumentoD.RESIDENCIA.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoResidencia(documento);
			}else if(CadTipoDocumentoD.REGISTRO_OAB.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoOab(documento);
			}
			for (CrpContatoEmail e : dto.getRepresentante().getPessoaFisica().getEmails()){
				if(e.getEmailPessoal() != null && e.getEmailPessoal().booleanValue()) {
					dto.setEmailPessoaFisica(e);
				}
			}
		}
		
		/*
		 Recupera os documentos da pessoa fisica, 
		 que para a empresa será mostrada no fieldset pessoa fisica.
		 que, neste caso, será o representante legal da Empresa.*/
		

		// Recupera os documentos do proponente
		for(CrpDocumentoPessoaFisica documento : dto.getRepresentante().getPessoaFisica().getDocumentos()){
			if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.CPF)) {
				dto.setCpf(documento);
			} else if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG)) {
				dto.setRg(documento);
			}
		}

		// Recupera os telefones do proponente
		for(CrpContatoTelefone telefone : dto.getRepresentante().getPessoaFisica().getTelefones()){
			if(telefone.getTipoTelefone().getId().equals(CrpTipoTelefone.CONTATO)) {
				dto.setTelefone(telefone);
			} else if(telefone.getTipoTelefone().getId().equals(CrpTipoTelefone.CELULAR)) {
				dto.setCelular(telefone);
			}
		}

		// Recupera o endereço
		for(CrpEndereco endereco : dto.getRepresentante().getPessoaFisica().getEnderecos()){
			if(CrpTipoEndereco.CORRESPONDENCIA.equals(endereco.getTipoEndereco().getId())){
				dto.setEndereco(endereco);
			}
		}

		// Carrega os dados da empresa caso haja apenas uma
		// Do contrário, o usuário deverá selecionar a empresa que quer pesquisar na lista
		if(!getModel().getEmpresas().isEmpty()){
			carregarDadosEmpresa(getModel().getEmpresas().get(0));
		}

	}
	private void carregarTela() {
		dto = new AnaliseCadastroPessoaJuridicaDto();
		dto.setCpfRepresentanteEmpresa(new CrpDocumentoPessoaFisica());
		dto.setDocumento(new CadDocumento());
		dto.setDocumentoCnpj(new CadDocumento());
		dto.setDocumentoCpf(new CadDocumento());
		dto.setDocumentoDeclaracao(new CadDocumento());
		dto.setDocumentoOab(new CadDocumento());
		dto.setDocumentoRepresentanteLegal(new CadDocumento());
		dto.setDocumentoResidencia(new CadDocumento());
		dto.setDocumentoRg(new CadDocumento());
		dto.setEmailEmpresa(new CrpContatoEmail());
		dto.setEmpresa(new CadEmpresa());
		dto.setEnderecoEmpresa(new CrpEndereco());
		dto.setRgRepresentanteEmpresa(new CrpDocumentoPessoaFisica());
		dto.setTelefoneEmpresa(new CrpContatoTelefone());
		dto.setServicos(empresaService.buscaServicos());

	}

	private void carregarDadosEmpresa(CadEmpresa empresa){
		
		// Define a empresa
		dto.setEmpresa(empresaService.buscaEmpresaPorId(empresa.getId()));
		dto.setRepresentanteEmpresa(empresaService.buscaRepresentantePorEmpresa(empresa.getId()));
		dto.setDescricaoAnaliseTecnicaEmpresa(empresa.getDescricaoAnaliseTecnica());
	//	Hibernate.initialize(empresa.getProponente());
		// Recupera os documentos da empresa
		for (CadDocumento documento : empresa.getDocumentos()) {
			if(CadTipoDocumentoD.ATOS_CONSTITUTIVOS.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoAtosConstitutivos(documento);
			} else if(CadTipoDocumentoD.CNPJ.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoCnpj(documento);
			} else if(CadTipoDocumentoD.REPRESENTANTE_LEGAL.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoRepresentanteLegal(documento);
			} else if(CadTipoDocumentoD.DECLARACAO_VERACIDADE.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoDeclaracao(documento);
			}
		}
		
		// Recupera os documentos de identificação do representante legal da empresa
		for(CrpDocumentoPessoaFisica documento : dto.getRepresentanteEmpresa().getPessoaFisica().getDocumentos()){
			if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.CPF)) {
				dto.setCpfRepresentanteEmpresa(documento);
			} else if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG)) {
				dto.setRgRepresentanteEmpresa(documento);
			}
		}
		
		// Recupera o telefone da empresa
		for(CrpContatoTelefone telefone : empresa.getPessoaTitular().getTelefones()){
			if(telefone.getTipoTelefone().getId().equals(CrpTipoTelefone.COMERCIAL)) {
				dto.setTelefoneEmpresa(telefone);
			}
		}
		
		// Recupera o e-mail da empresa
		for(CrpContatoEmail email : empresa.getPessoaTitular().getEmails()){
			if(email.getEmailPessoal() != null && !email.getEmailPessoal().booleanValue()) {
				dto.setEmailEmpresa(email);
			}
		}

		// Recupera o endereço da empresa
		for(CrpEndereco endereco : empresa.getPessoaTitular().getEnderecos()){
			if(CrpTipoEndereco.SEDE.equals(endereco.getTipoEndereco().getId()) 
					|| CrpTipoEndereco.COMERCIAL.equals(endereco.getTipoEndereco().getId())){
				dto.setEnderecoEmpresa(endereco);
			}
		}
		
	}
	
	
	
	
	public String aprovarEmpresa(){


		// Define a situação como aprovado e salva
		dto.getEmpresa().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.APROVADO));
		dto.getEmpresa().setAnalista(new SegUsuario(getSegBean().getUsuario().getId()));

		
		try {
			getUniversalManager().save(dto.getEmpresa());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			e.printStackTrace();
			FacesContext.getCurrentInstance().addMessage("aprovacaoPj", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de Aprova&ccedil;&atilde;o de cadastro" + 
									"Ocorreu um erro ao enviar e-mail de notifica&ccedil;&atilde;o ao proponente.", null));
			return ERROR;
		}
		// Envia e-mail notificando o proponente sobre a reprovação do cadastro
		try {
			EnvioMail.sendEmailAprovacaoEmpresa(dto.getEmpresa());
		} catch (Exception e) {
			// Para registrar no arquivo de log do servidor
			e.printStackTrace();
			return ERROR;
			
		
			
		}
		FacesContext.getCurrentInstance().addMessage("aprovacaoPj", 
				new FacesMessage(FacesMessage.SEVERITY_INFO,
						dto.getEmpresa().getSituacaoCadastro().getDescricao().toLowerCase() + " " + ConstantesARQ.COM_SUCESSO, null));
		return SUCCESS;
	}
	
	public String reprovarEmpresa(){

		// Verifica se a declaração de veracidade foi anexada ao cadastro
		if(StringUtil.isEmpty(dto.getEmpresa().getDescricaoAnaliseTecnica())){
			FacesContext.getCurrentInstance().addMessage("reprovacaoPj", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Para alterar o status do cadastro &eacute; necess&aacute;rio incluir um coment&aacute;rio!", null));
					return ERROR;
		}

		// Define a situação como reprovado e salva
		dto.getEmpresa().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.REPROVADO));
		dto.getEmpresa().setAnalista(new SegUsuario(getSegBean().getUsuario().getId()));

		try {
			getUniversalManager().save(dto.getEmpresa());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}

		// Envia e-mail notificando o proponente sobre a reprovação do cadastro
		try {
			EnvioMail.sendEmailReprovacaoEmpresa(dto.getEmpresa());
		} catch (Exception e) {
			// Para registrar no arquivo de log do servidor
			e.printStackTrace();
			FacesContext.getCurrentInstance().addMessage("reprovacaoPj", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de reprova&ccedil;&atilde;o de cadastro Ocorreu um erro ao enviar e-mail de notifica&ccedil;&atilde;o ao proponente.", null));
			return ERROR;
		}
		FacesContext.getCurrentInstance().addMessage("reprovacaoPj", new FacesMessage(FacesMessage.SEVERITY_INFO,dto.getEmpresa().getSituacaoCadastro().getDescricao().toLowerCase() + " " + ConstantesARQ.COM_SUCESSO, null));		
		return SUCCESS;
		
	}
	
	public String determinarStatusEmExigenciaEmpresa(){
		if(StringUtil.isEmpty(dto.getEmpresa().getDescricaoAnaliseTecnica())){
			FacesContext.getCurrentInstance().addMessage("emExgStatusPj", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
					"Para alterar o status do cadastro &eacute; necess&aacute;rio incluir um coment&aacute;rio!",null
					));
					return ERROR;
		}
		SegUsuario analista = new SegUsuario(getSegBean().getUsuario().getId());

		// Define o analista e a situação como em análise e salva
		dto.getEmpresa().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.EM_ANALISE));
		dto.getEmpresa().setAnalista(analista);

		try {
			EnvioMail.sendEmailMudancaStatusEmExigenciaEmpresa(dto.getEmpresa());
			getUniversalManager().save(dto.getEmpresa());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}

		FacesContext.getCurrentInstance().addMessage("emExgStatusPj", 
				new FacesMessage(FacesMessage.SEVERITY_INFO,
				"alterado para " + dto.getEmpresa().getSituacaoCadastro().getDescricao().toLowerCase(),null));
		return SUCCESS;
	}
	
	public void uploadDeclaracaoVeracidade(FileUploadEvent event) {
		try {
			dto.getDocumentoDeclaracao().setDocumento(new SerialBlob(event.getFile().getContents()));
			dto.getDocumentoDeclaracao().setTipoDocumento(new CadTipoDocumentoD(new Integer(8)));
			dto.getDocumentoDeclaracao().setNomDocumento(event.getFile().getFileName());
			if (dto.getDocumentoDeclaracao().getId() == null
					&& !dto.getEmpresa().getDocumentos().contains(dto.getDocumentoDeclaracao())) {
				dto.getEmpresa().addDocumento(dto.getDocumentoDeclaracao());
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
	}
	
	public StreamedContent getFile() {
		try {
			if (dto.getDocumento() != null && dto.getDocumento().getDocumento()!=null) {
				return new DefaultStreamedContent(dto.getDocumento()
						.getDocumento().getBinaryStream(), null, 
						StringUtil.replaceWhiteSpace(dto.getDocumento().getNomDocumento()));
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return null;
	}

	public void changeEmpresa(AjaxBehaviorEvent event){
		if(!getModel().getEmpresas().isEmpty()){
			dto.setEmpresa(empresaService.buscaEmpresaPorId(dto.getEmpresa().getId()));
			carregarDadosEmpresa(dto.getEmpresa());
		}
	}

	
	public boolean isRenderDadosEmpresa(){
		if(this.getModel().getEmpresas().size() > 1 
				&& dto.getEmpresa().getId() != null 
				&& !dto.getEmpresa().getId().equals(0)){
			return true;
		}else if(this.getModel().getEmpresas().size() == 1){
			return true;
		}
		return false;
	}
	
	public String aprovarProponente(){
		
		// Define a situação como aprovado e salva
		getModel().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.APROVADO));
		getModel().setAnalista(new SegUsuario(getSegBean().getUsuario().getId()));

		try {
			getUniversalManager().save(getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		// Envia e-mail notificando o proponente sobre a reprovação do cadastro
		try {
			EnvioMail.sendEmailAprovacaoProponente(getModel());
		} catch (Exception e) {
			// Para registrar no arquivo de log do servidor
			e.printStackTrace();
			
			FacesContext.getCurrentInstance().addMessage("aprovarPf", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de Aprova&ccedil;&atilde;o de cadastro" + 
									"Ocorreu um erro ao enviar e-mail de notifica&ccedil;&atilde;o ao proponente.", null));
			return ERROR;
		}

		FacesContext.getCurrentInstance().addMessage("aprovarPf", 
				new FacesMessage(FacesMessage.SEVERITY_INFO,
						this
						.getModel().getSituacaoCadastro().getDescricao().toLowerCase() + " " + ConstantesARQ.COM_SUCESSO, null));
		return SUCCESS;
	}
	

	public String reprovarProponente(){

		// Verifica se a declaração de veracidade foi anexada ao cadastro
		if(StringUtil.isEmpty(getModel().getDescricaoAnaliseTecnica())){
			
			FacesContext.getCurrentInstance().addMessage("reprovarPf", 
			new FacesMessage(FacesMessage.SEVERITY_ERROR,
			"Para alterar o status do cadastro &eacute; necess&aacute;rio incluir um coment&aacute;rio!", null));
			return ERROR;
		}

		// Define a situação como reprovado e salva
		getModel().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.REPROVADO));
		getModel().setAnalista(new SegUsuario(getSegBean().getUsuario().getId()));

		try {
			getUniversalManager().save(getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}

		// Envia e-mail notificando o proponente sobre a reprovação do cadastro
		try {
			EnvioMail.sendEmailReprovacaoProponente(getModel());
		} catch (Exception e) {
			// Para registrar no arquivo de log do servidor
			e.printStackTrace();
			
			FacesContext.getCurrentInstance().addMessage("reprovarPf", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de reprova&ccedil;&atilde;o de cadastro. Ocorreu um erro ao enviar e-mail de notifica&ccedil;&atilde;o ao proponente.", null));
			
			return ERROR;
		}

		FacesContext.getCurrentInstance().addMessage("reprovarPf", new FacesMessage(FacesMessage.SEVERITY_INFO,getModel().getSituacaoCadastro().getDescricao().toLowerCase() + " " + ConstantesARQ.COM_SUCESSO, null));
		return SUCCESS;
	}
	

	public String determinarStatusEmExigenciaProponente(){
		SegUsuario analista = new SegUsuario(getSegBean().getUsuario().getId());
			if(StringUtil.isEmpty(getModel().getDescricaoAnaliseTecnica())){
				FacesContext.getCurrentInstance().addMessage("emExgStatusPf", 
				new FacesMessage(FacesMessage.SEVERITY_ERROR,
				"Para alterar o status do cadastro &eacute; necess&aacute;rio incluir um coment&aacute;rio!",null
				));
				return ERROR;
			}
			
		// Define o analista e a situação como em análise e salva
		getModel().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.EM_ANALISE));
		getModel().setAnalista(analista);

		try {
			EnvioMail.sendEmailMudancaStatusEmExigenciaProponente(getModel());
			getUniversalManager().save(getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		
		FacesContext.getCurrentInstance().addMessage("emExgStatusPf", 
			new FacesMessage(FacesMessage.SEVERITY_INFO,
			"alterado para " + getModel().getSituacaoCadastro().getDescricao().toLowerCase(),null));
		return SUCCESS;
	}
	



	public SEGBean getSegBean() {
		return segBean;
	}

	public void setSegBean(SEGBean segBean) {
		this.segBean = segBean;
	}

	public GenericService getGenericService() {
		return genericService;
	}

	public void setGenericService(GenericService genericService) {
		this.genericService = genericService;
	}

	public String getComentarioAnaliseTecnica() {
		return comentarioAnaliseTecnica;
	}

	public void setComentarioAnaliseTecnica(String comentarioAnaliseTecnica) {
		this.comentarioAnaliseTecnica = comentarioAnaliseTecnica;
	}

	public CadProponente getProponente() {
		return proponente;
	}

	public void setProponente(CadProponente proponente) {
		this.proponente = proponente;
	}

	public ProponenteService getProponenteService() {
		return proponenteService;
	}

	public void setProponenteService(ProponenteService proponenteService) {
		this.proponenteService = proponenteService;
	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}


	public boolean isRenderTabEmpresa() {
		return renderTabEmpresa;
	}

	public void setRenderTabEmpresa(boolean renderTabEmpresa) {
		this.renderTabEmpresa = renderTabEmpresa;
	}

	public String getPROPONENTE() {
		return PROPONENTE;
	}

	public AnaliseCadastroPessoaJuridicaDto getDto() {
		return dto;
	}

	public void setDto(AnaliseCadastroPessoaJuridicaDto dto) {
		this.dto = dto;
	}


}
