package br.gov.mc.cadsei.bean.interno;

import javax.annotation.PostConstruct;
import javax.faces.application.FacesMessage;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.primefaces.model.DefaultStreamedContent;
import org.primefaces.model.StreamedContent;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.bean.SEGBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.mc.cadsei.entidade.CadDocumento;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.CadTipoDocumentoD;
import br.gov.mc.cadsei.entidade.dto.AnaliseCadastroProponenteDto;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.cadsei.service.ProponenteService;
import br.gov.mc.cadsei.util.CriptoUtil;
import br.gov.mc.cadsei.util.EnvioMail;
import br.gov.mc.cadsei.util.StringUtil;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpEndereco;
import br.gov.mc.corporativo.entidade.CrpTipoDocPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoEndereco;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;
import br.gov.mc.seg.entidade.SegUsuario;

@ManagedBean(name = "analisarCadastroProponenteBean")
@ViewScoped
public class AnalisarCadastroProponenteBean extends BaseBean<CadProponente>{

	private static final long serialVersionUID = 1L;
	
	private AnaliseCadastroProponenteDto dto;
	
	@ManagedProperty(value = "#{proponenteService}")
	private transient ProponenteService proponenteService;
	
	@ManagedProperty(value = "#{empresaService}")
	private transient EmpresaService empresaService;
	
	private CadProponente proponente;
	
	private String comentarioAnaliseTecnica;
	private boolean renderTabEmpresa;

	@ManagedProperty(value = "#{SEGBean}")
	private transient SEGBean segBean;
	
	private final String PROPONENTE = "idProponente";
	
	@ManagedProperty(value = "#{genericService}")
	private transient GenericService genericService;
	
	private String novaSenha = "";
	private String confirmarSenha = "";
	
	@Override
	public CadProponente createModel() {
		return new CadProponente();
	}
	public String voltar() {
		return this.redirect("listar-proponente-empresa.jsf");
	}
	
	public String voltarVisualizar() {
		return this.redirect("listar-proponente-empresa-consulta.jsf");
	}

	@Override
	public String getQualifiedName() {
		return "Cadastro";
	}

	@Override
	public boolean isFeminino() {
		return false;
	}
	
	public String analisar() {
		return this.redirect("analisar-cadastro.jsf");
	}
	
	
	@PostConstruct
	public void init(){
		carregarTela(); 
		setProponente(new CadProponente());
		
		Integer id = (Integer) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(PROPONENTE);
		setModel(proponenteService.buscaProponentePorId(id));
		dto.setDescricaoAnaliseTecnicaProponente(getModel().getDescricaoAnaliseTecnica());

		
		for (CadDocumento documento : getModel().getDocumentos()) {
			if(CadTipoDocumentoD.RG.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoRg(documento);
			}else if(CadTipoDocumentoD.CPF.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoCpf(documento);
			}else if(CadTipoDocumentoD.RESIDENCIA.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoResidencia(documento);
			}else if(CadTipoDocumentoD.REGISTRO_OAB.equals(documento.getTipoDocumento().getId())){
				dto.setDocumentoOab(documento);
			}
		}

		// Recupera os documentos do proponente
		for(CrpDocumentoPessoaFisica documento : getModel().getPessoaProponente().getDocumentos()){
			if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.CPF)) {
				dto.setCpf(documento);
			} else if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG)) {
				dto.setRg(documento);
			}
		}

		// Recupera os telefones do proponente
		for(CrpContatoTelefone telefone : getModel().getPessoaProponente().getTelefones()){
			if(telefone.getTipoTelefone().getId().equals(CrpTipoTelefone.CONTATO)) {
				dto.setTelefone(telefone);
			} else if(telefone.getTipoTelefone().getId().equals(CrpTipoTelefone.CELULAR)) {
				dto.setCelular(telefone);
			}
		}

		// Recupera o endereço do proponente
		for(CrpEndereco endereco : getModel().getPessoaProponente().getEnderecos()){
			if(CrpTipoEndereco.CORRESPONDENCIA.equals(endereco.getTipoEndereco().getId())){
				dto.setEndereco(endereco);
			}
		}

		}


	private void carregarTela() {
		dto = new AnaliseCadastroProponenteDto();
		dto.setCelular( new CrpContatoTelefone());
		dto.setCpf(new CrpDocumentoPessoaFisica());
		dto.setDocumento(new CadDocumento());
		dto.setDocumentoCpf(new CadDocumento());
		dto.setDocumentoOab(new CadDocumento());
		dto.setDocumentoResidencia(new CadDocumento());
		dto.setDocumentoRg(new CadDocumento());
		dto.setEndereco(new CrpEndereco());
		dto.setRg(new CrpDocumentoPessoaFisica());
		dto.setTelefone(new CrpContatoTelefone());

	}

		
	
	public StreamedContent getFile() {
		try {
			if (dto.getDocumento() != null) {
				return new DefaultStreamedContent(dto.getDocumento()
						.getDocumento().getBinaryStream(), null, 
						StringUtil.replaceWhiteSpace(dto.getDocumento().getNomDocumento()));
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return null;
	}

	
	public String alterarSenhaProponente(){
		try {
			if(this.novaSenha.isEmpty() && this.confirmarSenha.isEmpty()){
				FacesContext.getCurrentInstance().addMessage("atualiza_senha",
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "", "Favor informar a senha para seja recadastrada!"));
				return null;
			}

			if(!this.novaSenha.isEmpty() && this.confirmarSenha.isEmpty()){
				FacesContext.getCurrentInstance().addMessage("atualiza_senha",
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "", "Campo Confirmar Nova Senha n&atilde;o foi informado!"));
				return null;
			}
			if(this.novaSenha.isEmpty() && !this.confirmarSenha.isEmpty()){
				FacesContext.getCurrentInstance().addMessage("atualiza_senha",
						new FacesMessage(FacesMessage.SEVERITY_ERROR, "", "Campo Nova Senha n&atilde;o foi informado!"));
				return null;
			}
			
			if(!this.novaSenha.isEmpty() && !this.confirmarSenha.isEmpty()) {
				if(!this.novaSenha.equals(this.confirmarSenha)){
					FacesContext.getCurrentInstance().addMessage("atualiza_senha", new FacesMessage(FacesMessage.SEVERITY_ERROR,"",
							"Novas senhas n&atilde;o conferem!"));
					return null;
				}
				this.getModel().setSenhaProponente(CriptoUtil.getCriptografia(this.novaSenha));
				this.getGenericService().save(this.getModel());
				FacesContext.getCurrentInstance().addMessage("atualiza_senha", new FacesMessage(FacesMessage.SEVERITY_INFO,"",
						"Novas senhas recadastrada com sucesso!"));

			}
			
		} catch (Exception e) {
			e.printStackTrace();
			FacesContext.getCurrentInstance().addMessage("messages_senha", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de alteração de senha!" + 
									"Ocorreu um erro ao alterar a senha da pessoa física.", null));
			return ERROR;
		}
		return SUCCESS;
	}
	
	public String aprovarProponente(){
		
		// Define a situação como aprovado e salva
		getModel().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.APROVADO));
		getModel().setAnalista(new SegUsuario(getSegBean().getUsuario().getId()));

		try {
			getUniversalManager().save(getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		// Envia e-mail notificando o proponente sobre a reprovação do cadastro
		try {
			EnvioMail.sendEmailAprovacaoProponente(getModel());
		} catch (Exception e) {
			// Para registrar no arquivo de log do servidor
			e.printStackTrace();
			
			FacesContext.getCurrentInstance().addMessage("aprovarPf", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de Aprova&ccedil;&atilde;o de cadastro" + 
									"Ocorreu um erro ao enviar e-mail de notifica&ccedil;&atilde;o ao proponente.", null));
			return ERROR;
		}

		FacesContext.getCurrentInstance().addMessage("aprovarPf", 
				new FacesMessage(FacesMessage.SEVERITY_INFO,
						this
						.getModel().getSituacaoCadastro().getDescricao().toLowerCase() + " " + ConstantesARQ.COM_SUCESSO, null));
		return SUCCESS;
	}
	

	public String reprovarProponente(){

		// Verifica se a declaração de veracidade foi anexada ao cadastro
		if(StringUtil.isEmpty(getModel().getDescricaoAnaliseTecnica())){
			
			FacesContext.getCurrentInstance().addMessage("reprovarPf", 
			new FacesMessage(FacesMessage.SEVERITY_ERROR,
			"Para alterar o status do cadastro &eacute; necess&aacute;rio incluir um coment&aacute;rio!", null));
			return ERROR;
		}

		// Define a situação como reprovado e salva
		getModel().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.REPROVADO));
		getModel().setAnalista(new SegUsuario(getSegBean().getUsuario().getId()));

		try {
			getUniversalManager().save(getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}

		// Envia e-mail notificando o proponente sobre a reprovação do cadastro
		try {
			EnvioMail.sendEmailReprovacaoProponente(getModel());
		} catch (Exception e) {
			// Para registrar no arquivo de log do servidor
			e.printStackTrace();
			
			FacesContext.getCurrentInstance().addMessage("reprovarPf", 
					new FacesMessage(FacesMessage.SEVERITY_ERROR,
							"Notifica&ccedil;&atilde;o de reprova&ccedil;&atilde;o de cadastro. Ocorreu um erro ao enviar e-mail de notifica&ccedil;&atilde;o ao proponente.", null));
			
			return ERROR;
		}

		FacesContext.getCurrentInstance().addMessage("reprovarPf", new FacesMessage(FacesMessage.SEVERITY_INFO,getModel().getSituacaoCadastro().getDescricao().toLowerCase() + " " + ConstantesARQ.COM_SUCESSO, null));
		return SUCCESS;
	}
	

	public String determinarStatusEmExigenciaProponente(){
		SegUsuario analista = new SegUsuario(getSegBean().getUsuario().getId());
			if(StringUtil.isEmpty(getModel().getDescricaoAnaliseTecnica())){
				FacesContext.getCurrentInstance().addMessage("emExgStatusPf", 
				new FacesMessage(FacesMessage.SEVERITY_ERROR,
				"Para alterar o status do cadastro &eacute; necess&aacute;rio incluir um coment&aacute;rio!",null
				));
				return ERROR;
			}
			
		// Define o analista e a situação como em análise e salva
		getModel().setSituacaoCadastro((CadSituacaoCadastroD) getUniversalManager().get(CadSituacaoCadastroD.class, CadSituacaoCadastroD.AGUARDANDO_ANALISE));
		getModel().setAnalista(analista);

		try {
			EnvioMail.sendEmailMudancaStatusEmExigenciaProponente(getModel());
			getUniversalManager().save(getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		
		FacesContext.getCurrentInstance().addMessage("emExgStatusPf", 
			new FacesMessage(FacesMessage.SEVERITY_INFO,
			"alterado para " + getModel().getSituacaoCadastro().getDescricao().toLowerCase(),null));
		return SUCCESS;
	}
	



	public SEGBean getSegBean() {
		return segBean;
	}

	public void setSegBean(SEGBean segBean) {
		this.segBean = segBean;
	}

	public GenericService getGenericService() {
		return genericService;
	}

	public void setGenericService(GenericService genericService) {
		this.genericService = genericService;
	}

	public String getComentarioAnaliseTecnica() {
		return comentarioAnaliseTecnica;
	}

	public void setComentarioAnaliseTecnica(String comentarioAnaliseTecnica) {
		this.comentarioAnaliseTecnica = comentarioAnaliseTecnica;
	}

	public CadProponente getProponente() {
		return proponente;
	}

	public void setProponente(CadProponente proponente) {
		this.proponente = proponente;
	}

	public ProponenteService getProponenteService() {
		return proponenteService;
	}

	public void setProponenteService(ProponenteService proponenteService) {
		this.proponenteService = proponenteService;
	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}

	public AnaliseCadastroProponenteDto getDto() {
		return dto;
	}

	public void setDto(AnaliseCadastroProponenteDto dto) {
		this.dto = dto;
	}

	public boolean isRenderTabEmpresa() {
		return renderTabEmpresa;
	}

	public void setRenderTabEmpresa(boolean renderTabEmpresa) {
		this.renderTabEmpresa = renderTabEmpresa;
	}

	public String getPROPONENTE() {
		return PROPONENTE;
	}
	/**
	 * @return the novaSenha
	 */
	public String getNovaSenha() {
		return novaSenha;
	}
	/**
	 * @param novaSenha the novaSenha to set
	 */
	public void setNovaSenha(String novaSenha) {
		this.novaSenha = novaSenha;
	}
	/**
	 * @return the confirmarSenha
	 */
	public String getConfirmarSenha() {
		return confirmarSenha;
	}
	/**
	 * @param confirmarSenha the confirmarSenha to set
	 */
	public void setConfirmarSenha(String confirmarSenha) {
		this.confirmarSenha = confirmarSenha;
	}


}
