package br.gov.mc.cadsei.bean.interno;

import java.sql.SQLException;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.primefaces.context.RequestContext;
import org.primefaces.model.DefaultStreamedContent;
import org.primefaces.model.StreamedContent;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.bean.SEGBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.mc.cadsei.entidade.CadDocumentoPeticao;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadPeticao;
import br.gov.mc.cadsei.entidade.CadSituacaoPeticaoD;
import br.gov.mc.cadsei.util.StringUtil;
import br.gov.mc.seg.entidade.SegUsuario;

@ManagedBean(name = "analisarPeticaoBean")
@ViewScoped
public class AnalisarPeticaoBean extends BaseBean<CadPeticao> {

	private static final long serialVersionUID = 1L;
	
	private final String PETICAO = "peticao";
	
	private CadDocumentoPeticao documento;
	
	@ManagedProperty(value = "#{SEGBean}")
	private transient SEGBean segBean;
	
	@Override
	public CadPeticao createModel() {
		return new CadPeticao();
	}

	@Override
	public String getQualifiedName() {
		return "Peti&ccedil;&atilde;o";
	}
	
	@Override
	public boolean isFeminino() {
		return true;
	}
	
	@PostConstruct
	public void init(){
		Integer idPeticao = (Integer) FacesContext.getCurrentInstance().getExternalContext().getSessionMap().get(PETICAO);
		if(idPeticao != null){
			this.setModel((CadPeticao) this.getUniversalManager().get(CadPeticao.class, idPeticao));
			
		}
	}
	
	public String aceitar(){
		if(this.preSave(CadSituacaoPeticaoD.ACEITO)){
			FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), "aceita " + ConstantesARQ.COM_SUCESSO);
			return SUCCESS;		
		}
		return ERROR;
	}
	
	public String recusar(){
		if(this.preSave(CadSituacaoPeticaoD.NAO_ACEITO)){
			FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), "recusada " + ConstantesARQ.COM_SUCESSO);
			return SUCCESS;		
		}
		return ERROR;
	}

	public void limparPendente(){
		this.getModel().setPendencia(null);
		this.getModel().setProvidencia(null);
	}
	
	public String pendente(){
		
		if(StringUtil.isEmpty(this.getModel().getPendencia()) 
				|| StringUtil.isEmpty(this.getModel().getProvidencia())){
			FacesMessagesUtil.addErrorMessage(null, "Campos obrigat&oacute;rios!");
			RequestContext.getCurrentInstance().addCallbackParam("notValid", true);
			RequestContext.getCurrentInstance().addPartialUpdateTarget("pg-enter-pendencia");
			return ERROR;
		}
		
		if(this.preSave(CadSituacaoPeticaoD.PENDENTE)){
			FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), "pendente!");
			RequestContext.getCurrentInstance().addCallbackParam("notValid", false);
			RequestContext.getCurrentInstance().addPartialUpdateTarget("pg-info-pendente");
			RequestContext.getCurrentInstance().addPartialUpdateTarget("pg-buttons");
			RequestContext.getCurrentInstance().addPartialUpdateTarget("msgsPeticao");
			return SUCCESS;		
		}
		return ERROR;

	}
	
	private boolean preSave(Integer situacao){
		
		this.getModel().setSituacao(
				(CadSituacaoPeticaoD) this.getUniversalManager().get(
						CadSituacaoPeticaoD.class, situacao));
		this.getModel().setAnalista(new SegUsuario(this.getSegBean().getUsuario().getId()));
		
		if(this.getModel().getEmpresa().getId() != null){
			this.getModel().setEmpresa(new CadEmpresa(this.getModel().getEmpresa().getId()));
		}else{
			this.getModel().setEmpresa(null);
		}
		
		if(!situacao.equals(CadSituacaoPeticaoD.PENDENTE)){
			this.limparPendente();
		}
		
		// Salva a petição
		try {
			this.getUniversalManager().save(this.getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return false;
		}
		
		return true;
		
	}
	
	public StreamedContent getFile() {
		try {
			if (this.getDocumento() != null) {
				return new DefaultStreamedContent(this.getDocumento()
						.getArquivo().getBinaryStream(), null,
						StringUtil.replaceWhiteSpace(this.getDocumento().getNome()));
			}
		} catch (SQLException e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return null;
	}

	public CadDocumentoPeticao getDocumento() {
		if(this.documento == null){
			this.documento = new CadDocumentoPeticao();
		}
		return this.documento;
	}

	public void setDocumento(CadDocumentoPeticao documento) {
		this.documento = documento;
	}

	public boolean isSituacaoAceitarRecusarLiberada() {
		return CadSituacaoPeticaoD.AGUARDANDO_ANALISE.equals(this.getModel().getSituacao().getId()) 
				|| CadSituacaoPeticaoD.PENDENTE.equals(this.getModel().getSituacao().getId());
	}
	
	public boolean isSituacaoPendenteLiberada() {
		if(CadSituacaoPeticaoD.AGUARDANDO_ANALISE.equals(this.getModel().getSituacao().getId())){
			return true;
		}else if(CadSituacaoPeticaoD.PENDENTE.equals(this.getModel().getSituacao().getId())){
			return false;
		}else{
			return false;
		}
	}

	public SEGBean getSegBean() {
		return segBean;
	}

	public void setSegBean(SEGBean segBean) {
		this.segBean = segBean;
	}

}