package br.gov.mc.cadsei.bean.interno;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;

import org.primefaces.model.LazyDataModel;

import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.mc.cadsei.entidade.dto.FiltroEmailDTO;
import br.gov.mc.cadsei.service.EmailService;

@ManagedBean(name="listarEmailBean")
@ViewScoped
public class ListarEmailBean implements Serializable{
	

	private static final long serialVersionUID = 1L;
	private FiltroEmailDTO filtroPesquisa;
	
	@ManagedProperty(value = "#{emailService}")
	private transient EmailService emailService;
	
	private List<String> autoCompleteListNomePessoaFisica;
	private List<String> autoCompleteListNomePessoaJuridica;
	
	
	protected LazyDataModel<FiltroEmailDTO> listaEmailsPessoaJuridica;
	protected LazyDataModel<FiltroEmailDTO> listaEmailsPessoaFisica;
	@SuppressWarnings("rawtypes")
	private Map filters;
	
	
	@PostConstruct
	public void init() {
		filtroPesquisa= new FiltroEmailDTO();
		filters = new HashMap<String, String>();
		filtroPesquisa.setTipoConsulta("pessoaFisica");
	}
	
	public void pesquisar(){
		filters.clear();
		if(this.getFiltroPesquisa().isListaPessoaFisica()){
			preencheFiltersPesquisaPF();
		}
		if(this.getFiltroPesquisa().getTipoConsulta().equals("pessoaJuridica")){
			preencheFiltersPesquisaPj();
		}
		limparCampos();

	}

	private void limparCampos() {
		filtroPesquisa.setCnpjFiltroPesquisa("");
		filtroPesquisa.setCpfFiltroPesquisa("");
		filtroPesquisa.setNomeFiltroPesquisa("");
		filtroPesquisa.setRazaoSocialFiltroPesquisa("");

	}

@SuppressWarnings("unchecked")
private void preencheFiltersPesquisaPF() {
	if(this.getFiltroPesquisa().getNomeFiltroPesquisa()!=null && !getFiltroPesquisa().getNomeFiltroPesquisa().equals("")){
		this.filters.put("pp.nome", getFiltroPesquisa().getNomeFiltroPesquisa());
	}
	if(this.getFiltroPesquisa().getCpfFiltroPesquisa()!=null && !getFiltroPesquisa().getCpfFiltroPesquisa().equals("")){
		this.filters.put("p.loginProponente", getFiltroPesquisa().getCpfSemMascara());

	}
	setListaEmailsPessoaFisica(null);
}
	@SuppressWarnings("unchecked")
	private void preencheFiltersPesquisaPj() {
		if(this.getFiltroPesquisa().getRazaoSocialFiltroPesquisa()!=null && !getFiltroPesquisa().getRazaoSocialFiltroPesquisa().equals("")){
			this.filters.put("e.razaoSocial", getFiltroPesquisa().getRazaoSocialFiltroPesquisa());
		}
		if(this.getFiltroPesquisa().getCnpjFiltroPesquisa()!=null && !getFiltroPesquisa().getCnpjFiltroPesquisa().equals("")){
			this.filters.put("pt.cnpj", getFiltroPesquisa().getCnpjSemMascara());

		}
		setListaEmailsPessoaJuridica(null);
	}
	

	public List<String> autoCompleteNomePessoaFisica(String nome) {
		setAutoCompleteListNomePessoaFisica(emailService.buscaPessoaFisicaPorNome(nome));
		return autoCompleteListNomePessoaFisica;
	}
	
	public List<String> autoCompleteNomePessoaJuridica(String nome) {
		setAutoCompleteListNomePessoaJuridica(emailService.buscaPessoaJuridicasPorNome(nome));
		return autoCompleteListNomePessoaJuridica;
	}
	
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public LazyDataModel<FiltroEmailDTO> getListaEmailsPessoaFisica() {
		try {
			if (listaEmailsPessoaFisica == null)
				listaEmailsPessoaFisica = new LazyDataModel() {

					private static final long serialVersionUID = 2595093445602418759L;

					@Override
					public List<FiltroEmailDTO> load(int first, int pageSize, String sortField, boolean sortOrder, Map filters) {
						try {
							if (getFilters() != null && !getFilters().isEmpty()) {
								filters.putAll(getFilters());
							}
							// Reduzindo o contador depois de filtrado
							int rowCount = emailService.totalRegistrosPessoaFisica(filters).intValue();
							listaEmailsPessoaFisica.setRowCount(rowCount);
							List<FiltroEmailDTO> lista = emailService.listaEmailPessoaFisica(first, pageSize, sortField, sortOrder, getFilters());
							return lista;

						} catch (Exception e) {
							ExcecaoUtil.tratarExcecao(e);
						}
						return null;
					}

				};

			if (listaEmailsPessoaFisica.getRowCount() == 0) {
				if (getFilters() == null) {
					filters = new HashMap<String, String>();
					listaEmailsPessoaFisica.setRowCount(emailService.totalRegistrosPessoaFisica(getFilters()).intValue());
				} else {
					listaEmailsPessoaFisica.setRowCount(emailService.totalRegistrosPessoaFisica(filters).intValue());
				}
				listaEmailsPessoaFisica.setPageSize(listaEmailsPessoaFisica.getPageSize() != 0 ? listaEmailsPessoaFisica.getPageSize() : -1);
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return listaEmailsPessoaFisica;
	}
	
	
	@SuppressWarnings({ "unchecked", "rawtypes" })
	public LazyDataModel<FiltroEmailDTO> getListaEmailsPessoaJuridica() {
		try {
			if (listaEmailsPessoaJuridica == null)
				listaEmailsPessoaJuridica = new LazyDataModel() {

					private static final long serialVersionUID = 2595093445602418759L;

					@Override
					public List<FiltroEmailDTO> load(int first, int pageSize, String sortField, boolean sortOrder, Map filters) {
						try {
							if (getFilters() != null && !getFilters().isEmpty()) {
								filters.putAll(getFilters());
							}
							// Reduzindo o contador depois de filtrado
							int rowCount = emailService.totalRegistrosPessoaJuridica(filters).intValue();
							listaEmailsPessoaJuridica.setRowCount(rowCount);
							List<FiltroEmailDTO> lista = emailService.listaEmailPessoaJuridica(first, pageSize, sortField, sortOrder, getFilters());
							return lista;

						} catch (Exception e) {
							ExcecaoUtil.tratarExcecao(e);
						}
						return null;
					}

				};

			if (listaEmailsPessoaJuridica.getRowCount() == 0) {
				if (getFilters() == null) {
					filters = new HashMap<String, String>();
					listaEmailsPessoaJuridica.setRowCount(emailService.totalRegistrosPessoaJuridica(getFilters()).intValue());
				} else {
					listaEmailsPessoaJuridica.setRowCount(emailService.totalRegistrosPessoaJuridica(filters).intValue());
				}
				listaEmailsPessoaJuridica.setPageSize(listaEmailsPessoaJuridica.getPageSize() != 0 ? listaEmailsPessoaJuridica.getPageSize() : -1);
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return listaEmailsPessoaJuridica;
	}



	public EmailService getEmailService() {
		return emailService;
	}

	public void setEmailService(EmailService emailService) {
		this.emailService = emailService;
	}

	public List<String> getAutoCompleteListNomePessoaFisica() {
		return autoCompleteListNomePessoaFisica;
	}

	public void setAutoCompleteListNomePessoaFisica(List<String> autoCompleteListNomePessoaFisica) {
		this.autoCompleteListNomePessoaFisica = autoCompleteListNomePessoaFisica;
	}

	public List<String> getAutoCompleteListNomePessoaJuridica() {
		return autoCompleteListNomePessoaJuridica;
	}

	public void setAutoCompleteListNomePessoaJuridica(List<String> autoCompleteListNomePessoaJuridica) {
		this.autoCompleteListNomePessoaJuridica = autoCompleteListNomePessoaJuridica;
	}



	@SuppressWarnings("rawtypes")
	public Map getFilters() {
		if (this.filters == null) {
			this.filters = new HashMap<String, String>();
		}
		return filters;
	}

	public FiltroEmailDTO getFiltroPesquisa() {
		return filtroPesquisa;
	}


	public void setFiltroPesquisa(FiltroEmailDTO filtroPesquisa) {
		this.filtroPesquisa = filtroPesquisa;
	}

	public void setListaEmailsPessoaJuridica(LazyDataModel<FiltroEmailDTO> listaEmailsPessoaJuridica) {
		this.listaEmailsPessoaJuridica = listaEmailsPessoaJuridica;
	}

	public void setListaEmailsPessoaFisica(LazyDataModel<FiltroEmailDTO> listaEmailsPessoaFisica) {
		this.listaEmailsPessoaFisica = listaEmailsPessoaFisica;
	}


}
