package br.gov.mc.cadsei.bean.interno;

import java.text.Collator;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.SessionScoped;

import org.primefaces.model.LazyDataModel;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.dto.FiltroPesquisaProponenteDto;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.cadsei.util.CompareCrpUf;
import br.gov.mc.corporativo.entidade.CrpUf;

@ManagedBean(name = "listarEmpresaBean")
@SessionScoped
public class ListarEmpresaBean extends BaseBean<CadEmpresa> {
	/**
     * 
     */
	private static final long serialVersionUID = 1L;
	private CadEmpresa empresa;
	private List<CadSituacaoCadastroD> situacoes;
	private List<CadServico> servicos;
	private List<String> autoCompleteListRazaoSocialEmpresa;
	private FiltroPesquisaProponenteDto filtroPesquisa;
	protected LazyDataModel<CadEmpresa> lazyDataModel;
	@SuppressWarnings("rawtypes")
	private Map filters;
	private List<CrpUf> ufs;
	
	@ManagedProperty(value = "#{empresaService}")
	private transient EmpresaService empresaService;

	@SuppressWarnings("rawtypes")
	public Map l;

	@SuppressWarnings("unchecked")
	@PostConstruct
	public void init() {
		filtroPesquisa = new FiltroPesquisaProponenteDto();
		filtroPesquisa.setSituacao(new CadSituacaoCadastroD());
		filtroPesquisa.setServico(new CadServico());
		filtroPesquisa.setUf(new CrpUf());
		filters = new HashMap<String, String>();
		setServicos(this.getUniversalManager().getAll(CadServico.class));
		setSituacoes(this.situacoes = this.getUniversalManager().getAll(CadSituacaoCadastroD.class));
		carregaUfs();
		Collections.sort(this.ufs, new CompareCrpUf());
	}

	public List<String> autoCompleteRazaoSocialEmpresa(String nome) {
		setAutoCompleteListRazaoSocialEmpresa(empresaService.buscaEmpresaPorRazaoSocial(nome));
		return autoCompleteListRazaoSocialEmpresa;
	}

	@SuppressWarnings("unchecked")
	private void carregaUfs() {
		if (ufs == null || ufs.isEmpty()) {
			setUfs(this.getUniversalManager().getAll(CrpUf.class));
		}
	}

	public void limparcampos() {
		this.getFiltroPesquisa().setCnpj("");
		this.getFiltroPesquisa().setRazaoSocial("");
		//setLazyDataModel(null);
		filters = null;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public LazyDataModel<CadEmpresa> getLazyDataModel() {
		try {
			if (lazyDataModel == null)
				lazyDataModel = new LazyDataModel() {

					private static final long serialVersionUID = 1L;

					@Override
					public List<CadEmpresa> load(int first, int pageSize, String sortField, boolean sortOrder, Map filters) {
						try {
							if (getFilters() != null && !getFilters().isEmpty()) {
								filters.putAll(getFilters());
							}
							int rowCount = empresaService.totalRow(filters).intValue();
							lazyDataModel.setRowCount(rowCount);
							return empresaService.pesquisaParametrizada(first, pageSize, sortField, sortOrder, filters);
						} catch (Exception e) {
							ExcecaoUtil.tratarExcecao(e);
						}
						return null;
					}
				};

			if (lazyDataModel.getRowCount() == 0) {
				if (getFilters() == null) {
					filters = new HashMap<String, String>();
					lazyDataModel.setRowCount(empresaService.totalRow(filters).intValue());
				} else {
					lazyDataModel.setRowCount(empresaService.totalRow(filters).intValue());
				}
				lazyDataModel.setPageSize(lazyDataModel.getPageSize() != 0 ? lazyDataModel.getPageSize() : -1);
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return lazyDataModel;
	}

	public void setEmpresa(CadEmpresa empresa) {
		this.empresa = empresa;
	}

	public List<CadSituacaoCadastroD> getSituacoes() {
		return situacoes;
	}

	public void setSituacoes(List<CadSituacaoCadastroD> situacoes) {
		this.situacoes = situacoes;
	}

	public List<CadServico> getServicos() {
		return servicos;
	}

	public void setServicos(List<CadServico> servicos) {
		this.servicos = servicos;
	}

	public List<CrpUf> getUfs() {
		return ufs;
	}

	public void setUfs(List<CrpUf> ufs) {
		this.ufs = ufs;
	}

	public static long getSerialversionuid() {
		return serialVersionUID;
	}

	public List<String> getAutoCompleteListRazaoSocialEmpresa() {
		return autoCompleteListRazaoSocialEmpresa;
	}

	public void setAutoCompleteListRazaoSocialEmpresa(List<String> autoCompleteListRazaoSocialEmpresa) {
		this.autoCompleteListRazaoSocialEmpresa = autoCompleteListRazaoSocialEmpresa;
	}

	public EmpresaService getEmpresaService() {
		return empresaService;
	}

	public void setEmpresaService(EmpresaService empresaService) {
		this.empresaService = empresaService;
	}

	@Override
	public br.gov.mc.cadsei.entidade.CadEmpresa createModel() {
		return new CadEmpresa();
	}

	@Override
	public String getQualifiedName() {
		return "Empresa";
	}

	@Override
	public boolean isFeminino() {
		return false;
	}

	public FiltroPesquisaProponenteDto getFiltroPesquisa() {
		return filtroPesquisa;
	}

	public void setFiltroPesquisa(FiltroPesquisaProponenteDto filtroPesquisa) {
		this.filtroPesquisa = filtroPesquisa;
	}

	public CadEmpresa getEmpresa() {
		return empresa;
	}



	@SuppressWarnings("unchecked")
	public void pesquisar() {
		if(filters ==  null){
			filters = new HashMap<String, String>();
		}
		filters.clear();
		if (this.getFiltroPesquisa().getRazaoSocial() != null && !this.getFiltroPesquisa().getRazaoSocial().equals("")) {
			this.filters.put("p.razaoSocial", this.getFiltroPesquisa().getRazaoSocial());
		}

		if (this.getFiltroPesquisa().getSituacao() != null && this.getFiltroPesquisa().getSituacao().getId() != null) {
			this.filters.put("sc.id", this.getFiltroPesquisa().getSituacao().getId());
		}
		if (this.getFiltroPesquisa().getServico() != null && this.getFiltroPesquisa().getServico().getId() != null) {
			this.filters.put("s.id", this.getFiltroPesquisa().getServico().getId());
		}
		if (this.getFiltroPesquisa().getUf() != null && this.getFiltroPesquisa().getUf().getId() != null) {
			this.filters.put("u.id", this.getFiltroPesquisa().getUf().getId());
		}

		if (this.getFiltroPesquisa().getCnpj() != null && !this.getFiltroPesquisa().getCnpj().equals("")) {
			this.filters.put("p.cnpj", this.getFiltroPesquisa().getCnpjSemMascara().toString());
		}
		//limparcampos();
	}

	public int sortByNome(Object o1, Object o2) {
		return Collator.getInstance().compare(o1, o2);
	}

	public Map getFilters() {
		if (this.filters == null) {
			this.filters = new HashMap<String, String>();
		}
		return this.filters;
	}

	public void setLazyDataModel(LazyDataModel<CadEmpresa> lazyDataModel) {
		this.lazyDataModel = lazyDataModel;
	}

}
