package br.gov.mc.cadsei.bean.interno;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;

import br.gov.arquitetura.bean.PaginableBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.arquitetura.util.ConstantesARQ;
import br.gov.arquitetura.util.FacesMessagesUtil;
import br.gov.mc.cadsei.entidade.CadProcuracao;
import br.gov.mc.cadsei.entidade.CadStatusProcuracaoD;
import br.gov.mc.cadsei.entidade.dto.ResultadoOutorgadoDto;
import br.gov.mc.cadsei.service.ProcuracaoService;
import br.gov.mc.cadsei.util.Constantes;
import br.gov.mc.cadsei.util.SessionUtil;

@ManagedBean(name = "listarProcuracaoBean")
@ViewScoped
public class ListarProcuracaoBean extends PaginableBean<CadProcuracao> {

	private static final long serialVersionUID = 1L;

	private List<ResultadoOutorgadoDto> listOutorgadas;

	private List<ResultadoOutorgadoDto> listRecebidas;

	private Integer idStatusAtiva;

	private Integer idProcuracao;

	private static final String ACEITO = "aceito";

	@ManagedProperty(value = "#{procuracaoService}")
	private transient ProcuracaoService procuracaoService;

	@Override
	public CadProcuracao createModel() {
		return new CadProcuracao();
	}

	@Override
	public String getQualifiedName() {
		return "Procura&ccedil;&atilde;o";
	}

	@Override
	public boolean isFeminino() {
		return true;
	}

	@Override
	public Map<String, String> getFilters() {
		if (this.filters == null) {
			this.filters = new HashMap<String, String>();
		}
		return this.filters;
	}

	@PostConstruct
	public void init() {
		analisarProcuracaoVencidas();
	}

	private void analisarProcuracaoVencidas() {
		Calendar today = Calendar.getInstance();
		today.set(Calendar.HOUR_OF_DAY, 0);
		today.set(Calendar.MINUTE, 0);
		today.set(Calendar.SECOND, 0);
		today.set(Calendar.MILLISECOND, 0);

		for (CadProcuracao procuracao : this.getProcuracaoService().findAllProcuracaoAtivas()) {
			if (procuracao.getDataValidadeFinal().before(today.getTime())) {
				atualizarStatusProcuracao(procuracao);
			}
		}
	}

	private void atualizarStatusProcuracao(CadProcuracao procuracao) {
		procuracao.setStatusProcuracao(new CadStatusProcuracaoD(CadStatusProcuracaoD.VENCIDA));
		try {
			// Feito este metodo MERGE, pelo fato de não executar UPDATE com o
			// metodo SAVE
			this.getProcuracaoService().merge(procuracao);
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
	}

	public String revogarProcuracao() {
		try {
			this.getProcuracaoService().revogarProcuracao(this.getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		this.setListOutorgadas(null);
		FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), " Revogada " + ConstantesARQ.COM_SUCESSO);
		return SUCCESS;
	}

	@SuppressWarnings("unchecked")
	public String visualizarProcuracao() {
		this.getSessionMap().put("idProcuracao", this.getModel().getId());
		return this.redirect("cadastro-procuracao.jsf");
	}

	public String novaProcuracao() {
		return this.redirect("declaracao-cadastro-procuracao.jsf");
	}

	public String aceitarDeclaracao() {
		this.getSessionMap().put(ACEITO, Boolean.TRUE);
		return this.redirect("cadastro-procuracao.jsf");
	}

	@SuppressWarnings("unchecked")
	public String substabelecer() {
		if (!this.getModel().getFlgPermiteSubstabelecimento()) {
			FacesMessagesUtil.addErrorMessage(Constantes.NAO_PERMITE_SUBSTABELECER);
			return ERROR;
		}
		if (this.getModel().getStatusProcuracao().getId().equals(this.getIdStatusAtiva())) {
			this.getSessionMap().put("idProcuracao", this.getModel().getId());
			return this.redirect("cadastro-substabelecedor.jsf");
		}
		return null;
	}

	public String renunciarProcuracao() {
		this.getModel().setStatusProcuracao(new CadStatusProcuracaoD(CadStatusProcuracaoD.RENUNCIADA));
		try {
			this.getProcuracaoService().update(this.getModel());
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
			return ERROR;
		}
		this.setListRecebidas(null);
		FacesMessagesUtil.addInfoMessage(this.getQualifiedName(), " renunciada " + ConstantesARQ.COM_SUCESSO);
		return SUCCESS;
	}

	public List<ResultadoOutorgadoDto> getListOutorgadas() {
		if (listOutorgadas == null) {
			listOutorgadas = new ArrayList<ResultadoOutorgadoDto>();
			listOutorgadas = this.getProcuracaoService().findProcuracaoEnviadas(SessionUtil.getProponenteLogado().getLoginProponente());
		}
		return listOutorgadas;
	}

	public void setListOutorgadas(List<ResultadoOutorgadoDto> listOutorgadas) {
		this.listOutorgadas = listOutorgadas;
	}

	public ProcuracaoService getProcuracaoService() {
		return procuracaoService;
	}

	public void setProcuracaoService(ProcuracaoService procuracaoService) {
		this.procuracaoService = procuracaoService;
	}

	public List<ResultadoOutorgadoDto> getListRecebidas() {
		if (listRecebidas == null) {
			listRecebidas = new ArrayList<ResultadoOutorgadoDto>();
			listRecebidas = this.getProcuracaoService().findProcuracaoRecebidas(SessionUtil.getProponenteLogado().getLoginProponente(), false);
		}
		return listRecebidas;
	}

	public void setListRecebidas(List<ResultadoOutorgadoDto> listRecebidas) {
		this.listRecebidas = listRecebidas;
	}

	public Integer getIdProcuracao() {
		return idProcuracao;
	}

	public void setIdProcuracao(Integer idProcuracao) {
		this.idProcuracao = idProcuracao;
	}

	public Integer getIdStatusAtiva() {
		return CadStatusProcuracaoD.ATIVA;
	}

	public void setIdStatusAtiva(Integer idStatusAtiva) {
		this.idStatusAtiva = idStatusAtiva;
	}

}