package br.gov.mc.cadsei.bean.interno;

import java.text.Collator;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.ViewScoped;
import javax.faces.context.FacesContext;

import org.primefaces.model.LazyDataModel;

import br.gov.arquitetura.bean.BaseBean;
import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.dto.AnaliseCadastroProponenteDto;
import br.gov.mc.cadsei.entidade.dto.FiltroPesquisaProponenteDto;
import br.gov.mc.cadsei.service.ProponenteService;
import br.gov.mc.cadsei.util.CompareCrpUf;
import br.gov.mc.corporativo.entidade.CrpUf;

@ManagedBean(name = "listarProponenteBean")
@ViewScoped
public class ListarProponenteBean extends BaseBean<CadProponente> {

	private static final long serialVersionUID = 1L;
	private final String PROPONENTE = "idProponente";
	private final String EMPRESA = "idEmpresa";
	private CadProponente proponente;
	private CadEmpresa empresa;
	private List<CadSituacaoCadastroD> situacoes;
	private List<String> autoCompleteListNomeProponente;
	private FiltroPesquisaProponenteDto filtroPesquisa;
	protected LazyDataModel<CadProponente> lazyDataModel;
	private Map filters;
	private List<CrpUf> ufs;
	private AnaliseCadastroProponenteDto documento;

	@ManagedProperty(value = "#{proponenteService}")
	private transient ProponenteService proponenteService;

	@SuppressWarnings("unchecked")
	@PostConstruct
	public void init() {
		filtroPesquisa = new FiltroPesquisaProponenteDto();
		filtroPesquisa.setSituacao(new CadSituacaoCadastroD());
		filtroPesquisa.setServico(new CadServico());
		filtroPesquisa.setUf(new CrpUf());
		filtroPesquisa.setTipoConsulta("pessoaFisica");
		filters = new HashMap<String, String>();
		setProponente(new CadProponente());
		setEmpresa(new CadEmpresa());
		setSituacoes(this.situacoes = this.getUniversalManager().getAll(CadSituacaoCadastroD.class));
		carregaUfs();
		Collections.sort(this.ufs, new CompareCrpUf());
		documento = new AnaliseCadastroProponenteDto();
	}

	public List<String> autoCompleteNomeProponente(String nome) {
		setAutoCompleteListNomeProponente(proponenteService.buscaProponentePorNome(nome));
		return autoCompleteListNomeProponente;
	}

	@SuppressWarnings("unchecked")
	private void carregaUfs() {
		if (ufs == null || ufs.isEmpty()) {
			setUfs(this.getUniversalManager().getAll(CrpUf.class));
		}

	}

	@Override
	public CadProponente createModel() {
		return new CadProponente();
	}

	@Override
	public String getQualifiedName() {
		return "Proponente";
	}

	@Override
	public boolean isFeminino() {
		return false;
	}

	
	public String analisarProponente() {
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(PROPONENTE, this.getProponente().getId());
		return this.redirect("analisar-cadastro-proponente.xhtml");
	}
	
	public String analisarPessoaJuridica() {
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(PROPONENTE, this.getProponente().getId());
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(EMPRESA, this.getEmpresa().getId());
		return this.redirect("analisar-cadastro-pessoajuridica.xhtml");
	}
	
	public String visualizarProponente() {
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(PROPONENTE, this.getProponente().getId());
		return this.redirect("visualizar-cadastro-proponente.xhtml");
	}

	public String visualizarPessoaJuridica() {
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(PROPONENTE, this.getProponente().getId());
		FacesContext.getCurrentInstance().getExternalContext().getSessionMap().put(EMPRESA, this.getEmpresa().getId());
		return this.redirect("visualizar-cadastro-pessoajuridica.jsf");
	}
	
	@SuppressWarnings("unchecked")
	public void pesquisar() {
		filters.clear();
		if (this.getFiltroPesquisa().getSituacao() != null && this.getFiltroPesquisa().getSituacao().getId() != null) {
			this.filters.put("sit.id", this.getFiltroPesquisa().getSituacao().getId());
		}
		if (this.getFiltroPesquisa().getUf() != null && this.getFiltroPesquisa().getUf().getId() != null) {
			this.filters.put("cuf.id", this.getFiltroPesquisa().getUf().getId());
		}
		if (this.getFiltroPesquisa().getCpf() != null && !this.getFiltroPesquisa().getCpf().equals("")) {
			this.filters.put("p.loginProponente", this.getFiltroPesquisa().getCpfSemMascara());
		}
		if (this.getFiltroPesquisa().getNome() != null && !this.getFiltroPesquisa().getNome().equals("")) {
			this.filters.put("pf.nome", this.getFiltroPesquisa().getNome().trim());
		}
		//limparcampos();
	}

	public int sortByNome(Object o1, Object o2) {
		return Collator.getInstance().compare(o1, o2);
	}

	public void limparcampos() {
		this.getFiltroPesquisa().setCpf("");
		this.getFiltroPesquisa().setNome("");
		setLazyDataModel(null);
		filters = null;
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public LazyDataModel<CadProponente> getLazyDataModel() {
		try {
			if (lazyDataModel == null)
				lazyDataModel = new LazyDataModel() {

					private static final long serialVersionUID = 2595093445602418759L;

					@Override
					public List<CadProponente> load(int first, int pageSize, String sortField, boolean sortOrder, Map filters) {
						try {
							if (getFilters() != null && !getFilters().isEmpty()) {
								filters.putAll(getFilters());
							}
							// Reduzindo o contador depois de filtrado
							int rowCount = proponenteService.totalRow(filters).intValue();
							lazyDataModel.setRowCount(rowCount);
							List<CadProponente> lista = proponenteService.pesquisaParametrizada(first, pageSize, sortField, sortOrder, getFilters());
							return lista;

						} catch (Exception e) {
							ExcecaoUtil.tratarExcecao(e);
						}
						return null;
					}

				};

			if (lazyDataModel.getRowCount() == 0) {
				if (getFilters() == null) {
					filters = new HashMap<String, String>();
					lazyDataModel.setRowCount(proponenteService.totalRow(filters).intValue());
				} else {
					lazyDataModel.setRowCount(proponenteService.totalRow(filters).intValue());
				}
				lazyDataModel.setPageSize(lazyDataModel.getPageSize() != 0 ? lazyDataModel.getPageSize() : -1);
			}
		} catch (Exception e) {
			ExcecaoUtil.tratarExcecao(e);
		}
		return lazyDataModel;
	}


	public CadProponente getProponente() {
		return this.proponente;
	}

	public void setProponente(CadProponente proponente) {
		this.proponente = proponente;
	}

	public FiltroPesquisaProponenteDto getFiltroPesquisa() {
		return filtroPesquisa;
	}

	public void setFiltroPesquisa(FiltroPesquisaProponenteDto filtroPesquisa) {
		this.filtroPesquisa = filtroPesquisa;
	}

	public ProponenteService getProponenteService() {
		return proponenteService;
	}

	public void setProponenteService(ProponenteService proponenteService) {
		this.proponenteService = proponenteService;
	}

	public List<CrpUf> getUfs() {
		return ufs;
	}

	public void setUfs(List<CrpUf> ufs) {
		this.ufs = ufs;
	}

	public List<String> getAutoCompleteListNomeProponente() {
		return autoCompleteListNomeProponente;
	}

	public void setAutoCompleteListNomeProponente(List<String> autoCompleteListNomeProponente) {
		this.autoCompleteListNomeProponente = autoCompleteListNomeProponente;
	}

	public List<CadSituacaoCadastroD> getSituacoes() {
		return situacoes;
	}

	public void setSituacoes(List<CadSituacaoCadastroD> situacoes) {
		this.situacoes = situacoes;
	}

	@SuppressWarnings("rawtypes")
	public Map getFilters() {
		if (this.filters == null) {
			this.filters = new HashMap<String, String>();
		}
		return this.filters;
	}

	public void setLazyDataModel(LazyDataModel<CadProponente> lazyDataModel) {
		this.lazyDataModel = lazyDataModel;
	}

	public AnaliseCadastroProponenteDto getDocumento() {
		return documento;
	}

	public void setDocumento(AnaliseCadastroProponenteDto documento) {
		this.documento = documento;
	}

	public CadEmpresa getEmpresa() {
		return empresa;
	}

	public void setEmpresa(CadEmpresa empresa) {
		this.empresa = empresa;
	}

}