package br.gov.mc.cadsei.dao.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.cadsei.dao.EmailDao;
import br.gov.mc.cadsei.entidade.CadProcuracao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.dto.FiltroEmailDTO;
@Repository(value = "emailDao")
public class EmailDaoImpl extends HibernateDaoSupport implements EmailDao {
	
	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}

	@SuppressWarnings("unchecked")
	public List<String> buscaPessoaFisicaPorNome(String nome) {
		org.hibernate.Query q = this.getSession().createQuery("Select e.pessoaProponente.nome from CadProponente e WHERE e.pessoaProponente.nome LIKE ?");
		q.setParameter(0, "%" + nome + "%");
		return q.list();
	}

	@SuppressWarnings("unchecked")
	public List<String> buscaPessoaJuridicasPorNome(String nome) {
		org.hibernate.Query q = this.getSession().createQuery("Select f.razaoSocial from CadEmpresa e INNER JOIN e.pessoaTitular f WHERE f.razaoSocial LIKE ?");
		q.setParameter(0, "%" + nome + "%");
		return q.list();
	}

	@SuppressWarnings("unchecked")
	
	public List<FiltroEmailDTO> buscaEmailPessoaFisica(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filtros) {
		Query query = null;
		StringBuffer q = new StringBuffer();
		q.append(" Select distinct pp.nome, p.loginProponente, p.emailProponente from CadProponente p inner join p.pessoaProponente pp where 1=1 ");
		if(filtros.containsKey("pp.nome")){
			q.append(" and pp.nome Like :nome");
		}
		if(filtros.containsKey("p.loginProponente")){
			q.append(" and p.loginProponente =:cpf");
		}
		
		
		if (sortField != null) {
			if(sortField.equals("nome"))
			{
				sortField = "pp.nome";
				String ordenacao = sortOrder == true ? " ASC " : " DESC ";
				q.append(" ORDER BY ");
				q.append(sortField); 
				q.append(ordenacao);
			}else{
				String ordenacao = sortOrder == true ? " ASC " : " DESC ";
				q.append(" ORDER BY p.");
				q.append(sortField); 
				q.append(ordenacao);
				
			}
			
		}
		query = getSession().createQuery(q.toString());
		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("pp.nome"))
				query.setParameter("nome", "%" + v.getValue() + "%");
			
			if (v != null && v.getKey().equals("p.loginProponente"))
				query.setParameter("cpf", v.getValue());
		}
		
		query.setFirstResult(first);
		query.setMaxResults(pageSize);
		
		List<Object[]> proponente =  query.list();
		
		List<FiltroEmailDTO> lista = new ArrayList<FiltroEmailDTO>();
		FiltroEmailDTO f = new FiltroEmailDTO();
		for (Object[] o : proponente){
			f= new FiltroEmailDTO();
			f.setNome((String) o[0]);
			f.setCpf((String) o[1]);
			f.setEmailPF((String) o[2]);
			lista.add(f);
		}
		return lista;
		
	}

	@SuppressWarnings("unchecked")
	
	public List<FiltroEmailDTO> buscaEmailPessoaJuridica(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filtros) {
		Query query = null;
		StringBuffer q = new StringBuffer();
		q.append(" Select distinct pt.razaoSocial, pt.cnpj, em.descricao, e.emailAdicional1, e.emailAdicional2, e.emailAdicional3, e.emailAdicional4 from CadEmpresa e inner join e.pessoaTitular pt inner join pt.emails em where 1=1");
		if(filtros.containsKey("e.razaoSocial")){
			q.append(" and pt.razaoSocial Like :razaoSocial ");
		}
		if(filtros.containsKey("pt.cnpj")){
			q.append(" and pt.cnpj =:cnpj");
		}
		
		
		if (sortField != null) {
			String ordenacao = sortOrder == true ? " ASC " : " DESC ";
			q.append(" ORDER BY pt.");
			q.append(sortField); 
			q.append(ordenacao);
		}
		
		query = getSession().createQuery(q.toString());
		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("e.razaoSocial"))
				query.setParameter("razaoSocial", "%" + v.getValue() + "%");
			
			if (v != null && v.getKey().equals("pt.cnpj"))
				query.setParameter("cnpj", v.getValue());
		}
		
		query.setFirstResult(first);
		query.setMaxResults(pageSize);
		
		List<Object[]> proponente =  query.list();
		
		List<FiltroEmailDTO> lista = new ArrayList<FiltroEmailDTO>();
		FiltroEmailDTO f = new FiltroEmailDTO();
		for (Object[] o : proponente){
			f= new FiltroEmailDTO();
			f.setRazaoSocial((String) o[0]);
			f.setCnpj((String) o[1]);
			f.setEmailPJ((String) o[2]);
			
			if(!(o[3]).equals(""))
			f.setEmailPJ1((String) o[3]);
			
			if(!(o[4]).equals(""))
			f.setEmailPJ2((String) o[4]);
			
			if(!(o[5]).equals(""))
			f.setEmailPJ3((String) o[5]);
			
			if(!(o[6]).equals(""))
			f.setEmailPJ4((String) o[6]);
			lista.add(f);
		}
		return lista;
		
	}

	
	public Long totalRegistroPessoaFisica(Map<String, String> filtros)  {
		Query query = null;
		StringBuffer q = new StringBuffer();
		q.append(" Select count (distinct p.id) from CadProponente p inner join p.pessoaProponente pp where 1=1");
		if(filtros.containsKey("pp.nome")){
			q.append(" and pp.nome Like :nome");
		}
		if(filtros.containsKey("p.loginProponente")){
			q.append(" and p.loginProponente =:cpf");
		}
		query = getSession().createQuery(q.toString());
		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("pp.nome"))
				query.setParameter("nome", "%" + v.getValue() + "%");
			
			if (v != null && v.getKey().equals("p.loginProponente"))
				query.setParameter("cpf", v.getValue());
		}
		return (Long) query.uniqueResult();
	}
		

	
	public Long totalRegistroPessoaJuridica(Map<String, String> filtros) {
		Query query = null;
		StringBuffer q = new StringBuffer();
		q.append(" Select  count(e.id) from CadEmpresa e inner join e.pessoaTitular pt inner join pt.emails em where 1=1");
		if(filtros.containsKey("pt.razaoSocial")){
			q.append(" and e.razaoSocial Like :razaoSocial");
		}
		if(filtros.containsKey("pt.cnpj")){
			q.append(" and pt.cnpj =:cnpj");
		}
		query = getSession().createQuery(q.toString());
		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("pt.razaoSocial"))
				query.setParameter("razaoSocial", "%" + v.getValue() + "%");
			
			if (v != null && v.getKey().equals("pt.cnpj"))
				query.setParameter("cnpj", v.getValue());
		}
		return (Long) query.uniqueResult();
	}
	

}
