package br.gov.mc.cadsei.dao.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.cadsei.dao.EmpresaDao;
import br.gov.mc.cadsei.entidade.CadDocumento;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;

@Repository(value = "empresaDao")
public class EmpresaDaoImpl extends HibernateDaoSupport implements EmpresaDao {

	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}

	@SuppressWarnings("unchecked")
	public List<String> buscaEmpresaPorRazaoSocial(String nome) {
		org.hibernate.Query q = this.getSession().createQuery("Select f.razaoSocial from CadEmpresa e INNER JOIN e.pessoaTitular f WHERE f.razaoSocial LIKE ?");
		q.setParameter(0, "%" + nome + "%");
		return q.list();
	}

	@SuppressWarnings("unchecked")
	
	public List<CadEmpresa> buscaParametrizada(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filtros) {

		Query query = null;
		StringBuffer q = new StringBuffer();
		q.append(" Select distinct e.id, p.cnpj, p.razaoSocial, e.dataCadastro, sc.id,"
			+ " sc.descricao, ps.nome, pp.loginProponente, pp.dataCadastro, pp.id, "
			+ "pf.nome, dc.numero from CadEmpresa e");
		q.append(" inner join e.proponente pp ");
		q.append(" inner join pp.pessoaProponente ps ");
		q.append(" inner join e.pessoaTitular p ");
		q.append(" left join e.servicos s ");
		q.append(" inner join p.representantesLegais r");
		q.append(" inner join r.pessoaFisica pf");
		q.append(" inner join pf.documentos dc");
		q.append(" inner join e.situacaoCadastro sc ");
		if (filtros.containsKey("u.id")) {
			q.append(" inner join p.enderecos en ");
			q.append(" inner join en.municipio m ");
			q.append(" inner join m.uf u ");
		}
		
		q.append(" WHERE 1=1 ");
		q.append(" and dc.tipoDocumento.id = 1 and r.dataFim is null ");
		if (filtros.containsKey("p.razaoSocial")) {
			q.append(" and p.razaoSocial LIKE :razaosoc");
		}
		if (filtros.containsKey("sc.id")) {
			q.append(" and sc.id =:sitcad");
		}

		if (filtros.containsKey("s.id")) {
			q.append(" and s.id =:servicos");
		}

		if (filtros.containsKey("u.id")) {
			q.append("and u.id =:uf");
		}
		if (filtros.containsKey("p.cnpj")) {
			q.append(" and p.cnpj =:cnpj ");
		}
		if (sortField != null) {
			String ordenacao = sortOrder == true ? " ASC " : " DESC ";
			q.append(" ORDER BY e.");
			q.append(sortField );
			q.append(ordenacao);
		}
		query = getSession().createQuery(q.toString());

		for (Map.Entry<String, String> v : filtros.entrySet()) {
			
			if (v != null && v.getKey().equals("p.razaoSocial"))
			query.setParameter("razaosoc", "%" + v.getValue() + "%");

			if (v != null && v.getKey().equals("sc.id"))
				query.setParameter("sitcad", v.getValue());

			if (v != null && v.getKey().equals("s.id"))
				query.setParameter("servicos", v.getValue());

			if (v != null && v.getKey().equals("u.id"))
				query.setParameter("uf", v.getValue());

			if (v != null && v.getKey().equals("p.cnpj"))
				query.setParameter("cnpj", v.getValue());
		}
		query.setFirstResult(first);
		query.setMaxResults(pageSize);
		
		List<Object[]> empresas =  query.list();
		List<CadEmpresa> lista = new ArrayList<CadEmpresa>();
		CadEmpresa e = new CadEmpresa();
		
		for (Object[] o : empresas){
			e = new CadEmpresa();
			e.setId((Integer) o[0]);
			e.setPessoaTitular(new CrpPessoaJuridica());
			e.getPessoaTitular().setCnpj(((String)o[1]));
			e.getPessoaTitular().setRazaoSocial(((String)o[2]));
			e.setDataCadastro((Date)o[3]);
			e.setSituacaoCadastro(new CadSituacaoCadastroD());
			e.getSituacaoCadastro().setId((Integer) o[4]);
			e.getSituacaoCadastro().setDescricao(( String  ) o[5]);
			e.setProponente(new CadProponente());
			e.getProponente().setPessoaProponente(new CrpPessoaFisica());
			e.getProponente().getPessoaProponente().setNome(( String  ) o[6]);
			e.getProponente().setLoginProponente(( String  ) o[7]);
			e.getProponente().setDataCadastro(( Date) o[8]);
			e.getProponente().setId(( Integer) o[9]);
			e.setNomeRepresentante(( String  ) o[10]);
			e.setCpfRepresentante(( String  ) o[11]);
			lista.add(e);
		}
		return lista;
		
	}

	
	public Long rowCount(Map<String, String> filtros) {

		Query query = null;
		StringBuffer q = new StringBuffer();
		q.append("Select count(distinct e.id) from CadEmpresa e ");
		q.append(" inner join e.proponente pp ");
		q.append(" inner join pp.pessoaProponente ps ");
		q.append(" inner join e.pessoaTitular p ");
		q.append(" left join e.servicos s ");
		q.append(" inner join p.representantesLegais r");
		q.append(" inner join r.pessoaFisica pf");
		q.append(" inner join pf.documentos dc");
		q.append(" inner join e.situacaoCadastro sc ");
		if (filtros.containsKey("u.id")) {
			q.append(" inner join p.enderecos en ");
			q.append(" inner join en.municipio m ");
			q.append(" inner join m.uf u ");
		}
		q.append(" WHERE 1=1 ");
		if (filtros.containsKey("p.razaoSocial")) {
			q.append(" and p.razaoSocial LIKE :razsoc");
		}
		if (filtros.containsKey("sc.id")) {
			q.append(" and sc.id =:sitcad");
		}

		if (filtros.containsKey("s.id")) {
			q.append(" and s.id =:servicos");
		}

		if (filtros.containsKey("u.id")) {
			q.append("and u.id =:uf");
		}
		if (filtros.containsKey("p.cnpj")) {
			q.append(" and p.cnpj =:cnpj ");
		}
		query = getSession().createQuery(q.toString());

		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("p.razaoSocial"))
				query.setParameter("razsoc", "%" + v.getValue() + "%");

			if (v != null && v.getKey().equals("sc.id"))
				query.setParameter("sitcad", v.getValue());

			if (v != null && v.getKey().equals("s.id"))
				query.setParameter("servicos", v.getValue());

			if (v != null && v.getKey().equals("u.id"))
				query.setParameter("uf", v.getValue());

			if (v != null && v.getKey().equals("p.cnpj"))
				query.setParameter("cnpj", v.getValue());
		}
		return (Long) query.uniqueResult();

	}

	@SuppressWarnings("unchecked")
	
	public List<CadDocumento> buscaDocumentosPorEmpresa(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select d from CadEmpresa p inner join p.documentos d where p.id = ?");
		q.setParameter(0, id );
		return q.list();
	}
	
	
	public CrpRepresentantePesJur buscaRepresentantePorEmpresa(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select r from CadEmpresa e inner join e.pessoaTitular pt inner join pt.representantesLegais r inner join r.pessoaFisica pf inner join pf.documentos d where e.id = ? and r.dataFim is null");
		q.setParameter(0, id );
		return (CrpRepresentantePesJur) q.uniqueResult();
	}



	@SuppressWarnings("unchecked")
	
	public List<CadServico> buscaServicosPorEmpresa(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select s from CadEmpresa p inner join p.servicos s where p.id = ?");
		q.setParameter(0, id );
		return q.list();
	}
	
	@SuppressWarnings("unchecked")
	public List<CadServico> buscaServicos(){
		org.hibernate.Query q = this.getSession().createQuery("Select s from CadServico s"); 
		return q.list();
	}

	
	public CadServico buscaServicoPorId(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select s from CadServico s where s.id = ?");
		q.setParameter(0, id );
		return (CadServico) q.uniqueResult();
	}


	
	public CadEmpresa buscaEmpresaPorId(Integer id) {
			org.hibernate.Query q = this.getSession().createQuery("Select d from CadEmpresa d "
				+ "inner join d.proponente p inner join p.pessoaProponente "
				+ "inner join d.documentos where d.id = ?");
			q.setParameter(0, id );
			return (CadEmpresa) q.uniqueResult();
		}

	@SuppressWarnings("unchecked")
	
	public List<CadEmpresa> buscaEmpresaPorProponente(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select d from CadEmpresa d inner join d.pessoaTitular pt inner join d.proponente p where p.id = ?");
		q.setParameter(0, id );
		return q.list();
	}
	
	@SuppressWarnings("unchecked")
	
	public List<CadEmpresa> buscaEmpresaPorRepresentante(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select distinct e from CadEmpresa e inner join e.pessoaTitular pt inner join pt.representantesLegais r inner join r.pessoaFisica pf where r.id = ? and r.dataFim is null");
		q.setParameter(0, id );
		return q.list();
	}
	
	public Boolean validaCadastroEmpresa(String cpf) {
		org.hibernate.Query q = this.getSession().createQuery("Select distinct e from CadEmpresa e inner join e.proponente p inner join p.pessoaProponente pp inner join e.pessoaTitular pt inner join pt.representantesLegais r inner join r.pessoaFisica pf inner join pf.documentos d where d.numero = ? and r.dataFim is null and d.tipoDocumento.id = 1");
		q.setParameter(0, cpf );
		return q.list().size() == 0;
	}
	/**
	 Este método retornará a lista de empresas por representante com base no CPF do proponente (login).*/
	 
	@SuppressWarnings("unchecked")
	public List<CadEmpresa> listaEmpresaPorRepresentante(String cpf) {
		org.hibernate.Query q = this.getSession().createQuery(""
				+ "Select distinct "
				+ "e.id, "
				+ "e.anoConstituicao, "
				+ "pt.razaoSocial, "
				+ "pt.cnpj, "
				+ "pf.nome "
				+ " from CadEmpresa e inner join e.proponente "
				+ " p inner join p.pessoaProponente pp "
				+ " inner join e.pessoaTitular pt "
				+ " inner join pt.representantesLegais r "
				+ " inner join r.pessoaFisica pf "
				+ " inner join pf.documentos d where d.numero = ? "
				+ " and r.dataFim is null and d.tipoDocumento.id = 1");
		q.setParameter(0, cpf );
		
		List<Object[]> empresas = q.list();
		List<CadEmpresa> lista = new ArrayList<CadEmpresa>();
		CadEmpresa e = new CadEmpresa();
		for (Object[] o : empresas) {
			e = new CadEmpresa();
			e.setId((Integer) o[0]);
			e.setAnoConstituicao((Integer) o[1]);
			e.setPessoaTitular(new CrpPessoaJuridica());
			e.getPessoaTitular().setRazaoSocial(((String)o[2]));
			e.getPessoaTitular().setCnpj(((String)o[3]));
			e.setNomeRepresentante(((String)o[4]));
			lista.add(e);
}
		return lista;
		
	}
}