package br.gov.mc.cadsei.dao.impl;

import java.math.BigInteger;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SQLQuery;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.cadsei.dao.GenericDao;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.cadsei.entidade.dto.QuantitativoDto;
import br.gov.mc.cadsei.entidade.dto.QuantitativoUfDto;
import br.gov.mc.corporativo.entidade.CrpTipoEndereco;
import br.gov.mc.seg.entidade.SegUsuario;

@Repository(value = "genericDao")
public class GenericDaoImpl extends HibernateDaoSupport implements GenericDao {

	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}
	
	
	@SuppressWarnings("rawtypes")
	public Criteria createCriteria(Class klass) {
		return super.getSession().createCriteria(klass);
	}

	
	public Object save(Object o) throws Exception {
		getHibernateTemplate().saveOrUpdate(o);
		return o;
	}

	
	
	public boolean isEmAnalise(SegUsuario analista){
		SQLQuery cmd = super.getSession().createSQLQuery("SELECT COUNT(1) FROM CAD_PROPONENTE WHERE ID_ANALISTA = ? AND ID_SITUACAO_CADASTRO = ?");
		cmd.setParameter(0, analista.getId());
		cmd.setParameter(1, CadSituacaoCadastroD.AGUARDANDO_ANALISE);
		return ((Integer) cmd.uniqueResult()).intValue() > 0;
	}
	
	@SuppressWarnings("unchecked")
	public QuantitativoDto getDadosRelatorio(){
		
		// Define o objeto de retorno
		QuantitativoDto quantitativo = new QuantitativoDto();
		
		// Quantidade de Cadastros PF
		SQLQuery q1 = super.getSession().createSQLQuery("SELECT COUNT(1) FROM CAD_PROPONENTE");
		quantitativo.setQuantidadeProponente( new BigInteger(q1.uniqueResult().toString()).intValue());
		
		// Quantidade de Cadastros PF
		SQLQuery q2 = super.getSession().createSQLQuery("SELECT COUNT(1) FROM CAD_EMPRESA");
		quantitativo.setQuantidadeEmpresa(new BigInteger(q1.uniqueResult().toString()).intValue());
		
		// Quantidade de Cadastros PF por Estado
		StringBuilder sql = new StringBuilder();
		sql.append("SELECT uf.sgl_uf ");
		sql.append("     , COUNT(pr.id_proponente) ");
		sql.append("  FROM cad_proponente pr ");
		sql.append(" INNER JOIN corporativo.crp_pessoa_fisica pf ON (pr.id_pessoa         = pf.id_pessoa) ");
		sql.append(" INNER JOIN corporativo.crp_endereco      en ON (pf.id_pessoa          = en.id_pessoa) ");
		sql.append(" INNER JOIN corporativo.crp_municipio     mn ON (en.cod_municipio_ibge = mn.cod_municipio_ibge) ");
		sql.append(" INNER JOIN corporativo.crp_uf            uf ON (mn.cod_uf_ibge        = uf.cod_uf_ibge) ");
		sql.append(" WHERE en.id_tipo_endereco = ? ");
		sql.append(" GROUP BY uf.sgl_uf ");
		SQLQuery q3 = super.getSession().createSQLQuery(sql.toString());
		q3.setParameter(0, CrpTipoEndereco.CORRESPONDENCIA);
		List<Object> lista = q3.list();
		if(lista != null && !lista.isEmpty()){
			for (Object o : lista) {
				QuantitativoUfDto quantitativoUf = new QuantitativoUfDto();
				quantitativoUf.setSiglaUf(((Object[]) o)[0].toString());
				quantitativoUf.setQuantidade(Integer.valueOf(((Object[]) o)[1].toString()));
				quantitativo.addQuantitativoUf(quantitativoUf);
			}
		}
		
		// Quantidade de Cadastros Aguardando análise
		quantitativo.setQuantidadeAguardandoAnalise(this.getQuantidadeCadastroBySituacao(CadSituacaoCadastroD.AGUARDANDO_ANALISE));
		
		// Quantidade de Cadastros Em análise
		quantitativo.setQuantidadeEmAnalise(this.getQuantidadeCadastroBySituacao(CadSituacaoCadastroD.AGUARDANDO_ANALISE));
		
		// Quantidade de Cadastros Em análise
		quantitativo.setQuantidadeAprovado(this.getQuantidadeCadastroBySituacao(CadSituacaoCadastroD.APROVADO));		
		
		// Quantidade de Cadastros Em análise
		quantitativo.setQuantidadeReprovado(this.getQuantidadeCadastroBySituacao(CadSituacaoCadastroD.REPROVADO));
		
		return quantitativo;
		
	}
 
	private int getQuantidadeCadastroBySituacao(Integer idSituacao){
		SQLQuery cmd = super.getSession().createSQLQuery("SELECT COUNT(1) FROM CAD_PROPONENTE WHERE ID_SITUACAO_CADASTRO = ?");
		cmd.setParameter(0, idSituacao);
		return new BigInteger(cmd.uniqueResult().toString()).intValue();
	}

	public String obtemValorParametro(String nome){
		SQLQuery cmd = super.getSession().createSQLQuery("SELECT VALOR_PARAMETRO FROM CAD_PARAMETRO WHERE NOME_PARAMETRO = ?");
		cmd.setParameter(0, nome);
		return cmd.uniqueResult().toString();
	}

	
	
}