package br.gov.mc.cadsei.dao.impl;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Projections;
import org.hibernate.criterion.Restrictions;
import org.hibernate.transform.AliasToBeanResultTransformer;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.cadsei.dao.PeticaoDao;
import br.gov.mc.cadsei.entidade.CadArea;
import br.gov.mc.cadsei.entidade.CadAreaServico;
import br.gov.mc.cadsei.entidade.CadDocumentoPeticao;
import br.gov.mc.cadsei.entidade.CadPeticao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadSeiIntegracao;
import br.gov.mc.cadsei.entidade.CadServicoSolicitacao;
import br.gov.mc.cadsei.entidade.dto.CprdProtocoloDto;
import br.gov.mc.cadsei.util.Constantes;

@Repository(value = "peticaoPrtoDao")
public class PeticaoDaoImpl extends HibernateDaoSupport implements PeticaoDao {

	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}

	
	public CadPeticao save(CadPeticao peticao) throws Exception {
		super.getSession().saveOrUpdate(peticao);
		return peticao;
	}

	
	public CadPeticao getPeticaoByProponente(CadProponente proponente) {
		return (CadPeticao) super.getSession().createCriteria(CadPeticao.class).add(Restrictions.eq("proponente", proponente)).uniqueResult();
	}

	@SuppressWarnings("unchecked")
	
	public List<CadDocumentoPeticao> getDocumentosPeticao(CadPeticao peticao) {
		return super
				.getSession()
				.createCriteria(CadDocumentoPeticao.class, "d")
				.add(Restrictions.eq("peticao", peticao))
				.setProjection(
						Projections.projectionList().add(Projections.property("d.id").as("id")).add(Projections.property("d.peticao").as("peticao"))
								.add(Projections.property("d.nome").as("nome")))
				.setResultTransformer(new AliasToBeanResultTransformer(CadDocumentoPeticao.class)).list();
	}

	
	public CprdProtocoloDto getProtocolo(String numero) {

		// Monta consulta de protocolo
		StringBuilder cmd = new StringBuilder();
		cmd.append("SELECT pr.id_protocolo ");
		cmd.append("     , pr.num_protocolo ");
		cmd.append("     , tp.id_tipo_protocolo ");
		cmd.append("     , tp.nom_tipo_protocolo ");
		cmd.append("     , pr.hr_abertura ");
		cmd.append("     , pr.num_documento ");
		cmd.append("     , ed.dsc_especie_documento ");
		cmd.append("     , pr.dt_documento ");
		cmd.append("     , at.dsc_assunto ");
		cmd.append("  FROM cprod.cprd_protocolo pr");
		cmd.append(" INNER JOIN cprod.cprd_assunto_d           at ON (pr.id_assunto           = at.id_assunto)");
		cmd.append(" INNER JOIN cprod.cprd_tipo_protocolo_d    tp ON (pr.id_tipo_protocolo    = tp.id_tipo_protocolo)");
		cmd.append(" INNER JOIN cprod.cprd_especie_documento_d ed ON (pr.id_especie_documento = ed.id_especie_documento)");
		cmd.append(" WHERE pr.num_protocolo = ?");

		// Executa a consulta do protocolo
		CprdProtocoloDto protocolo = null;
		SQLQuery query = super.getSession().createSQLQuery(cmd.toString());
		query.setParameter(0, numero);
		Object[] obj = (Object[]) query.uniqueResult();
		if (obj != null) {
			protocolo = new CprdProtocoloDto();
			protocolo.setDadoSei(false);
			protocolo.setId((Integer) obj[0]);
			protocolo.setNumeroProtocolo((BigDecimal) obj[1]);
			protocolo.setIdTipoProtocolo((Integer) obj[2]);
			protocolo.setNomTipoProtocolo((String) obj[3]);
			protocolo.setDataAbertura((Date) obj[4]);
			protocolo.setNumeroDocumento(obj[5] != null ? (String) obj[5] : null);
			protocolo.setEspecieDocumento((String) obj[6]);
			protocolo.setDataDocumento((Date) obj[7]);
			protocolo.setAssunto((String) obj[8]);
		}

		return protocolo;

	}

	
	public CprdProtocoloDto getProtocoloSEI(String numero) {
		// Monta consulta de protocolo
		StringBuilder cmd = new StringBuilder();
		cmd.append("SELECT p.id_protocolo, p.protocolo_formatado_pesquisa, p.dta_geracao, tpro.nome FROM " + Constantes.LINKSERVER
				+ " SEI.protocolo AS p LEFT OUTER JOIN ( SELECT DISTINCT id_procedimento FROM " + Constantes.LINKSERVER
				+ "SEI.documento) AS doc ON doc.id_procedimento = p.id_protocolo LEFT OUTER JOIN " + Constantes.LINKSERVER
				+ "SEI.procedimento AS pro ON pro.id_procedimento = doc.id_procedimento LEFT OUTER JOIN " + Constantes.LINKSERVER
				+ "SEI.tipo_procedimento AS tpro ON tpro.id_tipo_procedimento = pro.id_tipo_procedimento WHERE p.protocolo_formatado_pesquisa = ?");

		// Executa a consulta do protocolo
		CprdProtocoloDto protocolo = null;
		SQLQuery query = super.getSession().createSQLQuery(cmd.toString());
		query.setParameter(0, numero);
		Object[] obj = (Object[]) query.uniqueResult();
		if (obj != null) {
			protocolo = new CprdProtocoloDto();
			protocolo.setDadoSei(true);
			protocolo.setId(obj[0] != null ? Integer.parseInt(obj[0].toString()) : null);
			protocolo.setNumeroProtocolo(obj[1] != null ? new BigDecimal(obj[1].toString()) : null);
			protocolo.setDataAbertura(obj[2] != null ? new Date(((java.sql.Timestamp) obj[2]).getTime()) : null);
			protocolo.setAssunto(obj[3] != null ? obj[3].toString() : null);
			// SEI - Sempre sera do tipo PROCESSO = 2
			protocolo.setIdTipoProtocolo(2);
			protocolo.setNomTipoProtocolo("Processo");
		}
		return protocolo;
	}

	@SuppressWarnings("unchecked")
	
	public List<CadArea> findAreaByProcuracao(Integer idProcuracao) {
		StringBuilder sb = new StringBuilder();
		sb.append("    SELECT distinct area ");
		sb.append("      FROM CadProcuracaoSolicitacao procuracaoSolicitacao ");
		sb.append(" LEFT JOIN procuracaoSolicitacao.area area ");
		sb.append("     WHERE 1 = 1 ");
		sb.append("       AND procuracaoSolicitacao.procuracao.id = :id ");
		Query query = super.getSession().createQuery(sb.toString());
		query.setParameter("id", idProcuracao);
		return query.list();
	}

	@SuppressWarnings("unchecked")
	
	public List<CadAreaServico> findServicoByProcuracao(Integer idArea, Integer idProcuracao) {
		StringBuilder sb = new StringBuilder();
		sb.append("    SELECT distinct areaServico ");
		sb.append("      FROM CadAreaServico areaServico ");
		sb.append(" INNER JOIN areaServico.area area ");
		sb.append(" INNER JOIN areaServico.servico servico ");
		sb.append(" INNER JOIN area.procuracaoSolicitacao procuracaoSolicitacao ");
		sb.append(" INNER JOIN servico.procuracaoSolicitacao procuracaoSolicitacaoServico ");
		sb.append("     WHERE 1 = 1 ");
		sb.append("       AND procuracaoSolicitacao.procuracao.id = :id ");
		sb.append("       AND procuracaoSolicitacao.area.id = :idArea ");
		Query query = super.getSession().createQuery(sb.toString());
		query.setParameter("id", idProcuracao);
		query.setParameter("idArea", idArea);
		return query.list();
	}

	@SuppressWarnings("unchecked")
	
	public List<CadServicoSolicitacao> findSolicitacaoByProcuracao(Integer idServico, Integer idProcuracao) {
		StringBuilder sb = new StringBuilder();
		sb.append("    SELECT distinct servicoSolicitacao ");
		sb.append("      FROM CadServicoSolicitacao servicoSolicitacao ");
		sb.append(" INNER JOIN servicoSolicitacao.servico servico ");
		sb.append(" INNER JOIN servicoSolicitacao.solicitacao solicitacao ");
		sb.append(" INNER JOIN servico.procuracaoSolicitacao procuracaoSolicitacao ");
		sb.append(" INNER JOIN solicitacao.procuracaoSolicitacao ps ");
		sb.append("     WHERE 1 = 1 ");
		sb.append("       AND procuracaoSolicitacao.procuracao.id = :id ");
		sb.append("       AND procuracaoSolicitacao.servico.id = :idServico ");
		Query query = super.getSession().createQuery(sb.toString());
		query.setParameter("id", idProcuracao);
		query.setParameter("idServico", idServico);
		return query.list();
	}

	
	public CadSeiIntegracao findSeiIntegracaoByIdPedticao(CadPeticao peticao) {
		return (CadSeiIntegracao) super.getSession().createCriteria(CadSeiIntegracao.class).add(Restrictions.eq("area.id", peticao.getArea().getId()))
				.add(Restrictions.eq("servico.id", peticao.getServico().getId())).add(Restrictions.eq("solicitacao.id", peticao.getSolicitacao().getId()))
				.uniqueResult();
	}

	@SuppressWarnings("unchecked")
	
	public List<CadPeticao> findPeticoes(CadProponente proponente, String tipoPessoa) {
		Criteria crit = this.getSession().createCriteria(CadPeticao.class);
		crit.add(Restrictions.eq("proponente", proponente));
		if (tipoPessoa != null && !tipoPessoa.isEmpty() && tipoPessoa.equalsIgnoreCase("F")) {
			crit.add(Restrictions.isNull("empresa"));
			crit.add(Restrictions.isNull("procuracao"));
		} else if (tipoPessoa != null && !tipoPessoa.isEmpty() && tipoPessoa.equalsIgnoreCase("J")) {
			crit.add(Restrictions.isNotNull("empresa"));
		} else if (tipoPessoa != null && !tipoPessoa.isEmpty() && tipoPessoa.equalsIgnoreCase("P")) {
			crit.add(Restrictions.isNotNull("procuracao"));
		}
		return crit.list();
	}

	
	public CadPeticao getByPeticao(CadPeticao peticao) {
		return (CadPeticao) super.getSession().createCriteria(CadPeticao.class).add(Restrictions.eq("id", peticao.getId())).uniqueResult();
	}	

}