package br.gov.mc.cadsei.dao.impl;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.cadsei.dao.ProcuracaoDao;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProcuracao;
import br.gov.mc.cadsei.entidade.CadStatusProcuracaoD;
import br.gov.mc.cadsei.entidade.dto.ResultadoOutorgadoDto;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpTipoDocPessoaFisica;

@Repository(value = "procuracaoDao")
public class ProcuracaoDaoImpl extends HibernateDaoSupport implements ProcuracaoDao {

	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}

	
	public CadProcuracao save(CadProcuracao procuracao) throws Exception {
		super.getSession().saveOrUpdate(procuracao);
		return procuracao;
	}

	
	public CadProcuracao merge(CadProcuracao procuracao) throws Exception {
		super.getSession().merge(procuracao);
		return procuracao;
	}

	
	public void update(CadProcuracao procuracao) throws Exception {
		SQLQuery cmd = super.getSession().createSQLQuery("UPDATE CAD_PROCURACAO SET ID_STATUS_PROCURACAO = ? WHERE ID_PROCURACAO = ?");
		cmd.setParameter(0, procuracao.getStatusProcuracao().getId());
		cmd.setParameter(1, procuracao.getId());
		cmd.executeUpdate();
	}

	
	public CadProcuracao getByProcuracao(CadProcuracao procuracao) {
		return (CadProcuracao) super.getSession().createCriteria(CadProcuracao.class).add(Restrictions.eq("id", procuracao.getId())).uniqueResult();
	}

	
	public CadProcuracao getByProcuracaoAtiva(CadProcuracao procuracao) {
		return (CadProcuracao) super.getSession().createCriteria(CadProcuracao.class).add(Restrictions.eq("id", procuracao.getId()))
				.add(Restrictions.eq("statusProcuracao.id", CadStatusProcuracaoD.ATIVA)).uniqueResult();
	}

	public CadProcuracao getByProcuracaoPai(CadProcuracao procuracao) {
		return (CadProcuracao) super.getSession().createCriteria(CadProcuracao.class).add(Restrictions.eq("procuracaoPai.id", procuracao.getId()))
				.uniqueResult();
	}

	public CadProcuracao getByProcuracaoPaiAtiva(CadProcuracao procuracao) {
		return (CadProcuracao) super.getSession().createCriteria(CadProcuracao.class).add(Restrictions.eq("procuracaoPai.id", procuracao.getId()))
				.add(Restrictions.eq("statusProcuracao.id", CadStatusProcuracaoD.ATIVA)).uniqueResult();
	}

	@SuppressWarnings("unchecked")
	
	public List<CadEmpresa> getEmpresaByProponente(Integer idProponenteLogado) {
		return (List<CadEmpresa>) super.getSession().createCriteria(CadEmpresa.class).add(Restrictions.eq("proponente.id", idProponenteLogado)).list();
	}

	public CadProcuracao findProcuracaoByOutorgadoCPF(String cpf, Integer idEmpresaOutorgante, Integer idProponenteOutorgante) {
		StringBuilder sb = new StringBuilder();
		sb.append("SELECT procuracao ");
		sb.append("  FROM CadProcuracao procuracao ");
		sb.append(" INNER JOIN procuracao.proponenteOutorgado proponente ");
		sb.append(" INNER JOIN procuracao.statusProcuracao status ");
		if (idEmpresaOutorgante != null) {
			sb.append(" INNER JOIN procuracao.empresaOutorgante empresaOutorgante ");
		} else {
			sb.append(" INNER JOIN procuracao.proponenteOutorgante proponenteOutorgante ");
		}
		sb.append(" WHERE proponente.loginProponente = :cpf ");
		if (idEmpresaOutorgante != null) {
			sb.append(" AND empresaOutorgante.id = :id ");
		} else {
			sb.append(" AND proponenteOutorgante.id = :id ");
		}
		sb.append(" AND status.id = :idStatus ");
		Query query = super.getSession().createQuery(sb.toString());
		query.setParameter("cpf", cpf);
		if (idEmpresaOutorgante != null) {
			query.setParameter("id", idEmpresaOutorgante);
		} else {
			query.setParameter("id", idProponenteOutorgante);
		}
		query.setParameter("idStatus", CadStatusProcuracaoD.ATIVA);
		return (CadProcuracao) query.uniqueResult();
	}

	public CadProcuracao findProcuracaoByOutorgadoCNPJ(String cnpj, Integer idEmpresaOutorgante, Integer idProponenteOutorgante) {
		StringBuilder sb = new StringBuilder();
		sb.append("SELECT procuracao ");
		sb.append("  FROM CadProcuracao procuracao ");
		sb.append(" INNER JOIN procuracao.empresaOutorgado empresa ");
		sb.append(" INNER JOIN procuracao.statusProcuracao status ");
		if (idEmpresaOutorgante != null) {
			sb.append(" INNER JOIN procuracao.empresaOutorgante empresaOutorgante ");
		} else {
			sb.append(" INNER JOIN procuracao.proponenteOutorgante proponenteOutorgante ");
		}
		sb.append(" WHERE empresa.pessoaTitular.cnpj = :cnpj ");
		if (idEmpresaOutorgante != null) {
			sb.append(" AND empresaOutorgante.id = :id ");
		} else {
			sb.append(" AND proponenteOutorgante.id = :id ");
		}
		sb.append(" AND status.id = :idStatus ");
		Query query = super.getSession().createQuery(sb.toString());
		query.setParameter("cnpj", cnpj);
		if (idEmpresaOutorgante != null) {
			query.setParameter("id", idEmpresaOutorgante);
		} else {
			query.setParameter("id", idProponenteOutorgante);
		}
		query.setParameter("idStatus", CadStatusProcuracaoD.ATIVA);
		return (CadProcuracao) query.uniqueResult();
	}

	
	public Integer getNumeroSequencial(Integer idProponente, Integer idEmpresa) {
		StringBuilder sb = new StringBuilder();
		sb.append(" SELECT CAST((CASE WHEN  MAX(NR_SEQUENCIAL) IS NULL  THEN 1  ELSE MAX(NR_SEQUENCIAL)+1 END) AS SIGNED) AS NR_SEQUENCIAL");
		sb.append("   FROM CAD_PROCURACAO ");
		sb.append("  WHERE DATE_FORMAT(DT_VALIDADE_INICIAL,'%Y') = DATE_FORMAT(CURRENT_DATE(),'%Y')  ");
		// if (idProponente != null) {
		// sb.append("   AND ID_PROPONENTE_OUTORGANTE = " + idProponente);
		// }
		// if (idEmpresa != null) {
		// sb.append("   AND ID_EMPRESA_OUTORGANTE = " + idEmpresa);
		// }
		Query query = super.getSession().createSQLQuery(sb.toString());
		return ((BigInteger) query.uniqueResult()).intValue();
	}

	
	public CadEmpresa recuperarEmpresa(String cnpj) {
		Criteria crit = this.getSession().createCriteria(CadEmpresa.class, "emp");
		crit.createAlias("emp.pessoaTitular", "pessoaTitular");
		crit.add(Restrictions.eq("pessoaTitular.cnpj", cnpj));
		return (CadEmpresa) crit.uniqueResult();
	}

	@SuppressWarnings("rawtypes")
	public List<ResultadoOutorgadoDto> findProcuracaoEnviadas(String cpf) {
		StringBuilder sb = new StringBuilder();
		sb.append(" SELECT ");
		sb.append("        proc_empresa.ID_PROCURACAO ");
//		sb.append("        CAST(pessoaJuridicaOutorgante.NUM_CNPJ AS nvarchar) cpfCnpjOutorgante, ");
//		sb.append("        pessoaJuridicaOutorgante.NOM_EMPRESARIAL nomeOutorgante,  ");
//		sb.append("        proponenteOutorgado.DSC_LOGIN_PROPONENTE, ");
//		sb.append("        CAST(pessoaJuridicaOutorgado.NUM_CNPJ AS nvarchar),  ");
//		sb.append("        pessoaFisicaOutorgado.NOM_PESSOA, ");
//		sb.append("        pessoaJuridicaOutorgado.NOM_EMPRESARIAL ");
		sb.append("  FROM CAD_PROCURACAO proc_empresa ");
		sb.append(" INNER JOIN CAD_STATUS_PROCURACAO_D status on proc_empresa.ID_STATUS_PROCURACAO = status.ID_STATUS_PROCURACAO ");
		sb.append("  LEFT OUTER JOIN CAD_EMPRESA empresaOutorgante on proc_empresa.ID_EMPRESA_OUTORGANTE=empresaOutorgante.ID_EMPRESA   ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pessoaJuridicaOutorgante on empresaOutorgante.ID_PESSOA_JURIDICA=pessoaJuridicaOutorgante.ID_PESSOA  ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA pessoa on pessoaJuridicaOutorgante.ID_PESSOA=pessoa.ID_PESSOA  ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_REPRESENTANTE_PES_JUR representante on pessoa.ID_PESSOA=representante.ID_PESSOA_JUR ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaF on representante.ID_PESSOA_REPR_PJ=pessoaF.ID_PESSOA  ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA pessoaF1 on pessoaF.ID_PESSOA=pessoaF1.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_DOCUMENTO_PESSOA_FISICA doc on pessoaF1.ID_PESSOA=doc.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_TIPO_DOC_PESSOA_FISICA tipoDoc on doc.ID_TIPO_DOC_PESSOA_FISICA=tipoDoc.ID_TIPO_DOC_PESSOA_FISICA ");
		sb.append("  LEFT OUTER JOIN CAD_EMPRESA empresaOutorgado on proc_empresa.ID_EMPRESA_OUTORGADO=empresaOutorgado.ID_EMPRESA  ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pessoaJuridicaOutorgado on empresaOutorgado.ID_PESSOA_JURIDICA=pessoaJuridicaOutorgado.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CAD_PROPONENTE proponenteOutorgado on proc_empresa.ID_PROPONENTE_OUTORGADO=proponenteOutorgado.ID_PROPONENTE ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaFisicaOutorgado on proponenteOutorgado.ID_PESSOA = pessoaFisicaOutorgado.ID_PESSOA ");
		sb.append("  WHERE 1=1 ");
		sb.append("    AND ID_EMPRESA_OUTORGANTE is not null ");

		sb.append("  AND doc.NUM_DOCUMENTO='" + cpf + "' ");
		sb.append("  AND tipoDoc.ID_TIPO_DOC_PESSOA_FISICA= " + CrpTipoDocPessoaFisica.CPF);

		sb.append(" UNION ");

		sb.append(" SELECT ");
		sb.append("        proc_proponente.ID_PROCURACAO ");
//		sb.append("        proponenteOutorgante.DSC_LOGIN_PROPONENTE cpfCnpjOutorgante, ");
//		sb.append("        pessoaFisicaOutorgante.NOM_PESSOA nomeOutorgante, ");
//		sb.append("        proponenteOutorgado.DSC_LOGIN_PROPONENTE, ");
//		sb.append("        CAST(pessoaJuridicaOutorgado.NUM_CNPJ AS nvarchar), ");
//		sb.append("        pessoaFisicaOutorgado.NOM_PESSOA, ");
//		sb.append("        pessoaJuridicaOutorgado.NOM_EMPRESARIAL ");
		sb.append("   FROM CAD_PROCURACAO proc_proponente ");
		sb.append("  INNER JOIN CAD_STATUS_PROCURACAO_D status on proc_proponente.ID_STATUS_PROCURACAO = status.ID_STATUS_PROCURACAO ");
		sb.append("   LEFT OUTER JOIN CAD_PROPONENTE proponenteOutorgante on proc_proponente.ID_PROPONENTE_OUTORGANTE=proponenteOutorgante.ID_PROPONENTE  ");
		sb.append("   LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaFisicaOutorgante on proponenteOutorgante.ID_PESSOA = pessoaFisicaOutorgante.ID_PESSOA  ");
		sb.append("   LEFT OUTER JOIN CAD_PROPONENTE proponenteOutorgado on proc_proponente.ID_PROPONENTE_OUTORGADO=proponenteOutorgado.ID_PROPONENTE  ");
		sb.append("   LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaFisicaOutorgado on proponenteOutorgado.ID_PESSOA = pessoaFisicaOutorgado.ID_PESSOA ");
		sb.append("   LEFT OUTER JOIN CAD_EMPRESA empresaOutorgado on proc_proponente.ID_EMPRESA_OUTORGADO=empresaOutorgado.ID_EMPRESA ");
		sb.append("   LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pessoaJuridicaOutorgado on empresaOutorgado.ID_PESSOA_JURIDICA=pessoaJuridicaOutorgado.ID_PESSOA ");
		sb.append("  WHERE 1=1  ");
		sb.append("    AND proc_proponente.ID_PROPONENTE_OUTORGANTE is not null ");
		sb.append("    AND proponenteOutorgante.DSC_LOGIN_PROPONENTE = '" + cpf + "' ");
		ResultadoOutorgadoDto result = null;
		List<ResultadoOutorgadoDto> list = new ArrayList<ResultadoOutorgadoDto>();
		SQLQuery query = super.getSession().createSQLQuery(sb.toString());

		Iterator it = query.list().iterator();
		while (it.hasNext()) {
			Object[] obj = (Object[]) it.next();
			result = new ResultadoOutorgadoDto();
			result.setIdProcuracao((Integer) obj[0]);
			result.setProcuracao(this.getByProcuracao(new CadProcuracao(result.getIdProcuracao())));
			result.setNumeroOutorgante(obj[1].toString());
			result.setNomeOutorgante(obj[2].toString());

			if (obj[3] == null) {
				result.setNumeroOutorgado(obj[4].toString());
				result.setNomeOutorgado(obj[6].toString());
			} else {
				result.setNumeroOutorgado(obj[3].toString());
				result.setNomeOutorgado(obj[5].toString());
			}

			// Recuperar a procuração que contenha uma procuração pai com o id
			// da procuração, sendo assim ira ser um substabelecedor.
			CadProcuracao procuracao = null;
			if (result.getProcuracao() != null) {
				if (result.getProcuracao().getProcuracaoPai().getId() != null) {
					// id = result.getProcuracao().getProcuracaoPai().getId();
					procuracao = this.getByProcuracaoAtiva(new CadProcuracao(result.getProcuracao().getProcuracaoPai().getId()));
					if (procuracao != null) {
						if (procuracao.getProcuracaoPai().getProponenteOutorgante() != null && procuracao.getProponenteOutorgante().getId() != null) {
							result.setNumeroSubstabelecedor(procuracao.getProponenteOutorgante().getLoginProponente());
							result.setNomeSubstabelecedor(procuracao.getProponenteOutorgante().getPessoaProponente().getNome());
						} else {
							result.setNumeroSubstabelecedor(procuracao.getEmpresaOutorgante().getPessoaTitular().getCnpj());
							result.setNomeSubstabelecedor(procuracao.getEmpresaOutorgante().getPessoaTitular().getRazaoSocial());
						}
					}
				} else {
					procuracao = this.getByProcuracaoPaiAtiva(new CadProcuracao(result.getProcuracao().getId()));
					if (procuracao != null) {
						if (procuracao.getProcuracaoPai().getProponenteOutorgado() != null && procuracao.getProponenteOutorgado().getId() != null) {
							result.setNumeroSubstabelecedor(procuracao.getProponenteOutorgado().getLoginProponente());
							result.setNomeSubstabelecedor(procuracao.getProponenteOutorgado().getPessoaProponente().getNome());
						} else {
							result.setNumeroSubstabelecedor(procuracao.getEmpresaOutorgado().getPessoaTitular().getCnpj());
							result.setNomeSubstabelecedor(procuracao.getEmpresaOutorgado().getPessoaTitular().getRazaoSocial());
						}
					}
				}
			}
			list.add(result);
		}
		return list;
	}

	public List<ResultadoOutorgadoDto> findProcuracaoRecebidas(String cpf, boolean ativo) {
		StringBuilder sb = new StringBuilder();
		sb.append(" SELECT ");
		sb.append("        proc_empresa.ID_PROCURACAO, ");
		sb.append("        pessoaJuridicaOutorgado.NUM_CNPJ cpfCnpjOutorgado, ");
		sb.append("        pessoaJuridicaOutorgado.NOM_EMPRESARIAL nomeOutorgado,  ");
		sb.append("        proponenteOutorgante.DSC_LOGIN_PROPONENTE, ");
		sb.append("        pessoaJuridicaOutorgante.NUM_CNPJ,  ");
		sb.append("        pessoaFisicaOutorgante.NOM_PESSOA, ");
		sb.append("        pessoaJuridicaOutorgante.NOM_EMPRESARIAL ");
		sb.append("  FROM CAD_PROCURACAO proc_empresa ");
		sb.append(" INNER JOIN CAD_STATUS_PROCURACAO_D status on proc_empresa.ID_STATUS_PROCURACAO = status.ID_STATUS_PROCURACAO ");
		sb.append("  LEFT OUTER JOIN CAD_EMPRESA empresaOutorgado on proc_empresa.ID_EMPRESA_OUTORGADO=empresaOutorgado.ID_EMPRESA ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pessoaJuridicaOutorgado on empresaOutorgado.ID_PESSOA_JURIDICA=pessoaJuridicaOutorgado.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA pessoa on pessoaJuridicaOutorgado.ID_PESSOA=pessoa.ID_PESSOA   ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_REPRESENTANTE_PES_JUR representante on pessoa.ID_PESSOA=representante.ID_PESSOA_JUR ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaF on representante.ID_PESSOA_REPR_PJ=pessoaF.ID_PESSOA  ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA pessoaF1 on pessoaF.ID_PESSOA=pessoaF1.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_DOCUMENTO_PESSOA_FISICA doc on pessoaF1.ID_PESSOA=doc.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_TIPO_DOC_PESSOA_FISICA tipoDoc on doc.ID_TIPO_DOC_PESSOA_FISICA=tipoDoc.ID_TIPO_DOC_PESSOA_FISICA ");
		sb.append("  LEFT OUTER JOIN CAD_EMPRESA empresaOutorgante on proc_empresa.ID_EMPRESA_OUTORGANTE=empresaOutorgante.ID_EMPRESA  ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pessoaJuridicaOutorgante on empresaOutorgante.ID_PESSOA_JURIDICA=pessoaJuridicaOutorgante.ID_PESSOA ");
		sb.append("  LEFT OUTER JOIN CAD_PROPONENTE proponenteOutorgante on proc_empresa.ID_PROPONENTE_OUTORGANTE=proponenteOutorgante.ID_PROPONENTE ");
		sb.append("  LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaFisicaOutorgante on proponenteOutorgante.ID_PESSOA = pessoaFisicaOutorgante.ID_PESSOA ");
		sb.append("  WHERE 1=1 ");
		sb.append("    AND ID_EMPRESA_OUTORGADO is not null ");
		sb.append("  AND doc.NUM_DOCUMENTO='" + cpf + "' ");
		sb.append("  AND tipoDoc.ID_TIPO_DOC_PESSOA_FISICA= " + CrpTipoDocPessoaFisica.CPF);

		if (ativo) {
			sb.append(" AND status.ID_STATUS_PROCURACAO = " + CadStatusProcuracaoD.ATIVA);
		}

		sb.append(" UNION ");

		sb.append(" SELECT ");
		sb.append("        proc_proponente.ID_PROCURACAO, ");
		sb.append("        proponenteOutorgado.DSC_LOGIN_PROPONENTE cpfCnpjOutorgado, ");
		sb.append("        pessoaFisicaOutorgado.NOM_PESSOA nomeOutorgado, ");
		sb.append("        proponenteOutorgante.DSC_LOGIN_PROPONENTE, ");
		sb.append("        pessoaJuridicaOutorgante.NUM_CNPJ, ");
		sb.append("        pessoaFisicaOutorgante.NOM_PESSOA, ");
		sb.append("        pessoaJuridicaOutorgante.NOM_EMPRESARIAL ");
		sb.append("   FROM CAD_PROCURACAO proc_proponente ");
		sb.append("  INNER JOIN CAD_STATUS_PROCURACAO_D status on proc_proponente.ID_STATUS_PROCURACAO = status.ID_STATUS_PROCURACAO ");
		sb.append("   LEFT OUTER JOIN CAD_PROPONENTE proponenteOutorgado on proc_proponente.ID_PROPONENTE_OUTORGADO=proponenteOutorgado.ID_PROPONENTE ");
		sb.append("   LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaFisicaOutorgado on (proponenteOutorgado.ID_PESSOA = pessoaFisicaOutorgado.ID_PESSOA) ");
		sb.append("   LEFT OUTER JOIN CAD_PROPONENTE proponenteOutorgante on (proc_proponente.ID_PROPONENTE_OUTORGANTE=proponenteOutorgante.ID_PROPONENTE) ");
		sb.append("   LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_FISICA pessoaFisicaOutorgante on (proponenteOutorgante.ID_PESSOA = pessoaFisicaOutorgante.ID_PESSOA) ");
		sb.append("   LEFT OUTER JOIN CAD_EMPRESA empresaOutorgante on proc_proponente.ID_EMPRESA_OUTORGANTE=empresaOutorgante.ID_EMPRESA ");
		sb.append("   LEFT OUTER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pessoaJuridicaOutorgante on empresaOutorgante.ID_PESSOA_JURIDICA=pessoaJuridicaOutorgante.ID_PESSOA ");
		sb.append("  WHERE 1=1  ");
		sb.append("    AND proc_proponente.ID_PROPONENTE_OUTORGADO is not null ");
		sb.append("    AND proponenteOutorgado.DSC_LOGIN_PROPONENTE = '" + cpf + "' ");

		if (ativo) {
			sb.append(" AND status.ID_STATUS_PROCURACAO = " + CadStatusProcuracaoD.ATIVA);
		}

		ResultadoOutorgadoDto result = null;
		List<ResultadoOutorgadoDto> list = new ArrayList<ResultadoOutorgadoDto>();
		SQLQuery query = super.getSession().createSQLQuery(sb.toString());

		@SuppressWarnings("rawtypes")
		Iterator it = query.list().iterator();
		while (it.hasNext()) {
			Object[] obj = (Object[]) it.next();
			result = new ResultadoOutorgadoDto();
			result.setIdProcuracao((Integer) obj[0]);
			result.setProcuracao(this.getByProcuracao(new CadProcuracao(result.getIdProcuracao())));
			result.setNumeroOutorgado(obj[1].toString());
			result.setNomeOutorgado(obj[2].toString());

			if (obj[3] == null) {
				result.setNumeroOutorgante(obj[4].toString());
				result.setNomeOutorgante(obj[6].toString());
			} else {
				result.setNumeroOutorgante(obj[3].toString());
				result.setNomeOutorgante(obj[5].toString());
			}

			// Recuperar a procuração que contenha uma procuração pai com o id
			// da procuração, sendo assim ira ser um substabelecedor.
			CadProcuracao procuracao = null;
			if (result.getProcuracao().getProcuracaoPai().getId() != null) {
				procuracao = this.getByProcuracaoAtiva(new CadProcuracao(result.getProcuracao().getProcuracaoPai().getId()));
				if (procuracao != null) {
					if (procuracao.getProcuracaoPai().getProponenteOutorgante() != null && procuracao.getProponenteOutorgante().getId() != null) {
						result.setNumeroSubstabelecedor(procuracao.getProponenteOutorgante().getLoginProponente());
						result.setNomeSubstabelecedor(procuracao.getProponenteOutorgante().getPessoaProponente().getNome());
					} else {
						result.setNumeroSubstabelecedor(procuracao.getEmpresaOutorgante().getPessoaTitular().getCnpj());
						result.setNomeSubstabelecedor(procuracao.getEmpresaOutorgante().getPessoaTitular().getRazaoSocial());
					}
				}
			} else {
				procuracao = this.getByProcuracaoPaiAtiva(new CadProcuracao(result.getProcuracao().getId()));
				if (procuracao != null) {
					if (procuracao.getProcuracaoPai().getProponenteOutorgado() != null && procuracao.getProponenteOutorgado().getId() != null) {
						result.setNumeroSubstabelecedor(procuracao.getProponenteOutorgado().getLoginProponente());
						result.setNomeSubstabelecedor(procuracao.getProponenteOutorgado().getPessoaProponente().getNome());
					} else {
						result.setNumeroSubstabelecedor(procuracao.getEmpresaOutorgado().getPessoaTitular().getCnpj());
						result.setNomeSubstabelecedor(procuracao.getEmpresaOutorgado().getPessoaTitular().getRazaoSocial());
					}
				}
			}
			list.add(result);
		}
		return list;
	}

	@SuppressWarnings("unchecked")
	public List<CadProcuracao> findAllProcuracaoAtivas() {
		return super.getSession().createCriteria(CadProcuracao.class).add(Restrictions.eq("statusProcuracao.id", CadStatusProcuracaoD.ATIVA)).list();
	}

	@SuppressWarnings("unchecked")
	public List<CrpContatoEmail> obtemEmailsPessoaTitularPorEmpresaOrtogado(Integer id) {
		Query q = this.getSession().createQuery("Select p.emails from CadProcuracao e inner join e.empresaOutorgado o inner join o.pessoaTitular p where o.id = ?");
		q.setParameter(0, id);
		return q.list();
	}

}
