package br.gov.mc.cadsei.dao.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.hibernate.SQLQuery;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;
import org.springframework.stereotype.Repository;

import br.gov.mc.cadsei.dao.ProponenteDao;
import br.gov.mc.cadsei.entidade.CadDocumento;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadSituacaoCadastroD;
import br.gov.mc.corporativo.entidade.CrpCorRaca;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;

@Repository(value = "proponenteDao")
public class ProponenteDaoImpl extends HibernateDaoSupport implements ProponenteDao {

	@Autowired(required = true)
	public void setFactory(SessionFactory factory) {
		super.setSessionFactory(factory);
	}

	@SuppressWarnings("unchecked")
	public List<CadProponente> getProponentes() {
		return getHibernateTemplate().find("select p from CadProponente p");
	}

	public void atualizaSenhaProponente(CadProponente proponente) {
		SQLQuery cmd = super.getSession().createSQLQuery("UPDATE CAD_PROPONENTE SET DSC_SENHA_PROPONENTE = ? WHERE ID_PROPONENTE = ?");
		cmd.setParameter(0, proponente.getSenhaProponente());
		cmd.setParameter(1, proponente.getId());
		cmd.executeUpdate();
	}

	@SuppressWarnings("unchecked")
	public List<String> buscaProponentePorNome(String nome) {
		org.hibernate.Query q = this.getSession().createQuery("Select e.pessoaProponente.nome from CadProponente e WHERE e.pessoaProponente.nome LIKE ?");
		q.setParameter(0, "%" + nome + "%");
		return q.list();
	}

	@SuppressWarnings("unchecked")
	public List<CadProponente> buscaParametrizada(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filtros) {

		SQLQuery cmd = null;
		StringBuffer sql = new StringBuffer();
		sql.append("     SELECT DISTINCT p.DSC_LOGIN_PROPONENTE, pf.NOM_PESSOA, p.DT_CADASTRO, sit.DSC_SITUACAO_CADASTRO, sit.ID_SITUACAO_CADASTRO, p.ID_PROPONENTE ");
		sql.append("       FROM CAD_PROPONENTE p ");
		sql.append(" INNER JOIN CORPORATIVO.CRP_PESSOA_FISICA pf on (p.ID_PESSOA = pf.ID_PESSOA) ");
		sql.append(" INNER JOIN CORPORATIVO.CRP_DOCUMENTO_PESSOA_FISICA doc on (doc.ID_PESSOA = pf.ID_PESSOA) ");
		sql.append(" INNER JOIN CAD_SITUACAO_CADASTRO_D sit on (sit.ID_SITUACAO_CADASTRO = p.ID_SITUACAO_CADASTRO) ");
		sql.append("  LEFT JOIN CORPORATIVO.CRP_UF cuf on (cuf.COD_UF_IBGE = p.ID_UF) ");

		sql.append("      WHERE  1 = 1 ");
//		sql.append("        AND p.ID_PESSOA NOT IN ( ");
//		sql.append("     SELECT r.ID_PESSOA_REPR_PJ ");
//		sql.append("       FROM CAD_EMPRESA e ");
//		sql.append(" INNER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pj on (e.ID_PESSOA_JURIDICA = pj.ID_PESSOA) ");
//		sql.append(" INNER JOIN CORPORATIVO.CRP_REPRESENTANTE_PES_JUR r on (pj.ID_PESSOA = r.ID_PESSOA_JUR) ");
//		sql.append(" WHERE r.DT_FIM is null ");
//		sql.append(" ) ");

		if (filtros.containsKey("sit.id")) {
			sql.append(" and sit.ID_SITUACAO_CADASTRO = :sitcad ");
		}
		if (filtros.containsKey("cuf.id")) {
			sql.append(" and cuf.COD_UF_IBGE = :uf ");
		}
		if (filtros.containsKey("p.loginProponente")) {
			sql.append(" and p.DSC_LOGIN_PROPONENTE = :cpf ");
		}
		if (filtros.containsKey("pf.nome")) {
			sql.append(" and UPPER(pf.NOM_PESSOA) LIKE :nome ");
		}
		if (sortField != null) {
			String ordenacao = sortOrder == true ? " ASC " : " DESC ";
			sql.append(" ORDER BY ");
			if (sortField.equalsIgnoreCase("pessoaProponente.nome")) {
				sql.append("pf.NOM_PESSOA");
			} else if (sortField.equalsIgnoreCase("loginProponente")) {
				sql.append("p.DSC_LOGIN_PROPONENTE");
			} else if (sortField.equalsIgnoreCase("dataCadastro")) {
				sql.append("p.DT_CADASTRO");
			} else if (sortField.equalsIgnoreCase("situacaoCadastro.descricao")) {
				sql.append("sit.DSC_SITUACAO_CADASTRO");
			}
			sql.append(ordenacao);
		}
		
		cmd = getSession().createSQLQuery(sql.toString());
		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("sit.id"))
				cmd.setParameter("sitcad", v.getValue());

			if (v != null && v.getKey().equals("cuf.id"))
				cmd.setParameter("uf", v.getValue());

			if (v != null && v.getKey().equals("p.loginProponente"))
				cmd.setParameter("cpf", v.getValue());

			if (v != null && v.getKey().equals("pf.nome"))
				cmd.setParameter("nome", "%" + v.getValue() + "%");

		}
		cmd.setFirstResult(first);
		cmd.setMaxResults(pageSize);
		
		List<Object[]> proponente = cmd.list();
		
		List<CadProponente> lista = new ArrayList<CadProponente>();
		CadProponente p = new CadProponente();
		
		for (Object[] o : proponente) {
			p = new CadProponente();
			p.setLoginProponente((String) o[0]);
			p.setPessoaProponente(new CrpPessoaFisica());
			p.getPessoaProponente().setNome((String) o[1]);
			p.setDataCadastro((Date) o[2]);
			p.setSituacaoCadastro(new  CadSituacaoCadastroD());
			p.getSituacaoCadastro().setDescricao((String) o[3]);
			p.getSituacaoCadastro().setId((Integer) o[4]);
			p.setId((Integer)o[5]);
			
			lista.add(p);
		}
			return lista;
			
		}

	
	public Long rowCount(Map<String, String> filtros) {

		SQLQuery cmd = null;
		StringBuffer sql = new StringBuffer();
		sql.append("     SELECT count(DISTINCT p.DSC_LOGIN_PROPONENTE) ");
		sql.append("       FROM CAD_PROPONENTE p ");
		sql.append(" INNER JOIN CORPORATIVO.CRP_PESSOA_FISICA pf on (p.ID_PESSOA = pf.ID_PESSOA) ");
		sql.append(" INNER JOIN CORPORATIVO.CRP_DOCUMENTO_PESSOA_FISICA doc on (doc.ID_PESSOA = pf.ID_PESSOA) ");
		sql.append(" INNER JOIN CAD_SITUACAO_CADASTRO_D sit on (sit.ID_SITUACAO_CADASTRO = p.ID_SITUACAO_CADASTRO) ");
		sql.append("  LEFT JOIN CORPORATIVO.CRP_UF cuf on (cuf.COD_UF_IBGE = p.ID_UF) ");

		sql.append("      WHERE  1 = 1 ");
		sql.append("        AND p.ID_PESSOA NOT IN ( ");
		sql.append("     SELECT r.ID_PESSOA_REPR_PJ ");
		sql.append("       FROM CAD_EMPRESA e ");
		sql.append(" INNER JOIN CORPORATIVO.CRP_PESSOA_JURIDICA pj on (e.ID_PESSOA_JURIDICA = pj.ID_PESSOA) ");
		sql.append(" INNER JOIN CORPORATIVO.CRP_REPRESENTANTE_PES_JUR r on (pj.ID_PESSOA = r.ID_PESSOA_JUR) ");
		sql.append(" ) ");

		if (filtros.containsKey("sit.id")) {
			sql.append(" and sit.ID_SITUACAO_CADASTRO = :sitcad ");
		}
		if (filtros.containsKey("cuf.id")) {
			sql.append(" and cuf.COD_UF_IBGE = :uf ");
		}
		if (filtros.containsKey("p.loginProponente")) {
			sql.append(" and p.DSC_LOGIN_PROPONENTE = :cpf ");
		}
		if (filtros.containsKey("pf.nome")) {
			sql.append(" and pf.NOM_PESSOA LIKE :nome ");
		}
		cmd = getSession().createSQLQuery(sql.toString());
		for (Map.Entry<String, String> v : filtros.entrySet()) {
			if (v != null && v.getKey().equals("sit.id"))
				cmd.setParameter("sitcad", v.getValue());

			if (v != null && v.getKey().equals("cuf.id"))
				cmd.setParameter("uf", v.getValue());

			if (v != null && v.getKey().equals("p.loginProponente"))
				cmd.setParameter("cpf", v.getValue());

			if (v != null && v.getKey().equals("pf.nome"))
				cmd.setParameter("nome", "%" + v.getValue() + "%");

		}
		return (Long.valueOf(cmd.uniqueResult().toString()));
	}

	@SuppressWarnings("unchecked")
	
	public List<CadDocumento> buscaDocumentosPorProponente(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select d from CadProponente p inner join p.documentos d where p.id = ?");
		q.setParameter(0, id );
		return q.list();
	}

	
	public CadProponente buscaProponentePorId(Integer id) {
		org.hibernate.Query q = this.getSession().createQuery("Select d from CadProponente d inner join d.documentos where d.id = ?  ");
		q.setParameter(0, id );
		return (CadProponente) q.uniqueResult();
	}

	@SuppressWarnings("unchecked")
	
	public List<CrpCorRaca> buscaCorRaca() {
		org.hibernate.Query q = this.getSession().createQuery("Select d from CrpCorRaca d order by d.nomCorRaca");
		return q.list();
	}

	
	public CadProponente   recuperarSenha(String email, Date dataNascimento, String cpf ) {
		org.hibernate.Query q = this
				.getSession()
				.createQuery(
						"Select e from CadProponente e inner join e.pessoaProponente pp inner join pp.documentos d WHERE e.emailProponente = ? and d.numero = ? and d.tipoDocumento.id =1");
		q.setParameter(0, email);
		//q.setParameter( 1 ,dataNascimento);
		q.setParameter(1 , cpf);
		return (CadProponente) q.uniqueResult();
	}
	
	
	public CadProponente   login(String login, String senha ) {
		org.hibernate.Query q = this
				.getSession()
				.createQuery(
						"Select e from CadProponente e inner join e.situacaoCadastro sc inner join e.pessoaProponente pp WHERE e.loginProponente = ? and e.senhaProponente = ?");
		q.setParameter(0, login);
		q.setParameter( 1 ,senha);
		return (CadProponente) q.uniqueResult();
	}

}
