package br.gov.mc.cadsei.entidade;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;

import br.gov.arquitetura.entidade.Entidade;
import br.gov.mc.cadsei.util.CpfCnpjUtil;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;
import br.gov.mc.seg.entidade.SegUsuario;
import br.gov.mj.pessoafisica.pojo.PessoaFisica;

@Entity
@Table(name = "CAD_EMPRESA", catalog = "CADSEI")
public class CadEmpresa extends Entidade<Integer> {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    @Column(name = "ID_EMPRESA", unique = true, nullable = false)
    private Integer id;

    @OneToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "ID_PESSOA_JURIDICA", nullable = false)
    private CrpPessoaJuridica pessoaTitular;

    @OneToOne(fetch = FetchType.EAGER)
    @JoinColumn(name = "ID_REPRESENTANTE", nullable = false)
    private CrpRepresentantePesJur representante;

    @OneToOne(fetch = FetchType.LAZY)
    @JoinColumn(name = "ID_PROPONENTE", nullable = false)
    private CadProponente proponente;

    @Column(name = "ANO_CONSTITUICAO")
    private Integer anoConstituicao;

    @Column(name = "QTD_EMPREGADOS")
    private BigDecimal qtdEmpregados;

    @Temporal(TemporalType.TIMESTAMP)
    @Column(name = "DT_CADASTRO", nullable = false)
    private Date dataCadastro;

    @Column(name = "DSC_EMAIL_ADICIONAL_1")
    private String emailAdicional1;

    @Column(name = "DSC_EMAIL_ADICIONAL_2")
    private String emailAdicional2;

    @Column(name = "DSC_EMAIL_ADICIONAL_3")
    private String emailAdicional3;

    @Column(name = "DSC_EMAIL_ADICIONAL_4")
    private String emailAdicional4;

    @Column(name = "DSC_TIPO_SERVICO_OUTRO")
    private String outros;

    @Cascade({ CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN })
    @OneToMany(fetch = FetchType.LAZY, mappedBy = "empresa")
    private List<CadDocumento> documentos;

    @Cascade({ CascadeType.MERGE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN })
    @OneToMany(mappedBy = "empresa", fetch = FetchType.LAZY)
    private List<CadHistoricoEmpresa> listaHistorico;

    @ManyToOne(fetch = FetchType.EAGER, optional = true)
    @JoinColumn(name = "ID_SITUACAO_CADASTRO")
    private CadSituacaoCadastroD situacaoCadastro;

    @ManyToMany
    @JoinTable(name = "CAD_EMPRESA_SERVICO", joinColumns = { @JoinColumn(name = "ID_EMPRESA") }, inverseJoinColumns = { @JoinColumn(name = "ID_SERVICO") })
    private List<CadServico> servicos;

    @ManyToOne(fetch = FetchType.EAGER, optional = true)
    @JoinColumn(name = "ID_ANALISTA")
    private SegUsuario analista;

    @Cascade({ CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN })
    @OneToMany(fetch = FetchType.LAZY, mappedBy = "empresaOutorgante")
    private List<CadProcuracao> procuracaoOutorgante;

    @Cascade({ CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN })
    @OneToMany(fetch = FetchType.LAZY, mappedBy = "empresaOutorgado")
    private List<CadProcuracao> procuracaoOutorgado;

    @Column(name = "DSC_ANALISE_TECNICA", nullable = true)
    private String descricaoAnaliseTecnica;

    @Transient
    private String nomeRepresentante;

    @Transient
    private String cpfRepresentante;

    public CadEmpresa(Integer id) {
	this.id = id;
    }

    public Integer getId() {
	return id;
    }

    public void setId(Integer id) {
	this.id = id;
    }

    public CrpPessoaJuridica getPessoaTitular() {
	return pessoaTitular;
    }

    public void setPessoaTitular(CrpPessoaJuridica pessoaTitular) {
	this.pessoaTitular = pessoaTitular;
    }

    public Integer getAnoConstituicao() {
	return anoConstituicao;
    }

    public void setAnoConstituicao(Integer anoConstituicao) {
	this.anoConstituicao = anoConstituicao;
    }

    public BigDecimal getQtdEmpregados() {
	return qtdEmpregados;
    }

    public void setQtdEmpregados(BigDecimal qtdEmpregados) {
	this.qtdEmpregados = qtdEmpregados;
    }

    public CadProponente getProponente() {
	return proponente;
    }

    public void setProponente(CadProponente proponente) {
	this.proponente = proponente;
    }

    public List<CadDocumento> getDocumentos() {
	if (this.documentos == null) {
	    this.documentos = new ArrayList<CadDocumento>();
	}
	return this.documentos;
    }

    public void setDocumentos(List<CadDocumento> documentos) {
	this.documentos = documentos;
    }

    public void addDocumento(CadDocumento documento) {
	documento.setEmpresa(this);
	this.getDocumentos().add(documento);
    }

    public CrpRepresentantePesJur getRepresentante() {
	if (this.representante == null) {
	    this.representante = new CrpRepresentantePesJur();
	    this.representante.setPessoaFisica(new CrpPessoaFisica()); 
	}
	return this.representante;
    }

    public void setRepresentante(CrpRepresentantePesJur representante) {
	this.representante = representante;
    }

    public Date getDataCadastro() {
	if (this.dataCadastro == null) {
	    this.dataCadastro = new Date();
	}
	return this.dataCadastro;
    }

    public void setDataCadastro(Date dataCadastro) {
	this.dataCadastro = dataCadastro;
    }

    public String getEmailAdicional1() {
	return emailAdicional1;
    }

    public void setEmailAdicional1(String emailAdicional1) {
	this.emailAdicional1 = emailAdicional1;
    }

    public String getEmailAdicional2() {
	return emailAdicional2;
    }

    public void setEmailAdicional2(String emailAdicional2) {
	this.emailAdicional2 = emailAdicional2;
    }

    public String getEmailAdicional3() {
	return emailAdicional3;
    }

    public void setEmailAdicional3(String emailAdicional3) {
	this.emailAdicional3 = emailAdicional3;
    }

    public String getEmailAdicional4() {
	return emailAdicional4;
    }

    public void setEmailAdicional4(String emailAdicional4) {
	this.emailAdicional4 = emailAdicional4;
    }

    public String getOutros() {
	return outros;
    }

    public void setOutros(String outros) {
	this.outros = outros;
    }

    public void setListaHistorico(List<CadHistoricoEmpresa> listaHistorico) {
	this.listaHistorico = listaHistorico;
    }

    public void setSituacaoCadastro(CadSituacaoCadastroD situacaoCadastro) {
	this.situacaoCadastro = situacaoCadastro;
    }

    @Override
    public int hashCode() {
	final int prime = 31;
	int result = 1;
	result = prime * result + ((id == null) ? 0 : id.hashCode());
	result = prime * result + ((pessoaTitular == null) ? 0 : pessoaTitular.hashCode());
	return result;
    }

    @Override
    public boolean equals(Object obj) {
	if (this == obj)
	    return true;
	if (obj == null)
	    return false;
	if (getClass() != obj.getClass())
	    return false;
	CadEmpresa other = (CadEmpresa) obj;
	if (id == null) {
	    if (other.id != null)
		return false;
	} else if (!id.equals(other.id))
	    return false;
	if (pessoaTitular == null) {
	    if (other.pessoaTitular != null)
		return false;
	} else if (!pessoaTitular.equals(other.pessoaTitular))
	    return false;
	return true;
    }

    public CadEmpresa() {
    }

    public List<CadHistoricoEmpresa> getListaHistorico() {
	return listaHistorico;
    }

    public CadSituacaoCadastroD getSituacaoCadastro() {
	return situacaoCadastro;
    }

    public SegUsuario getAnalista() {
	return analista;
    }

    public void setAnalista(SegUsuario analista) {
	this.analista = analista;
    }

    public String getDescricaoAnaliseTecnica() {
	return descricaoAnaliseTecnica;
    }

    public void setDescricaoAnaliseTecnica(String descricaoAnaliseTecnica) {
	this.descricaoAnaliseTecnica = descricaoAnaliseTecnica;
    }

    public List<CadServico> getServicos() {
	return servicos;
    }

    public void setServicos(List<CadServico> servicos) {
	this.servicos = servicos;
    }

    public List<CadProcuracao> getProcuracaoOutorgante() {
	if (procuracaoOutorgante == null) {
	    procuracaoOutorgante = new ArrayList<CadProcuracao>();
	}
	return procuracaoOutorgante;
    }

    public void setProcuracaoOutorgante(List<CadProcuracao> procuracaoOutorgante) {
	this.procuracaoOutorgante = procuracaoOutorgante;
    }

    public List<CadProcuracao> getProcuracaoOutorgado() {
	if (procuracaoOutorgado == null) {
	    procuracaoOutorgado = new ArrayList<CadProcuracao>();
	}
	return procuracaoOutorgado;
    }

    public void setProcuracaoOutorgado(List<CadProcuracao> procuracaoOutorgado) {
	this.procuracaoOutorgado = procuracaoOutorgado;
    }

    public String getNomeRepresentante() {
	return nomeRepresentante;
    }

    public void setNomeRepresentante(String nomeRepresentante) {
	this.nomeRepresentante = nomeRepresentante;
    }

    public String getCpfRepresentante() {
	if (!cpfRepresentante.isEmpty()) {
	    return CpfCnpjUtil.FormatCPF(cpfRepresentante);
	}
	return cpfRepresentante;
    }

    public void setCpfRepresentante(String cpfRepresentante) {
	this.cpfRepresentante = cpfRepresentante;
    }

}