package br.gov.mc.cadsei.entidade;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;

import br.gov.arquitetura.entidade.Entidade;
import br.gov.arquitetura.util.StringUtil;
import br.gov.mc.seg.entidade.SegUsuario;

@Entity
@Table(name = "CAD_PETICAO", catalog = "CADSEI")
public class CadPeticao extends Entidade<Integer> {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "ID_PETICAO", unique = true, nullable = false)
	private Integer id;

	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_PROPONENTE", nullable = false)
	private CadProponente proponente;	

	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_SITUACAO_PETICAO", nullable = false)
	private CadSituacaoPeticaoD situacao;
	
	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "ID_EMPRESA")
	private CadEmpresa empresa;	
	
	@ManyToOne(fetch=FetchType.LAZY)
	@JoinColumn(name = "ID_ANALISTA")
	private SegUsuario analista;	
	
	@Temporal(TemporalType.DATE)
	@Column(name = "DT_PETICAO", nullable = false)
	private Date dataPeticao;	

	@Column(name = "FLG_EXISTE_PROTOCOLO", nullable = false)
	private Boolean existeProtocolo;
	
	@Column(name = "NUM_PROTOCOLO")
	private BigDecimal protocolo;
	
	@Column(name = "DSC_ASSUNTO", nullable = false)
	private String assunto;
	
	@Column(name = "DSC_PENDENCIA")
	private String pendencia;
	
	@Column(name = "DSC_PROVIDENCIA")
	private String providencia;

	@Cascade({CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN})
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "peticao")
	private List<CadDocumentoPeticao> documentos;

	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_AREA")
	private CadArea area;

	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_SERVICO")
	private CadServico servico;

	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_SOLICITACAO")
	private CadSolicitacao solicitacao;

	@Transient
	private transient String numeroProtocolo;
	
	@Transient
	private transient String digitoNumeroProtocolo;
	
	
	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "ID_PROCURACAO")
	private CadProcuracao procuracao;


	public CadPeticao(){ }

	public CadPeticao(Integer id){ 
		this.id = id;
	}
	
	@Override
	public Integer getId() {
		return id;
	}

	@Override
	public void setId(Integer id) {
		this.id = id;
	}


	public CadProponente getProponente() {
		return proponente;
	}

	public void setProponente(CadProponente proponente) {
		this.proponente = proponente;
	}

	public CadSituacaoPeticaoD getSituacao() {
		if(this.situacao == null){
			this.situacao = new CadSituacaoPeticaoD();
		}
		return this.situacao;
	}

	public void setSituacao(CadSituacaoPeticaoD situacao) {
		this.situacao = situacao;
	}

	public CadEmpresa getEmpresa() {
		if(this.empresa == null){
			this.empresa = new CadEmpresa();
		}
		return this.empresa;
	}

	public void setEmpresa(CadEmpresa empresa) {
		this.empresa = empresa;
	}

	public SegUsuario getAnalista() {
		if(this.analista == null){
			this.analista = new SegUsuario();
		}
		return this.analista;
	}

	public void setAnalista(SegUsuario analista) {
		this.analista = analista;
	}

	public Date getDataPeticao() {
		return dataPeticao;
	}

	public void setDataPeticao(Date dataPeticao) {
		this.dataPeticao = dataPeticao;
	}

	public Boolean getExisteProtocolo() {
		return existeProtocolo;
	}

	public void setExisteProtocolo(Boolean existeProtocolo) {
		this.existeProtocolo = existeProtocolo;
	}

	public BigDecimal getProtocolo() {
		return protocolo;
	}

	public void setProtocolo(BigDecimal protocolo) {
		this.protocolo = protocolo;
	}

	public String getAssunto() {
		return assunto;
	}

	public void setAssunto(String assunto) {
		this.assunto = assunto;
	}

	public String getPendencia() {
		return pendencia;
	}

	public void setPendencia(String pendencia) {
		this.pendencia = pendencia;
	}

	public String getProvidencia() {
		return providencia;
	}

	public void setProvidencia(String providencia) {
		this.providencia = providencia;
	}

	public List<CadDocumentoPeticao> getDocumentos() {
		if(this.documentos == null){
			this.documentos = new ArrayList<CadDocumentoPeticao>();
		}
		return this.documentos;
	}

	public void setDocumentos(List<CadDocumentoPeticao> documentos) {
		this.documentos = documentos;
	}
	
	public void addDocumento(CadDocumentoPeticao documento){
		documento.setPeticao(this);
		this.getDocumentos().add(documento);
	}

	public String getNumeroProtocolo() {
		if(this.protocolo != null){
			if(this.protocolo.toString().length() == 17){
				return this.protocolo.toString().substring(0, 15);
			}
		}
		return this.numeroProtocolo;
	}

	public void setNumeroProtocolo(String numeroProtocolo) {
		this.numeroProtocolo = numeroProtocolo;
	}

	public String getDigitoNumeroProtocolo() {
		if(this.protocolo != null){
			if(this.protocolo.toString().length() == 17){
				return this.protocolo.toString().substring(15, 17);
			}
		}
		return this.digitoNumeroProtocolo;
	}

	public void setDigitoNumeroProtocolo(String digitoNumeroProtocolo) {
		this.digitoNumeroProtocolo = digitoNumeroProtocolo;
	}

	public String getNumeroProcessoFormatado(){
		if(this.protocolo != null){
		        if(this.protocolo.toString().length()==16){
			   return StringUtil.format("#####.######/####-##","0"+this.protocolo.toString());
		        }else{
		           return this.protocolo.toString();
		        }
		}
		return null;
	}

	public CadArea getArea() {
		if (area == null) {
			area = new CadArea();
		}
		return area;
	}

	public void setArea(CadArea area) {
		this.area = area;
	}

	public CadServico getServico() {
		if (servico == null) {
			servico = new CadServico();
		}
		return servico;
	}

	public void setServico(CadServico servico) {
		this.servico = servico;
	}

	public CadSolicitacao getSolicitacao() {
		if (solicitacao == null) {
			solicitacao = new CadSolicitacao();
		}
		return solicitacao;
	}

	public void setSolicitacao(CadSolicitacao solicitacao) {
		this.solicitacao = solicitacao;
	}

	public CadProcuracao getProcuracao() {
		if (procuracao == null) {
			procuracao = new CadProcuracao();
		}
		return procuracao;
	}

	public void setProcuracao(CadProcuracao procuracao) {
		this.procuracao = procuracao;
	}
	
}