package br.gov.mc.cadsei.entidade;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;

import br.gov.arquitetura.entidade.Entidade;

@Entity
@Table(name = "CAD_PROCURACAO", catalog = "CADSEI")
public class CadProcuracao extends Entidade<Integer> {

	private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "ID_PROCURACAO", unique = true, nullable = false)
	private Integer id;

	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_PROCURACAO_PAI")
	private CadProcuracao procuracaoPai;

	@OneToOne(fetch = FetchType.EAGER)
	@JoinColumn(name = "ID_STATUS_PROCURACAO", nullable = false)
	private CadStatusProcuracaoD statusProcuracao;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_PROPONENTE_OUTORGANTE")
	private CadProponente proponenteOutorgante;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_PROPONENTE_OUTORGADO")
	private CadProponente proponenteOutorgado;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_EMPRESA_OUTORGANTE")
	private CadEmpresa empresaOutorgante;

	@ManyToOne(fetch = FetchType.LAZY)
	@JoinColumn(name = "ID_EMPRESA_OUTORGADO")
	private CadEmpresa empresaOutorgado;

	@Column(name = "NR_SEQUENCIAL", nullable = false)
	private Integer nrSequencial;

	@Temporal(TemporalType.DATE)
	@Column(name = "DT_VALIDADE_INICIAL", nullable = false)
	private Date dataValidadeInicial;

	@Temporal(TemporalType.DATE)
	@Column(name = "DT_VALIDADE_FINAL", nullable = false)
	private Date dataValidadeFinal;

	@Column(name = "FLG_PERMITE_SUBESTABELECIMENTO")
	private Boolean flgPermiteSubstabelecimento;

	@Cascade({ CascadeType.ALL})
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "procuracao")
	private List<CadProcuracaoSolicitacao> procuracaoSolicitacao;
	
	@Transient
	private String ano;
	
	@Transient
	private String numeroSequencialZeros;

	public CadProcuracao() {
	}

	public CadProcuracao(Integer id) {
		this.id = id;
	}

	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public CadProcuracao getProcuracaoPai() {
		if (procuracaoPai == null) {
			procuracaoPai = new CadProcuracao();
		}
		return procuracaoPai;
	}

	public void setProcuracaoPai(CadProcuracao procuracaoPai) {
		this.procuracaoPai = procuracaoPai;
	}

	public CadStatusProcuracaoD getStatusProcuracao() {
		if (statusProcuracao == null) {
			statusProcuracao = new CadStatusProcuracaoD();
		}
		return statusProcuracao;
	}

	public void setStatusProcuracao(CadStatusProcuracaoD statusProcuracao) {
		this.statusProcuracao = statusProcuracao;
	}

	public CadProponente getProponenteOutorgante() {
		if (proponenteOutorgante == null) {
			proponenteOutorgante = new CadProponente();
		}
		return proponenteOutorgante;
	}

	public void setProponenteOutorgante(CadProponente proponenteOutorgante) {
		this.proponenteOutorgante = proponenteOutorgante;
	}

	public CadProponente getProponenteOutorgado() {
		if (proponenteOutorgado == null) {
			proponenteOutorgado = new CadProponente();
		}
		return proponenteOutorgado;
	}

	public void setProponenteOutorgado(CadProponente proponenteOutorgado) {
		this.proponenteOutorgado = proponenteOutorgado;
	}

	public CadEmpresa getEmpresaOutorgante() {
		if (empresaOutorgante == null) {
			empresaOutorgante = new CadEmpresa();
		}
		return empresaOutorgante;
	}

	public void setEmpresaOutorgante(CadEmpresa empresaOutorgante) {
		this.empresaOutorgante = empresaOutorgante;
	}

	public CadEmpresa getEmpresaOutorgado() {
		if (empresaOutorgado == null) {
			empresaOutorgado = new CadEmpresa();
		}
		return empresaOutorgado;
	}

	public void setEmpresaOutorgado(CadEmpresa empresaOutorgado) {
		this.empresaOutorgado = empresaOutorgado;
	}

	public Integer getNrSequencial() {
		return nrSequencial;
	}

	public void setNrSequencial(Integer nrSequencial) {
		this.nrSequencial = nrSequencial;
	}

	public Date getDataValidadeInicial() {
		return dataValidadeInicial;
	}

	public void setDataValidadeInicial(Date dataValidadeInicial) {
		this.dataValidadeInicial = dataValidadeInicial;
	}

	public Date getDataValidadeFinal() {
		return dataValidadeFinal;
	}

	public void setDataValidadeFinal(Date dataValidadeFinal) {
		this.dataValidadeFinal = dataValidadeFinal;
	}

	public Boolean getFlgPermiteSubstabelecimento() {
		return flgPermiteSubstabelecimento;
	}

	public void setFlgPermiteSubstabelecimento(Boolean flgPermiteSubstabelecimento) {
		this.flgPermiteSubstabelecimento = flgPermiteSubstabelecimento;
	}

	public List<CadProcuracaoSolicitacao> getProcuracaoSolicitacao() {
		if (procuracaoSolicitacao == null) {
			procuracaoSolicitacao = new ArrayList<CadProcuracaoSolicitacao>();
		}
		return procuracaoSolicitacao;
	}

	public void setProcuracaoSolicitacao(List<CadProcuracaoSolicitacao> procuracaoSolicitacao) {
		this.procuracaoSolicitacao = procuracaoSolicitacao;
	}
	
	public String getAno() {
		if(this.getDataValidadeInicial() != null){
			Calendar ano = Calendar.getInstance();
			ano.setTime(this.getDataValidadeInicial());
			return String.valueOf(ano.get(Calendar.YEAR)).substring(2, 4);
		}
		return "";
	}
	
	public String getNumeroSequencialZeros(){
		return String.format ("%04d", this.getNrSequencial());
	}
	
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((dataValidadeFinal == null) ? 0 : dataValidadeFinal.hashCode());
		result = prime * result + ((dataValidadeInicial == null) ? 0 : dataValidadeInicial.hashCode());
		result = prime * result + ((empresaOutorgante == null) ? 0 : empresaOutorgante.hashCode());
		result = prime * result + (flgPermiteSubstabelecimento ? 1231 : 1237);
		result = prime * result + ((id == null) ? 0 : id.hashCode());
		result = prime * result + ((nrSequencial == null) ? 0 : nrSequencial.hashCode());
		result = prime * result + ((procuracaoPai == null) ? 0 : procuracaoPai.hashCode());
		result = prime * result + ((procuracaoSolicitacao == null) ? 0 : procuracaoSolicitacao.hashCode());
		result = prime * result + ((proponenteOutorgado == null) ? 0 : proponenteOutorgado.hashCode());
		result = prime * result + ((proponenteOutorgante == null) ? 0 : proponenteOutorgante.hashCode());
		result = prime * result + ((statusProcuracao == null) ? 0 : statusProcuracao.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CadProcuracao other = (CadProcuracao) obj;
		if (dataValidadeFinal == null) {
			if (other.dataValidadeFinal != null)
				return false;
		} else if (!dataValidadeFinal.equals(other.dataValidadeFinal))
			return false;
		if (dataValidadeInicial == null) {
			if (other.dataValidadeInicial != null)
				return false;
		} else if (!dataValidadeInicial.equals(other.dataValidadeInicial))
			return false;
		if (empresaOutorgante == null) {
			if (other.empresaOutorgante != null)
				return false;
		} else if (!empresaOutorgante.equals(other.empresaOutorgante))
			return false;
		if (flgPermiteSubstabelecimento != other.flgPermiteSubstabelecimento)
			return false;
		if (id == null) {
			if (other.id != null)
				return false;
		} else if (!id.equals(other.id))
			return false;
		if (nrSequencial == null) {
			if (other.nrSequencial != null)
				return false;
		} else if (!nrSequencial.equals(other.nrSequencial))
			return false;
		if (procuracaoPai == null) {
			if (other.procuracaoPai != null)
				return false;
		} else if (!procuracaoPai.equals(other.procuracaoPai))
			return false;
		if (procuracaoSolicitacao == null) {
			if (other.procuracaoSolicitacao != null)
				return false;
		} else if (!procuracaoSolicitacao.equals(other.procuracaoSolicitacao))
			return false;
		if (proponenteOutorgado == null) {
			if (other.proponenteOutorgado != null)
				return false;
		} else if (!proponenteOutorgado.equals(other.proponenteOutorgado))
			return false;
		if (proponenteOutorgante == null) {
			if (other.proponenteOutorgante != null)
				return false;
		} else if (!proponenteOutorgante.equals(other.proponenteOutorgante))
			return false;
		if (statusProcuracao == null) {
			if (other.statusProcuracao != null)
				return false;
		} else if (!statusProcuracao.equals(other.statusProcuracao))
			return false;
		return true;
	}

}