package br.gov.mc.cadsei.entidade;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.annotations.Cascade;
import org.hibernate.annotations.CascadeType;
import org.hibernate.annotations.LazyCollection;
import org.hibernate.annotations.LazyCollectionOption;

import br.gov.arquitetura.entidade.Entidade;
import br.gov.mc.cadsei.util.CpfCnpjUtil;
import br.gov.mc.corporativo.entidade.CrpContatoTelefone;
import br.gov.mc.corporativo.entidade.CrpDocumentoPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoDocPessoaFisica;
import br.gov.mc.corporativo.entidade.CrpTipoTelefone;
import br.gov.mc.corporativo.entidade.CrpUf;
import br.gov.mc.seg.entidade.SegUsuario;

@Entity
@Table(name = "CAD_PROPONENTE", catalog = "CADSEI")
public class CadProponente extends Entidade<Integer> {

    private static final long serialVersionUID = 1L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	@Column(name = "ID_PROPONENTE", unique = true, nullable = false)
	private Integer id;
	
	@OneToOne(fetch = FetchType.LAZY)
	@JoinColumn(name="ID_PESSOA", nullable = false)
	private CrpPessoaFisica pessoaProponente;
	
	@ManyToOne(fetch = FetchType.LAZY, optional = true)
	@JoinColumn(name = "ID_UF")
	private CrpUf siglaUF;
	
	@ManyToOne(fetch=FetchType.LAZY, optional = true)
	@JoinColumn(name = "ID_ANALISTA")
	private SegUsuario analista;	
	
	@ManyToOne(fetch = FetchType.EAGER, optional = true)
	@JoinColumn(name = "ID_SITUACAO_CADASTRO")
	private CadSituacaoCadastroD situacaoCadastro;
	
	@Column(name = "NUM_IP_PROPONENTE", nullable = false)
	private String ipProponete;
	
	@Column(name = "DSC_LOGIN_PROPONENTE", nullable = false)
	private String loginProponente;
	
	@Column(name = "DSC_SENHA_PROPONENTE", nullable = false)
	private String senhaProponente;
	
	@Column(name = "DSC_EMAIL_PROPONENTE", nullable = false)
	private String emailProponente;

	@Temporal(TemporalType.TIMESTAMP)
	@Column(name = "DT_CADASTRO", nullable = false)
	private Date dataCadastro;

	@Column(name = "FLG_POSSUI_OAB", nullable = true)
	private Boolean possuiOab;
	
	@Column(name = "NUM_REGISTRO_OAB", nullable = true)
	private String numeroRegistroOab;
	
	@Column(name = "DSC_ANALISE_TECNICA", nullable = true)
	private String descricaoAnaliseTecnica;
	
	@Transient
	private transient Boolean evento;
	
	@Cascade({CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN})
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "proponente")
	private List<CadDocumento> documentos;

	@OneToMany(mappedBy = "proponente")
	@LazyCollection(LazyCollectionOption.FALSE)
	private List<CadEmpresa> empresas;
	
	@Cascade({CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN})
	@OneToMany( fetch = FetchType.EAGER ,mappedBy="proponente")
	private List<CadHistoricoProponente> listaHistorico;
	
	@Cascade({ CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN })
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "proponenteOutorgante")
	private List<CadProcuracao> procuracaoOutorgante;

	@Cascade({ CascadeType.SAVE_UPDATE, CascadeType.DELETE, CascadeType.DELETE_ORPHAN })
	@OneToMany(fetch = FetchType.LAZY, mappedBy = "proponenteOutorgado")
	private List<CadProcuracao> procuracaoOutorgado;

	
	
	@Transient
	private transient String senha;
	
	public CadProponente(){
	}
	
	public CadProponente(Integer id) {
		this.id = id;
	}
	
	public Integer getId() {
		return id;
	}

	public void setId(Integer id) {
		this.id = id;
	}

	public CrpPessoaFisica getPessoaProponente() {
		return pessoaProponente;
	}

	public void setPessoaProponente(CrpPessoaFisica pessoaProponente) {
		this.pessoaProponente = pessoaProponente;
	}

	public CrpUf getSiglaUF() {
		return siglaUF;
	}

	public void setSiglaUF(CrpUf siglaUF) {
		this.siglaUF = siglaUF;
	}

	public String getIpProponete() {
		return ipProponete;
	}

	public void setIpProponete(String ipProponete) {
		this.ipProponete = ipProponete;
	}

	public String getSenhaProponente() {
		return senhaProponente;
	}

	public void setSenhaProponente(String senhaProponente) {
		this.senhaProponente = senhaProponente;
	}

	public String getLoginProponente() {
		return loginProponente;
	}

	public void setLoginProponente(String loginProponente) {
		this.loginProponente = loginProponente;
	}	
	
	public void setCPFProponente(CrpDocumentoPessoaFisica documento){
		if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.CPF) ){
			if(this.getPessoaProponente().getDocumentos() == null){
				this.getPessoaProponente().setDocumentos(new ArrayList<CrpDocumentoPessoaFisica>());
			}
			this.getPessoaProponente().getDocumentos().add(documento);
		}
	}
	
	public String getCPFProponente(){
		for (CrpDocumentoPessoaFisica documento : this.getPessoaProponente().getDocumentos()) {
			if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.CPF)){
				return documento.getNumero();
			}
		}
		return null;
	}
	
	public void setTelefoneContato(Integer ddd, Integer numTelefone){
		CrpContatoTelefone telefone = new CrpContatoTelefone();
		telefone.setPessoa(pessoaProponente);
		telefone.setNumDdd(ddd.toString());
		telefone.setNumTelefone(numTelefone.toString());
		telefone.setTipoTelefone(new CrpTipoTelefone(CrpTipoTelefone.CONTATO));
		if(this.getPessoaProponente().getTelefones() == null){
			this.getPessoaProponente().setTelefones(new ArrayList<CrpContatoTelefone>());
		}
		this.getPessoaProponente().getTelefones().add(telefone);
	}
	
	public void setCelularContato(Integer ddd, Integer numCelular){
		CrpContatoTelefone celular = new CrpContatoTelefone();
		celular.setPessoa(pessoaProponente);
		celular.setNumDdd(ddd.toString());
		celular.setNumTelefone(numCelular.toString());
		celular.setTipoTelefone(new CrpTipoTelefone(CrpTipoTelefone.CELULAR));
		if(this.getPessoaProponente().getTelefones() == null){
			this.getPessoaProponente().setTelefones(new ArrayList<CrpContatoTelefone>());
		}
		this.getPessoaProponente().getTelefones().add(celular);
	}

	public void setRgProponente(CrpDocumentoPessoaFisica documento) {
		if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG)){
			if(this.getPessoaProponente() == null){
				this.setPessoaProponente(new CrpPessoaFisica());
			}
			this.getPessoaProponente().getDocumentos().add(documento);
		}
	}
	
	public String getRgProponente(){
		for (CrpDocumentoPessoaFisica documento : this.getPessoaProponente().getDocumentos()) {
			if(documento.getTipoDocumento().getId().equals(CrpTipoDocPessoaFisica.RG)){
				return documento.getNumero();
			}
		}
		return null;
	}

	public String getEmailProponente() {
		return emailProponente;
	}

	public void setEmailProponente(String emailProponente) {
		this.emailProponente = emailProponente;
	}

	public List<CadDocumento> getDocumentos() {
		if(this.documentos == null){
			this.documentos = new ArrayList<CadDocumento>();
		}
		return this.documentos;
	}

	public void setDocumentos(List<CadDocumento> documentos) {
		this.documentos = documentos;
	}
	
	public void addDocumento(CadDocumento documento){
		documento.setProponente(this);
		this.getDocumentos().add(documento);
	}

	public List<CadEmpresa> getEmpresas() {
		return this.empresas;
	}

	public void setEmpresas(List<CadEmpresa> empresas) {
		this.empresas = empresas;
	}
	
	public String getLoginFormated(){
		return CpfCnpjUtil.FormatCPF(getLoginProponente());
	}

	public Boolean getEvento() {
		return evento;
	}

	public void setEvento(Boolean evento) {
		this.evento = evento;
	}

	public Date getDataCadastro() {
		if(this.dataCadastro == null){
			this.dataCadastro = new Date();
		}
		return this.dataCadastro;
	}

	public void setDataCadastro(Date dataCadastro) {
		this.dataCadastro = dataCadastro;
	}

	public CadSituacaoCadastroD getSituacaoCadastro() {
		return this.situacaoCadastro;
	}

	public void setSituacaoCadastro(CadSituacaoCadastroD situacaoCadastral) {
		this.situacaoCadastro = situacaoCadastral;
	}

	public String getDescricaoAnaliseTecnica() {
		return descricaoAnaliseTecnica;
	}

	public void setDescricaoAnaliseTecnica(String descricaoAnaliseTecnica) {
		this.descricaoAnaliseTecnica = descricaoAnaliseTecnica;
	}

	public SegUsuario getAnalista() {
		if(this.analista == null){
			this.analista = new SegUsuario();
		}
		return this.analista;
	}

	public void setAnalista(SegUsuario analista) {
		this.analista = analista;
	}
	
	public Boolean getPossuiOab() {
		return possuiOab;
	}

	public void setPossuiOab(Boolean possuiOab) {
		this.possuiOab = possuiOab;
	}

	public String getNumeroRegistroOab() {
		return numeroRegistroOab;
	}

	public void setNumeroRegistroOab(String numeroOab) {
		this.numeroRegistroOab = numeroOab;
	}

	public String getSenha() {
		return senha;
	}

	public void setSenha(String senha) {
		this.senha = senha;
	}

	public List<CadHistoricoProponente> getListaHistorico() {
	    return listaHistorico;
	}

	public void setListaHistorico(List<CadHistoricoProponente> listaHistorico) {
	    this.listaHistorico = listaHistorico;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((pessoaProponente == null) ? 0 : pessoaProponente.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CadProponente other = (CadProponente) obj;
		if (pessoaProponente == null) {
			if (other.pessoaProponente != null)
				return false;
		} else if (!pessoaProponente.equals(other.pessoaProponente))
			return false;
		return true;
	}

	public List<CadProcuracao> getProcuracaoOutorgante() {
		return procuracaoOutorgante;
	}

	public void setProcuracaoOutorgante(List<CadProcuracao> procuracaoOutorgante) {
		this.procuracaoOutorgante = procuracaoOutorgante;
	}

	public List<CadProcuracao> getProcuracaoOutorgado() {
		return procuracaoOutorgado;
	}

	public void setProcuracaoOutorgado(List<CadProcuracao> procuracaoOutorgado) {
		this.procuracaoOutorgado = procuracaoOutorgado;
	}

}