package br.gov.mc.cadsei.service.impl;

import java.util.List;
import java.util.Map;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.mc.cadsei.dao.EmpresaDao;
import br.gov.mc.cadsei.entidade.CadDocumento;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadServico;
import br.gov.mc.cadsei.service.EmpresaService;
import br.gov.mc.corporativo.entidade.CrpRepresentantePesJur;

@Service(value = "empresaService")
@Transactional(readOnly = true, rollbackFor = Exception.class, propagation = Propagation.SUPPORTS)
public class EmpresaServiceImpl implements EmpresaService {

	@Autowired(required = true)
	@Qualifier(value = "empresaDao")
	protected EmpresaDao dao;

	
	public List<String> buscaEmpresaPorRazaoSocial(String nome) {
		return dao.buscaEmpresaPorRazaoSocial(nome);
	}

	
	public List<CadEmpresa> pesquisaParametrizada(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filtros) {
		List<CadEmpresa> empresas = dao.buscaParametrizada(first, pageSize, sortField, sortOrder, filtros);
		for (CadEmpresa e : empresas) {
			Hibernate.initialize(e.getProponente());
		}
		return empresas;
	}

	
	public Long totalRow(Map<String, String> filtros) {
		return dao.rowCount(filtros);
	}

	
	public List<CadDocumento> pesquisaDocumentosPorEmpresa(Integer id) {
		return dao.buscaDocumentosPorEmpresa(id);
	}
	
	public CrpRepresentantePesJur buscaRepresentantePorEmpresa(Integer id){
		return dao.buscaRepresentantePorEmpresa(id);
	}

	
	public List<CadServico> buscaServicos() {
		return dao.buscaServicos();
	}

	
	public CadServico buscaServicoPorId(Integer id) {
		CadServico s = dao.buscaServicoPorId(id);
		if(s.getProcuracaoSolicitacao()!= null)
		Hibernate.initialize(s.getProcuracaoSolicitacao());
		return s;
	}

	
	
	public CadEmpresa buscaEmpresaPorId(Integer id) {
		CadEmpresa e = dao.buscaEmpresaPorId(id);
		//Hibernate.initialize(e.getServicos());
		Hibernate.initialize(e.getDocumentos());
		Hibernate.initialize(e.getProponente());
		Hibernate.initialize(e.getProponente().getPessoaProponente());
		return e;
	}

	
	public List<CadEmpresa> buscaEmpresaPorProponente(Integer id) {
		return dao.buscaEmpresaPorProponente(id);
	}

	
	public List<CadEmpresa> buscaEmpresaPorRepresentante(Integer id) {
		return dao.buscaEmpresaPorRepresentante(id);
	}

	
	public List<CadServico> buscaServicosPorEmpresa(Integer idEmpresa) {
		return dao.buscaServicosPorEmpresa(idEmpresa);
	}

	
	public void validaCadastroEmpresa(String cpf) throws Exception {
		if(!dao.validaCadastroEmpresa(cpf)){
			throw new Exception("CNPJ j&aacute; cadastrado!");
	}
	}
	/**
	 *Este método retornará a lista de empresas por representante com base no CPF do proponente (login).
	 * */
	public List<CadEmpresa> listaEmpresaPorRepresentante(String cpf) {
		List<CadEmpresa> e = dao.listaEmpresaPorRepresentante(cpf);
		return e;
	}
}
