package br.gov.mc.cadsei.service.impl;

import org.hibernate.Criteria;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.mc.cadsei.dao.GenericDao;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.dto.QuantitativoDto;
import br.gov.mc.cadsei.service.GenericService;
import br.gov.mc.seg.entidade.SegUsuario;

@Service(value = "genericService")
@Transactional(readOnly = true, rollbackFor = Exception.class, propagation = Propagation.SUPPORTS)
public class GenericServiceImpl implements GenericService {

	@Autowired(required = true)
	@Qualifier(value = "genericDao")
	protected GenericDao dao;
	
	
	@SuppressWarnings("rawtypes")
	public Criteria createCriteria(Class klass) {
		return this.dao.createCriteria(klass);
	}

	
	@Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
	public Object save(Object o) throws Exception {
		return this.dao.save(o);
	}

	
	@Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
	public CadEmpresa saveEmpresa(CadEmpresa empresa) throws Exception {
		this.dao.save(empresa);
		empresa.getProponente().setDescricaoAnaliseTecnica(null);
		empresa.getProponente().setAnalista(null);
		this.dao.save(empresa.getProponente());
		return empresa;
	}
	
	

	
	public boolean isEmAnalise(SegUsuario analista) {
		return this.dao.isEmAnalise(analista);
	}

	
	public QuantitativoDto getDadosRelatorio() {
		return this.dao.getDadosRelatorio();
	}


	/* (non-Javadoc)
	 * @see br.gov.mc.cadsei.service.GenericService#obtemValorParametro(java.lang.String)
	 */
	public String obtemValorParametro(String nome) {
		return this.dao.obtemValorParametro(nome);
	}
	
	
}