package br.gov.mc.cadsei.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.mc.cadsei.dao.PeticaoDao;
import br.gov.mc.cadsei.entidade.CadArea;
import br.gov.mc.cadsei.entidade.CadAreaServico;
import br.gov.mc.cadsei.entidade.CadDocumentoPeticao;
import br.gov.mc.cadsei.entidade.CadPeticao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.entidade.CadSeiIntegracao;
import br.gov.mc.cadsei.entidade.CadServicoSolicitacao;
import br.gov.mc.cadsei.entidade.dto.CprdProtocoloDto;
import br.gov.mc.cadsei.service.PeticaoService;

@Service(value = "peticaoPrtoService")
@Transactional(readOnly = true, rollbackFor = Exception.class, propagation = Propagation.SUPPORTS)
public class PeticaoServiceImpl implements PeticaoService {

    @Autowired(required = true)
    @Qualifier(value = "peticaoPrtoDao")
    protected PeticaoDao dao;

    
    @Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
    public CadPeticao save(CadPeticao peticao) throws Exception {
	return this.dao.save(peticao);
    }

    
    public CadPeticao getPeticaoByProponente(CadProponente proponente) {
	return this.dao.getPeticaoByProponente(proponente);
    }

    
    public List<CadDocumentoPeticao> getDocumentosPeticao(CadPeticao peticao) {
	return this.dao.getDocumentosPeticao(peticao);
    }

    
    public CprdProtocoloDto getProtocolo(String numero) {
	CprdProtocoloDto dto = null;
	numero = numero.replace(".", "").replace("/", "").replace("-", "");
	dto = this.dao.getProtocoloSEI(numero);
	if (dto != null && dto.getId() != null) {
	    return dto;
	}
	return this.dao.getProtocolo(numero);
    }

    
    public List<CadArea> findAreaByProcuracao(Integer idProcuracao) {
	return this.dao.findAreaByProcuracao(idProcuracao);
    }

    
    public List<CadAreaServico> findServicoByProcuracao(Integer idArea, Integer idProcuracao) {
	return this.dao.findServicoByProcuracao(idArea, idProcuracao);
    }

    
    public List<CadServicoSolicitacao> findSolicitacaoByProcuracao(Integer idServico, Integer idProcuracao) {
	return this.dao.findSolicitacaoByProcuracao(idServico, idProcuracao);
    }

    
    public CadSeiIntegracao findSeiIntegracaoByIdPedticao(CadPeticao peticao) {
	return this.dao.findSeiIntegracaoByIdPedticao(peticao);
    }

	
	public List<CadPeticao> findPeticoes(CadProponente proponente, String tipoPessoa) {
		return this.dao.findPeticoes(proponente, tipoPessoa);
	}

	
	public CadPeticao getByPeticao(CadPeticao peticao) {
		return this.dao.getByPeticao(peticao);
	}

}