package br.gov.mc.cadsei.service.impl;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.mc.cadsei.dao.ProcuracaoDao;
import br.gov.mc.cadsei.entidade.CadEmpresa;
import br.gov.mc.cadsei.entidade.CadProcuracao;
import br.gov.mc.cadsei.entidade.CadStatusProcuracaoD;
import br.gov.mc.cadsei.entidade.dto.ResultadoOutorgadoDto;
import br.gov.mc.cadsei.service.ProcuracaoService;
import br.gov.mc.corporativo.entidade.CrpContatoEmail;
import br.gov.mc.corporativo.entidade.CrpPessoaJuridica;

@Service(value = "procuracaoService")
@Transactional(readOnly = true, rollbackFor = Exception.class, propagation = Propagation.SUPPORTS)
public class ProcuracaoServiceImpl implements ProcuracaoService {

	@Autowired(required = true)
	@Qualifier(value = "procuracaoDao")
	protected ProcuracaoDao dao;

	
	@Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
	public CadProcuracao save(CadProcuracao procuracao) throws Exception {
		return this.dao.save(procuracao);
	}
	
	
	@Transactional(readOnly = false, rollbackFor = Exception.class, propagation = Propagation.REQUIRED)
	public CadProcuracao merge(CadProcuracao procuracao) throws Exception {
		return this.dao.merge(procuracao);
	}

	
	public CadProcuracao getByProcuracao(CadProcuracao procuracao) {
		return this.dao.getByProcuracao(procuracao);	
	}
	
	
	public CadProcuracao getByProcuracaoPai(CadProcuracao procuracao) {
		return this.dao.getByProcuracaoPai(procuracao);	
	}


	
	public List<CadEmpresa> getEmpresaByProponente(Integer idProponenteLogado) {
		return this.dao.getEmpresaByProponente(idProponenteLogado);
	}

	
	public CadProcuracao findProcuracaoByOutorgadoCPF(String cpf, Integer idEmpresaOutorgante, Integer idProponenteOutorgante) {
		return this.dao.findProcuracaoByOutorgadoCPF(cpf, idEmpresaOutorgante, idProponenteOutorgante);
	}

	
	public Integer getNumeroSequencial(Integer idProponente, Integer idEmpresa) {
		return this.dao.getNumeroSequencial(idProponente, idEmpresa);
	}
	
	
	public CadProcuracao findProcuracaoByOutorgadoCNPJ(String cnpj, Integer idEmpresaOutorgante, Integer idProponenteOutorgante) {
		return this.dao.findProcuracaoByOutorgadoCNPJ(cnpj, idEmpresaOutorgante, idProponenteOutorgante);
	}

	
	public CadEmpresa recuperarEmpresa(String cnpj) {
		return this.dao.recuperarEmpresa(cnpj);
	}

	
	public List<ResultadoOutorgadoDto> findProcuracaoEnviadas(String cpf) {
		return this.dao.findProcuracaoEnviadas(cpf);
	}
	
	
	public List<ResultadoOutorgadoDto> findProcuracaoRecebidas(String cpf, boolean ativo) {
		return this.dao.findProcuracaoRecebidas(cpf, ativo);
	}

	
	public List<CadProcuracao> findAllProcuracaoAtivas() {
		return this.dao.findAllProcuracaoAtivas();
	}

	
	public void revogarProcuracao(CadProcuracao procuracao) throws Exception {
		procuracao.setStatusProcuracao(new CadStatusProcuracaoD(CadStatusProcuracaoD.REVOGADA));
		
		// Efetuado UPDATE na mão mesmo, devido ao hibernate não executar tanto com SAVE e MERGE.
		this.dao.update(procuracao);
		
		// Recupera procuracao Pai
		CadProcuracao procuracaoPai = this.dao.getByProcuracaoPai(procuracao);
		if(procuracaoPai != null){
			// Retornar ao metodo com a procuracao pai encontrada
			revogarProcuracao(procuracaoPai);
		}
	}

	
	public void update(CadProcuracao procuracao) throws Exception {
		this.dao.update(procuracao);
	}

	
	public List<CrpContatoEmail> obtemEmailsPessoaTitularPorEmpresaOrtogado(Integer id) {
		return dao.obtemEmailsPessoaTitularPorEmpresaOrtogado(id);
		
	}
	
}