package br.gov.mc.cadsei.service.impl;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.hibernate.Hibernate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import br.gov.mc.cadsei.dao.ProponenteDao;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.cadsei.service.ProponenteService;
import br.gov.mc.corporativo.entidade.CrpCorRaca;

@Service(value = "proponenteService")
@Transactional(readOnly = true, rollbackFor = Exception.class, propagation = Propagation.SUPPORTS)
public class ProponenteServiceImpl implements ProponenteService {

	@Autowired(required = true)
	@Qualifier(value = "proponenteDao")
	protected ProponenteDao dao;

	
	public List<CadProponente> getProponentes() {
		return this.dao.getProponentes();
	}

	
	public void atualizaSenhaProponente(CadProponente proponente) {
		this.dao.atualizaSenhaProponente(proponente);
	}

	
	public List<String> buscaProponentePorNome(String nome) {
		return dao.buscaProponentePorNome(nome);
	}

	
	public List<CadProponente> pesquisaParametrizada(int first, int pageSize, String sortField, boolean sortOrder, Map<String, String> filtros) {
		List<CadProponente> proponentes = dao.buscaParametrizada(first, pageSize, sortField, sortOrder, filtros);
		for (CadProponente c : proponentes) {
			Hibernate.initialize(c.getPessoaProponente());
			Hibernate.initialize(c.getPessoaProponente().getDocumentos());
		}
		return proponentes;
	}

	
	public Long totalRow(Map<String, String> filtros) {
		return dao.rowCount(filtros);
	}



	
	public CadProponente buscaProponentePorId(Integer id) {
		return dao.buscaProponentePorId(id);
	}

	
	public List<CrpCorRaca> buscaListaCorRaca() {
		return dao.buscaCorRaca();
	}

	
	public CadProponente recuperarSenha(String email, Date dataNascimento, String cpf ) {
		return dao.recuperarSenha(email, dataNascimento, cpf );
	}

	
	public CadProponente login(String login, String senha) {
		CadProponente p = dao.login(login, senha);
		if(p!=null){
		Hibernate.initialize(p.getSituacaoCadastro());
		Hibernate.initialize(p.getPessoaProponente());
		}
		return p;
	}

}
