package br.gov.mc.cadsei.util;

import java.util.Arrays;
import java.util.List;

import javax.faces.application.NavigationHandler;
import javax.faces.context.FacesContext;
import javax.faces.event.PhaseEvent;
import javax.faces.event.PhaseId;
import javax.faces.event.PhaseListener;
import javax.servlet.http.HttpServletRequest;

import br.gov.arquitetura.excecao.ExcecaoUtil;
import br.gov.mc.cadsei.entidade.CadProponente;
import br.gov.mc.seg.client.web.Constants;

/**
 * 
 * Filtra todas as fases do ciclo de vida de um ManageadBean e verifica se o
 * usuário esta conectado. Se não estiver, redireciona para a página de
 * login.
 * 
 */
public class AuthorizationListener implements PhaseListener {

    private static final long serialVersionUID = 1L;

    private String redirect(String page) {
	return page.concat("?faces-redirect=true");
    }

    public void afterPhase(PhaseEvent event) {

	// Recupera o contexto do evento
	FacesContext context = event.getFacesContext();

	// Recupera a página requisitada
	String viewId = context.getViewRoot().getViewId();

	// Verifica se a view requisitada é do sistema externo
	boolean externalRequest = viewId.contains("externo");

	// Somente continua a execução do método caso a requisição tenha
	// sido externa
	// Esse listener somente é utilizado para requisições externas, que
	// não são
	// geridas pelo SEG
	HttpServletRequest request = (HttpServletRequest) context.getExternalContext().getRequest();
	if (!externalRequest) {
	    // Definido o timeout da sessão para o sistema interno em 120
	    // minutos
	    request.getSession().setMaxInactiveInterval(7200);
	    br.gov.mc.seg.client.consulta.entidade.Usuario usuario = (br.gov.mc.seg.client.consulta.entidade.Usuario) request
		    .getSession().getAttribute(Constants.USER_PARAMETER);
	    if (usuario != null && usuario.getLogin()!=null) {
		return;
	    }

	    String login_valido = (String) request.getSession().getAttribute("LOGIN_VALIDO");
	    if (!viewId.contains("login") && !viewId.contains("login_externo") && usuario == null) {
		return;
	    }
	    if (login_valido == null) {
		request.getSession().setAttribute("LOGIN_VALIDO", "LOGIN_INVALIDO");
		// this.redirectPage(context, "/pages/login.jsf");
		return;
	    }
	    if (usuario == null && login_valido.equalsIgnoreCase("LOGIN_INVALIDO")) {
		// this.redirectPage(context, "/pages/login.jsf");
		return;
	    }

	    return;

	} else {
	    // Definido o timeout da sessão para o sistema externo em 30
	    // minutos
	    request.getSession().setMaxInactiveInterval(1800);
	}

	// Recupera a página de login
	String loginPage = null;
	try {
	    loginPage = context.getExternalContext().getInitParameter("login-page");
	    if (loginPage == null || loginPage.isEmpty()) {
		throw new Exception(
			"N&atilde;o foi poss&iacute;vel acessar a aplica&ccidil;&atilde;o! Par&acirc;metro login-page n&atilde;o foi definido no arquivo web.xml ou esta nulo.");
	    }
	} catch (Exception e1) {
	    ExcecaoUtil.tratarExcecao(e1);
	    return;
	}

	// Recuperar URL's que não serão validadas pelo PhaseListener
	List<String> authExceptions = null;
	try {
	    String[] exceptions = null;
	    if (context.getExternalContext().getInitParameter("authorization-exceptions") != null) {
		exceptions = context.getExternalContext().getInitParameter("authorization-exceptions").split(",");
	    }
	    if (exceptions == null || exceptions.length == 0 || exceptions[0].isEmpty()) {
		throw new Exception(
			"N&atilde;o foi poss&iacute;vel acessar a aplica&ccedil;&atilde;o! Par&acirc;metro authorization-exceptions n&atilde;o foi definido no arquivo web.xml ou esta nulo.");
	    }
	    authExceptions = Arrays.asList(exceptions);
	} catch (Exception e) {
	    ExcecaoUtil.tratarExcecao(e);
	    return;
	}

	// Recupera o proponente da sessão
	CadProponente proponente = (CadProponente) context.getExternalContext().getSessionMap().get("proponente");
	if (proponente != null && viewId.equals(loginPage)) {
	    this.redirectPage(context, "principal");
	}

	// Se a página requisitada não existir na lista de exceções, valida
	// o usuário
	if (!authExceptions.contains(viewId)) {

	    // Caso o proponente ainda não exista na sessão redireciona para a
	    // página de login,
	    // se já existir, direciona para a página principal da
	    // aplicação.
	    if (proponente == null) {
		this.redirectPage(context, loginPage);
	    }

	}

    }

    private void redirectPage(FacesContext context, String page) {
	// Recupera o handler de nevegação para redirecionamento
	NavigationHandler handler = context.getApplication().getNavigationHandler();
	handler.handleNavigation(context, null, this.redirect(page));
	context.renderResponse();
    }

    public void beforePhase(PhaseEvent event) {
    }

    public PhaseId getPhaseId() {
	return PhaseId.RESTORE_VIEW;
    }

}